@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2003, 2025, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2014, 2021, Red Hat Inc. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -23,7 +23,6 @@
  *
  */
 
-#include "precompiled.hpp"
 
 #include "asm/assembler.hpp"
 #include "asm/assembler.inline.hpp"
@@ -231,27 +230,27 @@ public:
         br(Assembler::EQ, _rounds_52);
       }
       break;
-    case  2:  aes_round(_data, _subkeys +  0);  break;
-    case  3:  aes_round(_data, _subkeys +  1);  break;
+    case  2:  aes_round(_data, as_FloatRegister(_subkeys->encoding() + 0));  break;
+    case  3:  aes_round(_data, as_FloatRegister(_subkeys->encoding() + 1));  break;
     case  4:
       if (_once)  bind(_rounds_52);
       break;
-    case  5:  aes_round(_data, _subkeys +  2);  break;
-    case  6:  aes_round(_data, _subkeys +  3);  break;
+    case  5:  aes_round(_data, as_FloatRegister(_subkeys->encoding() + 2));  break;
+    case  6:  aes_round(_data, as_FloatRegister(_subkeys->encoding() + 3));  break;
     case  7:
       if (_once)  bind(_rounds_44);
       break;
-    case  8:  aes_round(_data, _subkeys +  4);  break;
-    case  9:  aes_round(_data, _subkeys +  5);  break;
-    case 10:  aes_round(_data, _subkeys +  6);  break;
-    case 11:  aes_round(_data, _subkeys +  7);  break;
-    case 12:  aes_round(_data, _subkeys +  8);  break;
-    case 13:  aes_round(_data, _subkeys +  9);  break;
-    case 14:  aes_round(_data, _subkeys + 10);  break;
-    case 15:  aes_round(_data, _subkeys + 11);  break;
-    case 16:  aes_round(_data, _subkeys + 12);  break;
-    case 17:  aese(_data, _subkeys + 13);  break;
-    case 18:  eor(_data, T16B, _data, _subkeys + 14);  break;
+    case  8:  aes_round(_data, as_FloatRegister(_subkeys->encoding() +  4));  break;
+    case  9:  aes_round(_data, as_FloatRegister(_subkeys->encoding() +  5));  break;
+    case 10:  aes_round(_data, as_FloatRegister(_subkeys->encoding() +  6));  break;
+    case 11:  aes_round(_data, as_FloatRegister(_subkeys->encoding() +  7));  break;
+    case 12:  aes_round(_data, as_FloatRegister(_subkeys->encoding() +  8));  break;
+    case 13:  aes_round(_data, as_FloatRegister(_subkeys->encoding() +  9));  break;
+    case 14:  aes_round(_data, as_FloatRegister(_subkeys->encoding() + 10));  break;
+    case 15:  aes_round(_data, as_FloatRegister(_subkeys->encoding() + 11));  break;
+    case 16:  aes_round(_data, as_FloatRegister(_subkeys->encoding() + 12));  break;
+    case 17:  aese(_data, as_FloatRegister(_subkeys->encoding() + 13));  break;
+    case 18:  eor(_data, T16B, _data, as_FloatRegister(_subkeys->encoding() + 14));  break;
     case 19:
       if (_to != noreg) {
         st1(_data, T16B, _to);
@@ -264,7 +263,7 @@ public:
   virtual KernelGenerator *next() {
     return new AESKernelGenerator(this, _unrolls,
                                   _from, _to, _keylen,
-                                  _data + 1, _subkeys, /*once*/false);
+                                  _data->successor(), _subkeys, /*once*/false);
   }
 
   virtual int length() { return 20; }
@@ -359,7 +358,7 @@ public:
                          FloatRegister b,
                          /* non-offsetted (shared) registers */
                          FloatRegister a, FloatRegister a1_xor_a0, FloatRegister p, FloatRegister vzr,
-                         /* offseted (temp) registers */
+                         /* offsetted (temp) registers */
                          FloatRegister tmp1, FloatRegister tmp2, FloatRegister tmp3)
     : KernelGenerator(as, unrolls),
       _result_lo(result_lo), _result_hi(result_hi), _b(b),
@@ -409,14 +408,14 @@ public:
     }
   }
 
-  virtual KernelGenerator *next() {
-    GHASHMultiplyGenerator *result = new GHASHMultiplyGenerator(*this);
-    result->_result_lo += register_stride;
-    result->_result_hi += register_stride;
-    result->_b += register_stride;
-    result->_tmp1 += register_stride;
-    result->_tmp2 += register_stride;
-    result->_tmp3 += register_stride;
+  virtual KernelGenerator* next() {
+    GHASHMultiplyGenerator* result = new GHASHMultiplyGenerator(*this);
+    result->_result_lo = as_FloatRegister(result->_result_lo->encoding() + register_stride);
+    result->_result_hi = as_FloatRegister(result->_result_hi->encoding() + register_stride);
+    result->_b         = as_FloatRegister(result->_b        ->encoding() + register_stride);
+    result->_tmp1      = as_FloatRegister(result->_tmp1     ->encoding() + register_stride);
+    result->_tmp2      = as_FloatRegister(result->_tmp2     ->encoding() + register_stride);
+    result->_tmp3      = as_FloatRegister(result->_tmp3     ->encoding() + register_stride);
     return result;
   }
 
@@ -436,7 +435,7 @@ public:
                        FloatRegister result, FloatRegister lo, FloatRegister hi,
                        /* non-offsetted (shared) registers */
                        FloatRegister p, FloatRegister vzr, FloatRegister data,
-                       /* offseted (temp) registers */
+                       /* offsetted (temp) registers */
                        FloatRegister t1)
     : KernelGenerator(as, unrolls),
       _result(result), _lo(lo), _hi(hi),
@@ -477,17 +476,17 @@ public:
     if (_data->is_valid() && _once) {
       assert(length() >= unrolls(), "not enough room for inteleaved loads");
       if (index < unrolls()) {
-        ld1((_data + index*register_stride), T16B, post(r2, 0x10));
+        ld1(as_FloatRegister(_data->encoding() + index*register_stride), T16B, post(r2, 0x10));
       }
     }
   }
 
   virtual KernelGenerator *next() {
     GHASHReduceGenerator *result = new GHASHReduceGenerator(*this);
-    result->_result += register_stride;
-    result->_hi += register_stride;
-    result->_lo += register_stride;
-    result->_t1 += register_stride;
+    result->_result = as_FloatRegister(result->_result->encoding() + register_stride);
+    result->_hi     = as_FloatRegister(result->_hi    ->encoding() + register_stride);
+    result->_lo     = as_FloatRegister(result->_lo    ->encoding() + register_stride);
+    result->_t1     = as_FloatRegister(result->_t1    ->encoding() + register_stride);
     result->_once = false;
     return result;
   }
@@ -508,7 +507,7 @@ void MacroAssembler::ghash_modmul(FloatRegister result,
 //
 // Clobbers all vector registers.
 //
-void MacroAssembler::ghash_processBlocks_wide(address field_polynomial, Register state,
+void MacroAssembler::ghash_processBlocks_wide(Label& field_polynomial, Register state,
                                               Register subkeyH,
                                               Register data, Register blocks, int unrolls) {
   int register_stride = 7;
@@ -521,7 +520,7 @@ void MacroAssembler::ghash_processBlocks_wide(address field_polynomial, Register
   // everything big-endian or reverse the bits in each byte and do
   // it little-endian.  On AArch64 it's more idiomatic to reverse
   // the bits in each byte (we have an instruction, RBIT, to do
-  // that) and keep the data in little-endian bit order throught the
+  // that) and keep the data in little-endian bit order through the
   // calculation, bit-reversing the inputs and outputs.
 
   assert(unrolls * register_stride < 32, "out of registers");
@@ -532,7 +531,10 @@ void MacroAssembler::ghash_processBlocks_wide(address field_polynomial, Register
   FloatRegister p = v31;
   eor(vzr, T16B, vzr, vzr); // zero register
 
-  ldrq(p, field_polynomial);    // The field polynomial
+  // load polynomial via label which must identify local data in the
+  // same code stub
+  adr(rscratch1, field_polynomial);
+  ldrq(p, rscratch1);    // The field polynomial
 
   ldrq(v0, Address(state));
   ldrq(Hprime, Address(subkeyH));
@@ -582,7 +584,8 @@ void MacroAssembler::ghash_processBlocks_wide(address field_polynomial, Register
   // v0 contains the initial state. Clear the others.
   for (int i = 1; i < unrolls; i++) {
     int ofs = register_stride * i;
-    eor(ofs+v0, T16B, ofs+v0, ofs+v0); // zero each state register
+    FloatRegister v0_ofs = as_FloatRegister(v0->encoding() + ofs);
+    eor(v0_ofs, T16B, v0_ofs, v0_ofs); // zero each state register
   }
 
   ext(a1_xor_a0, T16B, Hprime, Hprime, 0x08); // long-swap subkeyH into a1_xor_a0
@@ -590,7 +593,8 @@ void MacroAssembler::ghash_processBlocks_wide(address field_polynomial, Register
 
   // Load #unrolls blocks of data
   for (int ofs = 0; ofs < unrolls * register_stride; ofs += register_stride) {
-    ld1(v2+ofs, T16B, post(data, 0x10));
+    FloatRegister v2_ofs = as_FloatRegister(v2->encoding() + ofs);
+    ld1(v2_ofs, T16B, post(data, 0x10));
   }
 
   // Register assignments, replicated across 4 clones, v0 ... v23
@@ -623,8 +627,10 @@ void MacroAssembler::ghash_processBlocks_wide(address field_polynomial, Register
 
     // Xor data into current state
     for (int ofs = 0; ofs < unrolls * register_stride; ofs += register_stride) {
-      rbit((v2+ofs), T16B, (v2+ofs));
-      eor((v2+ofs), T16B, v0+ofs, (v2+ofs));   // bit-swapped data ^ bit-swapped state
+      FloatRegister v0_ofs = as_FloatRegister(v0->encoding() + ofs);
+      FloatRegister v2_ofs = as_FloatRegister(v2->encoding() + ofs);
+      rbit(v2_ofs, T16B, v2_ofs);
+      eor(v2_ofs, T16B, v0_ofs, v2_ofs);   // bit-swapped data ^ bit-swapped state
     }
 
     // Generate fully-unrolled multiply-reduce in two stages.
@@ -651,24 +657,31 @@ void MacroAssembler::ghash_processBlocks_wide(address field_polynomial, Register
   // First, we multiply/reduce each clone by the appropriate power of H.
   for (int i = 0; i < unrolls; i++) {
     int ofs = register_stride * i;
+    FloatRegister v0_ofs = as_FloatRegister(v0->encoding() + ofs);
+    FloatRegister v1_ofs = as_FloatRegister(v1->encoding() + ofs);
+    FloatRegister v2_ofs = as_FloatRegister(v2->encoding() + ofs);
+    FloatRegister v3_ofs = as_FloatRegister(v3->encoding() + ofs);
+    FloatRegister v4_ofs = as_FloatRegister(v4->encoding() + ofs);
+    FloatRegister v5_ofs = as_FloatRegister(v5->encoding() + ofs);
+
     ldrq(Hprime, Address(subkeyH, 16 * (unrolls - i - 1)));
 
-    rbit(v2+ofs, T16B, v2+ofs);
-    eor(v2+ofs, T16B, ofs+v0, v2+ofs);   // bit-swapped data ^ bit-swapped state
+    rbit(v2_ofs, T16B, v2_ofs);
+    eor(v2_ofs, T16B, v0_ofs, v2_ofs);   // bit-swapped data ^ bit-swapped state
 
     rev64(Hprime, T16B, Hprime);
     rbit(Hprime, T16B, Hprime);
     ext(a1_xor_a0, T16B, Hprime, Hprime, 0x08); // long-swap subkeyH into a1_xor_a0
     eor(a1_xor_a0, T16B, a1_xor_a0, Hprime);    // xor subkeyH into subkeyL (Karatsuba: (A1+A0))
-    ghash_modmul(/*result*/v0+ofs, /*result_lo*/v5+ofs, /*result_hi*/v4+ofs, /*b*/v2+ofs,
+    ghash_modmul(/*result*/v0_ofs, /*result_lo*/v5_ofs, /*result_hi*/v4_ofs, /*b*/v2_ofs,
                  Hprime, vzr, a1_xor_a0, p,
-                 /*temps*/v1+ofs, v3+ofs, /* reuse b*/v2+ofs);
+                 /*temps*/v1_ofs, v3_ofs, /* reuse b*/v2_ofs);
   }
 
   // Then we sum the results.
-  for (int i = 0; i < unrolls - 1; i++) {
-    int ofs = register_stride * i;
-    eor(v0, T16B, v0, v0 + register_stride + ofs);
+  for (int i = 1; i < unrolls; i++) {
+    FloatRegister v0_ofs = as_FloatRegister(v0->encoding() + register_stride * i);
+    eor(v0, T16B, v0, v0_ofs);
   }
 
   sub(blocks, blocks, (unsigned char)unrolls);
