@@ -41,25 +41,8 @@ void C1SafepointPollStub::emit_code(LIR_Assembler* ce) {
   if (UseSIGTRAP) {
     DEBUG_ONLY( __ should_not_reach_here("C1SafepointPollStub::emit_code"); )
   } else {
-    assert(SharedRuntime::polling_page_return_handler_blob() != nullptr,
-           "polling page return stub not created yet");
-    address stub = SharedRuntime::polling_page_return_handler_blob()->entry_point();
-
     __ bind(_entry);
-    // Using pc relative address computation.
-    {
-      Label next_pc;
-      __ bl(next_pc);
-      __ bind(next_pc);
-    }
-    int current_offset = __ offset();
-    __ mflr(R12);
-    __ add_const_optimized(R12, R12, safepoint_offset() - current_offset);
-    __ std(R12, in_bytes(JavaThread::saved_exception_pc_offset()), R16_thread);
-
-    __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(stub));
-    __ mtctr(R0);
-    __ bctr();
+    __ jump_to_polling_page_return_handler_blob(safepoint_offset());
   }
 }
 
@@ -67,19 +50,19 @@ void RangeCheckStub::emit_code(LIR_Assembler* ce) {
   __ bind(_entry);
 
   if (_info->deoptimize_on_exception()) {
-    address a = Runtime1::entry_for(C1StubId::predicate_failed_trap_id);
+    address a = Runtime1::entry_for(StubId::c1_predicate_failed_trap_id);
     //__ load_const_optimized(R0, a);
     __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(a));
     __ mtctr(R0);
     __ bctrl();
     ce->add_call_info_here(_info);
     ce->verify_oop_map(_info);
-    debug_only(__ illtrap());
+    DEBUG_ONLY(__ illtrap());
     return;
   }
 
-  address stub = _throw_index_out_of_bounds_exception ? Runtime1::entry_for(C1StubId::throw_index_exception_id)
-                                                      : Runtime1::entry_for(C1StubId::throw_range_check_failed_id);
+  address stub = _throw_index_out_of_bounds_exception ? Runtime1::entry_for(StubId::c1_throw_index_exception_id)
+                                                      : Runtime1::entry_for(StubId::c1_throw_range_check_failed_id);
   //__ load_const_optimized(R0, stub);
   __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(stub));
   __ mtctr(R0);
@@ -98,7 +81,7 @@ void RangeCheckStub::emit_code(LIR_Assembler* ce) {
   __ bctrl();
   ce->add_call_info_here(_info);
   ce->verify_oop_map(_info);
-  debug_only(__ illtrap());
+  DEBUG_ONLY(__ illtrap());
 }
 
 
@@ -108,14 +91,14 @@ PredicateFailedStub::PredicateFailedStub(CodeEmitInfo* info) {
 
 void PredicateFailedStub::emit_code(LIR_Assembler* ce) {
   __ bind(_entry);
-  address a = Runtime1::entry_for(C1StubId::predicate_failed_trap_id);
+  address a = Runtime1::entry_for(StubId::c1_predicate_failed_trap_id);
   //__ load_const_optimized(R0, a);
   __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(a));
   __ mtctr(R0);
   __ bctrl();
   ce->add_call_info_here(_info);
   ce->verify_oop_map(_info);
-  debug_only(__ illtrap());
+  DEBUG_ONLY(__ illtrap());
 }
 
 
@@ -132,7 +115,7 @@ void CounterOverflowStub::emit_code(LIR_Assembler* ce) {
   __ load_const_optimized(R0, md.value());
   __ std(R0, -8, R1_SP);
 
-  address a = Runtime1::entry_for(C1StubId::counter_overflow_id);
+  address a = Runtime1::entry_for(StubId::c1_counter_overflow_id);
   //__ load_const_optimized(R0, a);
   __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(a));
   __ mtctr(R0);
@@ -149,14 +132,14 @@ void DivByZeroStub::emit_code(LIR_Assembler* ce) {
     ce->compilation()->implicit_exception_table()->append(_offset, __ offset());
   }
   __ bind(_entry);
-  address stub = Runtime1::entry_for(C1StubId::throw_div0_exception_id);
+  address stub = Runtime1::entry_for(StubId::c1_throw_div0_exception_id);
   //__ load_const_optimized(R0, stub);
   __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(stub));
   __ mtctr(R0);
   __ bctrl();
   ce->add_call_info_here(_info);
   ce->verify_oop_map(_info);
-  debug_only(__ illtrap());
+  DEBUG_ONLY(__ illtrap());
 }
 
 
@@ -164,9 +147,9 @@ void ImplicitNullCheckStub::emit_code(LIR_Assembler* ce) {
   address a;
   if (_info->deoptimize_on_exception()) {
     // Deoptimize, do not throw the exception, because it is probably wrong to do it here.
-    a = Runtime1::entry_for(C1StubId::predicate_failed_trap_id);
+    a = Runtime1::entry_for(StubId::c1_predicate_failed_trap_id);
   } else {
-    a = Runtime1::entry_for(C1StubId::throw_null_pointer_exception_id);
+    a = Runtime1::entry_for(StubId::c1_throw_null_pointer_exception_id);
   }
 
   if (ImplicitNullChecks || TrapBasedNullChecks) {
@@ -179,7 +162,7 @@ void ImplicitNullCheckStub::emit_code(LIR_Assembler* ce) {
   __ bctrl();
   ce->add_call_info_here(_info);
   ce->verify_oop_map(_info);
-  debug_only(__ illtrap());
+  DEBUG_ONLY(__ illtrap());
 }
 
 
@@ -193,19 +176,19 @@ void SimpleExceptionStub::emit_code(LIR_Assembler* ce) {
   __ mtctr(R0);
   __ bctrl();
   ce->add_call_info_here(_info);
-  debug_only( __ illtrap(); )
+  DEBUG_ONLY( __ illtrap(); )
 }
 
 
 // Implementation of NewInstanceStub
-NewInstanceStub::NewInstanceStub(LIR_Opr klass_reg, LIR_Opr result, ciInstanceKlass* klass, CodeEmitInfo* info, C1StubId stub_id) {
+NewInstanceStub::NewInstanceStub(LIR_Opr klass_reg, LIR_Opr result, ciInstanceKlass* klass, CodeEmitInfo* info, StubId stub_id) {
   _result = result;
   _klass = klass;
   _klass_reg = klass_reg;
   _info = new CodeEmitInfo(info);
-  assert(stub_id == C1StubId::new_instance_id                 ||
-         stub_id == C1StubId::fast_new_instance_id            ||
-         stub_id == C1StubId::fast_new_instance_init_check_id,
+  assert(stub_id == StubId::c1_new_instance_id                 ||
+         stub_id == StubId::c1_fast_new_instance_id            ||
+         stub_id == StubId::c1_fast_new_instance_init_check_id,
          "need new_instance id");
   _stub_id = stub_id;
 }
@@ -235,7 +218,7 @@ NewTypeArrayStub::NewTypeArrayStub(LIR_Opr klass_reg, LIR_Opr length, LIR_Opr re
 void NewTypeArrayStub::emit_code(LIR_Assembler* ce) {
   __ bind(_entry);
 
-  address entry = Runtime1::entry_for(C1StubId::new_type_array_id);
+  address entry = Runtime1::entry_for(StubId::c1_new_type_array_id);
   //__ load_const_optimized(R0, entry);
   __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(entry));
   __ mr_if_needed(/*op->tmp1()->as_register()*/ R5_ARG3, _length->as_register()); // already sign-extended
@@ -258,7 +241,7 @@ NewObjectArrayStub::NewObjectArrayStub(LIR_Opr klass_reg, LIR_Opr length, LIR_Op
 void NewObjectArrayStub::emit_code(LIR_Assembler* ce) {
   __ bind(_entry);
 
-  address entry = Runtime1::entry_for(C1StubId::new_object_array_id);
+  address entry = Runtime1::entry_for(StubId::c1_new_object_array_id);
   //__ load_const_optimized(R0, entry);
   __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(entry));
   __ mr_if_needed(/*op->tmp1()->as_register()*/ R5_ARG3, _length->as_register()); // already sign-extended
@@ -271,7 +254,7 @@ void NewObjectArrayStub::emit_code(LIR_Assembler* ce) {
 
 void MonitorEnterStub::emit_code(LIR_Assembler* ce) {
   __ bind(_entry);
-  address stub = Runtime1::entry_for(ce->compilation()->has_fpu_code() ? C1StubId::monitorenter_id : C1StubId::monitorenter_nofpu_id);
+  address stub = Runtime1::entry_for(ce->compilation()->has_fpu_code() ? StubId::c1_monitorenter_id : StubId::c1_monitorenter_nofpu_id);
   //__ load_const_optimized(R0, stub);
   __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(stub));
   __ mr_if_needed(/*scratch_opr()->as_register()*/ R4_ARG2, _obj_reg->as_register());
@@ -288,7 +271,7 @@ void MonitorExitStub::emit_code(LIR_Assembler* ce) {
   if (_compute_lock) {
     ce->monitor_address(_monitor_ix, _lock_reg);
   }
-  address stub = Runtime1::entry_for(ce->compilation()->has_fpu_code() ? C1StubId::monitorexit_id : C1StubId::monitorexit_nofpu_id);
+  address stub = Runtime1::entry_for(ce->compilation()->has_fpu_code() ? StubId::c1_monitorexit_id : StubId::c1_monitorexit_nofpu_id);
   //__ load_const_optimized(R0, stub);
   __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(stub));
   assert(_lock_reg->as_register() == R4_ARG2, "");
@@ -402,12 +385,12 @@ void PatchingStub::emit_code(LIR_Assembler* ce) {
   address target = nullptr;
   relocInfo::relocType reloc_type = relocInfo::none;
   switch (_id) {
-    case access_field_id:  target = Runtime1::entry_for(C1StubId::access_field_patching_id); break;
-    case load_klass_id:    target = Runtime1::entry_for(C1StubId::load_klass_patching_id);
+    case access_field_id:  target = Runtime1::entry_for(StubId::c1_access_field_patching_id); break;
+    case load_klass_id:    target = Runtime1::entry_for(StubId::c1_load_klass_patching_id);
                            reloc_type = relocInfo::metadata_type; break;
-    case load_mirror_id:   target = Runtime1::entry_for(C1StubId::load_mirror_patching_id);
+    case load_mirror_id:   target = Runtime1::entry_for(StubId::c1_load_mirror_patching_id);
                            reloc_type = relocInfo::oop_type; break;
-    case load_appendix_id: target = Runtime1::entry_for(C1StubId::load_appendix_patching_id);
+    case load_appendix_id: target = Runtime1::entry_for(StubId::c1_load_appendix_patching_id);
                            reloc_type = relocInfo::oop_type; break;
     default: ShouldNotReachHere();
   }
@@ -433,7 +416,7 @@ void PatchingStub::emit_code(LIR_Assembler* ce) {
 
 void DeoptimizeStub::emit_code(LIR_Assembler* ce) {
   __ bind(_entry);
-  address stub = Runtime1::entry_for(C1StubId::deoptimize_id);
+  address stub = Runtime1::entry_for(StubId::c1_deoptimize_id);
   //__ load_const_optimized(R0, stub);
   __ add_const_optimized(R0, R29_TOC, MacroAssembler::offset_to_global_toc(stub));
   __ mtctr(R0);
@@ -441,7 +424,7 @@ void DeoptimizeStub::emit_code(LIR_Assembler* ce) {
   __ load_const_optimized(R0, _trap_request); // Pass trap request in R0.
   __ bctrl();
   ce->add_call_info_here(_info);
-  debug_only(__ illtrap());
+  DEBUG_ONLY(__ illtrap());
 }
 
 
