@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2021, 2025, Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2021, 2024 SAP SE. All rights reserved.
+ * Copyright (c) 2021, 2025 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -114,7 +114,6 @@ public:
   }
 };
 
-
 void ZBarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorators, BasicType type,
                                    Register base, RegisterOrConstant ind_or_offs, Register dst,
                                    Register tmp1, Register tmp2,
@@ -169,7 +168,7 @@ void ZBarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorators
   // if the pointer is not dirty.
   // Only dirty pointers must be processed by this barrier, so we can skip it in case the latter condition holds true.
   __ and_(tmp1, tmp1, dst);
-  __ beq(CCR0, uncolor);
+  __ beq(CR0, uncolor);
 
   /* ==== Invoke barrier ==== */
   {
@@ -193,8 +192,8 @@ void ZBarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorators
 
   // Slow-path has already uncolored
   if (L_handle_null != nullptr) {
-    __ cmpdi(CCR0, dst, 0);
-    __ beq(CCR0, *L_handle_null);
+    __ cmpdi(CR0, dst, 0);
+    __ beq(CR0, *L_handle_null);
   }
   __ b(done);
 
@@ -203,7 +202,7 @@ void ZBarrierSetAssembler::load_at(MacroAssembler* masm, DecoratorSet decorators
     __ srdi(dst, dst, ZPointerLoadShift);
   } else {
     __ srdi_(dst, dst, ZPointerLoadShift);
-    __ beq(CCR0, *L_handle_null);
+    __ beq(CR0, *L_handle_null);
   }
 
   __ bind(done);
@@ -234,7 +233,7 @@ static void emit_store_fast_path_check(MacroAssembler* masm, Register base, Regi
     // A not relocatable object could have spurious raw null pointers in its fields after
     // getting promoted to the old generation.
     __ relocate(barrier_Relocation::spec(), ZBarrierRelocationFormatStoreGoodBits);
-    __ cmplwi(CCR0, R0, barrier_Relocation::unpatched);
+    __ cmplwi(CR0, R0, barrier_Relocation::unpatched);
   } else {
     __ ld(R0, ind_or_offs, base);
     // Stores on relocatable objects never need to deal with raw null pointers in fields.
@@ -244,7 +243,7 @@ static void emit_store_fast_path_check(MacroAssembler* masm, Register base, Regi
     __ relocate(barrier_Relocation::spec(), ZBarrierRelocationFormatStoreBadMask);
     __ andi_(R0, R0, barrier_Relocation::unpatched);
   }
-  __ bc_far_optimized(Assembler::bcondCRbiIs0, __ bi0(CCR0, Assembler::equal), medium_path);
+  __ bc_far_optimized(Assembler::bcondCRbiIs0, __ bi0(CR0, Assembler::equal), medium_path);
 }
 
 void ZBarrierSetAssembler::store_barrier_fast(MacroAssembler* masm,
@@ -274,7 +273,7 @@ void ZBarrierSetAssembler::store_barrier_fast(MacroAssembler* masm,
     __ ld(R0, ind_or_offset, ref_base);
     __ ld(rnew_zpointer, in_bytes(ZThreadLocalData::store_bad_mask_offset()), R16_thread);
     __ and_(R0, R0, rnew_zpointer);
-    __ bne(CCR0, medium_path);
+    __ bne(CR0, medium_path);
     __ bind(medium_path_continuation);
     __ ld(rnew_zpointer, in_bytes(ZThreadLocalData::store_good_mask_offset()), R16_thread);
   }
@@ -293,7 +292,7 @@ static void store_barrier_buffer_add(MacroAssembler* masm,
   // Combined pointer bump and check if the buffer is disabled or full
   __ ld(R0, in_bytes(ZStoreBarrierBuffer::current_offset()), tmp1);
   __ addic_(R0, R0, -(int)sizeof(ZStoreBarrierEntry));
-  __ blt(CCR0, slow_path);
+  __ blt(CR0, slow_path);
   __ std(R0, in_bytes(ZStoreBarrierBuffer::current_offset()), tmp1);
 
   // Entry is at ZStoreBarrierBuffer (tmp1) + buffer_offset + scaled index (R0)
@@ -327,8 +326,8 @@ void ZBarrierSetAssembler::store_barrier_medium(MacroAssembler* masm,
     // Atomic accesses can get to the medium fast path because the value was a
     // raw null value. If it was not null, then there is no doubt we need to take a slow path.
     __ ld(tmp, ind_or_offs, ref_base);
-    __ cmpdi(CCR0, tmp, 0);
-    __ bne(CCR0, slow_path);
+    __ cmpdi(CR0, tmp, 0);
+    __ bne(CR0, slow_path);
 
     // If we get this far, we know there is a young raw null value in the field.
     // Try to self-heal null values for atomic accesses
@@ -338,12 +337,12 @@ void ZBarrierSetAssembler::store_barrier_medium(MacroAssembler* masm,
       need_restore = true;
     }
     __ ld(R0, in_bytes(ZThreadLocalData::store_good_mask_offset()), R16_thread);
-    __ cmpxchgd(CCR0, tmp, (intptr_t)0, R0, ref_base,
+    __ cmpxchgd(CR0, tmp, (intptr_t)0, R0, ref_base,
                 MacroAssembler::MemBarNone, MacroAssembler::cmpxchgx_hint_atomic_update(),
                 noreg, need_restore ? nullptr : &slow_path);
     if (need_restore) {
       __ sub(ref_base, ref_base, ind_or_offs);
-      __ bne(CCR0, slow_path);
+      __ bne(CR0, slow_path);
     }
   } else {
     // A non-atomic relocatable object won't get to the medium fast path due to a
@@ -447,7 +446,7 @@ void ZBarrierSetAssembler::copy_load_at_fast(MacroAssembler* masm,
                                              Label& continuation) const {
   __ ldx(zpointer, addr);
   __ and_(R0, zpointer, load_bad_mask);
-  __ bne(CCR0, slow_path);
+  __ bne(CR0, slow_path);
   __ bind(continuation);
 }
 void ZBarrierSetAssembler::copy_load_at_slow(MacroAssembler* masm,
@@ -480,7 +479,7 @@ void ZBarrierSetAssembler::copy_store_at_fast(MacroAssembler* masm,
   if (!dest_uninitialized) {
     __ ldx(R0, addr);
     __ and_(R0, R0, store_bad_mask);
-    __ bne(CCR0, medium_path);
+    __ bne(CR0, medium_path);
     __ bind(continuation);
   }
   __ rldimi(zpointer, store_good_mask, 0, 64 - ZPointerLoadShift); // Replace color bits.
@@ -515,8 +514,8 @@ void ZBarrierSetAssembler::copy_store_at_slow(MacroAssembler* masm,
 void ZBarrierSetAssembler::generate_disjoint_oop_copy(MacroAssembler* masm, bool dest_uninitialized) {
   const Register zpointer = R2, tmp = R9;
   Label done, loop, load_bad, load_good, store_bad, store_good;
-  __ cmpdi(CCR0, R5_ARG3, 0);
-  __ beq(CCR0, done);
+  __ cmpdi(CR0, R5_ARG3, 0);
+  __ beq(CR0, done);
   __ mtctr(R5_ARG3);
 
   __ align(32);
@@ -539,7 +538,7 @@ void ZBarrierSetAssembler::generate_conjoint_oop_copy(MacroAssembler* masm, bool
   const Register zpointer = R2, tmp = R9;
   Label done, loop, load_bad, load_good, store_bad, store_good;
   __ sldi_(R0, R5_ARG3, 3);
-  __ beq(CCR0, done);
+  __ beq(CR0, done);
   __ mtctr(R5_ARG3);
   // Point behind last elements and copy backwards.
   __ add(R3_ARG1, R3_ARG1, R0);
@@ -561,7 +560,6 @@ void ZBarrierSetAssembler::generate_conjoint_oop_copy(MacroAssembler* masm, bool
   copy_store_at_slow(masm, R4_ARG2, tmp, store_bad, store_good, dest_uninitialized);
 }
 
-
 // Verify a colored pointer.
 void ZBarrierSetAssembler::check_oop(MacroAssembler *masm, Register obj, const char* msg) {
   if (!VerifyOops) {
@@ -570,12 +568,12 @@ void ZBarrierSetAssembler::check_oop(MacroAssembler *masm, Register obj, const c
   Label done, skip_uncolor;
   // Skip (colored) null.
   __ srdi_(R0, obj, ZPointerLoadShift);
-  __ beq(CCR0, done);
+  __ beq(CR0, done);
 
   // Check if ZAddressHeapBase << ZPointerLoadShift is set. If so, we need to uncolor.
   __ rldicl_(R0, obj, 64 - ZAddressHeapBaseShift - ZPointerLoadShift, 63);
   __ mr(R0, obj);
-  __ beq(CCR0, skip_uncolor);
+  __ beq(CR0, skip_uncolor);
   __ srdi(R0, obj, ZPointerLoadShift);
   __ bind(skip_uncolor);
 
@@ -583,7 +581,6 @@ void ZBarrierSetAssembler::check_oop(MacroAssembler *masm, Register obj, const c
   __ bind(done);
 }
 
-
 void ZBarrierSetAssembler::try_resolve_jobject_in_native(MacroAssembler* masm, Register dst, Register jni_env,
                                                          Register obj, Register tmp, Label& slowpath) {
   __ block_comment("try_resolve_jobject_in_native (zgc) {");
@@ -594,7 +591,7 @@ void ZBarrierSetAssembler::try_resolve_jobject_in_native(MacroAssembler* masm, R
 
   // Test for tag
   __ andi_(tmp, obj, JNIHandles::tag_mask);
-  __ bne(CCR0, tagged);
+  __ bne(CR0, tagged);
 
   // Resolve local handle
   __ ld(dst, 0, obj);
@@ -605,7 +602,7 @@ void ZBarrierSetAssembler::try_resolve_jobject_in_native(MacroAssembler* masm, R
   // Test for weak tag
   __ andi_(tmp, obj, JNIHandles::TypeTag::weak_global);
   __ clrrdi(dst, obj, JNIHandles::tag_size); // Untag.
-  __ bne(CCR0, weak_tagged);
+  __ bne(CR0, weak_tagged);
 
   // Resolve global handle
   __ ld(dst, 0, dst);
@@ -620,7 +617,7 @@ void ZBarrierSetAssembler::try_resolve_jobject_in_native(MacroAssembler* masm, R
 
   __ bind(check_color);
   __ and_(tmp, tmp, dst);
-  __ bne(CCR0, slowpath);
+  __ bne(CR0, slowpath);
 
   // Uncolor
   __ srdi(dst, dst, ZPointerLoadShift);
@@ -666,7 +663,7 @@ void ZBarrierSetAssembler::generate_c1_load_barrier(LIR_Assembler* ce,
                                                     ZLoadBarrierStubC1* stub,
                                                     bool on_non_strong) const {
   check_color(ce, ref, on_non_strong);
-  __ bc_far_optimized(Assembler::bcondCRbiIs0, __ bi0(CCR0, Assembler::equal), *stub->entry());
+  __ bc_far_optimized(Assembler::bcondCRbiIs0, __ bi0(CR0, Assembler::equal), *stub->entry());
   z_uncolor(ce, ref);
   __ bind(*stub->continuation());
 }
