@@ -78,15 +78,48 @@ private:
 
   void save() {
     MacroAssembler* masm = _masm;
-    __ push(rax);
-    __ push(rcx);
-    __ push(rdx);
-    __ push(rdi);
-    __ push(rsi);
-    __ push(r8);
-    __ push(r9);
-    __ push(r10);
-    __ push(r11);
+    if (VM_Version::supports_apx_f()) {
+      if (_result != rax) {
+        __ pushp(rax);
+      }
+      __ pushp(rcx);
+      // Save current stack pointer into rcx
+      __ movptr(rcx, rsp);
+      // Align stack pointer to 16 byte boundary. This is hard constraint
+      // for push2/pop2 with PPX hints.
+      __ andptr(rsp, -StackAlignmentInBytes);
+      // Push original stack pointer.
+      __ push(rcx);
+      // Restore the original contents of RCX register.
+      __ movptr(rcx, Address(rcx));
+      // Now push remaining caller save GPRs and EGPRs on 16B aligned stack.
+      // Note: For PPX to work properly, a PPX-marked PUSH2 (respectively, POP2) should always
+      // be matched with a PPX-marked POP2 (PUSH2), not with two PPX-marked POPs (PUSHs).
+      __ pushp(rdx);
+      __ push2p(rdi, rsi);
+      __ push2p(r8, r9);
+      __ push2p(r10, r11);
+      __ push2p(r16, r17);
+      __ push2p(r18, r19);
+      __ push2p(r20, r21);
+      __ push2p(r22, r23);
+      __ push2p(r24, r25);
+      __ push2p(r26, r27);
+      __ push2p(r28, r29);
+      __ push2p(r30, r31);
+    } else {
+      if (_result != rax) {
+        __ push(rax);
+      }
+      __ push(rcx);
+      __ push(rdx);
+      __ push(rdi);
+      __ push(rsi);
+      __ push(r8);
+      __ push(r9);
+      __ push(r10);
+      __ push(r11);
+    }
 
     if (_xmm_spill_size != 0) {
       __ subptr(rsp, _xmm_spill_size);
@@ -139,21 +172,43 @@ private:
       __ addptr(rsp, _xmm_spill_size);
     }
 
-    __ pop(r11);
-    __ pop(r10);
-    __ pop(r9);
-    __ pop(r8);
-    __ pop(rsi);
-    __ pop(rdi);
-    __ pop(rdx);
-    __ pop(rcx);
-    if (_result == noreg) {
-      __ pop(rax);
-    } else if (_result == rax) {
-      __ addptr(rsp, wordSize);
+    if (VM_Version::supports_apx_f()) {
+      __ pop2p(r31, r30);
+      __ pop2p(r29, r28);
+      __ pop2p(r27, r26);
+      __ pop2p(r25, r24);
+      __ pop2p(r23, r22);
+      __ pop2p(r21, r20);
+      __ pop2p(r19, r18);
+      __ pop2p(r17, r16);
+      __ pop2p(r11, r10);
+      __ pop2p(r9, r8);
+      __ pop2p(rsi, rdi);
+      __ popp(rdx);
+      // Re-instantiate original stack pointer.
+      __ movptr(rsp, Address(rsp));
+      __ popp(rcx);
+      if (_result != rax) {
+        if (_result != noreg) {
+          __ movptr(_result, rax);
+        }
+        __ popp(rax);
+      }
     } else {
-      __ movptr(_result, rax);
-      __ pop(rax);
+      __ pop(r11);
+      __ pop(r10);
+      __ pop(r9);
+      __ pop(r8);
+      __ pop(rsi);
+      __ pop(rdi);
+      __ pop(rdx);
+      __ pop(rcx);
+      if (_result != rax) {
+        if (_result != noreg) {
+          __ movptr(_result, rax);
+        }
+        __ pop(rax);
+      }
     }
   }
 
@@ -220,11 +275,10 @@ void ZBarrierSetAssembler::load_at(MacroAssembler* masm,
                                    BasicType type,
                                    Register dst,
                                    Address src,
-                                   Register tmp1,
-                                   Register tmp_thread) {
+                                   Register tmp1) {
   if (!ZBarrierSet::barrier_needed(decorators, type)) {
     // Barrier not needed
-    BarrierSetAssembler::load_at(masm, decorators, type, dst, src, tmp1, tmp_thread);
+    BarrierSetAssembler::load_at(masm, decorators, type, dst, src, tmp1);
     return;
   }
 
@@ -234,7 +288,7 @@ void ZBarrierSetAssembler::load_at(MacroAssembler* masm,
   Register scratch = tmp1;
   if (tmp1 == noreg) {
     scratch = r12;
-    __ push(scratch);
+    __ push_ppx(scratch);
   }
 
   assert_different_registers(dst, scratch);
@@ -294,7 +348,7 @@ void ZBarrierSetAssembler::load_at(MacroAssembler* masm,
 
   // Restore scratch register
   if (tmp1 == noreg) {
-    __ pop(scratch);
+    __ pop_ppx(scratch);
   }
 
   BLOCK_COMMENT("} ZBarrierSetAssembler::load_at");
@@ -408,10 +462,10 @@ void ZBarrierSetAssembler::store_barrier_fast(MacroAssembler* masm,
       __ movptr(rnew_zpointer, rnew_zaddress);
     }
     assert_different_registers(rcx, rnew_zpointer);
-    __ push(rcx);
+    __ push_ppx(rcx);
     __ movptr(rcx, ExternalAddress((address)&ZPointerLoadShift));
     __ shlq(rnew_zpointer);
-    __ pop(rcx);
+    __ pop_ppx(rcx);
     __ orq(rnew_zpointer, Address(r15_thread, ZThreadLocalData::store_good_mask_offset()));
   }
 }
@@ -429,7 +483,7 @@ static void store_barrier_buffer_add(MacroAssembler* masm,
   __ jcc(Assembler::equal, slow_path);
 
   Register tmp2 = r15_thread;
-  __ push(tmp2);
+  __ push_ppx(tmp2);
 
   // Bump the pointer
   __ movq(tmp2, Address(tmp1, ZStoreBarrierBuffer::current_offset()));
@@ -447,7 +501,7 @@ static void store_barrier_buffer_add(MacroAssembler* masm,
   __ movptr(tmp1, Address(tmp1, 0));
   __ movptr(Address(tmp2, in_bytes(ZStoreBarrierEntry::prev_offset())), tmp1);
 
-  __ pop(tmp2);
+  __ pop_ppx(tmp2);
 }
 
 void ZBarrierSetAssembler::store_barrier_medium(MacroAssembler* masm,
@@ -474,9 +528,9 @@ void ZBarrierSetAssembler::store_barrier_medium(MacroAssembler* masm,
 
     // If we get this far, we know there is a young raw null value in the field.
     // Try to self-heal null values for atomic accesses
-    __ push(rax);
-    __ push(rbx);
-    __ push(rcx);
+    __ push_ppx(rax);
+    __ push_ppx(rbx);
+    __ push_ppx(rcx);
 
     __ lea(rcx, ref_addr);
     __ xorq(rax, rax);
@@ -485,9 +539,9 @@ void ZBarrierSetAssembler::store_barrier_medium(MacroAssembler* masm,
     __ lock();
     __ cmpxchgq(rbx, Address(rcx, 0));
 
-    __ pop(rcx);
-    __ pop(rbx);
-    __ pop(rax);
+    __ pop_ppx(rcx);
+    __ pop_ppx(rbx);
+    __ pop_ppx(rax);
 
     __ jcc(Assembler::notEqual, slow_path);
 
@@ -529,10 +583,10 @@ void ZBarrierSetAssembler::store_at(MacroAssembler* masm,
       } else {
         __ movptr(tmp1, src);
       }
-      __ push(rcx);
+      __ push_ppx(rcx);
       __ movptr(rcx, ExternalAddress((address)&ZPointerLoadShift));
       __ shlq(tmp1);
-      __ pop(rcx);
+      __ pop_ppx(rcx);
       __ orq(tmp1, Address(r15_thread, ZThreadLocalData::store_good_mask_offset()));
     } else {
       Label done;
@@ -953,10 +1007,10 @@ void ZBarrierSetAssembler::try_resolve_jobject_in_native(MacroAssembler* masm,
     __ shrq(tmp);
     __ movptr(obj, tmp);
   } else {
-    __ push(rcx);
+    __ push_ppx(rcx);
     __ movptr(rcx, ExternalAddress((address)&ZPointerLoadShift));
     __ shrq(obj);
-    __ pop(rcx);
+    __ pop_ppx(rcx);
   }
 
   __ bind(done);
@@ -1035,7 +1089,7 @@ void ZBarrierSetAssembler::generate_c1_load_barrier_stub(LIR_Assembler* ce,
 
   // Save rax unless it is the result or tmp register
   if (ref != rax && tmp != rax) {
-    __ push(rax);
+    __ push_ppx(rax);
   }
 
   // Setup arguments and call runtime stub
@@ -1055,7 +1109,7 @@ void ZBarrierSetAssembler::generate_c1_load_barrier_stub(LIR_Assembler* ce,
 
   // Restore rax unless it is the result or tmp register
   if (ref != rax && tmp != rax) {
-    __ pop(rax);
+    __ pop_ppx(rax);
   }
 
   // Stub exit
@@ -1329,7 +1383,13 @@ void ZBarrierSetAssembler::patch_barrier_relocation(address addr, int format) {
   const uint16_t value = patch_barrier_relocation_value(format);
   uint8_t* const patch_addr = (uint8_t*)addr + offset;
   if (format == ZBarrierRelocationFormatLoadGoodBeforeShl) {
-    *patch_addr = (uint8_t)value;
+    if (VM_Version::supports_apx_f()) {
+      NativeInstruction* instruction = nativeInstruction_at(addr);
+      uint8_t* const rex2_patch_addr = patch_addr + (instruction->has_rex2_prefix() ? 1 : 0);
+      *rex2_patch_addr = (uint8_t)value;
+    } else {
+      *patch_addr = (uint8_t)value;
+    }
   } else {
     *(uint16_t*)patch_addr = value;
   }
@@ -1391,12 +1451,12 @@ void ZBarrierSetAssembler::check_oop(MacroAssembler* masm, Register obj, Registe
   // Uncolor presumed zpointer
   assert(obj != rcx, "bad choice of register");
   if (rcx != tmp1 && rcx != tmp2) {
-    __ push(rcx);
+    __ push_ppx(rcx);
   }
   __ movl(rcx, Address(tmp2, tmp1, Address::times_4, 0));
   __ shrq(obj);
   if (rcx != tmp1 && rcx != tmp2) {
-    __ pop(rcx);
+    __ pop_ppx(rcx);
   }
 
   __ jmp(check_zaddress);
