@@ -1,5 +1,5 @@
 /*
- *  Copyright (c) 2016, 2021, Intel Corporation. All rights reserved.
+ * Copyright (c) 2016, 2025, Intel Corporation. All rights reserved.
  * Intel Math Library (LIBM) Source Code
  *
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
@@ -173,7 +173,8 @@
 #define __ _masm->
 
 address StubGenerator::generate_libmCos() {
-  StubCodeMark mark(this, "StubRoutines", "libmCos");
+  StubId stub_id = StubId::stubgen_dcos_id;
+  StubCodeMark mark(this, stub_id);
   address start = __ pc();
 
   Label L_2TAG_PACKET_0_0_1, L_2TAG_PACKET_1_0_1, L_2TAG_PACKET_2_0_1, L_2TAG_PACKET_3_0_1;
@@ -184,11 +185,11 @@ address StubGenerator::generate_libmCos() {
   __ enter(); // required for proper stackwalking of RuntimeStub frame
 
 #ifdef _WIN64
-  __ push(rsi);
-  __ push(rdi);
+  __ push_ppx(rsi);
+  __ push_ppx(rdi);
 #endif
 
-  __ push(rbx);
+  __ push_ppx(rbx);
   __ subq(rsp, 16);
   __ movsd(Address(rsp, 8), xmm0);
 
@@ -608,11 +609,11 @@ address StubGenerator::generate_libmCos() {
 
   __ bind(B1_4);
   __ addq(rsp, 16);
-  __ pop(rbx);
+  __ pop_ppx(rbx);
 
 #ifdef _WIN64
-  __ pop(rdi);
-  __ pop(rsi);
+  __ pop_ppx(rdi);
+  __ pop_ppx(rsi);
 #endif
 
   __ leave(); // required for proper stackwalking of RuntimeStub frame
