@@ -1,6 +1,6 @@
 /*
-* Copyright (c) 2016, 2021, Intel Corporation. All rights reserved.
-* Copyright (C) 2021 THL A29 Limited, a Tencent company. All rights reserved.
+* Copyright (c) 2016, 2025, Intel Corporation. All rights reserved.
+* Copyright (C) 2021, Tencent. All rights reserved.
 * Intel Math Library (LIBM) Source Code
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
@@ -176,7 +176,8 @@ ATTRIBUTE_ALIGNED(16) static const juint _coeff[] =
 #define __ _masm->
 
 address StubGenerator::generate_libmLog() {
-  StubCodeMark mark(this, "StubRoutines", "libmLog");
+  StubId stub_id = StubId::stubgen_dlog_id;
+  StubCodeMark mark(this, stub_id);
   address start = __ pc();
 
   Label L_2TAG_PACKET_0_0_2, L_2TAG_PACKET_1_0_2, L_2TAG_PACKET_2_0_2, L_2TAG_PACKET_3_0_2;
@@ -514,7 +515,8 @@ ATTRIBUTE_ALIGNED(16) static const juint _coeff_log10[] =
 };
 
 address StubGenerator::generate_libmLog10() {
-  StubCodeMark mark(this, "StubRoutines", "libmLog10");
+  StubId stub_id = StubId::stubgen_dlog10_id;
+  StubCodeMark mark(this, stub_id);
   address start = __ pc();
 
   Label L_2TAG_PACKET_0_0_2, L_2TAG_PACKET_1_0_2, L_2TAG_PACKET_2_0_2, L_2TAG_PACKET_3_0_2;
