@@ -32,61 +32,33 @@
 // Implementation of the platform-specific part of StubRoutines - for
 // a description of how to extend it, see the stubRoutines.hpp file.
 
-address StubRoutines::x86::_verify_mxcsr_entry = nullptr;
-address StubRoutines::x86::_upper_word_mask_addr = nullptr;
-address StubRoutines::x86::_shuffle_byte_flip_mask_addr = nullptr;
+// define fields for arch-specific entries
+
+#define DEFINE_ARCH_ENTRY(arch, blob_name, stub_name, field_name, getter_name) \
+  address StubRoutines:: arch :: STUB_FIELD_NAME(field_name)  = nullptr;
+
+#define DEFINE_ARCH_ENTRY_INIT(arch, blob_name, stub_name, field_name, getter_name, init_function) \
+  address StubRoutines:: arch :: STUB_FIELD_NAME(field_name)  = CAST_FROM_FN_PTR(address, init_function);
+
+STUBGEN_ARCH_ENTRIES_DO(DEFINE_ARCH_ENTRY, DEFINE_ARCH_ENTRY_INIT)
+
+#undef DEFINE_ARCH_ENTRY_INIT
+#undef DEFINE_ARCH_ENTRY
+
+address StubRoutines::crc_table_addr() {
+  return (address)StubRoutines::x86::_crc_table;
+}
+address StubRoutines::crc32c_table_addr() {
+  if (StubRoutines::x86::_crc32c_table == nullptr) {
+    bool supports_clmul = VM_Version::supports_clmul();
+    StubRoutines::x86::generate_CRC32C_table(supports_clmul);
+  }
+  return (address)StubRoutines::x86::_crc32c_table;
+}
+
 address StubRoutines::x86::_k256_adr = nullptr;
-address StubRoutines::x86::_vector_short_to_byte_mask = nullptr;
-address StubRoutines::x86::_vector_int_to_byte_mask = nullptr;
-address StubRoutines::x86::_vector_int_to_short_mask = nullptr;
-address StubRoutines::x86::_vector_all_bits_set = nullptr;
-address StubRoutines::x86::_vector_byte_shuffle_mask = nullptr;
-address StubRoutines::x86::_vector_int_mask_cmp_bits = nullptr;
-address StubRoutines::x86::_vector_short_shuffle_mask = nullptr;
-address StubRoutines::x86::_vector_int_shuffle_mask = nullptr;
-address StubRoutines::x86::_vector_long_shuffle_mask = nullptr;
-address StubRoutines::x86::_vector_float_sign_mask = nullptr;
-address StubRoutines::x86::_vector_float_sign_flip = nullptr;
-address StubRoutines::x86::_vector_double_sign_mask = nullptr;
-address StubRoutines::x86::_vector_double_sign_flip = nullptr;
-address StubRoutines::x86::_vector_byte_perm_mask = nullptr;
-address StubRoutines::x86::_vector_long_sign_mask = nullptr;
-address StubRoutines::x86::_vector_iota_indices = nullptr;
-address StubRoutines::x86::_vector_reverse_bit_lut = nullptr;
-address StubRoutines::x86::_vector_reverse_byte_perm_mask_long = nullptr;
-address StubRoutines::x86::_vector_reverse_byte_perm_mask_int = nullptr;
-address StubRoutines::x86::_vector_reverse_byte_perm_mask_short = nullptr;
-address StubRoutines::x86::_vector_popcount_lut = nullptr;
-address StubRoutines::x86::_vector_count_leading_zeros_lut = nullptr;
-address StubRoutines::x86::_vector_32_bit_mask = nullptr;
-address StubRoutines::x86::_vector_64_bit_mask = nullptr;
-#ifdef _LP64
 address StubRoutines::x86::_k256_W_adr = nullptr;
 address StubRoutines::x86::_k512_W_addr = nullptr;
-address StubRoutines::x86::_pshuffle_byte_flip_mask_addr_sha512 = nullptr;
-// Base64 masks
-address StubRoutines::x86::_encoding_table_base64 = nullptr;
-address StubRoutines::x86::_shuffle_base64 = nullptr;
-address StubRoutines::x86::_avx2_shuffle_base64 = nullptr;
-address StubRoutines::x86::_avx2_input_mask_base64 = nullptr;
-address StubRoutines::x86::_avx2_lut_base64 = nullptr;
-address StubRoutines::x86::_avx2_decode_tables_base64 = nullptr;
-address StubRoutines::x86::_avx2_decode_lut_tables_base64 = nullptr;
-address StubRoutines::x86::_lookup_lo_base64 = nullptr;
-address StubRoutines::x86::_lookup_hi_base64 = nullptr;
-address StubRoutines::x86::_lookup_lo_base64url = nullptr;
-address StubRoutines::x86::_lookup_hi_base64url = nullptr;
-address StubRoutines::x86::_pack_vec_base64 = nullptr;
-address StubRoutines::x86::_join_0_1_base64 = nullptr;
-address StubRoutines::x86::_join_1_2_base64 = nullptr;
-address StubRoutines::x86::_join_2_3_base64 = nullptr;
-address StubRoutines::x86::_decoding_table_base64 = nullptr;
-address StubRoutines::x86::_compress_perm_table32 = nullptr;
-address StubRoutines::x86::_compress_perm_table64 = nullptr;
-address StubRoutines::x86::_expand_perm_table32 = nullptr;
-address StubRoutines::x86::_expand_perm_table64 = nullptr;
-#endif
-address StubRoutines::x86::_pshuffle_byte_flip_mask_addr = nullptr;
 
 const uint64_t StubRoutines::x86::_crc_by128_masks[] =
 {
@@ -183,7 +155,6 @@ const juint StubRoutines::x86::_crc_table[] =
     0x2d02ef8dUL
 };
 
-#ifdef _LP64
 const juint StubRoutines::x86::_crc_table_avx512[] =
 {
     0xe95c1271UL, 0x00000000UL, 0xce3371cbUL, 0x00000000UL,
@@ -230,7 +201,6 @@ const juint StubRoutines::x86::_shuf_table_crc32_avx512[] =
     0x83828100UL, 0x87868584UL, 0x8b8a8988UL, 0x8f8e8d8cUL,
     0x03020100UL, 0x07060504UL, 0x0b0a0908UL, 0x000e0d0cUL
 };
-#endif // _LP64
 
 const jint StubRoutines::x86::_arrays_hashcode_powers_of_31[] =
 {
@@ -332,7 +302,7 @@ static uint32_t crc32c_f_pow_n(uint32_t n) {
   return result;
 }
 
-juint *StubRoutines::x86::_crc32c_table;
+juint* StubRoutines::x86::_crc32c_table = nullptr;
 
 void StubRoutines::x86::generate_CRC32C_table(bool is_pclmulqdq_table_supported) {
 
@@ -393,7 +363,6 @@ ATTRIBUTE_ALIGNED(64) const juint StubRoutines::x86::_k256[] =
     0x90befffaUL, 0xa4506cebUL, 0xbef9a3f7UL, 0xc67178f2UL
 };
 
-#ifdef _LP64
 // used in MacroAssembler::sha256_AVX2
 // dynamically built from _k256
 ATTRIBUTE_ALIGNED(64) juint StubRoutines::x86::_k256_W[2*sizeof(StubRoutines::x86::_k256)];
@@ -442,4 +411,3 @@ ATTRIBUTE_ALIGNED(64) const julong StubRoutines::x86::_k512_W[] =
     0x4cc5d4becb3e42b6ULL, 0x597f299cfc657e2aULL,
     0x5fcb6fab3ad6faecULL, 0x6c44198c4a475817ULL,
 };
-#endif
