@@ -22,12 +22,12 @@
  *
  */
 
-#include "classfile/vmSymbols.hpp"
 #include "c1/c1_InstructionPrinter.hpp"
 #include "c1/c1_ValueStack.hpp"
 #include "ci/ciArray.hpp"
 #include "ci/ciInstance.hpp"
 #include "ci/ciObject.hpp"
+#include "classfile/vmSymbols.hpp"
 
 
 #ifndef PRODUCT
@@ -779,12 +779,6 @@ void InstructionPrinter::do_ExceptionObject(ExceptionObject* x) {
   output()->print("incoming exception");
 }
 
-
-void InstructionPrinter::do_RoundFP(RoundFP* x) {
-  output()->print("round_fp ");
-  print_value(x->input());
-}
-
 void InstructionPrinter::do_UnsafeGet(UnsafeGet* x) {
   print_unsafe_op(x, x->is_raw() ? "UnsafeGet (raw)" : "UnsafeGet");
   output()->put(')');
