@@ -23,6 +23,7 @@
  */
 
 #include "cds/aotClassLocation.hpp"
+#include "cds/aotLogging.hpp"
 #include "cds/archiveBuilder.hpp"
 #include "cds/cdsConfig.hpp"
 #include "cds/dynamicArchive.hpp"
@@ -44,9 +45,10 @@
 #include "utilities/formatBuffer.hpp"
 #include "utilities/stringUtils.hpp"
 
-#include <sys/stat.h>
 #include <errno.h>
+#include <sys/stat.h>
 
+Array<ClassPathZipEntry*>* AOTClassLocationConfig::_dumptime_jar_files = nullptr;
 AOTClassLocationConfig* AOTClassLocationConfig::_dumptime_instance = nullptr;
 const AOTClassLocationConfig* AOTClassLocationConfig::_runtime_instance = nullptr;
 
@@ -249,7 +251,7 @@ AOTClassLocation* AOTClassLocation::allocate(JavaThread* current, const char* pa
     // We allow the file to not exist, as long as it also doesn't exist during runtime.
     type = FileType::NOT_EXIST;
   } else {
-    log_error(cds)("Unable to open file %s.", path);
+    aot_log_error(aot)("Unable to open file %s.", path);
     MetaspaceShared::unrecoverable_loading_error();
   }
 
@@ -358,7 +360,7 @@ char* AOTClassLocation::get_cpattr() const {
       if (found != nullptr) {
         // Same behavior as jdk/src/share/classes/java/util/jar/Attributes.java
         // If duplicated entries are found, the last one is used.
-        log_warning(cds)("Warning: Duplicate name in Manifest: %s.\n"
+        log_warning(aot)("Warning: Duplicate name in Manifest: %s.\n"
                          "Ensure that the manifest does not have duplicate entries, and\n"
                          "that blank lines separate individual sections in both your\n"
                          "manifest and in the META-INF/MANIFEST.MF entry in the jar file:\n%s\n", tag, path());
@@ -391,31 +393,31 @@ bool AOTClassLocation::check(const char* runtime_path, bool has_aot_linked_class
   struct stat st;
   if (os::stat(runtime_path, &st) != 0) {
     if (_file_type != FileType::NOT_EXIST) {
-      log_warning(cds)("Required classpath entry does not exist: %s", runtime_path);
+      aot_log_warning(aot)("Required classpath entry does not exist: %s", runtime_path);
       return false;
     }
   } else if ((st.st_mode & S_IFMT) == S_IFDIR) {
     if (_file_type == FileType::NOT_EXIST) {
-      log_warning(cds)("'%s' must not exist", runtime_path);
+      aot_log_warning(aot)("'%s' must not exist", runtime_path);
       return false;
     }
     if (_file_type == FileType::NORMAL) {
-      log_warning(cds)("'%s' must be a file", runtime_path);
+      aot_log_warning(aot)("'%s' must be a file", runtime_path);
       return false;
     }
     if (!os::dir_is_empty(runtime_path)) {
-      log_warning(cds)("directory is not empty: '%s'", runtime_path);
+      aot_log_warning(aot)("directory is not empty: '%s'", runtime_path);
       return false;
     }
   } else {
     if (_file_type == FileType::NOT_EXIST) {
-      log_warning(cds)("'%s' must not exist", runtime_path);
+      aot_log_warning(aot)("'%s' must not exist", runtime_path);
       if (has_aot_linked_classes) {
-        log_error(cds)("CDS archive has aot-linked classes. It cannot be used because the "
+        aot_log_error(aot)("CDS archive has aot-linked classes. It cannot be used because the "
                        "file %s exists", runtime_path);
         return false;
       } else {
-        log_warning(cds)("Archived non-system classes are disabled because the "
+        aot_log_warning(aot)("Archived non-system classes are disabled because the "
                          "file %s exists", runtime_path);
         FileMapInfo::current_info()->set_has_platform_or_app_classes(false);
         if (DynamicArchive::is_mapped()) {
@@ -424,13 +426,13 @@ bool AOTClassLocation::check(const char* runtime_path, bool has_aot_linked_class
       }
     }
     if (_file_type == FileType::DIR) {
-      log_warning(cds)("'%s' must be a directory", runtime_path);
+      aot_log_warning(aot)("'%s' must be a directory", runtime_path);
       return false;
     }
     bool size_differs = _filesize != st.st_size;
     bool time_differs = _check_time && (_timestamp != st.st_mtime);
     if (size_differs || time_differs) {
-      log_warning(cds)("This file is not the one used while building the shared archive file: '%s'%s%s",
+      aot_log_warning(aot)("This file is not the one used while building the shared archive file: '%s'%s%s",
                        runtime_path,
                        time_differs ? ", timestamp has changed" : "",
                        size_differs ? ", size has changed" : "");
@@ -462,6 +464,7 @@ void AOTClassLocationConfig::dumptime_init_helper(TRAPS) {
   AOTClassLocation* jrt = AOTClassLocation::allocate(THREAD, ClassLoader::get_jrt_entry()->name(),
                                                0, Group::MODULES_IMAGE,
                                                /*from_cpattr*/false, /*is_jrt*/true);
+  log_info(class, path)("path [%d] = (modules image)", tmp_array.length());
   tmp_array.append(jrt);
 
   parse(THREAD, tmp_array, all_css.boot_cp(), Group::BOOT_CLASSPATH, /*parse_manifest*/true);
@@ -479,10 +482,17 @@ void AOTClassLocationConfig::dumptime_init_helper(TRAPS) {
     _class_locations->at_put(i, tmp_array.at(i));
   }
 
+  _dumptime_jar_files = MetadataFactory::new_array<ClassPathZipEntry*>(ClassLoaderData::the_null_class_loader_data(),
+                                                                       tmp_array.length(), CHECK);
+  for (int i = 1; i < tmp_array.length(); i++) {
+    ClassPathZipEntry* jar_file = ClassLoader::create_class_path_zip_entry(tmp_array.at(i)->path());
+    _dumptime_jar_files->at_put(i, jar_file); // may be null if the path is not a valid JAR file
+  }
+
   const char* lcp = find_lcp(all_css.boot_and_app_cp(), _dumptime_lcp_len);
   if (_dumptime_lcp_len > 0) {
-    os::free((void*)lcp);
     log_info(class, path)("Longest common prefix = %s (%zu chars)", lcp, _dumptime_lcp_len);
+    os::free((void*)lcp);
   } else {
     assert(_dumptime_lcp_len == 0, "sanity");
     log_info(class, path)("Longest common prefix = <none> (0 chars)");
@@ -564,6 +574,7 @@ void AOTClassLocationConfig::parse(JavaThread* current, GrowableClassLocationArr
 void AOTClassLocationConfig::add_class_location(JavaThread* current, GrowableClassLocationArray& tmp_array,
                                                 const char* path, Group group, bool parse_manifest, bool from_cpattr) {
   AOTClassLocation* cs = AOTClassLocation::allocate(current, path, tmp_array.length(), group, from_cpattr);
+  log_info(class, path)("path [%d] = %s%s", tmp_array.length(), path, from_cpattr ? " (from cpattr)" : "");
   tmp_array.append(cs);
 
   if (!parse_manifest) {
@@ -681,7 +692,7 @@ void AOTClassLocationConfig::check_nonempty_dirs() const {
     }
     if (cs->is_dir()) {
       if (!os::dir_is_empty(cs->path())) {
-        log_error(cds)("Error: non-empty directory '%s'", cs->path());
+        aot_log_error(aot)("Error: non-empty directory '%s'", cs->path());
         has_nonempty_dir = true;
       }
     }
@@ -693,7 +704,32 @@ void AOTClassLocationConfig::check_nonempty_dirs() const {
   }
 }
 
+// It's possible to use reflection+setAccessible to call into ClassLoader::defineClass() to
+// pretend that a dynamically generated class comes from a JAR file in the classpath.
+// Detect such classes so that they can be excluded from the archive.
+bool AOTClassLocationConfig::is_valid_classpath_index(int classpath_index, InstanceKlass* ik) {
+  if (1 <= classpath_index && classpath_index < length()) {
+    ClassPathZipEntry *zip = _dumptime_jar_files->at(classpath_index);
+    if (zip != nullptr) {
+      JavaThread* current = JavaThread::current();
+      ResourceMark rm(current);
+      const char* const class_name = ik->name()->as_C_string();
+      const char* const file_name = ClassLoader::file_name_for_class_name(class_name,
+                                                                          ik->name()->utf8_length());
+      if (!zip->has_entry(current, file_name)) {
+        aot_log_warning(aot)("class %s cannot be archived because it was not defined from %s as claimed",
+                         class_name, zip->name());
+        return false;
+      }
+    }
+  }
+
+  return true;
+}
+
 AOTClassLocationConfig* AOTClassLocationConfig::write_to_archive() const {
+  log_locations(CDSConfig::output_archive_path(), /*is_write=*/true);
+
   Array<AOTClassLocation*>* archived_copy = ArchiveBuilder::new_ro_array<AOTClassLocation*>(_class_locations->length());
   for (int i = 0; i < _class_locations->length(); i++) {
     archived_copy->at_put(i, _class_locations->at(i)->write_to_archive());
@@ -741,11 +777,11 @@ bool AOTClassLocationConfig::check_classpaths(bool is_boot_classpath, bool has_a
       effective_dumptime_path = substitute(effective_dumptime_path, _dumptime_lcp_len, runtime_lcp, runtime_lcp_len);
     }
 
-    log_info(class, path)("Checking '%s' %s%s", effective_dumptime_path, cs->file_type_string(),
+    log_info(class, path)("Checking [%d] '%s' %s%s", i, effective_dumptime_path, cs->file_type_string(),
                           cs->from_cpattr() ? " (from JAR manifest ClassPath attribute)" : "");
     if (!cs->from_cpattr() && file_exists(effective_dumptime_path)) {
       if (!runtime_css.has_next()) {
-        log_warning(cds)("%s classpath has fewer elements than expected", which);
+        aot_log_warning(aot)("%s classpath has fewer elements than expected", which);
         return false;
       }
       const char* runtime_path = runtime_css.get_next();
@@ -753,7 +789,7 @@ bool AOTClassLocationConfig::check_classpaths(bool is_boot_classpath, bool has_a
         runtime_path = runtime_css.get_next();
       }
       if (!os::same_files(effective_dumptime_path, runtime_path)) {
-        log_warning(cds)("The name of %s classpath [%d] does not match: expected '%s', got '%s'",
+        aot_log_warning(aot)("The name of %s classpath [%d] does not match: expected '%s', got '%s'",
                          which, runtime_css.current(), effective_dumptime_path, runtime_path);
         return false;
       }
@@ -769,7 +805,7 @@ bool AOTClassLocationConfig::check_classpaths(bool is_boot_classpath, bool has_a
   if (is_boot_classpath && runtime_css.has_next() && (need_to_check_app_classpath() || num_module_paths() > 0)) {
     // the check passes if all the extra runtime boot classpath entries are non-existent
     if (check_paths_existence(runtime_css)) {
-      log_warning(cds)("boot classpath is longer than expected");
+      aot_log_warning(aot)("boot classpath is longer than expected");
       return false;
     }
   }
@@ -838,7 +874,7 @@ bool AOTClassLocationConfig::check_module_paths(bool has_aot_linked_classes, int
 
     while (true) {
       if (!runtime_css.has_next()) {
-        log_warning(cds)("module path has fewer elements than expected");
+        aot_log_warning(aot)("module path has fewer elements than expected");
         *has_extra_module_paths = true;
         return true;
       }
@@ -929,11 +965,14 @@ bool AOTClassLocationConfig::need_lcp_match_helper(int start, int end, ClassLoca
   return true;
 }
 
-bool AOTClassLocationConfig::validate(bool has_aot_linked_classes, bool* has_extra_module_paths) const {
+bool AOTClassLocationConfig::validate(const char* cache_filename, bool has_aot_linked_classes, bool* has_extra_module_paths) const {
   ResourceMark rm;
   AllClassLocationStreams all_css;
 
+  log_locations(cache_filename, /*is_write=*/false);
+
   const char* jrt = ClassLoader::get_jrt_entry()->name();
+  log_info(class, path)("Checking [0] (modules image)");
   bool success = class_location_at(0)->check(jrt, has_aot_linked_classes);
   log_info(class, path)("Modules image %s validation: %s", jrt, success ? "passed" : "failed");
   if (!success) {
@@ -990,16 +1029,63 @@ bool AOTClassLocationConfig::validate(bool has_aot_linked_classes, bool* has_ext
         "" : " (hint: enable -Xlog:class+path=info to diagnose the failure)";
     if (RequireSharedSpaces && !PrintSharedArchiveAndExit) {
       if (CDSConfig::is_dumping_final_static_archive()) {
-        log_error(cds)("class path and/or module path are not compatible with the "
+        aot_log_error(aot)("class path and/or module path are not compatible with the "
                        "ones specified when the AOTConfiguration file was recorded%s", hint_msg);
         vm_exit_during_initialization("Unable to use create AOT cache.", nullptr);
       } else {
-        log_error(cds)("%s%s", mismatch_msg, hint_msg);
+        aot_log_error(aot)("%s%s", mismatch_msg, hint_msg);
         MetaspaceShared::unrecoverable_loading_error();
       }
     } else {
-      log_warning(cds)("%s%s", mismatch_msg, hint_msg);
+      MetaspaceShared::report_loading_error("%s%s", mismatch_msg, hint_msg);
     }
   }
   return success;
 }
+
+void AOTClassLocationConfig::log_locations(const char* cache_filename, bool is_write) const {
+  if (log_is_enabled(Info, class, path)) {
+    LogStreamHandle(Info, class, path) st;
+    st.print_cr("%s classpath(s) %s %s (size = %d)",
+                is_write ? "Writing" : "Reading",
+                is_write ? "into" : "from",
+                cache_filename, class_locations()->length());
+    print_on(&st);
+  }
+}
+
+void AOTClassLocationConfig::print() {
+  if (CDSConfig::is_dumping_archive()) {
+    tty->print_cr("AOTClassLocationConfig::_dumptime_instance = %p", _dumptime_instance);
+    if (_dumptime_instance != nullptr) {
+      _dumptime_instance->print_on(tty);
+    }
+  }
+  if (CDSConfig::is_using_archive()) {
+    tty->print_cr("AOTClassLocationConfig::_runtime_instance = %p", _runtime_instance);
+    if (_runtime_instance != nullptr) {
+      _runtime_instance->print_on(tty);
+    }
+  }
+}
+
+void AOTClassLocationConfig::print_on(outputStream* st) const {
+  const char* type = "boot";
+  int n = class_locations()->length();
+  for (int i = 0; i < n; i++) {
+    if (i >= boot_cp_end_index()) {
+      type = "app";
+    }
+    if (i >= app_cp_end_index()) {
+      type = "module";
+    }
+    const AOTClassLocation* cs = class_location_at(i);
+    const char* path;
+    if (i == 0) {
+      path = ClassLoader::get_jrt_entry()->name();
+    } else {
+      path = cs->path();
+    }
+    st->print_cr("(%-6s) [%d] = %s", type, i, path);
+  }
+}
