@@ -22,10 +22,10 @@
  *
  */
 
+#include "cds/aotMetaspace.hpp"
 #include "cds/archiveHeapLoader.inline.hpp"
 #include "cds/cdsConfig.hpp"
 #include "cds/heapShared.hpp"
-#include "cds/metaspaceShared.hpp"
 #include "classfile/classLoaderDataShared.hpp"
 #include "classfile/systemDictionaryShared.hpp"
 #include "gc/shared/collectedHeap.hpp"
@@ -167,7 +167,7 @@ void ArchiveHeapLoader::patch_compressed_embedded_pointers(BitMapView bm,
                                                   MemRegion region) {
   narrowOop dt_encoded_bottom = info->encoded_heap_region_dumptime_address();
   narrowOop rt_encoded_bottom = CompressedOops::encode_not_null(cast_to_oop(region.start()));
-  log_info(cds)("patching heap embedded pointers: narrowOop 0x%8x -> 0x%8x",
+  log_info(aot)("patching heap embedded pointers: narrowOop 0x%8x -> 0x%8x",
                   (uint)dt_encoded_bottom, (uint)rt_encoded_bottom);
 
   // Optimization: if dumptime shift is the same as runtime shift, we can perform a
@@ -175,15 +175,15 @@ void ArchiveHeapLoader::patch_compressed_embedded_pointers(BitMapView bm,
   narrowOop* patching_start = (narrowOop*)region.start() + FileMapInfo::current_info()->heap_oopmap_start_pos();
   if (_narrow_oop_shift == CompressedOops::shift()) {
     uint32_t quick_delta = (uint32_t)rt_encoded_bottom - (uint32_t)dt_encoded_bottom;
-    log_info(cds)("CDS heap data relocation quick delta = 0x%x", quick_delta);
+    log_info(aot)("heap data relocation quick delta = 0x%x", quick_delta);
     if (quick_delta == 0) {
-      log_info(cds)("CDS heap data relocation unnecessary, quick_delta = 0");
+      log_info(aot)("heap data relocation unnecessary, quick_delta = 0");
     } else {
       PatchCompressedEmbeddedPointersQuick patcher(patching_start, quick_delta);
       bm.iterate(&patcher);
     }
   } else {
-    log_info(cds)("CDS heap data quick relocation not possible");
+    log_info(aot)("heap data quick relocation not possible");
     PatchCompressedEmbeddedPointers patcher(patching_start);
     bm.iterate(&patcher);
   }
@@ -261,7 +261,7 @@ class ArchiveHeapLoader::PatchLoadedRegionPointers: public BitMapClosure {
 bool ArchiveHeapLoader::init_loaded_region(FileMapInfo* mapinfo, LoadedArchiveHeapRegion* loaded_region,
                                            MemRegion& archive_space) {
   size_t total_bytes = 0;
-  FileMapRegion* r = mapinfo->region_at(MetaspaceShared::hp);
+  FileMapRegion* r = mapinfo->region_at(AOTMetaspace::hp);
   r->assert_is_heap_region();
   if (r->used() == 0) {
     return false;
@@ -269,7 +269,7 @@ bool ArchiveHeapLoader::init_loaded_region(FileMapInfo* mapinfo, LoadedArchiveHe
 
   assert(is_aligned(r->used(), HeapWordSize), "must be");
   total_bytes += r->used();
-  loaded_region->_region_index = MetaspaceShared::hp;
+  loaded_region->_region_index = AOTMetaspace::hp;
   loaded_region->_region_size = r->used();
   loaded_region->_dumptime_base = (uintptr_t)mapinfo->heap_region_dumptime_address();
 
@@ -301,12 +301,12 @@ bool ArchiveHeapLoader::load_heap_region_impl(FileMapInfo* mapinfo, LoadedArchiv
   if (!mapinfo->read_region(loaded_region->_region_index, (char*)load_address, r->used(), /* do_commit = */ false)) {
     // There's no easy way to free the buffer, so we will fill it with zero later
     // in fill_failed_loaded_heap(), and it will eventually be GC'ed.
-    log_warning(cds)("Loading of heap region %d has failed. Archived objects are disabled", loaded_region->_region_index);
+    log_warning(aot)("Loading of heap region %d has failed. Archived objects are disabled", loaded_region->_region_index);
     _loading_failed = true;
     return false;
   }
   assert(r->mapped_base() == (char*)load_address, "sanity");
-  log_info(cds)("Loaded heap    region #%d at base " INTPTR_FORMAT " top " INTPTR_FORMAT
+  log_info(aot)("Loaded heap    region #%d at base " INTPTR_FORMAT " top " INTPTR_FORMAT
                 " size %6zu delta %zd",
                 loaded_region->_region_index, load_address, load_address + loaded_region->_region_size,
                 loaded_region->_region_size, loaded_region->_runtime_offset);
@@ -348,10 +348,10 @@ bool ArchiveHeapLoader::load_heap_region(FileMapInfo* mapinfo) {
 }
 
 class VerifyLoadedHeapEmbeddedPointers: public BasicOopIterateClosure {
-  ResourceHashtable<uintptr_t, bool>* _table;
+  HashTable<uintptr_t, bool>* _table;
 
  public:
-  VerifyLoadedHeapEmbeddedPointers(ResourceHashtable<uintptr_t, bool>* table) : _table(table) {}
+  VerifyLoadedHeapEmbeddedPointers(HashTable<uintptr_t, bool>* table) : _table(table) {}
 
   virtual void do_oop(narrowOop* p) {
     // This should be called before the loaded region is modified, so all the embedded pointers
@@ -408,10 +408,10 @@ void ArchiveHeapLoader::finish_loaded_heap() {
 }
 
 void ArchiveHeapLoader::verify_loaded_heap() {
-  log_info(cds, heap)("Verify all oops and pointers in loaded heap");
+  log_info(aot, heap)("Verify all oops and pointers in loaded heap");
 
   ResourceMark rm;
-  ResourceHashtable<uintptr_t, bool> table;
+  HashTable<uintptr_t, bool> table;
   VerifyLoadedHeapEmbeddedPointers verifier(&table);
   HeapWord* bottom = (HeapWord*)_loaded_heap_bottom;
   HeapWord* top    = (HeapWord*)_loaded_heap_top;
@@ -447,20 +447,20 @@ class PatchNativePointers: public BitMapClosure {
 
   bool do_bit(size_t offset) {
     Metadata** p = _start + offset;
-    *p = (Metadata*)(address(*p) + MetaspaceShared::relocation_delta());
+    *p = (Metadata*)(address(*p) + AOTMetaspace::relocation_delta());
     return true;
   }
 };
 
 void ArchiveHeapLoader::patch_native_pointers() {
-  if (MetaspaceShared::relocation_delta() == 0) {
+  if (AOTMetaspace::relocation_delta() == 0) {
     return;
   }
 
-  FileMapRegion* r = FileMapInfo::current_info()->region_at(MetaspaceShared::hp);
+  FileMapRegion* r = FileMapInfo::current_info()->region_at(AOTMetaspace::hp);
   if (r->mapped_base() != nullptr && r->has_ptrmap()) {
-    log_info(cds, heap)("Patching native pointers in heap region");
-    BitMapView bm = FileMapInfo::current_info()->ptrmap_view(MetaspaceShared::hp);
+    log_info(aot, heap)("Patching native pointers in heap region");
+    BitMapView bm = FileMapInfo::current_info()->ptrmap_view(AOTMetaspace::hp);
     PatchNativePointers patcher((Metadata**)r->mapped_base() + FileMapInfo::current_info()->heap_ptrmap_start_pos());
     bm.iterate(&patcher);
   }
