@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,8 +30,7 @@
 #include "oops/symbol.hpp"
 #include "utilities/globalDefinitions.hpp"
 #include "utilities/growableArray.hpp"
-#include "utilities/resourceHash.hpp"
-
+#include "utilities/hashTable.hpp"
 
 /**
  * Bytecode Assembler
@@ -66,7 +65,6 @@ class BytecodeCPEntry {
   union {
     Symbol* utf8;
     u2 klass;
-    u2 string;
     struct {
       u2 name_index;
       u2 type_index;
@@ -93,9 +91,9 @@ class BytecodeCPEntry {
     return bcpe;
   }
 
-  static BytecodeCPEntry string(u2 index) {
+  static BytecodeCPEntry string(Symbol* symbol) {
     BytecodeCPEntry bcpe(STRING);
-    bcpe._u.string = index;
+    bcpe._u.utf8 = symbol;
     return bcpe;
   }
 
@@ -114,6 +112,8 @@ class BytecodeCPEntry {
   }
 
   static bool equals(BytecodeCPEntry const& e0, BytecodeCPEntry const& e1) {
+    // The hash is the "union trick" value of the information saved for the tag,
+    // so can be compared for equality.
     return e0._tag == e1._tag && e0._u.hash == e1._u.hash;
   }
 
@@ -122,47 +122,57 @@ class BytecodeCPEntry {
   }
 };
 
-class BytecodeConstantPool : ResourceObj {
+class BytecodeConstantPool : public ResourceObj {
  private:
-  typedef ResourceHashtable<BytecodeCPEntry, u2,
+  typedef HashTable<BytecodeCPEntry, u2,
       256, AnyObj::RESOURCE_AREA, mtInternal,
       &BytecodeCPEntry::hash, &BytecodeCPEntry::equals> IndexHash;
 
   ConstantPool* _orig;
-  GrowableArray<BytecodeCPEntry> _entries;
-  IndexHash _indices;
+  GrowableArray<BytecodeCPEntry> _added_entries;
+  IndexHash _index_map;
+  int _orig_cp_added;
 
-  u2 find_or_add(BytecodeCPEntry const& bcpe);
+  u2 find_or_add(BytecodeCPEntry const& bcpe, TRAPS);
 
+  void init();
  public:
 
-  BytecodeConstantPool(ConstantPool* orig) : _orig(orig) {}
+  BytecodeConstantPool(ConstantPool* orig) : _orig(orig), _orig_cp_added(0) {
+    init();
+  }
 
-  BytecodeCPEntry const& at(u2 index) const { return _entries.at(index); }
+  BytecodeCPEntry const& at(u2 index) const { return _added_entries.at(index); }
 
   InstanceKlass* pool_holder() const {
     return _orig->pool_holder();
   }
 
-  u2 utf8(Symbol* sym) {
-    return find_or_add(BytecodeCPEntry::utf8(sym));
+  u2 utf8(Symbol* sym, TRAPS) {
+    return find_or_add(BytecodeCPEntry::utf8(sym), THREAD);
   }
 
-  u2 klass(Symbol* class_name) {
-    return find_or_add(BytecodeCPEntry::klass(utf8(class_name)));
+  u2 klass(Symbol* class_name, TRAPS) {
+    u2 utf8_entry = utf8(class_name, CHECK_0);
+    return find_or_add(BytecodeCPEntry::klass(utf8_entry), THREAD);
   }
 
-  u2 string(Symbol* str) {
-    return find_or_add(BytecodeCPEntry::string(utf8(str)));
+  u2 string(Symbol* str, TRAPS) {
+    // Create the utf8_entry in the hashtable but use Symbol for matching.
+    (void)utf8(str, CHECK_0);
+    return find_or_add(BytecodeCPEntry::string(str), THREAD);
   }
 
-  u2 name_and_type(Symbol* name, Symbol* sig) {
-    return find_or_add(BytecodeCPEntry::name_and_type(utf8(name), utf8(sig)));
+  u2 name_and_type(Symbol* name, Symbol* sig, TRAPS) {
+    u2 utf8_name = utf8(name, CHECK_0);
+    u2 utf8_sig  = utf8(sig, CHECK_0);
+    return find_or_add(BytecodeCPEntry::name_and_type(utf8_name, utf8_sig), THREAD);
   }
 
-  u2 methodref(Symbol* class_name, Symbol* name, Symbol* sig) {
-    return find_or_add(BytecodeCPEntry::methodref(
-        klass(class_name), name_and_type(name, sig)));
+  u2 methodref(Symbol* class_name, Symbol* name, Symbol* sig, TRAPS) {
+    u2 klass_entry = klass(class_name, CHECK_0);
+    u2 type_entry = name_and_type(name, sig, CHECK_0);
+    return find_or_add(BytecodeCPEntry::methodref(klass_entry, type_entry), THREAD);
   }
 
   ConstantPool* create_constant_pool(TRAPS) const;
@@ -179,37 +189,22 @@ class BytecodeAssembler : StackObj {
   void append(u2 imm_u2);
   void append(u4 imm_u4);
 
-  void xload(u4 index, u1 quick, u1 twobyte);
+  void athrow();
+  void dup();
+  void invokespecial(Symbol* cls, Symbol* name, Symbol* sig, TRAPS);
+  void ldc(u1 index);
+  void ldc_w(u2 index);
+  void _new(Symbol* sym, TRAPS);
+  void load_string(Symbol* sym, TRAPS);
 
  public:
   BytecodeAssembler(BytecodeBuffer* buffer, BytecodeConstantPool* cp)
     : _code(buffer), _cp(cp) {}
 
-  void aload(u4 index);
-  void areturn();
-  void athrow();
-  void checkcast(Symbol* sym);
-  void dload(u4 index);
-  void dreturn();
-  void dup();
-  void fload(u4 index);
-  void freturn();
-  void iload(u4 index);
-  void invokespecial(Method* method);
-  void invokespecial(Symbol* cls, Symbol* name, Symbol* sig);
-  void invokevirtual(Method* method);
-  void invokevirtual(Symbol* cls, Symbol* name, Symbol* sig);
-  void ireturn();
-  void ldc(u1 index);
-  void ldc_w(u2 index);
-  void lload(u4 index);
-  void lreturn();
-  void _new(Symbol* sym);
-  void _return();
-
-  void load_string(Symbol* sym);
-  void load(BasicType bt, u4 index);
-  void _return(BasicType bt);
+  static int assemble_method_error(BytecodeConstantPool* cp,
+                                   BytecodeBuffer* buffer,
+                                   Symbol* errorName,
+                                   Symbol* message, TRAPS);
 };
 
 #endif // SHARE_CLASSFILE_BYTECODEASSEMBLER_HPP
