@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -58,10 +58,6 @@ public:
 
   virtual bool is_modules_image() const { return false; }
   virtual bool is_jar_file() const { return false; }
-  virtual bool is_multi_release_jar() const { return false; }
-  virtual void set_multi_release_jar() {}
-  // Is this entry created from the "Class-path" attribute from a JAR Manifest?
-  virtual bool from_class_path_attr() const { return false; }
   virtual const char* name() const = 0;
   virtual JImageFile* jimage() const { return nullptr; }
   virtual void close_jimage() {}
@@ -92,16 +88,12 @@ class ClassPathZipEntry: public ClassPathEntry {
  private:
   jzfile* _zip;              // The zip archive
   const char*   _zip_name;   // Name of zip archive
-  bool _from_class_path_attr; // From the "Class-path" attribute of a jar file
-  bool _multi_release;       // multi-release jar
  public:
   bool is_jar_file() const { return true;  }
-  bool is_multi_release_jar() const { return _multi_release; }
-  void set_multi_release_jar() { _multi_release = true; }
-  bool from_class_path_attr() const { return _from_class_path_attr; }
   const char* name() const { return _zip_name; }
-  ClassPathZipEntry(jzfile* zip, const char* zip_name, bool is_boot_append, bool from_class_path_attr, bool multi_release);
+  ClassPathZipEntry(jzfile* zip, const char* zip_name);
   virtual ~ClassPathZipEntry();
+  bool has_entry(JavaThread* current, const char* name);
   u1* open_entry(JavaThread* current, const char* name, jint* filesize, bool nul_terminate);
   ClassFileStream* open_stream(JavaThread* current, const char* name);
 };
@@ -226,22 +218,7 @@ class ClassLoader: AllStatic {
   // Last entry in linked list of appended ClassPathEntry instances
   static ClassPathEntry* volatile _last_append_entry;
 
-  // Info used by CDS
-  CDS_ONLY(static ClassPathEntry* _app_classpath_entries;)
-  CDS_ONLY(static ClassPathEntry* _last_app_classpath_entry;)
-  CDS_ONLY(static ClassPathEntry* _module_path_entries;)
-  CDS_ONLY(static ClassPathEntry* _last_module_path_entry;)
-  CDS_ONLY(static void setup_app_search_path(JavaThread* current, const char* class_path);)
-  CDS_ONLY(static void setup_module_search_path(JavaThread* current, const char* path);)
-  static bool add_to_app_classpath_entries(JavaThread* current,
-                                           ClassPathEntry* entry,
-                                           bool check_for_duplicates);
-  CDS_ONLY(static void add_to_module_path_entries(const char* path,
-                                           ClassPathEntry* entry);)
-
  public:
-  CDS_ONLY(static ClassPathEntry* app_classpath_entries() {return _app_classpath_entries;})
-  CDS_ONLY(static ClassPathEntry* module_path_entries() {return _module_path_entries;})
 
   static bool has_bootclasspath_append() { return first_append_entry() != nullptr; }
 
@@ -263,10 +240,7 @@ class ClassLoader: AllStatic {
   static void* zip_library_handle();
   static jzfile* open_zip_file(const char* canonical_path, char** error_msg, JavaThread* thread);
   static ClassPathEntry* create_class_path_entry(JavaThread* current,
-                                                 const char *path, const struct stat* st,
-                                                 bool is_boot_append,
-                                                 bool from_class_path_attr,
-                                                 bool is_multi_release = false);
+                                                 const char *path, const struct stat* st);
 
   // Canonicalizes path names, so strcmp will work properly. This is mainly
   // to avoid confusing the zip library
@@ -276,10 +250,7 @@ class ClassLoader: AllStatic {
   static PackageEntry* get_package_entry(Symbol* pkg_name, ClassLoaderData* loader_data);
   static int crc32(int crc, const char* buf, int len);
   static bool update_class_path_entry_list(JavaThread* current,
-                                           const char *path,
-                                           bool check_for_duplicates,
-                                           bool is_boot_append,
-                                           bool from_class_path_attr);
+                                           const char *path);
   static void print_bootclasspath();
 
   // Timing
@@ -363,8 +334,6 @@ class ClassLoader: AllStatic {
   // Initialization
   static void initialize(TRAPS);
   static void classLoader_init2(JavaThread* current);
-  CDS_ONLY(static void initialize_shared_path(JavaThread* current);)
-  CDS_ONLY(static void initialize_module_path(TRAPS);)
 
   static int compute_Object_vtable();
 
@@ -373,26 +342,12 @@ class ClassLoader: AllStatic {
   static bool is_in_patch_mod_entries(Symbol* module_name);
 
 #if INCLUDE_CDS
-  // Sharing dump and restore
-
-  // Helper function used by CDS code to get the number of boot classpath
-  // entries during shared classpath setup time.
-  static int num_boot_classpath_entries();
-
-  static ClassPathEntry* get_next_boot_classpath_entry(ClassPathEntry* e);
-
-  // Helper function used by CDS code to get the number of app classpath
-  // entries during shared classpath setup time.
-  static int num_app_classpath_entries();
-
-  // Helper function used by CDS code to get the number of module path
-  // entries during shared classpath setup time.
-  static int num_module_path_entries();
-  static void  exit_with_path_failure(const char* error, const char* message);
   static char* uri_to_path(const char* uri);
   static void  record_result(JavaThread* current, InstanceKlass* ik,
                              const ClassFileStream* stream, bool redefined);
+  static void record_result_for_builtin_loader(s2 classpath_index, InstanceKlass* result, bool redefined);
   static void record_hidden_class(InstanceKlass* ik);
+  static void append_boot_classpath(ClassPathEntry* new_entry);
 #endif
 
   static char* lookup_vm_options();
@@ -419,7 +374,7 @@ class ClassLoader: AllStatic {
   static void add_to_boot_append_entries(ClassPathEntry* new_entry);
 
   // creates a class path zip entry (returns null if JAR file cannot be opened)
-  static ClassPathZipEntry* create_class_path_zip_entry(const char *apath, bool is_boot_append);
+  static ClassPathZipEntry* create_class_path_zip_entry(const char *path);
 
   static bool string_ends_with(const char* str, const char* str_to_find);
 
