@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,17 +22,18 @@
  *
  */
 
-#include "precompiled.hpp"
+#include "cds/aotClassLocation.hpp"
 #include "cds/archiveBuilder.hpp"
 #include "cds/archiveUtils.hpp"
 #include "cds/cdsConfig.hpp"
-#include "cds/filemap.hpp"
 #include "cds/heapShared.hpp"
 #include "classfile/classLoader.hpp"
 #include "classfile/classLoaderData.inline.hpp"
+#include "classfile/classLoaderDataShared.hpp"
 #include "classfile/javaClasses.inline.hpp"
 #include "classfile/moduleEntry.hpp"
 #include "classfile/systemDictionary.hpp"
+#include "classfile/systemDictionaryShared.hpp"
 #include "jni.h"
 #include "logging/log.hpp"
 #include "logging/logStream.hpp"
@@ -44,13 +45,13 @@
 #include "runtime/safepoint.hpp"
 #include "utilities/events.hpp"
 #include "utilities/growableArray.hpp"
+#include "utilities/hashTable.hpp"
 #include "utilities/ostream.hpp"
 #include "utilities/quickSort.hpp"
-#include "utilities/resourceHash.hpp"
 
 ModuleEntry* ModuleEntryTable::_javabase_module = nullptr;
 
-oop ModuleEntry::module() const { return _module.resolve(); }
+oop ModuleEntry::module_oop() const { return _module_handle.resolve(); }
 
 void ModuleEntry::set_location(Symbol* location) {
   // _location symbol's refcounts are managed by ModuleEntry,
@@ -62,7 +63,7 @@ void ModuleEntry::set_location(Symbol* location) {
   if (location != nullptr) {
     location->increment_refcount();
     CDS_ONLY(if (CDSConfig::is_using_archive()) {
-        _shared_path_index = FileMapInfo::get_module_shared_path_index(location);
+        _shared_path_index = AOTClassLocationConfig::runtime()->get_module_shared_path_index(_location);
       });
   }
 }
@@ -285,7 +286,7 @@ ModuleEntry::ModuleEntry(Handle module_handle,
   }
 
   if (!module_handle.is_null()) {
-    _module = loader_data->add_handle(module_handle);
+    _module_handle = loader_data->add_handle(module_handle);
   }
 
   set_version(version);
@@ -318,6 +319,15 @@ ModuleEntry* ModuleEntry::create_unnamed_module(ClassLoaderData* cld) {
   // corresponding unnamed module can be found in the java.lang.ClassLoader object.
   oop module = java_lang_ClassLoader::unnamedModule(cld->class_loader());
 
+#if INCLUDE_CDS_JAVA_HEAP
+  ModuleEntry* archived_unnamed_module = ClassLoaderDataShared::archived_unnamed_module(cld);
+  if (archived_unnamed_module != nullptr) {
+    archived_unnamed_module->load_from_archive(cld);
+    archived_unnamed_module->restore_archived_oops(cld);
+    return archived_unnamed_module;
+  }
+#endif
+
   // Ensure that the unnamed module was correctly set when the class loader was constructed.
   // Guarantee will cause a recognizable crash if the user code has circumvented calling the ClassLoader constructor.
   ResourceMark rm;
@@ -334,6 +344,16 @@ ModuleEntry* ModuleEntry::create_unnamed_module(ClassLoaderData* cld) {
 }
 
 ModuleEntry* ModuleEntry::create_boot_unnamed_module(ClassLoaderData* cld) {
+#if INCLUDE_CDS_JAVA_HEAP
+  ModuleEntry* archived_unnamed_module = ClassLoaderDataShared::archived_boot_unnamed_module();
+  if (archived_unnamed_module != nullptr) {
+    archived_unnamed_module->load_from_archive(cld);
+    // It's too early to call archived_unnamed_module->restore_archived_oops(cld).
+    // We will do it inside Modules::set_bootloader_unnamed_module()
+    return archived_unnamed_module;
+  }
+#endif
+
   // For the boot loader, the java.lang.Module for the unnamed module
   // is not known until a call to JVM_SetBootLoaderUnnamedModule is made. At
   // this point initially create the ModuleEntry for the unnamed module.
@@ -346,7 +366,6 @@ ModuleEntry* ModuleEntry::create_boot_unnamed_module(ClassLoaderData* cld) {
 // This is okay because the unnamed module gets created before the ClassLoaderData
 // is available to other threads.
 ModuleEntry* ModuleEntry::new_unnamed_module_entry(Handle module_handle, ClassLoaderData* cld) {
-
   ModuleEntry* entry = new ModuleEntry(module_handle, /*is_open*/true, /*name*/nullptr,
                                        /*version*/ nullptr, /*location*/ nullptr,
                                        cld);
@@ -384,7 +403,7 @@ void ModuleEntry::set_loader_data(ClassLoaderData* cld) {
 }
 
 #if INCLUDE_CDS_JAVA_HEAP
-typedef ResourceHashtable<
+typedef HashTable<
   const ModuleEntry*,
   ModuleEntry*,
   557, // prime number
@@ -396,12 +415,17 @@ static int _num_archived_module_entries = 0;
 static int _num_inited_module_entries = 0;
 #endif
 
+bool ModuleEntry::should_be_archived() const {
+  return SystemDictionaryShared::is_builtin_loader(loader_data());
+}
+
 ModuleEntry* ModuleEntry::allocate_archived_entry() const {
-  assert(is_named(), "unnamed packages/modules are not archived");
+  precond(should_be_archived());
+  precond(CDSConfig::is_dumping_full_module_graph());
   ModuleEntry* archived_entry = (ModuleEntry*)ArchiveBuilder::rw_region_alloc(sizeof(ModuleEntry));
   memcpy((void*)archived_entry, (void*)this, sizeof(ModuleEntry));
-  archived_entry->_archived_module_index = -1;
 
+  archived_entry->_archived_module_index = HeapShared::append_root(module_oop());
   if (_archive_modules_entries == nullptr) {
     _archive_modules_entries = new (mtClass)ArchivedModuleEntries();
   }
@@ -409,9 +433,23 @@ ModuleEntry* ModuleEntry::allocate_archived_entry() const {
   _archive_modules_entries->put(this, archived_entry);
   DEBUG_ONLY(_num_archived_module_entries++);
 
-  if (log_is_enabled(Info, cds, module)) {
+  if (CDSConfig::is_dumping_final_static_archive()) {
+    OopHandle null_handle;
+    archived_entry->_shared_pd = null_handle;
+  } else {
+    assert(archived_entry->shared_protection_domain() == nullptr, "never set during -Xshare:dump");
+  }
+
+  // Clear handles and restore at run time. Handles cannot be archived.
+  OopHandle null_handle;
+  archived_entry->_module_handle = null_handle;
+
+  // For verify_archived_module_entries()
+  DEBUG_ONLY(_num_inited_module_entries++);
+
+  if (log_is_enabled(Info, aot, module)) {
     ResourceMark rm;
-    LogStream ls(Log(cds, module)::info());
+    LogStream ls(Log(aot, module)::info());
     ls.print("Stored in archive: ");
     archived_entry->print(&ls);
   }
@@ -471,10 +509,14 @@ void ModuleEntry::init_as_archived_entry() {
   set_archived_reads(write_growable_array(reads()));
 
   _loader_data = nullptr;  // re-init at runtime
-  _shared_path_index = FileMapInfo::get_module_shared_path_index(_location);
   if (name() != nullptr) {
+    _shared_path_index = AOTClassLocationConfig::dumptime()->get_module_shared_path_index(_location);
     _name = ArchiveBuilder::get_buffered_symbol(_name);
     ArchivePtrMarker::mark_pointer((address*)&_name);
+  } else {
+    // _shared_path_index is used only by SystemDictionary::is_shared_class_visible_impl()
+    // for checking classes in named modules.
+    _shared_path_index = -1;
   }
   if (_version != nullptr) {
     _version = ArchiveBuilder::get_buffered_symbol(_version);
@@ -489,22 +531,6 @@ void ModuleEntry::init_as_archived_entry() {
   ArchivePtrMarker::mark_pointer((address*)&_location);
 }
 
-void ModuleEntry::update_oops_in_archived_module(int root_oop_index) {
-  assert(CDSConfig::is_dumping_full_module_graph(), "sanity");
-  assert(_archived_module_index == -1, "must be set exactly once");
-  assert(root_oop_index >= 0, "sanity");
-
-  _archived_module_index = root_oop_index;
-
-  assert(shared_protection_domain() == nullptr, "never set during -Xshare:dump");
-  // Clear handles and restore at run time. Handles cannot be archived.
-  OopHandle null_handle;
-  _module = null_handle;
-
-  // For verify_archived_module_entries()
-  DEBUG_ONLY(_num_inited_module_entries++);
-}
-
 #ifndef PRODUCT
 void ModuleEntry::verify_archived_module_entries() {
   assert(_num_archived_module_entries == _num_inited_module_entries,
@@ -524,7 +550,7 @@ void ModuleEntry::restore_archived_oops(ClassLoaderData* loader_data) {
   assert(CDSConfig::is_using_archive(), "runtime only");
   Handle module_handle(Thread::current(), HeapShared::get_root(_archived_module_index, /*clear=*/true));
   assert(module_handle.not_null(), "huh");
-  set_module(loader_data->add_handle(module_handle));
+  set_module_handle(loader_data->add_handle(module_handle));
 
   // This was cleared to zero during dump time -- we didn't save the value
   // because it may be affected by archive relocation.
@@ -533,9 +559,9 @@ void ModuleEntry::restore_archived_oops(ClassLoaderData* loader_data) {
   assert(java_lang_Module::loader(module_handle()) == loader_data->class_loader(),
          "must be set in dump time");
 
-  if (log_is_enabled(Info, cds, module)) {
+  if (log_is_enabled(Info, aot, module)) {
     ResourceMark rm;
-    LogStream ls(Log(cds, module)::info());
+    LogStream ls(Log(aot, module)::info());
     ls.print("Restored from archive: ");
     print(&ls);
   }
@@ -660,7 +686,7 @@ void ModuleEntryTable::finalize_javabase(Handle module_handle, Symbol* version,
   jb_module->set_location(location);
   // Once java.base's ModuleEntry _module field is set with the known
   // java.lang.Module, java.base is considered "defined" to the VM.
-  jb_module->set_module(boot_loader_data->add_handle(module_handle));
+  jb_module->set_module_handle(boot_loader_data->add_handle(module_handle));
 
   // Store pointer to the ModuleEntry for java.base in the java.lang.Module object.
   java_lang_Module::set_module_entry(module_handle(), jb_module);
@@ -671,6 +697,8 @@ void ModuleEntryTable::finalize_javabase(Handle module_handle, Symbol* version,
 // classes needing their module field set are added to the fixup_module_list.
 // Their module field is set once java.base's java.lang.Module is known to the VM.
 void ModuleEntryTable::patch_javabase_entries(JavaThread* current, Handle module_handle) {
+  assert(!CDSConfig::is_using_aot_linked_classes(), "patching is not necessary with AOT-linked classes");
+
   if (module_handle.is_null()) {
     fatal("Unable to patch the module field of classes loaded prior to "
           JAVA_BASE_NAME "'s definition, invalid java.lang.Module");
@@ -698,7 +726,7 @@ void ModuleEntryTable::patch_javabase_entries(JavaThread* current, Handle module
       // We allow -XX:ArchiveHeapTestClass to archive additional classes
       // into the CDS heap, but these must be in the unnamed module.
       ModuleEntry* unnamed_module = ClassLoaderData::the_null_class_loader_data()->unnamed_module();
-      Handle unnamed_module_handle(current, unnamed_module->module());
+      Handle unnamed_module_handle(current, unnamed_module->module_oop());
       java_lang_Class::fixup_module_field(k, unnamed_module_handle);
     } else
 #endif
@@ -739,11 +767,11 @@ void ModuleEntryTable::modules_do(ModuleClosure* closure) {
   _table.iterate_all(do_f);
 }
 
-void ModuleEntry::print(outputStream* st) {
+void ModuleEntry::print(outputStream* st) const {
   st->print_cr("entry " PTR_FORMAT " name %s module " PTR_FORMAT " loader %s version %s location %s strict %s",
                p2i(this),
                name_as_C_string(),
-               p2i(module()),
+               p2i(module_oop()),
                loader_data()->loader_name_and_id(),
                version() != nullptr ? version()->as_C_string() : "nullptr",
                location() != nullptr ? location()->as_C_string() : "nullptr",
