@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,7 +30,7 @@
 #include "oops/symbolHandle.hpp"
 #include "runtime/atomic.hpp"
 #include "utilities/growableArray.hpp"
-#include "utilities/resourceHash.hpp"
+#include "utilities/hashTable.hpp"
 #include "utilities/macros.hpp"
 #include "utilities/ostream.hpp"
 #if INCLUDE_JFR
@@ -151,7 +151,7 @@ public:
   // return true.
   bool has_qual_exports_list() const {
     assert_locked_or_safepoint(Module_lock);
-    return (!is_unqual_exported() && _qualified_exports != NULL);
+    return (!is_unqual_exported() && _qualified_exports != nullptr);
   }
   bool is_exported_allUnnamed() const {
     assert_locked_or_safepoint(Module_lock);
@@ -208,6 +208,7 @@ public:
   void print(outputStream* st = tty);
 
 #if INCLUDE_CDS_JAVA_HEAP
+  bool should_be_archived() const;
   void iterate_symbols(MetaspaceClosure* closure);
   PackageEntry* allocate_archived_entry() const;
   void init_as_archived_entry();
@@ -225,19 +226,14 @@ public:
   }
   void set_defined_by_cds_in_class_path(int idx) {
     assert(idx < max_index_for_defined_in_class_path(), "sanity");
-    int old_val = 0;
-    int new_val = 0;
-    do {
-      old_val = Atomic::load(&_defined_by_cds_in_class_path);
-      new_val = old_val | ((int)1 << idx);
-    } while (Atomic::cmpxchg(&_defined_by_cds_in_class_path, old_val, new_val) != old_val);
+    Atomic::fetch_then_or(&_defined_by_cds_in_class_path, ((int)1 << idx));
   }
 };
 
 // The PackageEntryTable is a Hashtable containing a list of all packages defined
 // by a particular class loader.  Each package is represented as a PackageEntry node.
 class PackageEntryTable : public CHeapObj<mtModule> {
-  ResourceHashtable<SymbolHandle, PackageEntry*, 109, ResourceObj::C_HEAP, mtModule,
+  HashTable<SymbolHandle, PackageEntry*, 109, AnyObj::C_HEAP, mtModule,
                     SymbolHandle::compute_hash> _table;
 public:
   PackageEntryTable();
