@@ -143,10 +143,10 @@ void StackMapTable::check_jump_target(
 }
 
 void StackMapTable::print_on(outputStream* str) const {
-  str->indent().print_cr("StackMapTable: frame_count = %d", _frame_count);
-  str->indent().print_cr("table = { ");
+  str->print_cr("StackMapTable: frame_count = %d", _frame_count);
+  str->print_cr("table = {");
   {
-    streamIndentor si(str);
+    StreamIndentor si(str, 2);
     for (int32_t i = 0; i < _frame_count; ++i) {
       _frame_array->at(i)->print_on(str);
     }
@@ -244,7 +244,7 @@ StackMapFrame* StackMapReader::next_helper(TRAPS) {
   int offset;
   VerificationType* locals = nullptr;
   u1 frame_type = _stream->get_u1(CHECK_NULL);
-  if (frame_type < 64) {
+  if (frame_type <= SAME_FRAME_END) {
     // same_frame
     if (_first) {
       offset = frame_type;
@@ -266,17 +266,17 @@ StackMapFrame* StackMapReader::next_helper(TRAPS) {
     _first = false;
     return frame;
   }
-  if (frame_type < 128) {
+  if (frame_type <= SAME_LOCALS_1_STACK_ITEM_FRAME_END) {
     // same_locals_1_stack_item_frame
     if (_first) {
-      offset = frame_type - 64;
+      offset = frame_type - SAME_LOCALS_1_STACK_ITEM_FRAME_START;
       // Can't share the locals array since that is updated by the verifier.
       if (_prev_frame->locals_size() > 0) {
         locals = NEW_RESOURCE_ARRAY_IN_THREAD(
           THREAD, VerificationType, _prev_frame->locals_size());
       }
     } else {
-      offset = _prev_frame->offset() + frame_type - 63;
+      offset = _prev_frame->offset() + frame_type - (SAME_LOCALS_1_STACK_ITEM_FRAME_START - 1);
       locals = _prev_frame->locals();
     }
     VerificationType* stack = NEW_RESOURCE_ARRAY_IN_THREAD(
@@ -340,13 +340,14 @@ StackMapFrame* StackMapReader::next_helper(TRAPS) {
     return frame;
   }
 
-  if (frame_type <= SAME_EXTENDED) {
+  if (frame_type <= SAME_FRAME_EXTENDED) {
     // chop_frame or same_frame_extended
     locals = _prev_frame->locals();
     int length = _prev_frame->locals_size();
-    int chops = SAME_EXTENDED - frame_type;
+    int chops = SAME_FRAME_EXTENDED - frame_type;
     int new_length = length;
     u1 flags = _prev_frame->flags();
+    assert(chops == 0 || (frame_type >= CHOP_FRAME_START && frame_type <= CHOP_FRAME_END), "should be");
     if (chops != 0) {
       new_length = chop(locals, length, chops);
       check_verification_type_array_size(
@@ -380,9 +381,10 @@ StackMapFrame* StackMapReader::next_helper(TRAPS) {
     }
     _first = false;
     return frame;
-  } else if (frame_type < SAME_EXTENDED + 4) {
+  } else if (frame_type <= APPEND_FRAME_END) {
     // append_frame
-    int appends = frame_type - SAME_EXTENDED;
+    assert(frame_type >= APPEND_FRAME_START && frame_type <= APPEND_FRAME_END, "should be");
+    int appends = frame_type - APPEND_FRAME_START + 1;
     int real_length = _prev_frame->locals_size();
     int new_length = real_length + appends*2;
     locals = NEW_RESOURCE_ARRAY_IN_THREAD(THREAD, VerificationType, new_length);
@@ -412,7 +414,7 @@ StackMapFrame* StackMapReader::next_helper(TRAPS) {
     _first = false;
     return frame;
   }
-  if (frame_type == FULL) {
+  if (frame_type == FULL_FRAME) {
     // full_frame
     u1 flags = 0;
     u2 locals_size = _stream->get_u2(CHECK_NULL);
