@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,18 +22,17 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "code/debugInfo.hpp"
 #include "code/debugInfoRec.hpp"
 #include "code/nmethod.hpp"
 #include "gc/shared/collectedHeap.hpp"
 #include "memory/universe.hpp"
 #include "oops/oop.inline.hpp"
-#include "runtime/stackValue.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/interfaceSupport.inline.hpp"
 #include "runtime/javaThread.hpp"
 #include "runtime/jniHandles.inline.hpp"
+#include "runtime/stackValue.hpp"
 
 // Constructors
 
@@ -53,15 +52,9 @@ void DebugInfoWriteStream::write_metadata(Metadata* h) {
 }
 
 oop DebugInfoReadStream::read_oop() {
-  nmethod* nm = const_cast<CompiledMethod*>(code())->as_nmethod_or_null();
-  oop o;
-  if (nm != nullptr) {
-    // Despite these oops being found inside nmethods that are on-stack,
-    // they are not kept alive by all GCs (e.g. G1 and Shenandoah).
-    o = nm->oop_at_phantom(read_int());
-  } else {
-    o = code()->oop_at(read_int());
-  }
+  // Despite these oops being found inside nmethods that are on-stack,
+  // they are not kept alive by all GCs (e.g. G1 and Shenandoah).
+  oop o = code()->oop_at_phantom(read_int());
   assert(oopDesc::is_oop_or_null(o), "oop only");
   return o;
 }
@@ -250,14 +243,13 @@ ObjectValue* ObjectMergeValue::select(frame& fr, RegisterMap& reg_map) {
   // the description of the scalar replaced object.
   if (selector == -1) {
     StackValue* sv_merge_pointer = StackValue::create_stack_value(&fr, &reg_map, _merge_pointer);
-    _selected = new ObjectValue(id());
+    _selected = new ObjectValue(id(), nullptr, false);
 
     // Retrieve the pointer to the real object and use it as if we had
     // allocated it during the deoptimization
     _selected->set_value(sv_merge_pointer->get_obj()());
 
-    // No need to rematerialize
-    return nullptr;
+    return _selected;
   } else {
     assert(selector < _possible_objects.length(), "sanity");
     _selected = (ObjectValue*) _possible_objects.at(selector);
@@ -265,6 +257,14 @@ ObjectValue* ObjectMergeValue::select(frame& fr, RegisterMap& reg_map) {
   }
 }
 
+Handle ObjectMergeValue::value() const {
+  if (_selected != nullptr) {
+    return _selected->value();
+  } else {
+    return Handle();
+  }
+}
+
 void ObjectMergeValue::read_object(DebugInfoReadStream* stream) {
   _selector = read_from(stream);
   _merge_pointer = read_from(stream);
@@ -381,7 +381,7 @@ void ConstantOopReadValue::print_on(outputStream* st) const {
   if (value()() != nullptr) {
     value()()->print_value_on(st);
   } else {
-    st->print("nullptr");
+    st->print("null");
   }
 }
 
