@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -46,8 +46,7 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
 
  public:
   enum GCParPhases {
-    RetireTLABsAndFlushLogs,
-    NonJavaThreadFlushLogs,
+    RetireTLABs,
     GCWorkerStart,
     ExtRootScan,
     ThreadRoots,
@@ -59,7 +58,7 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     MergeER = StrongOopStorageSetRoots + EnumRange<OopStorageSet::StrongId>().size(),
     MergeRS,
     OptMergeRS,
-    MergeLB,
+    SweepRT,
     ScanHR,
     OptScanHR,
     CodeRoots,
@@ -71,7 +70,6 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     Other,
     GCWorkerTotal,
     GCWorkerEnd,
-    RedirtyCards,
     FreeCollectionSet,
     YoungFreeCSet,
     NonYoungFreeCSet,
@@ -111,16 +109,19 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     MergeRSHowlArrayOfCards,
     MergeRSHowlBitmap,
     MergeRSHowlFull,
-    MergeRSCards,
+    MergeRSFromRemSetCards,
+    MergeRSTotalCards,
     MergeRSContainersSentinel
   };
 
   static constexpr const char* GCMergeRSWorkItemsStrings[MergeRSContainersSentinel] =
     { "Merged Inline:", "Merged ArrayOfCards:", "Merged Howl:", "Merged Full:",
       "Merged Howl Inline:", "Merged Howl ArrayOfCards:", "Merged Howl BitMap:", "Merged Howl Full:",
-      "Merged Cards:" };
+      "Merged From RS Cards:", "Total Cards:" };
 
   enum GCScanHRWorkItems {
+    ScanHRPendingCards,
+    ScanHRScannedEmptyCards,
     ScanHRScannedCards,
     ScanHRScannedBlocks,
     ScanHRClaimedChunks,
@@ -129,11 +130,6 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     ScanHRUsedMemory
   };
 
-  enum GCMergeLBWorkItems {
-    MergeLBDirtyCards,
-    MergeLBSkippedCards
-  };
-
   enum GCCodeRootsWorkItems {
     CodeRootsScannedNMethods
   };
@@ -143,7 +139,10 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     MergePSSLABSize,
     MergePSSLABWasteBytes,
     MergePSSLABUndoWasteBytes,
-    MergePSSEvacFailExtra
+    MergePSSPendingCards,      // To be scanned cards generated by GC (from cross-references and evacuation failure).
+    MergePSSToYoungGenCards,   // To-young-gen cards generated by GC.
+    MergePSSEvacFail,          // Evacuation failure generated dirty cards by GC.
+    MergePSSMarked,            // Total newly marked cards.
   };
 
   enum RestoreEvacFailureRegionsWorkItems {
@@ -176,22 +175,22 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
   double _cur_collection_nmethod_list_cleanup_time_ms;
 
   double _cur_merge_heap_roots_time_ms;
+  // Merge refinement table time. Note that this time is included in _cur_merge_heap_roots_time_ms.
+  double _cur_merge_refinement_table_time_ms;
   double _cur_optional_merge_heap_roots_time_ms;
 
   double _cur_prepare_merge_heap_roots_time_ms;
   double _cur_optional_prepare_merge_heap_roots_time_ms;
 
-  double _cur_distribute_log_buffers_time_ms;
-
   double _cur_pre_evacuate_prepare_time_ms;
 
   double _cur_post_evacuate_cleanup_1_time_ms;
   double _cur_post_evacuate_cleanup_2_time_ms;
 
-  double _cur_expand_heap_time_ms;
+  double _cur_resize_heap_time_ms;
   double _cur_ref_proc_time_ms;
 
-  double _cur_collection_start_sec;
+  // Not included in _gc_pause_time_ms
   double _root_region_scan_wait_time_ms;
 
   double _external_accounted_time_ms;
@@ -211,9 +210,13 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
 
   double _cur_region_register_time;
 
+  // Not included in _gc_pause_time_ms
   double _cur_verify_before_time_ms;
   double _cur_verify_after_time_ms;
 
+  // Time spent to trigger concurrent tasks of ResetMarkingState and NoteStartOfMark.
+  double _cur_prepare_concurrent_task_time_ms;
+
   ReferenceProcessorPhaseTimes _ref_phase_times;
   WeakProcessorTimes _weak_phase_times;
 
@@ -278,8 +281,8 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     _cur_pre_evacuate_prepare_time_ms = ms;
   }
 
-  void record_expand_heap_time(double ms) {
-    _cur_expand_heap_time_ms = ms;
+  void record_resize_heap_time(double ms) {
+    _cur_resize_heap_time_ms = ms;
   }
 
   void record_initial_evac_time(double ms) {
@@ -298,6 +301,10 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     _cur_merge_heap_roots_time_ms += ms;
   }
 
+  void record_merge_refinement_table_time(double ms) {
+    _cur_merge_refinement_table_time_ms = ms;
+  }
+
   void record_or_add_optional_merge_heap_roots_time(double ms) {
     _cur_optional_merge_heap_roots_time_ms += ms;
   }
@@ -306,10 +313,6 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     _cur_prepare_merge_heap_roots_time_ms += ms;
   }
 
-  void record_distribute_log_buffers_time_ms(double ms) {
-    _cur_distribute_log_buffers_time_ms += ms;
-  }
-
   void record_or_add_optional_prepare_merge_heap_roots_time(double ms) {
     _cur_optional_prepare_merge_heap_roots_time_ms += ms;
   }
@@ -318,6 +321,10 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     _cur_ref_proc_time_ms = ms;
   }
 
+  void record_prepare_concurrent_task_time_ms(double ms) {
+    _cur_prepare_concurrent_task_time_ms = ms;
+  }
+
   void record_root_region_scan_wait_time(double time_ms) {
     _root_region_scan_wait_time_ms = time_ms;
   }
@@ -358,10 +365,6 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     _recorded_prepare_for_mutator_time_ms = time_ms;
   }
 
-  void record_cur_collection_start_sec(double time_ms) {
-    _cur_collection_start_sec = time_ms;
-  }
-
   void record_verify_before_time_ms(double time_ms) {
     _cur_verify_before_time_ms = time_ms;
   }
@@ -378,23 +381,22 @@ class G1GCPhaseTimes : public CHeapObj<mtGC> {
     _recorded_prepare_heap_roots_time_ms = recorded_prepare_heap_roots_time_ms;
   }
 
-  double cur_collection_start_sec() {
-    return _cur_collection_start_sec;
-  }
-
-  double cur_distribute_log_buffers_time_ms() {
-    return _cur_distribute_log_buffers_time_ms;
-  }
-
   double cur_collection_par_time_ms() {
     return _cur_collection_initial_evac_time_ms +
            _cur_optional_evac_time_ms +
+           _cur_prepare_merge_heap_roots_time_ms +
            _cur_merge_heap_roots_time_ms +
-           _cur_optional_merge_heap_roots_time_ms;
+           _cur_optional_prepare_merge_heap_roots_time_ms +
+           _cur_optional_merge_heap_roots_time_ms +
+           _cur_collection_nmethod_list_cleanup_time_ms;
   }
 
-  double cur_expand_heap_time_ms() {
-    return _cur_expand_heap_time_ms;
+  double cur_merge_refinement_table_time() const {
+    return _cur_merge_refinement_table_time_ms;
+  }
+
+  double cur_resize_heap_time_ms() {
+    return _cur_resize_heap_time_ms;
   }
 
   double root_region_scan_wait_time_ms() {
