@@ -27,7 +27,6 @@
 #include "gc/g1/g1CollectedHeap.inline.hpp"
 #include "gc/g1/g1ConcurrentRefine.hpp"
 #include "gc/g1/g1ConcurrentRefineThread.hpp"
-#include "gc/g1/g1DirtyCardQueue.hpp"
 #include "gc/g1/g1HeapRegion.hpp"
 #include "gc/g1/g1HeapRegionRemSet.inline.hpp"
 #include "gc/g1/g1RemSet.hpp"
@@ -37,39 +36,61 @@
 #include "runtime/javaThread.hpp"
 
 void G1RemSetSummary::update() {
-  class CollectData : public ThreadClosure {
+  G1ConcurrentRefine* refine = G1CollectedHeap::heap()->concurrent_refine();
+
+  class CollectWorkerData : public ThreadClosure {
     G1RemSetSummary* _summary;
     uint _counter;
   public:
-    CollectData(G1RemSetSummary * summary) : _summary(summary),  _counter(0) {}
+    CollectWorkerData(G1RemSetSummary* summary) : _summary(summary),  _counter(0) {}
     virtual void do_thread(Thread* t) {
       G1ConcurrentRefineThread* crt = static_cast<G1ConcurrentRefineThread*>(t);
-      _summary->set_rs_thread_vtime(_counter, crt->vtime_accum());
+      _summary->set_worker_thread_cpu_time(_counter, crt->cpu_time());
       _counter++;
     }
   } collector(this);
 
-  G1CollectedHeap* g1h = G1CollectedHeap::heap();
-  g1h->concurrent_refine()->threads_do(&collector);
+  refine->worker_threads_do(&collector);
+
+  class CollectControlData : public ThreadClosure {
+    G1RemSetSummary* _summary;
+  public:
+    CollectControlData(G1RemSetSummary* summary) : _summary(summary) {}
+    virtual void do_thread(Thread* t) {
+      G1ConcurrentRefineThread* crt = static_cast<G1ConcurrentRefineThread*>(t);
+      _summary->set_control_thread_cpu_time(crt->cpu_time());
+    }
+  } control(this);
+
+  refine->control_thread_do(&control);
 }
 
-void G1RemSetSummary::set_rs_thread_vtime(uint thread, double value) {
-  assert(_rs_threads_vtimes != nullptr, "just checking");
-  assert(thread < _num_vtimes, "just checking");
-  _rs_threads_vtimes[thread] = value;
+void G1RemSetSummary::set_worker_thread_cpu_time(uint thread, jlong value) {
+  assert(_worker_threads_cpu_times != nullptr, "just checking");
+  assert(thread < _num_worker_threads, "just checking");
+  _worker_threads_cpu_times[thread] = value;
 }
 
-double G1RemSetSummary::rs_thread_vtime(uint thread) const {
-  assert(_rs_threads_vtimes != nullptr, "just checking");
-  assert(thread < _num_vtimes, "just checking");
-  return _rs_threads_vtimes[thread];
+void G1RemSetSummary::set_control_thread_cpu_time(jlong value) {
+  _control_thread_cpu_time = value;
+}
+
+jlong G1RemSetSummary::worker_thread_cpu_time(uint thread) const {
+  assert(_worker_threads_cpu_times != nullptr, "just checking");
+  assert(thread < _num_worker_threads, "just checking");
+  return _worker_threads_cpu_times[thread];
+}
+
+jlong G1RemSetSummary::control_thread_cpu_time() const {
+  return _control_thread_cpu_time;
 }
 
 G1RemSetSummary::G1RemSetSummary(bool should_update) :
-  _num_vtimes(G1ConcRefinementThreads),
-  _rs_threads_vtimes(NEW_C_HEAP_ARRAY(double, _num_vtimes, mtGC)) {
+  _num_worker_threads(G1ConcRefinementThreads),
+  _worker_threads_cpu_times(NEW_C_HEAP_ARRAY(jlong, _num_worker_threads, mtGC)),
+  _control_thread_cpu_time(0) {
 
-  memset(_rs_threads_vtimes, 0, sizeof(double) * _num_vtimes);
+  memset(_worker_threads_cpu_times, 0, sizeof(jlong) * _num_worker_threads);
 
   if (should_update) {
     update();
@@ -77,26 +98,28 @@ G1RemSetSummary::G1RemSetSummary(bool should_update) :
 }
 
 G1RemSetSummary::~G1RemSetSummary() {
-  FREE_C_HEAP_ARRAY(double, _rs_threads_vtimes);
+  FREE_C_HEAP_ARRAY(jlong, _worker_threads_cpu_times);
 }
 
 void G1RemSetSummary::set(G1RemSetSummary* other) {
   assert(other != nullptr, "just checking");
-  assert(_num_vtimes == other->_num_vtimes, "just checking");
+  assert(_num_worker_threads == other->_num_worker_threads, "just checking");
 
-  memcpy(_rs_threads_vtimes, other->_rs_threads_vtimes, sizeof(double) * _num_vtimes);
+  memcpy(_worker_threads_cpu_times, other->_worker_threads_cpu_times, sizeof(jlong) * _num_worker_threads);
+  _control_thread_cpu_time = other->_control_thread_cpu_time;
 }
 
 void G1RemSetSummary::subtract_from(G1RemSetSummary* other) {
   assert(other != nullptr, "just checking");
-  assert(_num_vtimes == other->_num_vtimes, "just checking");
+  assert(_num_worker_threads == other->_num_worker_threads, "just checking");
 
-  for (uint i = 0; i < _num_vtimes; i++) {
-    set_rs_thread_vtime(i, other->rs_thread_vtime(i) - rs_thread_vtime(i));
+  for (uint i = 0; i < _num_worker_threads; i++) {
+    set_worker_thread_cpu_time(i, other->worker_thread_cpu_time(i) - worker_thread_cpu_time(i));
   }
+  _control_thread_cpu_time = other->_control_thread_cpu_time - _control_thread_cpu_time;
 }
 
-class RegionTypeCounter {
+class G1PerRegionTypeRemSetCounters {
 private:
   const char* _name;
 
@@ -130,7 +153,7 @@ private:
 
 public:
 
-  RegionTypeCounter(const char* name) : _name(name), _rs_unused_mem_size(0), _rs_mem_size(0), _cards_occupied(0),
+  G1PerRegionTypeRemSetCounters(const char* name) : _name(name), _rs_unused_mem_size(0), _rs_mem_size(0), _cards_occupied(0),
     _amount(0), _amount_tracked(0), _code_root_mem_size(0), _code_root_elems(0) { }
 
   void add(size_t rs_unused_mem_size, size_t rs_mem_size, size_t cards_occupied,
@@ -180,13 +203,12 @@ public:
 };
 
 
-class HRRSStatsIter: public G1HeapRegionClosure {
-private:
-  RegionTypeCounter _young;
-  RegionTypeCounter _humongous;
-  RegionTypeCounter _free;
-  RegionTypeCounter _old;
-  RegionTypeCounter _all;
+class G1HeapRegionStatsClosure: public G1HeapRegionClosure {
+  G1PerRegionTypeRemSetCounters _young;
+  G1PerRegionTypeRemSetCounters _humongous;
+  G1PerRegionTypeRemSetCounters _free;
+  G1PerRegionTypeRemSetCounters _old;
+  G1PerRegionTypeRemSetCounters _all;
 
   size_t _max_rs_mem_sz;
   G1HeapRegion* _max_rs_mem_sz_region;
@@ -214,7 +236,7 @@ private:
   G1HeapRegion* max_code_root_mem_sz_region() const { return _max_code_root_mem_sz_region; }
 
 public:
-  HRRSStatsIter() : _young("Young"), _humongous("Humongous"),
+  G1HeapRegionStatsClosure() : _young("Young"), _humongous("Humongous"),
     _free("Free"), _old("Old"), _all("All"),
     _max_rs_mem_sz(0), _max_rs_mem_sz_region(nullptr),
     _max_code_root_mem_sz(0), _max_code_root_mem_sz_region(nullptr),
@@ -229,7 +251,7 @@ public:
 
     // Accumulate card set details for regions that are assigned to single region
     // groups. G1HeapRegionRemSet::mem_size() includes the size of the code roots
-    if (hrrs->is_added_to_cset_group() && hrrs->cset_group()->length() == 1) {
+    if (hrrs->has_cset_group() && hrrs->cset_group()->length() == 1) {
       G1CardSet* card_set = hrrs->cset_group()->card_set();
 
       rs_mem_sz = hrrs->mem_size() + card_set->mem_size();
@@ -249,7 +271,7 @@ public:
     }
     size_t code_root_elems = hrrs->code_roots_list_length();
 
-    RegionTypeCounter* current = nullptr;
+    G1PerRegionTypeRemSetCounters* current = nullptr;
     if (r->is_free()) {
       current = &_free;
     } else if (r->is_young()) {
@@ -269,49 +291,44 @@ public:
     return false;
   }
 
-  void do_cset_groups() {
-    G1CollectedHeap* g1h = G1CollectedHeap::heap();
-    G1CSetCandidateGroup* young_only_cset_group = g1h->young_regions_cset_group();
-
+  void accumulate_stats_for_group(G1CSetCandidateGroup* group, G1PerRegionTypeRemSetCounters* gen_counter) {
     // If the group has only a single region, then stats were accumulated
-    // during region iteration.
-    if (young_only_cset_group->length() > 1) {
-      G1CardSet* young_only_card_set = young_only_cset_group->card_set();
-      size_t rs_mem_sz = young_only_card_set->mem_size();
-      size_t rs_unused_mem_sz = young_only_card_set->unused_mem_size();
-      size_t occupied_cards = young_only_card_set->occupied();
+    // during region iteration. Skip these.
+    if (group->length() > 1) {
+      G1CardSet* card_set = group->card_set();
 
-      _max_group_cardset_mem_sz = rs_mem_sz;
-      _max_cardset_mem_sz_group = young_only_cset_group;
+      size_t rs_mem_sz = card_set->mem_size();
+      size_t rs_unused_mem_sz = card_set->unused_mem_size();
+      size_t occupied_cards = card_set->occupied();
 
-      // Only update cardset details
-      _young.add(rs_unused_mem_sz, rs_mem_sz, occupied_cards, 0, 0, false);
+      if (rs_mem_sz > _max_group_cardset_mem_sz) {
+        _max_group_cardset_mem_sz = rs_mem_sz;
+        _max_cardset_mem_sz_group = group;
+      }
+
+      gen_counter->add(rs_unused_mem_sz, rs_mem_sz, occupied_cards, 0, 0, false);
       _all.add(rs_unused_mem_sz, rs_mem_sz, occupied_cards, 0, 0, false);
     }
+  }
 
+  void do_cset_groups() {
+    G1CollectedHeap* g1h = G1CollectedHeap::heap();
 
-    RegionTypeCounter* current = &_old;
-    for (G1CSetCandidateGroup* group : g1h->policy()->candidates()->from_marking_groups()) {
-      if (group->length() > 1) {
-        G1CardSet* group_card_set = group->card_set();
-        size_t rs_mem_sz = group_card_set->mem_size();
-        size_t rs_unused_mem_sz = group_card_set->unused_mem_size();
-        size_t occupied_cards = group_card_set->occupied();
+    accumulate_stats_for_group(g1h->young_regions_cset_group(), &_young);
 
-        if (rs_mem_sz > _max_group_cardset_mem_sz) {
-          _max_group_cardset_mem_sz = rs_mem_sz;
-          _max_cardset_mem_sz_group = group;
-        }
-
-        // Only update cardset details
-        _old.add(rs_unused_mem_sz, rs_mem_sz, occupied_cards, 0, 0, false);
-        _all.add(rs_unused_mem_sz, rs_mem_sz, occupied_cards, 0, 0, false);
-      }
+    G1CollectionSetCandidates* candidates = g1h->policy()->candidates();
+    for (G1CSetCandidateGroup* group : candidates->from_marking_groups()) {
+      accumulate_stats_for_group(group, &_old);
+    }
+    // Skip gathering statistics for retained regions. Just verify that they have
+    // the expected amount of regions.
+    for (G1CSetCandidateGroup* group : candidates->retained_groups()) {
+      assert(group->length() == 1, "must be");
     }
   }
 
   void print_summary_on(outputStream* out) {
-    RegionTypeCounter* counters[] = { &_young, &_humongous, &_free, &_old, nullptr };
+    G1PerRegionTypeRemSetCounters* counters[] = { &_young, &_humongous, &_free, &_old, nullptr };
 
     out->print_cr(" Current rem set statistics");
     out->print_cr("  Total per region rem sets sizes = %zu"
@@ -319,13 +336,13 @@ public:
                   total_rs_mem_sz(),
                   max_rs_mem_sz(),
                   total_rs_unused_mem_sz());
-    for (RegionTypeCounter** current = &counters[0]; *current != nullptr; current++) {
+    for (G1PerRegionTypeRemSetCounters** current = &counters[0]; *current != nullptr; current++) {
       (*current)->print_rs_mem_info_on(out, total_rs_mem_sz());
     }
 
     out->print_cr("    %zu occupied cards represented.",
                   total_cards_occupied());
-    for (RegionTypeCounter** current = &counters[0]; *current != nullptr; current++) {
+    for (G1PerRegionTypeRemSetCounters** current = &counters[0]; *current != nullptr; current++) {
       (*current)->print_cards_occupied_info_on(out, total_cards_occupied());
     }
 
@@ -341,7 +358,6 @@ public:
 
     if (max_cardset_mem_sz_group() != nullptr) {
       G1CSetCandidateGroup* cset_group = max_cardset_mem_sz_group();
-      G1HeapRegionRemSet* rem_set = max_rs_mem_sz_region()->rem_set();
       out->print_cr("    Collectionset Candidate Group with largest cardset = %u:(%u regions), "
                     "size = %zu occupied = %zu",
                     cset_group->group_id(), cset_group->length(),
@@ -361,13 +377,13 @@ public:
                   proper_unit_for_byte_size(total_code_root_mem_sz()),
                   byte_size_in_proper_unit(max_code_root_rem_set->code_roots_mem_size()),
                   proper_unit_for_byte_size(max_code_root_rem_set->code_roots_mem_size()));
-    for (RegionTypeCounter** current = &counters[0]; *current != nullptr; current++) {
+    for (G1PerRegionTypeRemSetCounters** current = &counters[0]; *current != nullptr; current++) {
       (*current)->print_code_root_mem_info_on(out, total_code_root_mem_sz());
     }
 
     out->print_cr("    %zu code roots represented.",
                   total_code_root_elems());
-    for (RegionTypeCounter** current = &counters[0]; *current != nullptr; current++) {
+    for (G1PerRegionTypeRemSetCounters** current = &counters[0]; *current != nullptr; current++) {
       (*current)->print_code_root_elems_info_on(out, total_code_root_elems());
     }
 
@@ -383,14 +399,14 @@ public:
 void G1RemSetSummary::print_on(outputStream* out, bool show_thread_times) {
   if (show_thread_times) {
     out->print_cr(" Concurrent refinement threads times (s)");
+    out->print_cr(" Control %5.2f Workers", (double)control_thread_cpu_time() / NANOSECS_PER_SEC);
     out->print("     ");
-    for (uint i = 0; i < _num_vtimes; i++) {
-      out->print("    %5.2f", rs_thread_vtime(i));
+    for (uint i = 0; i < _num_worker_threads; i++) {
+      out->print("    %5.2f", (double)worker_thread_cpu_time(i) / NANOSECS_PER_SEC);
     }
     out->cr();
   }
-
-  HRRSStatsIter blk;
+  G1HeapRegionStatsClosure blk;
   G1CollectedHeap::heap()->heap_region_iterate(&blk);
   blk.do_cset_groups();
   blk.print_summary_on(out);
