@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "gc/g1/g1CollectedHeap.inline.hpp"
 #include "gc/g1/g1CollectionSetChooser.hpp"
 #include "gc/g1/g1HeapRegion.inline.hpp"
@@ -105,6 +104,17 @@ void G1RemSetTrackingPolicy::update_after_rebuild(G1HeapRegion* r) {
                                            r->rem_set()->clear(true /* only_cardset */);
                                          });
     }
+
+    size_t remset_bytes = r->rem_set()->mem_size();
+    size_t occupied = 0;
+    // per region cardset details only valid if group contains a single region.
+    if (r->rem_set()->has_cset_group() &&
+        r->rem_set()->cset_group()->length() == 1 ) {
+        G1CardSet *card_set = r->rem_set()->cset_group()->card_set();
+        remset_bytes += card_set->mem_size();
+        occupied = card_set->occupied();
+    }
+
     G1ConcurrentMark* cm = G1CollectedHeap::heap()->concurrent_mark();
     log_trace(gc, remset, tracking)("After rebuild region %u "
                                     "(tams " PTR_FORMAT " "
@@ -114,7 +124,7 @@ void G1RemSetTrackingPolicy::update_after_rebuild(G1HeapRegion* r) {
                                     r->hrm_index(),
                                     p2i(cm->top_at_mark_start(r)),
                                     cm->live_bytes(r->hrm_index()),
-                                    r->rem_set()->occupied(),
-                                    r->rem_set()->mem_size());
+                                    occupied,
+                                    remset_bytes);
   }
 }
