@@ -37,19 +37,18 @@
 #include "utilities/align.hpp"
 
 PSOldGen::PSOldGen(ReservedSpace rs, size_t initial_size, size_t min_size,
-                   size_t max_size, const char* perf_data_name, int level):
+                   size_t max_size):
   _min_gen_size(min_size),
   _max_gen_size(max_size)
 {
-  initialize(rs, initial_size, GenAlignment, perf_data_name, level);
+  initialize(rs, initial_size, SpaceAlignment);
 }
 
-void PSOldGen::initialize(ReservedSpace rs, size_t initial_size, size_t alignment,
-                          const char* perf_data_name, int level) {
+void PSOldGen::initialize(ReservedSpace rs, size_t initial_size, size_t alignment) {
   initialize_virtual_space(rs, initial_size, alignment);
-  initialize_work(perf_data_name, level);
+  initialize_work();
 
-  initialize_performance_counters(perf_data_name, level);
+  initialize_performance_counters();
 }
 
 void PSOldGen::initialize_virtual_space(ReservedSpace rs,
@@ -63,13 +62,10 @@ void PSOldGen::initialize_virtual_space(ReservedSpace rs,
   }
 }
 
-void PSOldGen::initialize_work(const char* perf_data_name, int level) {
+void PSOldGen::initialize_work() {
   MemRegion const reserved_mr = reserved();
   assert(reserved_mr.byte_size() == max_gen_size(), "invariant");
 
-  // Object start stuff: for all reserved memory
-  start_array()->initialize(reserved_mr);
-
   // Card table stuff: for all committed memory
   MemRegion committed_mr((HeapWord*)virtual_space()->low(),
                          (HeapWord*)virtual_space()->high());
@@ -108,13 +104,14 @@ void PSOldGen::initialize_work(const char* perf_data_name, int level) {
                              &ParallelScavengeHeap::heap()->workers());
 
   // Update the start_array
+  _start_array = new ObjectStartArray(reserved_mr);
   start_array()->set_covered_region(committed_mr);
 }
 
-void PSOldGen::initialize_performance_counters(const char* perf_data_name, int level) {
-  // Generation Counters, generation 'level', 1 subspace
-  _gen_counters = new PSGenerationCounters(perf_data_name, level, 1, min_gen_size(),
-                                           max_gen_size(), virtual_space());
+void PSOldGen::initialize_performance_counters() {
+  const char* perf_data_name = "old";
+  _gen_counters = new GenerationCounters(perf_data_name, 1, 1, min_gen_size(),
+                                         max_gen_size(), virtual_space()->committed_size());
   _space_counters = new SpaceCounters(perf_data_name, 0,
                                       virtual_space()->reserved_size(),
                                       _object_space, _gen_counters);
@@ -175,12 +172,24 @@ bool PSOldGen::expand_for_allocate(size_t word_size) {
       result = expand(word_size*HeapWordSize);
     }
   }
-  if (GCExpandToAllocateDelayMillis > 0) {
-    os::naked_sleep(GCExpandToAllocateDelayMillis);
-  }
   return result;
 }
 
+void PSOldGen::try_expand_till_size(size_t target_capacity_bytes) {
+  if (target_capacity_bytes <= capacity_in_bytes()) {
+    // Current capacity is enough
+    return;
+  }
+
+  if (capacity_in_bytes() == max_gen_size()) {
+    // Already at max size
+    return;
+  }
+
+  size_t to_expand_bytes = target_capacity_bytes - capacity_in_bytes();
+  expand(to_expand_bytes);
+}
+
 bool PSOldGen::expand(size_t bytes) {
 #ifdef ASSERT
   if (!Thread::current()->is_VM_thread()) {
@@ -189,8 +198,12 @@ bool PSOldGen::expand(size_t bytes) {
   assert_locked_or_safepoint(Heap_lock);
   assert(bytes > 0, "precondition");
 #endif
+  const size_t remaining_bytes = virtual_space()->uncommitted_size();
+  if (remaining_bytes == 0) {
+    return false;
+  }
   const size_t alignment = virtual_space()->alignment();
-  size_t aligned_bytes  = align_up(bytes, alignment);
+  size_t aligned_bytes = align_up(MIN2(bytes, remaining_bytes), alignment);
   size_t aligned_expand_bytes = align_up(MinHeapDeltaBytes, alignment);
 
   if (UseNUMA) {
@@ -198,13 +211,6 @@ bool PSOldGen::expand(size_t bytes) {
     // providing a page per lgroup. Alignment is larger or equal to the page size.
     aligned_expand_bytes = MAX2(aligned_expand_bytes, alignment * os::numa_get_groups_num());
   }
-  if (aligned_bytes == 0) {
-    // The alignment caused the number of bytes to wrap.  A call to expand
-    // implies a best effort to expand by "bytes" but not a guarantee.  Align
-    // down to give a best effort.  This is likely the most that the generation
-    // can expand since it has some capacity to start with.
-    aligned_bytes = align_down(bytes, alignment);
-  }
 
   bool success = false;
   if (aligned_expand_bytes > aligned_bytes) {
@@ -217,9 +223,6 @@ bool PSOldGen::expand(size_t bytes) {
     success = expand_to_reserved();
   }
 
-  if (success && GCLocker::is_active_and_needs_gc()) {
-    log_debug(gc)("Garbage collection disabled, expanded heap instead");
-  }
   return success;
 }
 
@@ -243,7 +246,7 @@ bool PSOldGen::expand_by(size_t bytes) {
     post_resize();
     if (UsePerfData) {
       _space_counters->update_capacity();
-      _gen_counters->update_all();
+      _gen_counters->update_capacity(_virtual_space->committed_size());
     }
   }
 
@@ -268,8 +271,9 @@ bool PSOldGen::expand_to_reserved() {
 }
 
 void PSOldGen::shrink(size_t bytes) {
-  assert_lock_strong(PSOldGenExpand_lock);
-  assert_locked_or_safepoint(Heap_lock);
+  assert(Thread::current()->is_VM_thread(), "precondition");
+  assert(SafepointSynchronize::is_at_safepoint(), "precondition");
+  assert(bytes > 0, "precondition");
 
   size_t size = align_down(bytes, virtual_space()->alignment());
   if (size > 0) {
@@ -287,19 +291,15 @@ void PSOldGen::complete_loaded_archive_space(MemRegion archive_space) {
   HeapWord* cur = archive_space.start();
   while (cur < archive_space.end()) {
     size_t word_size = cast_to_oop(cur)->size();
-    _start_array.update_for_block(cur, cur + word_size);
+    _start_array->update_for_block(cur, cur + word_size);
     cur += word_size;
   }
 }
 
-void PSOldGen::resize(size_t desired_free_space) {
+void PSOldGen::resize(size_t desired_capacity) {
   const size_t alignment = virtual_space()->alignment();
   const size_t size_before = virtual_space()->committed_size();
-  size_t new_size = used_in_bytes() + desired_free_space;
-  if (new_size < used_in_bytes()) {
-    // Overflowed the addition.
-    new_size = max_gen_size();
-  }
+  size_t new_size = desired_capacity;
   // Adjust according to our min and max
   new_size = clamp(new_size, min_gen_size(), max_gen_size());
 
@@ -308,10 +308,10 @@ void PSOldGen::resize(size_t desired_free_space) {
   const size_t current_size = capacity_in_bytes();
 
   log_trace(gc, ergo)("AdaptiveSizePolicy::old generation size: "
-    "desired free: %zu used: %zu"
-    " new size: %zu current size %zu"
+    "used: %zu"
+    " capacity %zu -> %zu"
     " gen limits: %zu / %zu",
-    desired_free_space, used_in_bytes(), new_size, current_size,
+    used_in_bytes(), current_size, new_size,
     max_gen_size(), min_gen_size());
 
   if (new_size == current_size) {
@@ -320,11 +320,9 @@ void PSOldGen::resize(size_t desired_free_space) {
   }
   if (new_size > current_size) {
     size_t change_bytes = new_size - current_size;
-    MutexLocker x(PSOldGenExpand_lock);
     expand(change_bytes);
   } else {
     size_t change_bytes = current_size - new_size;
-    MutexLocker x(PSOldGenExpand_lock);
     shrink(change_bytes);
   }
 
@@ -365,21 +363,18 @@ void PSOldGen::post_resize() {
 
 void PSOldGen::print() const { print_on(tty);}
 void PSOldGen::print_on(outputStream* st) const {
-  st->print(" %-15s", name());
-  st->print(" total %zuK, used %zuK",
-              capacity_in_bytes()/K, used_in_bytes()/K);
-  st->print_cr(" [" PTR_FORMAT ", " PTR_FORMAT ", " PTR_FORMAT ")",
-                p2i(virtual_space()->low_boundary()),
-                p2i(virtual_space()->high()),
-                p2i(virtual_space()->high_boundary()));
+  st->print("%-15s", name());
+  st->print(" total %zuK, used %zuK ", capacity_in_bytes() / K, used_in_bytes() / K);
+  virtual_space()->print_space_boundaries_on(st);
 
-  st->print("  object"); object_space()->print_on(st);
+  StreamIndentor si(st, 1);
+  object_space()->print_on(st, "object ");
 }
 
 void PSOldGen::update_counters() {
   if (UsePerfData) {
     _space_counters->update_all();
-    _gen_counters->update_all();
+    _gen_counters->update_capacity(_virtual_space->committed_size());
   }
 }
 
