@@ -27,7 +27,7 @@
 #include "gc/parallel/parallelScavengeHeap.hpp"
 #include "gc/parallel/psOldGen.hpp"
 #include "gc/parallel/psPromotionManager.inline.hpp"
-#include "gc/parallel/psScavenge.inline.hpp"
+#include "gc/parallel/psScavenge.hpp"
 #include "gc/shared/continuationGCSupport.inline.hpp"
 #include "gc/shared/gcTrace.hpp"
 #include "gc/shared/partialArraySplitter.inline.hpp"
@@ -86,15 +86,6 @@ void PSPromotionManager::initialize() {
   }
 }
 
-// Helper functions to get around the circular dependency between
-// psScavenge.inline.hpp and psPromotionManager.inline.hpp.
-bool PSPromotionManager::should_scavenge(oop* p, bool check_to_space) {
-  return PSScavenge::should_scavenge(p, check_to_space);
-}
-bool PSPromotionManager::should_scavenge(narrowOop* p, bool check_to_space) {
-  return PSScavenge::should_scavenge(p, check_to_space);
-}
-
 PSPromotionManager* PSPromotionManager::gc_thread_promotion_manager(uint index) {
   assert(index < ParallelGCThreads, "index out of range");
   assert(_manager_array != nullptr, "Sanity");
@@ -166,7 +157,7 @@ PartialArrayTaskStats* PSPromotionManager::partial_array_task_stats() {
 
 // Most members are initialized either by initialize() or reset().
 PSPromotionManager::PSPromotionManager()
-  : _partial_array_splitter(_partial_array_state_manager, ParallelGCThreads)
+  : _partial_array_splitter(_partial_array_state_manager, ParallelGCThreads, ParGCArrayScanChunk)
 {
   // We set the old lab's start array.
   _old_lab.set_start_array(old_gen()->start_array());
@@ -193,6 +184,7 @@ void PSPromotionManager::reset() {
   // Do not prefill the LAB's, save heap wastage!
   HeapWord* lab_base = young_space()->top();
   _young_lab.initialize(MemRegion(lab_base, (size_t)0));
+  _young_gen_has_alloc_failure = false;
   _young_gen_is_full = false;
 
   lab_base = old_gen()->object_space()->top();
@@ -211,7 +203,7 @@ void PSPromotionManager::restore_preserved_marks() {
   _preserved_marks_set->restore(&ParallelScavengeHeap::heap()->workers());
 }
 
-void PSPromotionManager::drain_stacks_depth(bool totally_drain) {
+void PSPromotionManager::drain_stacks(bool totally_drain) {
   const uint threshold = totally_drain ? 0
                                        : _target_stack_size;
 
@@ -251,7 +243,7 @@ void PSPromotionManager::flush_labs() {
     _old_lab.flush();
 
   // Let PSScavenge know if we overflowed
-  if (_young_gen_is_full) {
+  if (_young_gen_is_full || _young_gen_has_alloc_failure) {
     PSScavenge::set_survivor_overflow(true);
   }
 }
