@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2023, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -61,14 +61,4 @@ class CSpaceCounters: public CHeapObj<mtGC> {
   const char* name_space() const        { return _name_space; }
 };
 
-class ContiguousSpaceUsedHelper : public PerfLongSampleHelper {
-  private:
-    ContiguousSpace* _space;
-
-  public:
-    ContiguousSpaceUsedHelper(ContiguousSpace* space) : _space(space) { }
-
-    jlong take_sample() override;
-};
-
 #endif // SHARE_GC_SERIAL_CSPACECOUNTERS_HPP
