@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,14 +22,13 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "gc/shared/oopStorage.inline.hpp"
 #include "gc/shared/oopStorageParState.inline.hpp"
 #include "logging/log.hpp"
 #include "logging/logStream.hpp"
 #include "memory/allocation.inline.hpp"
 #include "nmt/memTracker.hpp"
-#include "runtime/atomic.hpp"
+#include "runtime/atomicAccess.hpp"
 #include "runtime/globals.hpp"
 #include "runtime/handles.inline.hpp"
 #include "runtime/interfaceSupport.inline.hpp"
@@ -149,16 +148,16 @@ size_t OopStorage::ActiveArray::block_count() const {
 }
 
 size_t OopStorage::ActiveArray::block_count_acquire() const {
-  return Atomic::load_acquire(&_block_count);
+  return AtomicAccess::load_acquire(&_block_count);
 }
 
 void OopStorage::ActiveArray::increment_refcount() const {
-  int new_value = Atomic::add(&_refcount, 1);
+  int new_value = AtomicAccess::add(&_refcount, 1);
   assert(new_value >= 1, "negative refcount %d", new_value - 1);
 }
 
 bool OopStorage::ActiveArray::decrement_refcount() const {
-  int new_value = Atomic::sub(&_refcount, 1);
+  int new_value = AtomicAccess::sub(&_refcount, 1);
   assert(new_value >= 0, "negative refcount %d", new_value);
   return new_value == 0;
 }
@@ -170,7 +169,7 @@ bool OopStorage::ActiveArray::push(Block* block) {
     *block_ptr(index) = block;
     // Use a release_store to ensure all the setup is complete before
     // making the block visible.
-    Atomic::release_store(&_block_count, index + 1);
+    AtomicAccess::release_store(&_block_count, index + 1);
     return true;
   } else {
     return false;
@@ -273,8 +272,8 @@ uintx OopStorage::Block::bitmask_for_entry(const oop* ptr) const {
 bool OopStorage::Block::is_safe_to_delete() const {
   assert(is_empty(), "precondition");
   OrderAccess::loadload();
-  return (Atomic::load_acquire(&_release_refcount) == 0) &&
-         (Atomic::load_acquire(&_deferred_updates_next) == nullptr);
+  return (AtomicAccess::load_acquire(&_release_refcount) == 0) &&
+         (AtomicAccess::load_acquire(&_deferred_updates_next) == nullptr);
 }
 
 OopStorage::Block* OopStorage::Block::deferred_updates_next() const {
@@ -300,7 +299,12 @@ void OopStorage::Block::set_active_index(size_t index) {
 
 size_t OopStorage::Block::active_index_safe(const Block* block) {
   STATIC_ASSERT(sizeof(intptr_t) == sizeof(block->_active_index));
-  return SafeFetchN((intptr_t*)&block->_active_index, 0);
+  // Be careful, because block could be a false positive from block_for_ptr.
+  assert(block != nullptr, "precondition");
+  uintptr_t block_addr = reinterpret_cast<uintptr_t>(block);
+  uintptr_t index_loc = block_addr + offset_of(Block, _active_index);
+  static_assert(sizeof(size_t) == sizeof(intptr_t), "assumption");
+  return static_cast<size_t>(SafeFetchN(reinterpret_cast<intptr_t*>(index_loc), 0));
 }
 
 unsigned OopStorage::Block::get_index(const oop* ptr) const {
@@ -317,8 +321,8 @@ void OopStorage::Block::atomic_add_allocated(uintx add) {
   // we can use an atomic add to implement the operation.  The assert post
   // facto verifies the precondition held; if there were any set bits in
   // common, then after the add at least one of them will be zero.
-  uintx sum = Atomic::add(&_allocated_bitmask, add);
-  assert((sum & add) == add, "some already present: " UINTX_FORMAT ":" UINTX_FORMAT,
+  uintx sum = AtomicAccess::add(&_allocated_bitmask, add);
+  assert((sum & add) == add, "some already present: %zu:%zu",
          sum, add);
 }
 
@@ -366,21 +370,23 @@ void OopStorage::Block::delete_block(const Block& block) {
 OopStorage::Block*
 OopStorage::Block::block_for_ptr(const OopStorage* owner, const oop* ptr) {
   STATIC_ASSERT(_data_pos == 0);
-  // Const-ness of ptr is not related to const-ness of containing block.
+  assert(ptr != nullptr, "precondition");
   // Blocks are allocated section-aligned, so get the containing section.
-  oop* section_start = align_down(const_cast<oop*>(ptr), block_alignment);
+  uintptr_t section_start = align_down(reinterpret_cast<uintptr_t>(ptr), block_alignment);
   // Start with a guess that the containing section is the last section,
   // so the block starts section_count-1 sections earlier.
-  oop* section = section_start - (section_size * (section_count - 1));
+  size_t section_size_in_bytes = sizeof(oop) * section_size;
+  uintptr_t section = section_start - (section_size_in_bytes * (section_count - 1));
   // Walk up through the potential block start positions, looking for
   // the owner in the expected location.  If we're below the actual block
   // start position, the value at the owner position will be some oop
   // (possibly null), which can never match the owner.
   intptr_t owner_addr = reinterpret_cast<intptr_t>(owner);
-  for (unsigned i = 0; i < section_count; ++i, section += section_size) {
-    Block* candidate = reinterpret_cast<Block*>(section);
-    if (SafeFetchN(&candidate->_owner_address, 0) == owner_addr) {
-      return candidate;
+  for (unsigned i = 0; i < section_count; ++i, section += section_size_in_bytes) {
+    uintptr_t owner_loc = section + offset_of(Block, _owner_address);
+    static_assert(sizeof(OopStorage*) == sizeof(intptr_t), "assumption");
+    if (SafeFetchN(reinterpret_cast<intptr_t*>(owner_loc), 0) == owner_addr) {
+      return reinterpret_cast<Block*>(section);
     }
   }
   return nullptr;
@@ -446,7 +452,7 @@ oop* OopStorage::allocate() {
   oop* result = block->allocate();
   assert(result != nullptr, "allocation failed");
   assert(!block->is_empty(), "postcondition");
-  Atomic::inc(&_allocation_count); // release updates outside lock.
+  AtomicAccess::inc(&_allocation_count); // release updates outside lock.
   if (block->is_full()) {
     // Transitioning from not full to full.
     // Remove full blocks from consideration by future allocates.
@@ -484,7 +490,7 @@ size_t OopStorage::allocate(oop** ptrs, size_t size) {
     assert(!is_empty_bitmask(taken), "invariant");
   } // Drop lock, now that we've taken all available entries from block.
   size_t num_taken = population_count(taken);
-  Atomic::add(&_allocation_count, num_taken);
+  AtomicAccess::add(&_allocation_count, num_taken);
   // Fill ptrs from those taken entries.
   size_t limit = MIN2(num_taken, size);
   for (size_t i = 0; i < limit; ++i) {
@@ -500,7 +506,7 @@ size_t OopStorage::allocate(oop** ptrs, size_t size) {
     assert(size == limit, "invariant");
     assert(num_taken == (limit + population_count(taken)), "invariant");
     block->release_entries(taken, this);
-    Atomic::sub(&_allocation_count, num_taken - limit);
+    AtomicAccess::sub(&_allocation_count, num_taken - limit);
   }
   log_trace(oopstorage, ref)("%s: bulk allocate %zu, returned %zu",
                              name(), limit, num_taken - limit);
@@ -572,7 +578,7 @@ bool OopStorage::expand_active_array() {
   assert_lock_strong(_allocation_mutex);
   ActiveArray* old_array = _active_array;
   size_t new_size = 2 * old_array->size();
-  log_debug(oopstorage, blocks)("%s: expand active array " SIZE_FORMAT,
+  log_debug(oopstorage, blocks)("%s: expand active array %zu",
                                 name(), new_size);
   ActiveArray* new_array = ActiveArray::create(new_size,
                                                mem_tag(),
@@ -593,7 +599,7 @@ void OopStorage::replace_active_array(ActiveArray* new_array) {
   // Update new_array refcount to account for the new reference.
   new_array->increment_refcount();
   // Install new_array, ensuring its initialization is complete first.
-  Atomic::release_store(&_active_array, new_array);
+  AtomicAccess::release_store(&_active_array, new_array);
   // Wait for any readers that could read the old array from _active_array.
   // Can't use GlobalCounter here, because this is called from allocate(),
   // which may be called in the scope of a GlobalCounter critical section
@@ -611,7 +617,7 @@ void OopStorage::replace_active_array(ActiveArray* new_array) {
 // using it.
 OopStorage::ActiveArray* OopStorage::obtain_active_array() const {
   SingleWriterSynchronizer::CriticalSection cs(&_protect_active);
-  ActiveArray* result = Atomic::load_acquire(&_active_array);
+  ActiveArray* result = AtomicAccess::load_acquire(&_active_array);
   result->increment_refcount();
   return result;
 }
@@ -643,8 +649,7 @@ public:
   }
 };
 
-OopStorage::Block* OopStorage::find_block_or_null(const oop* ptr) const {
-  assert(ptr != nullptr, "precondition");
+OopStorage::Block* OopStorage::block_for_ptr(const oop* ptr) const {
   return Block::block_for_ptr(this, ptr);
 }
 
@@ -667,14 +672,14 @@ static void log_release_transitions(uintx releasing,
 void OopStorage::Block::release_entries(uintx releasing, OopStorage* owner) {
   assert(releasing != 0, "preconditon");
   // Prevent empty block deletion when transitioning to empty.
-  Atomic::inc(&_release_refcount);
+  AtomicAccess::inc(&_release_refcount);
 
   // Atomically update allocated bitmask.
   uintx old_allocated = _allocated_bitmask;
   while (true) {
     assert((releasing & ~old_allocated) == 0, "releasing unallocated entries");
     uintx new_value = old_allocated ^ releasing;
-    uintx fetched = Atomic::cmpxchg(&_allocated_bitmask, old_allocated, new_value);
+    uintx fetched = AtomicAccess::cmpxchg(&_allocated_bitmask, old_allocated, new_value);
     if (fetched == old_allocated) break; // Successful update.
     old_allocated = fetched;             // Retry with updated bitmask.
   }
@@ -693,12 +698,12 @@ void OopStorage::Block::release_entries(uintx releasing, OopStorage* owner) {
     // then someone else has made such a claim and the deferred update has not
     // yet been processed and will include our change, so we don't need to do
     // anything further.
-    if (Atomic::replace_if_null(&_deferred_updates_next, this)) {
+    if (AtomicAccess::replace_if_null(&_deferred_updates_next, this)) {
       // Successfully claimed.  Push, with self-loop for end-of-list.
       Block* head = owner->_deferred_updates;
       while (true) {
         _deferred_updates_next = (head == nullptr) ? this : head;
-        Block* fetched = Atomic::cmpxchg(&owner->_deferred_updates, head, this);
+        Block* fetched = AtomicAccess::cmpxchg(&owner->_deferred_updates, head, this);
         if (fetched == head) break; // Successful update.
         head = fetched;             // Retry with updated head.
       }
@@ -715,7 +720,7 @@ void OopStorage::Block::release_entries(uintx releasing, OopStorage* owner) {
     }
   }
   // Release hold on empty block deletion.
-  Atomic::dec(&_release_refcount);
+  AtomicAccess::dec(&_release_refcount);
 }
 
 // Process one available deferred update.  Returns true if one was processed.
@@ -724,13 +729,13 @@ bool OopStorage::reduce_deferred_updates() {
   // Atomically pop a block off the list, if any available.
   // No ABA issue because this is only called by one thread at a time.
   // The atomicity is wrto pushes by release().
-  Block* block = Atomic::load_acquire(&_deferred_updates);
+  Block* block = AtomicAccess::load_acquire(&_deferred_updates);
   while (true) {
     if (block == nullptr) return false;
     // Try atomic pop of block from list.
     Block* tail = block->deferred_updates_next();
     if (block == tail) tail = nullptr; // Handle self-loop end marker.
-    Block* fetched = Atomic::cmpxchg(&_deferred_updates, block, tail);
+    Block* fetched = AtomicAccess::cmpxchg(&_deferred_updates, block, tail);
     if (fetched == block) break; // Update successful.
     block = fetched;             // Retry with updated block.
   }
@@ -771,18 +776,18 @@ static inline void check_release_entry(const oop* entry) {
 
 void OopStorage::release(const oop* ptr) {
   check_release_entry(ptr);
-  Block* block = find_block_or_null(ptr);
+  Block* block = block_for_ptr(ptr);
   assert(block != nullptr, "%s: invalid release " PTR_FORMAT, name(), p2i(ptr));
   log_trace(oopstorage, ref)("%s: releasing " PTR_FORMAT, name(), p2i(ptr));
   block->release_entries(block->bitmask_for_entry(ptr), this);
-  Atomic::dec(&_allocation_count);
+  AtomicAccess::dec(&_allocation_count);
 }
 
 void OopStorage::release(const oop* const* ptrs, size_t size) {
   size_t i = 0;
   while (i < size) {
     check_release_entry(ptrs[i]);
-    Block* block = find_block_or_null(ptrs[i]);
+    Block* block = block_for_ptr(ptrs[i]);
     assert(block != nullptr, "%s: invalid release " PTR_FORMAT, name(), p2i(ptrs[i]));
     size_t count = 0;
     uintx releasing = 0;
@@ -801,7 +806,7 @@ void OopStorage::release(const oop* const* ptrs, size_t size) {
     }
     // Release the contiguous entries that are in block.
     block->release_entries(releasing, this);
-    Atomic::sub(&_allocation_count, count);
+    AtomicAccess::sub(&_allocation_count, count);
   }
 }
 
@@ -815,7 +820,7 @@ static Mutex* make_oopstorage_mutex(const char* storage_name,
                                     const char* kind,
                                     Mutex::Rank rank) {
   char name[256];
-  os::snprintf(name, sizeof(name), "%s %s lock", storage_name, kind);
+  os::snprintf_checked(name, sizeof(name), "%s %s lock", storage_name, kind);
   return new PaddedMutex(rank, name);
 }
 
@@ -901,12 +906,12 @@ const jlong cleanup_defer_period = 500 * NANOSECS_PER_MILLISEC;
 bool OopStorage::has_cleanup_work_and_reset() {
   assert_lock_strong(Service_lock);
 
-  if (Atomic::load_acquire(&needs_cleanup_requested) &&
+  if (AtomicAccess::load_acquire(&needs_cleanup_requested) &&
       os::javaTimeNanos() > cleanup_permit_time) {
     cleanup_permit_time =
       os::javaTimeNanos() + cleanup_defer_period;
     // Set the request flag false and return its old value.
-    Atomic::release_store(&needs_cleanup_requested, false);
+    AtomicAccess::release_store(&needs_cleanup_requested, false);
     return true;
   } else {
     return false;
@@ -918,22 +923,22 @@ bool OopStorage::has_cleanup_work_and_reset() {
 void OopStorage::record_needs_cleanup() {
   // Set local flag first, else ServiceThread could wake up and miss
   // the request.
-  Atomic::release_store(&_needs_cleanup, true);
-  Atomic::release_store_fence(&needs_cleanup_requested, true);
+  AtomicAccess::release_store(&_needs_cleanup, true);
+  AtomicAccess::release_store_fence(&needs_cleanup_requested, true);
 }
 
 bool OopStorage::delete_empty_blocks() {
   // ServiceThread might have oopstorage work, but not for this object.
   // But check for deferred updates, which might provide cleanup work.
-  if (!Atomic::load_acquire(&_needs_cleanup) &&
-      (Atomic::load_acquire(&_deferred_updates) == nullptr)) {
+  if (!AtomicAccess::load_acquire(&_needs_cleanup) &&
+      (AtomicAccess::load_acquire(&_deferred_updates) == nullptr)) {
     return false;
   }
 
   MutexLocker ml(_allocation_mutex, Mutex::_no_safepoint_check_flag);
 
   // Clear the request before processing.
-  Atomic::release_store_fence(&_needs_cleanup, false);
+  AtomicAccess::release_store_fence(&_needs_cleanup, false);
 
   // Other threads could be adding to the empty block count or the
   // deferred update list while we're working.  Set an upper bound on
@@ -989,7 +994,8 @@ bool OopStorage::delete_empty_blocks() {
 }
 
 OopStorage::EntryStatus OopStorage::allocation_status(const oop* ptr) const {
-  const Block* block = find_block_or_null(ptr);
+  if (ptr == nullptr) return INVALID_ENTRY;
+  const Block* block = block_for_ptr(ptr);
   if (block != nullptr) {
     // Prevent block deletion and _active_array modification.
     MutexLocker ml(_allocation_mutex, Mutex::_no_safepoint_check_flag);
@@ -1078,7 +1084,7 @@ void OopStorage::BasicParState::update_concurrent_iteration_count(int value) {
 
 bool OopStorage::BasicParState::claim_next_segment(IterationData* data) {
   data->_processed += data->_segment_end - data->_segment_start;
-  size_t start = Atomic::load_acquire(&_next_block);
+  size_t start = AtomicAccess::load_acquire(&_next_block);
   if (start >= _block_count) {
     return finish_iteration(data); // No more blocks available.
   }
@@ -1091,11 +1097,11 @@ bool OopStorage::BasicParState::claim_next_segment(IterationData* data) {
   size_t max_step = 10;
   size_t remaining = _block_count - start;
   size_t step = MIN2(max_step, 1 + (remaining / _estimated_thread_count));
-  // Atomic::add with possible overshoot.  This can perform better
+  // AtomicAccess::add with possible overshoot.  This can perform better
   // than a CAS loop on some platforms when there is contention.
   // We can cope with the uncertainty by recomputing start/end from
   // the result of the add, and dealing with potential overshoot.
-  size_t end = Atomic::add(&_next_block, step);
+  size_t end = AtomicAccess::add(&_next_block, step);
   // _next_block may have changed, so recompute start from result of add.
   start = end - step;
   // _next_block may have changed so much that end has overshot.
@@ -1114,27 +1120,47 @@ bool OopStorage::BasicParState::claim_next_segment(IterationData* data) {
 
 bool OopStorage::BasicParState::finish_iteration(const IterationData* data) const {
   log_info(oopstorage, blocks, stats)
-          ("Parallel iteration on %s: blocks = " SIZE_FORMAT
-           ", processed = " SIZE_FORMAT " (%2.f%%)",
+          ("Parallel iteration on %s: blocks = %zu"
+           ", processed = %zu (%2.f%%)",
            _storage->name(), _block_count, data->_processed,
            percent_of(data->_processed, _block_count));
   return false;
 }
 
 size_t OopStorage::BasicParState::num_dead() const {
-  return Atomic::load(&_num_dead);
+  return AtomicAccess::load(&_num_dead);
 }
 
 void OopStorage::BasicParState::increment_num_dead(size_t num_dead) {
-  Atomic::add(&_num_dead, num_dead);
+  AtomicAccess::add(&_num_dead, num_dead);
 }
 
 void OopStorage::BasicParState::report_num_dead() const {
-  _storage->report_num_dead(Atomic::load(&_num_dead));
+  _storage->report_num_dead(AtomicAccess::load(&_num_dead));
 }
 
 const char* OopStorage::name() const { return _name; }
 
+bool OopStorage::print_containing(const oop* addr, outputStream* st) {
+  if (addr != nullptr) {
+    Block* block = block_for_ptr(addr);
+    if (block != nullptr && block->print_containing(addr, st)) {
+      st->print(" in oop storage \"%s\"", name());
+      return true;
+    }
+  }
+  return false;
+}
+
+bool OopStorage::Block::print_containing(const oop* addr, outputStream* st) {
+  if (contains(addr)) {
+    st->print(PTR_FORMAT " is a pointer %u/%zu into block %zu",
+              p2i(addr), get_index(addr), ARRAY_SIZE(_data), _active_index);
+    return true;
+  }
+  return false;
+}
+
 #ifndef PRODUCT
 
 void OopStorage::print_on(outputStream* st) const {
@@ -1144,7 +1170,7 @@ void OopStorage::print_on(outputStream* st) const {
   double data_size = section_size * section_count;
   double alloc_percentage = percent_of((double)allocations, blocks * data_size);
 
-  st->print("%s: " SIZE_FORMAT " entries in " SIZE_FORMAT " blocks (%.F%%), " SIZE_FORMAT " bytes",
+  st->print("%s: %zu entries in %zu blocks (%.F%%), %zu bytes",
             name(), allocations, blocks, alloc_percentage, total_memory_usage());
   if (_concurrent_iteration_count > 0) {
     st->print(", concurrent iteration active");
