@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,26 +28,12 @@
 #include "memory/allocation.hpp"
 #include "memory/padded.hpp"
 #include "oops/oopsHierarchy.hpp"
-#include "runtime/atomic.hpp"
+#include "runtime/atomicAccess.hpp"
 #include "utilities/debug.hpp"
 #include "utilities/globalDefinitions.hpp"
 #include "utilities/ostream.hpp"
 #include "utilities/stack.hpp"
 
-// Simple TaskQueue stats that are collected by default in debug builds.
-
-#if !defined(TASKQUEUE_STATS) && defined(ASSERT)
-#define TASKQUEUE_STATS 1
-#elif !defined(TASKQUEUE_STATS)
-#define TASKQUEUE_STATS 0
-#endif
-
-#if TASKQUEUE_STATS
-#define TASKQUEUE_STATS_ONLY(code) code
-#else
-#define TASKQUEUE_STATS_ONLY(code)
-#endif // TASKQUEUE_STATS
-
 #if TASKQUEUE_STATS
 class TaskQueueStats {
 public:
@@ -56,7 +42,11 @@ public:
     pop,              // number of taskqueue pops
     pop_slow,         // subset of taskqueue pops that were done slow-path
     steal_attempt,    // number of taskqueue steal attempts
-    steal,            // number of taskqueue steals
+    steal_empty,      // number of empty taskqueues
+    steal_contended,  // number of contended steals
+    steal_success,    // number of successful steals
+    steal_max_contended_in_a_row, // maximum number of contended steals in a row
+    steal_bias_drop,  // number of times the bias has been dropped
     overflow,         // number of overflow pushes
     overflow_max_len, // max length of overflow stack
     last_stat_id
@@ -68,8 +58,16 @@ public:
   inline void record_push()          { ++_stats[push]; }
   inline void record_pop()           { ++_stats[pop]; }
   inline void record_pop_slow()      { record_pop(); ++_stats[pop_slow]; }
-  inline void record_steal_attempt() { ++_stats[steal_attempt]; }
-  inline void record_steal()         { ++_stats[steal]; }
+  inline void record_steal_attempt(uint kind) {
+    ++_stats[steal_attempt];
+    ++_stats[steal_empty + kind];
+  }
+  inline void record_contended_in_a_row(uint in_a_row) {
+    if (_stats[steal_max_contended_in_a_row] < in_a_row) {
+      _stats[steal_max_contended_in_a_row] = in_a_row;
+    }
+  }
+  inline void record_bias_drop() { ++_stats[steal_bias_drop]; }
   inline void record_overflow(size_t new_length);
 
   TaskQueueStats & operator +=(const TaskQueueStats & addend);
@@ -81,9 +79,9 @@ public:
 
   // Print the specified line of the header (does not include a line separator).
   static void print_header(unsigned int line, outputStream* const stream = tty,
-                           unsigned int width = 10);
+                           unsigned int width = 11);
   // Print the statistics (does not include a line separator).
-  void print(outputStream* const stream = tty, unsigned int width = 10) const;
+  void print(outputStream* const stream = tty, unsigned int width = 11) const;
 
   DEBUG_ONLY(void verify() const;)
 
@@ -104,8 +102,8 @@ void TaskQueueStats::reset() {
 
 // TaskQueueSuper collects functionality common to all GenericTaskQueue instances.
 
-template <unsigned int N, MEMFLAGS F>
-class TaskQueueSuper: public CHeapObj<F> {
+template <unsigned int N, MemTag MT>
+class TaskQueueSuper: public CHeapObj<MT> {
 protected:
   // Internal type for indexing the queue; also used for the tag.
   typedef NOT_LP64(uint16_t) LP64_ONLY(uint32_t) idx_t;
@@ -142,36 +140,36 @@ protected:
   };
 
   uint bottom_relaxed() const {
-    return Atomic::load(&_bottom);
+    return AtomicAccess::load(&_bottom);
   }
 
   uint bottom_acquire() const {
-    return Atomic::load_acquire(&_bottom);
+    return AtomicAccess::load_acquire(&_bottom);
   }
 
   void set_bottom_relaxed(uint new_bottom) {
-    Atomic::store(&_bottom, new_bottom);
+    AtomicAccess::store(&_bottom, new_bottom);
   }
 
   void release_set_bottom(uint new_bottom) {
-    Atomic::release_store(&_bottom, new_bottom);
+    AtomicAccess::release_store(&_bottom, new_bottom);
   }
 
   Age age_relaxed() const {
-    return Age(Atomic::load(&_age._data));
+    return Age(AtomicAccess::load(&_age._data));
   }
 
   void set_age_relaxed(Age new_age) {
-    Atomic::store(&_age._data, new_age._data);
+    AtomicAccess::store(&_age._data, new_age._data);
   }
 
   Age cmpxchg_age(Age old_age, Age new_age) {
-    return Age(Atomic::cmpxchg(&_age._data, old_age._data, new_age._data));
+    return Age(AtomicAccess::cmpxchg(&_age._data, old_age._data, new_age._data));
   }
 
   idx_t age_top_relaxed() const {
     // Atomically accessing a subfield of an "atomic" member.
-    return Atomic::load(&_age._fields._top);
+    return AtomicAccess::load(&_age._fields._top);
   }
 
   // These both operate mod N.
@@ -221,11 +219,11 @@ protected:
   }
 
 private:
-  DEFINE_PAD_MINUS_SIZE(0, DEFAULT_CACHE_LINE_SIZE, 0);
+  DEFINE_PAD_MINUS_SIZE(0, DEFAULT_PADDING_SIZE, 0);
 
   // Index of the first free element after the last one pushed (mod N).
   volatile uint _bottom;
-  DEFINE_PAD_MINUS_SIZE(1, DEFAULT_CACHE_LINE_SIZE, sizeof(uint));
+  DEFINE_PAD_MINUS_SIZE(1, DEFAULT_PADDING_SIZE, sizeof(uint));
 
   // top() is the index of the oldest pushed element (mod N), and tag()
   // is the associated epoch, to distinguish different modifications of
@@ -233,7 +231,7 @@ private:
   // (_bottom - top()) mod N == N-1; the latter indicates underflow
   // during concurrent pop_local/pop_global.
   volatile Age _age;
-  DEFINE_PAD_MINUS_SIZE(2, DEFAULT_CACHE_LINE_SIZE, sizeof(Age));
+  DEFINE_PAD_MINUS_SIZE(2, DEFAULT_PADDING_SIZE, sizeof(Age));
 
   NONCOPYABLE(TaskQueueSuper);
 
@@ -268,6 +266,16 @@ public:
   // in GenericTaskQueue.
   uint max_elems() const { return N - 2; }
 
+  // The result of a pop_global operation. The value order of this must correspond
+  // to the order in the corresponding TaskQueueStats StatId.
+  enum class PopResult : uint {
+    Empty     = 0, // Queue has been empty. t is undefined.
+    Contended = 1, // Contention prevented successful retrieval, queue most likely contains elements. t is undefined.
+    Success   = 2  // Successfully retrieved an element, t contains it.
+  };
+
+  TASKQUEUE_STATS_ONLY(void record_steal_attempt(PopResult kind) { stats.record_steal_attempt((uint)kind); })
+
   TASKQUEUE_STATS_ONLY(TaskQueueStats stats;)
 };
 
@@ -302,37 +310,39 @@ public:
 // practice of parallel programming (PPoPP 2013), 69-80
 //
 
-template <class E, MEMFLAGS F, unsigned int N = TASKQUEUE_SIZE>
-class GenericTaskQueue: public TaskQueueSuper<N, F> {
+template <class E, MemTag MT, unsigned int N = TASKQUEUE_SIZE>
+class GenericTaskQueue: public TaskQueueSuper<N, MT> {
 protected:
-  typedef typename TaskQueueSuper<N, F>::Age Age;
-  typedef typename TaskQueueSuper<N, F>::idx_t idx_t;
+  typedef typename TaskQueueSuper<N, MT>::Age Age;
+  typedef typename TaskQueueSuper<N, MT>::idx_t idx_t;
 
-  using TaskQueueSuper<N, F>::MOD_N_MASK;
+  using TaskQueueSuper<N, MT>::MOD_N_MASK;
 
-  using TaskQueueSuper<N, F>::bottom_relaxed;
-  using TaskQueueSuper<N, F>::bottom_acquire;
+  using TaskQueueSuper<N, MT>::bottom_relaxed;
+  using TaskQueueSuper<N, MT>::bottom_acquire;
 
-  using TaskQueueSuper<N, F>::set_bottom_relaxed;
-  using TaskQueueSuper<N, F>::release_set_bottom;
+  using TaskQueueSuper<N, MT>::set_bottom_relaxed;
+  using TaskQueueSuper<N, MT>::release_set_bottom;
 
-  using TaskQueueSuper<N, F>::age_relaxed;
-  using TaskQueueSuper<N, F>::set_age_relaxed;
-  using TaskQueueSuper<N, F>::cmpxchg_age;
-  using TaskQueueSuper<N, F>::age_top_relaxed;
+  using TaskQueueSuper<N, MT>::age_relaxed;
+  using TaskQueueSuper<N, MT>::set_age_relaxed;
+  using TaskQueueSuper<N, MT>::cmpxchg_age;
+  using TaskQueueSuper<N, MT>::age_top_relaxed;
 
-  using TaskQueueSuper<N, F>::increment_index;
-  using TaskQueueSuper<N, F>::decrement_index;
-  using TaskQueueSuper<N, F>::dirty_size;
-  using TaskQueueSuper<N, F>::clean_size;
-  using TaskQueueSuper<N, F>::assert_not_underflow;
+  using TaskQueueSuper<N, MT>::increment_index;
+  using TaskQueueSuper<N, MT>::decrement_index;
+  using TaskQueueSuper<N, MT>::dirty_size;
+  using TaskQueueSuper<N, MT>::clean_size;
+  using TaskQueueSuper<N, MT>::assert_not_underflow;
 
 public:
-  using TaskQueueSuper<N, F>::max_elems;
-  using TaskQueueSuper<N, F>::size;
+  typedef typename TaskQueueSuper<N, MT>::PopResult PopResult;
+
+  using TaskQueueSuper<N, MT>::max_elems;
+  using TaskQueueSuper<N, MT>::size;
 
 #if  TASKQUEUE_STATS
-  using TaskQueueSuper<N, F>::stats;
+  using TaskQueueSuper<N, MT>::stats;
 #endif
 
 private:
@@ -357,7 +367,7 @@ public:
 
   // Like pop_local(), but uses the "global" end of the queue (the least
   // recently pushed).
-  bool pop_global(E& t);
+  PopResult pop_global(E& t);
 
   // Delete any resource associated with the queue.
   ~GenericTaskQueue();
@@ -372,7 +382,7 @@ private:
   // Element array.
   E* _elems;
 
-  DEFINE_PAD_MINUS_SIZE(1, DEFAULT_CACHE_LINE_SIZE, sizeof(E*));
+  DEFINE_PAD_MINUS_SIZE(1, DEFAULT_PADDING_SIZE, sizeof(E*));
   // Queue owner local variables. Not to be accessed by other threads.
 
   static const uint InvalidQueueId = uint(-1);
@@ -380,14 +390,17 @@ private:
 
   int _seed; // Current random seed used for selecting a random queue during stealing.
 
-  DEFINE_PAD_MINUS_SIZE(2, DEFAULT_CACHE_LINE_SIZE, sizeof(uint) + sizeof(int));
+  DEFINE_PAD_MINUS_SIZE(2, DEFAULT_PADDING_SIZE, sizeof(uint) + sizeof(int));
 public:
   int next_random_queue_id();
 
   void set_last_stolen_queue_id(uint id)     { _last_stolen_queue_id = id; }
   uint last_stolen_queue_id() const          { return _last_stolen_queue_id; }
   bool is_last_stolen_queue_id_valid() const { return _last_stolen_queue_id != InvalidQueueId; }
-  void invalidate_last_stolen_queue_id()     { _last_stolen_queue_id = InvalidQueueId; }
+  void invalidate_last_stolen_queue_id()     {
+    TASKQUEUE_STATS_ONLY(stats.record_bias_drop();)
+    _last_stolen_queue_id = InvalidQueueId;
+  }
 };
 
 // OverflowTaskQueue is a TaskQueue that also includes an overflow stack for
@@ -401,12 +414,12 @@ public:
 // Note that size() is not hidden--it returns the number of elements in the
 // TaskQueue, and does not include the size of the overflow stack.  This
 // simplifies replacement of GenericTaskQueues with OverflowTaskQueues.
-template<class E, MEMFLAGS F, unsigned int N = TASKQUEUE_SIZE>
-class OverflowTaskQueue: public GenericTaskQueue<E, F, N>
+template<class E, MemTag MT, unsigned int N = TASKQUEUE_SIZE>
+class OverflowTaskQueue: public GenericTaskQueue<E, MT, N>
 {
 public:
-  typedef Stack<E, F>               overflow_t;
-  typedef GenericTaskQueue<E, F, N> taskqueue_t;
+  typedef Stack<E, MT>               overflow_t;
+  typedef GenericTaskQueue<E, MT, N> taskqueue_t;
 
   TASKQUEUE_STATS_ONLY(using taskqueue_t::stats;)
 
@@ -440,19 +453,23 @@ public:
   virtual uint tasks() const = 0;
 };
 
-template <MEMFLAGS F> class TaskQueueSetSuperImpl: public CHeapObj<F>, public TaskQueueSetSuper {
+template <MemTag MT> class TaskQueueSetSuperImpl: public CHeapObj<MT>, public TaskQueueSetSuper {
 };
 
-template<class T, MEMFLAGS F>
-class GenericTaskQueueSet: public TaskQueueSetSuperImpl<F> {
+template<class T, MemTag MT>
+class GenericTaskQueueSet: public TaskQueueSetSuperImpl<MT> {
 public:
   typedef typename T::element_type E;
+  typedef typename T::PopResult PopResult;
 
 private:
   uint _n;
   T** _queues;
 
-  bool steal_best_of_2(uint queue_num, E& t);
+  // Attempts to steal an element from a foreign queue (!= queue_num), setting
+  // the result in t. Validity of this value and the return value is the same
+  // as for the last pop_global() operation.
+  PopResult steal_best_of_2(uint queue_num, E& t);
 
 public:
   GenericTaskQueueSet(uint n);
@@ -473,30 +490,43 @@ public:
   virtual uint tasks() const;
 
   uint size() const { return _n; }
+
+#if TASKQUEUE_STATS
+private:
+  static void print_taskqueue_stats_hdr(outputStream* const st, const char* label);
+public:
+  void print_taskqueue_stats(outputStream* const st, const char* label);
+  void reset_taskqueue_stats();
+
+  // Prints taskqueue set statistics into gc+task+stats=trace and resets
+  // its statistics.
+  void print_and_reset_taskqueue_stats(const char* label);
+#endif // TASKQUEUE_STATS
 };
 
-template<class T, MEMFLAGS F> void
-GenericTaskQueueSet<T, F>::register_queue(uint i, T* q) {
+template<class T, MemTag MT> void
+GenericTaskQueueSet<T, MT>::register_queue(uint i, T* q) {
   assert(i < _n, "index out of range.");
   _queues[i] = q;
 }
 
-template<class T, MEMFLAGS F> T*
-GenericTaskQueueSet<T, F>::queue(uint i) {
+template<class T, MemTag MT> T*
+GenericTaskQueueSet<T, MT>::queue(uint i) {
+  assert(i < _n, "index out of range.");
   return _queues[i];
 }
 
 #ifdef ASSERT
-template<class T, MEMFLAGS F>
-void GenericTaskQueueSet<T, F>::assert_empty() const {
+template<class T, MemTag MT>
+void GenericTaskQueueSet<T, MT>::assert_empty() const {
   for (uint j = 0; j < _n; j++) {
     _queues[j]->assert_empty();
   }
 }
 #endif // ASSERT
 
-template<class T, MEMFLAGS F>
-uint GenericTaskQueueSet<T, F>::tasks() const {
+template<class T, MemTag MT>
+uint GenericTaskQueueSet<T, MT>::tasks() const {
   uint n = 0;
   for (uint j = 0; j < _n; j++) {
     n += _queues[j]->size();
@@ -513,7 +543,7 @@ public:
 class ObjArrayTask
 {
 public:
-  ObjArrayTask(oop o = NULL, int idx = 0): _obj(o), _index(idx) { }
+  ObjArrayTask(oop o = nullptr, int idx = 0): _obj(o), _index(idx) { }
   ObjArrayTask(oop o, size_t idx): _obj(o), _index(int(idx)) {
     assert(idx <= size_t(max_jint), "too big");
   }
@@ -529,22 +559,12 @@ private:
   int _index;
 };
 
-// Wrapper over an oop that is a partially scanned array.
-// Can be converted to a ScannerTask for placement in associated task queues.
-// Refers to the partially copied source array oop.
-class PartialArrayScanTask {
-  oop _src;
+class PartialArrayState;
 
-public:
-  PartialArrayScanTask() : _src() {}
-  explicit PartialArrayScanTask(oop src_array) : _src(src_array) {}
-  // Trivially copyable.
-
-  oop to_source_array() const { return _src; }
-};
-
-// Discriminated union over oop*, narrowOop*, and PartialArrayScanTask.
+// Discriminated union over oop/oop*, narrowOop*, and PartialArrayState.
 // Uses a low tag in the associated pointer to identify the category.
+// Oop/oop* are overloaded using the same tag because they can not appear at the
+// same time.
 // Used as a task queue element type.
 class ScannerTask {
   void* _p;
@@ -575,14 +595,16 @@ class ScannerTask {
   }
 
 public:
-  ScannerTask() : _p(NULL) {}
+  ScannerTask() : _p(nullptr) {}
+
+  explicit ScannerTask(oop p) : _p(encode(p, OopTag)) {}
 
   explicit ScannerTask(oop* p) : _p(encode(p, OopTag)) {}
 
   explicit ScannerTask(narrowOop* p) : _p(encode(p, NarrowOopTag)) {}
 
-  explicit ScannerTask(PartialArrayScanTask t) :
-    _p(encode(t.to_source_array(), PartialArrayTag)) {}
+  explicit ScannerTask(PartialArrayState* state) :
+    _p(encode(state, PartialArrayTag)) {}
 
   // Trivially copyable.
 
@@ -596,7 +618,7 @@ public:
     return (raw_value() & NarrowOopTag) != 0;
   }
 
-  bool is_partial_array_task() const {
+  bool is_partial_array_state() const {
     return (raw_value() & PartialArrayTag) != 0;
   }
 
@@ -604,12 +626,16 @@ public:
     return static_cast<oop*>(decode(OopTag));
   }
 
+  oop to_oop() const {
+    return cast_to_oop(decode(OopTag));
+  }
+
   narrowOop* to_narrow_oop_ptr() const {
     return static_cast<narrowOop*>(decode(NarrowOopTag));
   }
 
-  PartialArrayScanTask to_partial_array_task() const {
-    return PartialArrayScanTask(cast_to_oop(decode(PartialArrayTag)));
+  PartialArrayState* to_partial_array_state() const {
+    return static_cast<PartialArrayState*>(decode(PartialArrayTag));
   }
 };
 
