@@ -124,7 +124,7 @@ void ZTracer::initialize() {
 void ZTracer::send_stat_counter(const ZStatCounter& counter, uint64_t increment, uint64_t value) {
   NoSafepointVerifier nsv;
 
-  EventZStatisticsCounter e;
+  JfrNonReentrant<EventZStatisticsCounter> e;
   if (e.should_commit()) {
     e.set_id(counter.id());
     e.set_increment(increment);
@@ -136,7 +136,7 @@ void ZTracer::send_stat_counter(const ZStatCounter& counter, uint64_t increment,
 void ZTracer::send_stat_sampler(const ZStatSampler& sampler, uint64_t value) {
   NoSafepointVerifier nsv;
 
-  EventZStatisticsSampler e;
+  JfrNonReentrant<EventZStatisticsSampler> e;
   if (e.should_commit()) {
     e.set_id(sampler.id());
     e.set_value(value);
