@@ -35,6 +35,8 @@
 #include "jfr/recorder/checkpoint/types/traceid/jfrTraceIdLoadBarrier.inline.hpp"
 #include "jfr/recorder/jfrRecorder.hpp"
 #include "jfr/support/jfrKlassUnloading.hpp"
+#include "jfr/support/methodtracer/jfrInstrumentedClass.hpp"
+#include "jfr/support/methodtracer/jfrMethodTracer.hpp"
 #include "jfr/utilities/jfrHashtable.hpp"
 #include "jfr/utilities/jfrTypes.hpp"
 #include "jfr/writers/jfrTypeWriterHost.hpp"
@@ -480,13 +482,65 @@ static void do_primitives() {
   write_primitive(_writer, nullptr); // void.class
 }
 
+static void do_method_tracer_klasses() {
+  assert(JfrTraceIdEpoch::has_method_tracer_changed_tag_state(), "invariant");
+  assert_locked_or_safepoint(ClassLoaderDataGraph_lock);
+  assert(_subsystem_callback != nullptr, "invariant");
+  GrowableArray<JfrInstrumentedClass>* const instrumented = JfrMethodTracer::instrumented_classes();
+  assert(instrumented != nullptr, "invariant");
+  assert(instrumented->length() > 0, "invariant");
+  for (int i = 0; i < instrumented->length(); ++i) {
+    JfrInstrumentedClass& jic = instrumented->at(i);
+    if (jic.unloaded()) {
+      continue;
+    }
+    if (JfrKlassUnloading::is_unloaded(jic.trace_id(), previous_epoch())) {
+      jic.set_unloaded(true);
+      continue;
+    }
+    assert(jic.trace_id() == JfrTraceId::load_raw(jic.instance_klass()), "invariant");
+    assert(JfrTraceId::has_sticky_bit(jic.instance_klass()), "invariant");
+    if (current_epoch()) {
+      JfrTraceId::load(jic.instance_klass()); // enqueue klass for this epoch
+    } else {
+      _subsystem_callback->do_artifact(jic.instance_klass()); // process directly
+    }
+  }
+  JfrTraceIdEpoch::reset_method_tracer_tag_state();
+}
+
+static void clear_method_tracer_klasses() {
+  assert_locked_or_safepoint (ClassLoaderDataGraph_lock);
+  assert(previous_epoch(), "invariant");
+  GrowableArray<JfrInstrumentedClass>* const instrumented = JfrMethodTracer::instrumented_classes();
+  assert(instrumented != nullptr, "invariant");
+  const int length = instrumented->length();
+  bool trim = false;
+  for (int i = 0; i < length; ++i) {
+    JfrInstrumentedClass& jic = instrumented->at(i);
+    if (jic.unloaded()) {
+      trim = true;
+      continue;
+    }
+    if (JfrKlassUnloading::is_unloaded(jic.trace_id(), true)) {
+      jic.set_unloaded(true);
+      trim = true;
+    }
+  }
+  JfrMethodTracer::trim_instrumented_classes(trim);
+}
+
 static void do_unloading_klass(Klass* klass) {
   assert(klass != nullptr, "invariant");
   assert(_subsystem_callback != nullptr, "invariant");
-  if (klass->is_instance_klass() && InstanceKlass::cast(klass)->is_scratch_class()) {
-    return;
+  if (!used(klass) && klass->is_instance_klass() && InstanceKlass::cast(klass)->is_scratch_class()) {
+    SET_TRANSIENT(klass);
+    assert(used(klass), "invariant");
   }
   if (JfrKlassUnloading::on_unload(klass)) {
+    if (JfrTraceId::has_sticky_bit(klass)) {
+      JfrMethodTracer::add_to_unloaded_set(klass);
+    }
     _subsystem_callback->do_artifact(klass);
   }
 }
@@ -504,9 +558,13 @@ static void do_klasses() {
     return;
   }
   if (is_initial_typeset_for_chunk()) {
-    // Only write the primitive classes once per chunk.
+    // Only write the primitive and method tracer classes once per chunk.
     do_primitives();
   }
+  if (JfrTraceIdEpoch::has_method_tracer_changed_tag_state()) {
+    do_method_tracer_klasses();
+  }
+
   JfrTraceIdLoadBarrier::do_klasses(&do_klass, previous_epoch());
 }
 
@@ -990,19 +1048,15 @@ class MethodIteratorHost {
  private:
   MethodCallback _method_cb;
   KlassCallback _klass_cb;
-  KlassUsedPredicate _klass_used_predicate;
-  MethodUsedPredicate _method_used_predicate;
   MethodFlagPredicate<leakp> _method_flag_predicate;
  public:
   MethodIteratorHost(JfrCheckpointWriter* writer) :
     _method_cb(writer, unloading(), false),
     _klass_cb(writer, unloading(), false),
-    _klass_used_predicate(current_epoch()),
-    _method_used_predicate(current_epoch()),
     _method_flag_predicate(current_epoch()) {}
 
   bool operator()(KlassPtr klass) {
-    if (_method_used_predicate(klass)) {
+    if (klass->is_instance_klass()) {
       const InstanceKlass* ik = InstanceKlass::cast(klass);
       while (ik != nullptr) {
         const int len = ik->methods()->length();
@@ -1017,7 +1071,7 @@ class MethodIteratorHost {
         ik = ik->previous_versions();
       }
     }
-    return _klass_used_predicate(klass) ? _klass_cb(klass) : true;
+    return _klass_cb(klass);
   }
 
   int count() const { return _method_cb.count(); }
@@ -1206,6 +1260,7 @@ static size_t teardown() {
   const size_t total_count = _artifacts->total_count();
   if (previous_epoch()) {
     clear_klasses_and_methods();
+    clear_method_tracer_klasses();
     JfrKlassUnloading::clear();
     _artifacts->increment_checkpoint_id();
     _initial_type_set = true;
@@ -1221,10 +1276,11 @@ static void setup(JfrCheckpointWriter* writer, JfrCheckpointWriter* leakp_writer
   _class_unload = class_unload;
   _flushpoint = flushpoint;
   if (_artifacts == nullptr) {
-    _artifacts = new JfrArtifactSet(class_unload);
+    _artifacts = new JfrArtifactSet(class_unload, previous_epoch());
   } else {
-    _artifacts->initialize(class_unload);
+    _artifacts->initialize(class_unload, previous_epoch());
   }
+  assert(current_epoch() || _leakp_writer != nullptr, "invariant");
   assert(_artifacts != nullptr, "invariant");
   assert(!_artifacts->has_klass_entries(), "invariant");
 }
