@@ -29,18 +29,23 @@
 #include "oops/oop.inline.hpp"
 #include "oops/symbol.hpp"
 
-JfrArtifactSet::JfrArtifactSet(bool class_unload) : _symbol_table(nullptr),
-                                                    _klass_list(nullptr),
-                                                    _total_count(0),
-                                                    _class_unload(class_unload) {
-  initialize(class_unload);
-  assert(_klass_list != nullptr, "invariant");
+JfrArtifactSet::JfrArtifactSet(bool class_unload, bool previous_epoch) : _symbol_table(nullptr),
+                                                                         _klass_set(nullptr),
+                                                                         _klass_loader_set(nullptr),
+                                                                         _klass_loader_leakp_set(nullptr),
+                                                                         _total_count(0),
+                                                                         _class_unload(class_unload) {
+  initialize(class_unload, previous_epoch);
+  assert(!previous_epoch || _klass_loader_leakp_set != nullptr, "invariant");
+  assert(_klass_loader_set != nullptr, "invariant");
+  assert(_klass_set != nullptr, "invariant");
 }
 
-static const size_t initial_klass_list_size = 256;
-const int initial_klass_loader_set_size = 64;
+static unsigned initial_klass_set_size = 4096;
+static unsigned initial_klass_loader_set_size = 64;
+static unsigned initial_klass_loader_leakp_set_size = 64;
 
-void JfrArtifactSet::initialize(bool class_unload) {
+void JfrArtifactSet::initialize(bool class_unload, bool previous_epoch) {
   _class_unload = class_unload;
   if (_symbol_table == nullptr) {
     _symbol_table = JfrSymbolTable::create();
@@ -49,10 +54,12 @@ void JfrArtifactSet::initialize(bool class_unload) {
   assert(_symbol_table != nullptr, "invariant");
   _symbol_table->set_class_unload(class_unload);
   _total_count = 0;
-  // resource allocation
-  _klass_list = new GrowableArray<const Klass*>(initial_klass_list_size);
-  _klass_loader_set = new GrowableArray<const Klass*>(initial_klass_loader_set_size);
-  _klass_loader_leakp_set = new GrowableArray<const Klass*>(initial_klass_loader_set_size);
+  // Resource allocations. Keep in this allocation order.
+  if (previous_epoch) {
+    _klass_loader_leakp_set = new JfrKlassSet(initial_klass_loader_leakp_set_size);
+  }
+  _klass_loader_set = new JfrKlassSet(initial_klass_loader_set_size);
+  _klass_set = new JfrKlassSet(initial_klass_set_size);
 }
 
 void JfrArtifactSet::clear() {
@@ -63,7 +70,8 @@ void JfrArtifactSet::clear() {
 
 JfrArtifactSet::~JfrArtifactSet() {
   delete _symbol_table;
-  // _klass_list and _klass_loader_list will be cleared by a ResourceMark
+  // _klass_loader_set, _klass_loader_leakp_set and
+  // _klass_list will be cleared by a ResourceMark
 }
 
 traceid JfrArtifactSet::bootstrap_name(bool leakp) {
@@ -92,17 +100,12 @@ traceid JfrArtifactSet::mark(uintptr_t hash, const char* const str, bool leakp)
 }
 
 bool JfrArtifactSet::has_klass_entries() const {
-  return _klass_list->is_nonempty();
+  return _klass_set->is_nonempty();
 }
-
-int JfrArtifactSet::entries() const {
-  return _klass_list->length();
-}
-
-static inline bool not_in_set(GrowableArray<const Klass*>* set, const Klass* k) {
+static inline bool not_in_set(JfrArtifactSet::JfrKlassSet* set, const Klass* k) {
   assert(set != nullptr, "invariant");
   assert(k != nullptr, "invariant");
-  return !JfrMutablePredicate<const Klass*, compare_klasses>::test(set, k);
+  return set->add(k);
 }
 
 bool JfrArtifactSet::should_do_cld_klass(const Klass* k, bool leakp) {
@@ -115,16 +118,21 @@ bool JfrArtifactSet::should_do_cld_klass(const Klass* k, bool leakp) {
 void JfrArtifactSet::register_klass(const Klass* k) {
   assert(k != nullptr, "invariant");
   assert(IS_SERIALIZED(k), "invariant");
-  assert(_klass_list != nullptr, "invariant");
-  _klass_list->append(k);
+  assert(_klass_set != nullptr, "invariant");
+  _klass_set->add(k);
 }
 
 size_t JfrArtifactSet::total_count() const {
+  assert(_klass_set != nullptr, "invariant");
+  initial_klass_set_size = MAX2(initial_klass_set_size, _klass_set->table_size());
+  assert(_klass_loader_set != nullptr, "invariant");
+  initial_klass_loader_set_size = MAX2(initial_klass_loader_set_size, _klass_loader_set->table_size());
   return _total_count;
 }
 
 void JfrArtifactSet::increment_checkpoint_id() {
   assert(_symbol_table != nullptr, "invariant");
   _symbol_table->increment_checkpoint_id();
+  assert(_klass_loader_leakp_set != nullptr, "invariant");
+  initial_klass_loader_leakp_set_size = MAX2(initial_klass_loader_leakp_set_size, _klass_loader_leakp_set->table_size());
 }
-
