@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "jfr/jfrEvents.hpp"
 #include "jfr/jni/jfrJavaSupport.hpp"
 #include "jfr/leakprofiler/leakProfiler.hpp"
@@ -33,7 +32,7 @@
 #include "jvm_io.h"
 #include "logging/log.hpp"
 #include "runtime/arguments.hpp"
-#include "runtime/atomic.hpp"
+#include "runtime/atomicAccess.hpp"
 #include "runtime/globals.hpp"
 #include "runtime/javaThread.hpp"
 #include "runtime/mutexLocker.hpp"
@@ -109,8 +108,6 @@ static void close_emergency_dump_file() {
 }
 
 static const char* create_emergency_dump_path() {
-  assert(is_path_empty(), "invariant");
-
   const size_t path_len = get_dump_directory();
   if (path_len == 0) {
     return nullptr;
@@ -526,7 +523,7 @@ static bool prepare_for_emergency_dump(Thread* thread) {
 static volatile int jfr_shutdown_lock = 0;
 
 static bool guard_reentrancy() {
-  return Atomic::cmpxchg(&jfr_shutdown_lock, 0, 1) == 0;
+  return AtomicAccess::cmpxchg(&jfr_shutdown_lock, 0, 1) == 0;
 }
 
 class JavaThreadInVMAndNative : public StackObj {
@@ -559,22 +556,22 @@ class JavaThreadInVMAndNative : public StackObj {
   }
 };
 
-static void post_events(bool exception_handler, Thread* thread) {
-  if (exception_handler) {
+static void post_events(bool emit_old_object_samples, bool emit_event_shutdown, Thread* thread) {
+  if (emit_old_object_samples) {
+    LeakProfiler::emit_events(max_jlong, false, false);
+  }
+  if (emit_event_shutdown) {
     EventShutdown e;
     e.set_reason("VM Error");
     e.commit();
-  } else {
-    // OOM
-    LeakProfiler::emit_events(max_jlong, false, false);
   }
   EventDumpReason event;
-  event.set_reason(exception_handler ? "Crash" : "Out of Memory");
+  event.set_reason(emit_old_object_samples ? "Out of Memory" : "Crash");
   event.set_recordingId(-1);
   event.commit();
 }
 
-void JfrEmergencyDump::on_vm_shutdown(bool exception_handler) {
+void JfrEmergencyDump::on_vm_shutdown(bool emit_old_object_samples, bool emit_event_shutdown) {
   if (!guard_reentrancy()) {
     return;
   }
@@ -587,7 +584,7 @@ void JfrEmergencyDump::on_vm_shutdown(bool exception_handler) {
   if (!prepare_for_emergency_dump(thread)) {
     return;
   }
-  post_events(exception_handler, thread);
+  post_events(emit_old_object_samples, emit_event_shutdown, thread);
   // if JavaThread, transition to _thread_in_native to issue a final flushpoint
   NoHandleMark nhm;
   jtivm.transition_to_native();
