@@ -106,7 +106,8 @@ bool JfrVirtualMemorySegment::initialize(size_t reservation_size_request_bytes)
   assert(is_aligned(reservation_size_request_bytes, os::vm_allocation_granularity()), "invariant");
   _rs = MemoryReserver::reserve(reservation_size_request_bytes,
                                 os::vm_allocation_granularity(),
-                                os::vm_page_size());
+                                os::vm_page_size(),
+                                mtTracing);
   if (!_rs.is_reserved()) {
     return false;
   }
@@ -119,7 +120,7 @@ bool JfrVirtualMemorySegment::initialize(size_t reservation_size_request_bytes)
                               _rs.base(),
                               _rs.size(),
                               os::vm_page_size());
-  MemTracker::record_virtual_memory_tag((address)_rs.base(), mtTracing);
+  MemTracker::record_virtual_memory_tag(_rs, mtTracing);
   assert(is_aligned(_rs.base(), os::vm_page_size()), "invariant");
   assert(is_aligned(_rs.size(), os::vm_page_size()), "invariant");
 
