@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -72,7 +72,7 @@ class OopIterateClosure : public OopClosure {
 
  protected:
   OopIterateClosure(ReferenceDiscoverer* rd) : _ref_discoverer(rd) { }
-  OopIterateClosure() : _ref_discoverer(NULL) { }
+  OopIterateClosure() : _ref_discoverer(nullptr) { }
   ~OopIterateClosure() { }
 
   void set_ref_discoverer_internal(ReferenceDiscoverer* rd) { _ref_discoverer = rd; }
@@ -84,7 +84,6 @@ class OopIterateClosure : public OopClosure {
   // the below enum describes the different alternatives.
   enum ReferenceIterationMode {
     DO_DISCOVERY,                // Apply closure and discover references
-    DO_DISCOVERED_AND_DISCOVERY, // Apply closure to discovered field and do discovery
     DO_FIELDS,                   // Apply closure to all fields
     DO_FIELDS_EXCEPT_REFERENT    // Apply closure to all fields except the referent field
   };
@@ -114,7 +113,7 @@ class OopIterateClosure : public OopClosure {
 // An OopIterateClosure that can be used when there's no need to visit the Metadata.
 class BasicOopIterateClosure : public OopIterateClosure {
 public:
-  BasicOopIterateClosure(ReferenceDiscoverer* rd = NULL) : OopIterateClosure(rd) {}
+  BasicOopIterateClosure(ReferenceDiscoverer* rd = nullptr) : OopIterateClosure(rd) {}
 
   virtual bool do_metadata() { return false; }
   virtual void do_klass(Klass* k) { ShouldNotReachHere(); }
@@ -123,11 +122,18 @@ public:
   virtual void do_nmethod(nmethod* nm) { ShouldNotReachHere(); }
 };
 
+// Interface for applying an OopClosure to a set of oops.
+class OopIterator {
+public:
+  virtual void oops_do(OopClosure* cl) = 0;
+};
+
+enum class derived_base : intptr_t;
 enum class derived_pointer : intptr_t;
 class DerivedOopClosure : public Closure {
  public:
   enum { SkipNull = true };
-  virtual void do_derived_oop(oop* base, derived_pointer* derived) = 0;
+  virtual void do_derived_oop(derived_base* base, derived_pointer* derived) = 0;
 };
 
 class KlassClosure : public Closure {
@@ -170,7 +176,7 @@ class ClaimMetadataVisitingOopIterateClosure : public OopIterateClosure {
   const int _claim;
 
  public:
-  ClaimMetadataVisitingOopIterateClosure(int claim, ReferenceDiscoverer* rd = NULL) :
+  ClaimMetadataVisitingOopIterateClosure(int claim, ReferenceDiscoverer* rd = nullptr) :
       OopIterateClosure(rd),
       _claim(claim) { }
 
@@ -186,7 +192,7 @@ class ClaimMetadataVisitingOopIterateClosure : public OopIterateClosure {
 // It's used to proxy through the metadata to the oops defined in them.
 class MetadataVisitingOopIterateClosure: public ClaimMetadataVisitingOopIterateClosure {
  public:
-  MetadataVisitingOopIterateClosure(ReferenceDiscoverer* rd = NULL);
+  MetadataVisitingOopIterateClosure(ReferenceDiscoverer* rd = nullptr);
 };
 
 // ObjectClosure is used for iterating through an object space
@@ -197,12 +203,16 @@ class ObjectClosure : public Closure {
   virtual void do_object(oop obj) = 0;
 };
 
-
 class BoolObjectClosure : public Closure {
  public:
   virtual bool do_object_b(oop obj) = 0;
 };
 
+class OopFieldClosure {
+public:
+  virtual void do_field(oop base, oop* p) = 0;
+};
+
 class AlwaysTrueClosure: public BoolObjectClosure {
  public:
   bool do_object_b(oop p) { return true; }
@@ -222,74 +232,37 @@ public:
   ObjectToOopClosure(OopIterateClosure* cl) : _cl(cl) {}
 };
 
-// SpaceClosure is used for iterating over spaces
-
-class Space;
-class CompactibleSpace;
-
-class SpaceClosure : public StackObj {
- public:
-  // Called for each space
-  virtual void do_space(Space* s) = 0;
-};
-
-class CompactibleSpaceClosure : public StackObj {
- public:
-  // Called for each compactible space
-  virtual void do_space(CompactibleSpace* s) = 0;
-};
-
-
-// CodeBlobClosure is used for iterating through code blobs
+// NMethodClosure is used for iterating through nmethods
 // in the code cache or on thread stacks
 
-class CodeBlobClosure : public Closure {
- public:
-  // Called for each code blob.
-  virtual void do_code_blob(CodeBlob* cb) = 0;
-};
-
-// Applies an oop closure to all ref fields in code blobs
-// iterated over in an object iteration.
-class CodeBlobToOopClosure : public CodeBlobClosure {
- protected:
-  OopClosure* _cl;
-  bool _fix_relocations;
-  void do_nmethod(nmethod* nm);
- public:
-  // If fix_relocations(), then cl must copy objects to their new location immediately to avoid
-  // patching nmethods with the old locations.
-  CodeBlobToOopClosure(OopClosure* cl, bool fix_relocations) : _cl(cl), _fix_relocations(fix_relocations) {}
-  virtual void do_code_blob(CodeBlob* cb);
-
-  bool fix_relocations() const { return _fix_relocations; }
-  const static bool FixRelocations = true;
-};
-
-class MarkingCodeBlobClosure : public CodeBlobToOopClosure {
-  bool _keepalive_nmethods;
-
- public:
-  MarkingCodeBlobClosure(OopClosure* cl, bool fix_relocations, bool keepalive_nmethods) :
-      CodeBlobToOopClosure(cl, fix_relocations),
-      _keepalive_nmethods(keepalive_nmethods) {}
-
-  // Called for each code blob, but at most once per unique blob.
-  virtual void do_code_blob(CodeBlob* cb);
-};
-
 class NMethodClosure : public Closure {
  public:
   virtual void do_nmethod(nmethod* n) = 0;
 };
 
-class CodeBlobToNMethodClosure : public CodeBlobClosure {
-  NMethodClosure* const _nm_cl;
+// Applies an oop closure to all ref fields in nmethods
+// iterated over in an object iteration.
+class NMethodToOopClosure : public NMethodClosure {
+ protected:
+  OopClosure* _cl;
+  bool _fix_relocations;
+ public:
+  // If fix_relocations(), then cl must copy objects to their new location immediately to avoid
+  // patching nmethods with the old locations.
+  NMethodToOopClosure(OopClosure* cl, bool fix_relocations) : _cl(cl), _fix_relocations(fix_relocations) {}
+  void do_nmethod(nmethod* nm) override;
+
+  const static bool FixRelocations = true;
+};
+
+class MarkingNMethodClosure : public NMethodClosure {
+  OopClosure* _cl;
 
  public:
-  CodeBlobToNMethodClosure(NMethodClosure* nm_cl) : _nm_cl(nm_cl) {}
+  MarkingNMethodClosure(OopClosure* cl) : _cl(cl) {}
 
-  virtual void do_code_blob(CodeBlob* cb);
+  // Called for each nmethod.
+  virtual void do_nmethod(nmethod* nm);
 };
 
 // MonitorClosure is used for iterating over monitors in the monitors cache
@@ -325,39 +298,6 @@ public:
  virtual bool should_return_fine_grain() { return false; }
 };
 
-// Abstract closure for serializing data (read or write).
-
-class SerializeClosure : public Closure {
-public:
-  // Return bool indicating whether closure implements read or write.
-  virtual bool reading() const = 0;
-
-  // Read/write the void pointer pointed to by p.
-  virtual void do_ptr(void** p) = 0;
-
-  // Read/write the 32-bit unsigned integer pointed to by p.
-  virtual void do_u4(u4* p) = 0;
-
-  // Read/write the bool pointed to by p.
-  virtual void do_bool(bool* p) = 0;
-
-  // Read/write the region specified.
-  virtual void do_region(u_char* start, size_t size) = 0;
-
-  // Check/write the tag.  If reading, then compare the tag against
-  // the passed in value and fail is they don't match.  This allows
-  // for verification that sections of the serialized data are of the
-  // correct length.
-  virtual void do_tag(int tag) = 0;
-
-  // Read/write the oop
-  virtual void do_oop(oop* o) = 0;
-
-  bool writing() {
-    return !reading();
-  }
-};
-
 class SymbolClosure : public StackObj {
  public:
   virtual void do_symbol(Symbol**) = 0;
