@@ -29,10 +29,12 @@
 #include "memory/allocation.hpp"
 #include "metaprogramming/enableIf.hpp"
 #include "oops/array.hpp"
+#include "utilities/debug.hpp"
 #include "utilities/globalDefinitions.hpp"
 #include "utilities/growableArray.hpp"
 #include "utilities/macros.hpp"
-#include "utilities/resizeableResourceHash.hpp"
+#include "utilities/resizableHashTable.hpp"
+
 #include <type_traits>
 
 // The metadata hierarchy is separate from the oop hierarchy
@@ -104,6 +106,18 @@ public:
   //         Symbol*     bar() { return (Symbol*)    _obj; }
   //
   // [2] All Array<T> dimensions are statically declared.
+  //
+  // Pointer Tagging
+  //
+  // All metaspace pointers are at least 4 byte aligned. Therefore, it's possible for
+  // certain pointers to contain "tags" in their lowest 2 bits.
+  //
+  // Ref::obj() clears the tag bits in the return values. As a result, most
+  // callers who just want walk a closure of metaspace objects do not need to worry
+  // about the tag bits.
+  //
+  // If you need to use the tags, you can access the tagged pointer with Ref::addr()
+  // and manipulate its parts with strip_tags(), decode_tags() and add_tags()
   class Ref : public CHeapObj<mtMetaspace> {
     Writability _writability;
     address _enclosing_obj;
@@ -123,7 +137,7 @@ public:
     virtual ~Ref() {}
 
     address obj() const {
-      return *addr();
+      return strip_tags(*addr());
     }
 
     address* addr() const {
@@ -143,12 +157,35 @@ public:
     Ref* next() const               { return _next; }
   };
 
+  // Pointer tagging support
+  constexpr static uintx TAG_MASK = 0x03;
+
+  template <typename T>
+  static T strip_tags(T ptr_with_tags) {
+    uintx n = (uintx)ptr_with_tags;
+    return (T)(n & ~TAG_MASK);
+  }
+
+  template <typename T>
+  static uintx decode_tags(T ptr_with_tags) {
+    uintx n = (uintx)ptr_with_tags;
+    return (n & TAG_MASK);
+  }
+
+  template <typename T>
+  static T add_tags(T ptr, uintx tags) {
+    uintx n = (uintx)ptr;
+    assert((n & TAG_MASK) == 0, "sanity");
+    assert(tags <= TAG_MASK, "sanity");
+    return (T)(n | tags);
+  }
+
 private:
   // MSORef -- iterate an instance of MetaspaceObj
   template <class T> class MSORef : public Ref {
     T** _mpp;
     T* dereference() const {
-      return *_mpp;
+      return strip_tags(*_mpp);
     }
   protected:
     virtual void** mpp() const {
@@ -176,7 +213,7 @@ private:
     Array<T>** _mpp;
   protected:
     Array<T>* dereference() const {
-      return *_mpp;
+      return strip_tags(*_mpp);
     }
     virtual void** mpp() const {
       return (void**)_mpp;
@@ -338,7 +375,7 @@ public:
   UniqueMetaspaceClosure() : _has_been_visited(INITIAL_TABLE_SIZE, MAX_TABLE_SIZE) {}
 
 private:
-  ResizeableResourceHashtable<address, bool, AnyObj::C_HEAP,
+  ResizeableHashTable<address, bool, AnyObj::C_HEAP,
                               mtClassShared> _has_been_visited;
 };
 
