@@ -43,8 +43,6 @@
 //------------------------------ResourceArea-----------------------------------
 // A ResourceArea is an Arena that supports safe usage of ResourceMark.
 class ResourceArea: public Arena {
-  friend class VMStructs;
-
 #ifdef ASSERT
   int _nesting;                 // current # of nested ResourceMarks
   void verify_has_resource_mark();
@@ -54,8 +52,12 @@ public:
   ResourceArea(MemTag mem_tag = mtThread) :
     Arena(mem_tag, Arena::Tag::tag_ra) DEBUG_ONLY(COMMA _nesting(0)) {}
 
-  ResourceArea(size_t init_size, MemTag mem_tag = mtThread) :
-    Arena(mem_tag, Arena::Tag::tag_ra, init_size) DEBUG_ONLY(COMMA _nesting(0)) {
+  ResourceArea(size_t init_size, MemTag mem_tag = mtThread, Arena::Tag arena_tag = Arena::Tag::tag_ra) :
+    Arena(mem_tag, arena_tag, init_size) DEBUG_ONLY(COMMA _nesting(0)) {
+  }
+
+  ResourceArea(MemTag mem_tag, Arena::Tag arena_tag) :
+    Arena(mem_tag, arena_tag) DEBUG_ONLY(COMMA _nesting(0)) {
   }
 
   char* allocate_bytes(size_t size, AllocFailType alloc_failmode = AllocFailStrategy::EXIT_OOM);
@@ -141,6 +143,14 @@ public:
       assert(_max == state._max,     "Sanity check: idempotence");
     }
   }
+
+  static char* strdup(const char* src) {
+    return static_cast<Arena*>(Thread::current()->resource_area())->strdup(src);
+  }
+
+  static char* strdup(Thread* thread, const char* src) {
+    return static_cast<Arena*>(thread->resource_area())->strdup(src);
+  }
 };
 
 
