@@ -22,6 +22,7 @@
  *
  */
 #include "cds/filemap.hpp"
+#include "logging/log.hpp"
 #include "memory/metaspace.hpp"
 #include "memory/metaspaceUtils.hpp"
 #include "nmt/mallocTracker.hpp"
@@ -29,6 +30,8 @@
 #include "nmt/memReporter.hpp"
 #include "nmt/memTracker.hpp"
 #include "nmt/memoryFileTracker.hpp"
+#include "nmt/regionsTree.hpp"
+#include "nmt/regionsTree.inline.hpp"
 #include "nmt/threadStackTracker.hpp"
 #include "nmt/virtualMemoryTracker.hpp"
 #include "utilities/debug.hpp"
@@ -36,7 +39,7 @@
 #include "utilities/ostream.hpp"
 
 #define INDENT_BY(num_chars, CODE) { \
-  streamIndentor si(out, num_chars); \
+  StreamIndentor si(out, num_chars); \
   { CODE }                           \
 }
 
@@ -51,7 +54,7 @@ static ssize_t counter_diff(size_t c1, size_t c2) {
 }
 
 MemReporterBase::MemReporterBase(outputStream* out, size_t scale) :
-  _scale(scale), _output(out), _auto_indentor(out) {}
+  _scale(scale), _output(out) {}
 
 size_t MemReporterBase::reserved_total(const MallocMemory* malloc, const VirtualMemory* vm) {
   return malloc->malloc_size() + malloc->arena_size() + vm->reserved();
@@ -79,7 +82,7 @@ void MemReporterBase::print_malloc(const MemoryCounter* c, MemTag mem_tag) const
   const size_t count = c->count();
 
   if (mem_tag != mtNone) {
-    out->print("(%s%zu%s type=%s", alloc_type,
+    out->print("(%s%zu%s tag=%s", alloc_type,
       amount_in_current_scale(amount), scale, NMTUtil::tag_to_name(mem_tag));
   } else {
     out->print("(%s%zu%s", alloc_type,
@@ -181,14 +184,14 @@ void MemSummaryReporter::report() {
     MemTag mem_tag = NMTUtil::index_to_tag(index);
     // thread stack is reported as part of thread category
     if (mem_tag == mtThreadStack) continue;
-    MallocMemory* malloc_memory = _malloc_snapshot->by_type(mem_tag);
-    VirtualMemory* virtual_memory = _vm_snapshot->by_type(mem_tag);
+    MallocMemory* malloc_memory = _malloc_snapshot->by_tag(mem_tag);
+    VirtualMemory* virtual_memory = _vm_snapshot->by_tag(mem_tag);
 
-    report_summary_of_type(mem_tag, malloc_memory, virtual_memory);
+    report_summary_of_tag(mem_tag, malloc_memory, virtual_memory);
   }
 }
 
-void MemSummaryReporter::report_summary_of_type(MemTag mem_tag,
+void MemSummaryReporter::report_summary_of_tag(MemTag mem_tag,
   MallocMemory*  malloc_memory, VirtualMemory* virtual_memory) {
 
   size_t reserved_amount  = reserved_total (malloc_memory, virtual_memory);
@@ -197,7 +200,7 @@ void MemSummaryReporter::report_summary_of_type(MemTag mem_tag,
   // Count thread's native stack in "Thread" category
   if (mem_tag == mtThread) {
     const VirtualMemory* thread_stack_usage =
-      (const VirtualMemory*)_vm_snapshot->by_type(mtThreadStack);
+      (const VirtualMemory*)_vm_snapshot->by_tag(mtThreadStack);
     reserved_amount  += thread_stack_usage->reserved();
     committed_amount += thread_stack_usage->committed();
   } else if (mem_tag == mtNMT) {
@@ -230,7 +233,7 @@ void MemSummaryReporter::report_summary_of_type(MemTag mem_tag,
 #endif
   out->print_cr(")");
 
-  streamIndentor si(out, indent);
+  StreamIndentor si(out, indent);
 
   if (mem_tag == mtClass) {
     // report class count
@@ -239,7 +242,7 @@ void MemSummaryReporter::report_summary_of_type(MemTag mem_tag,
                   _instance_class_count, _array_class_count);
   } else if (mem_tag == mtThread) {
     const VirtualMemory* thread_stack_usage =
-     _vm_snapshot->by_type(mtThreadStack);
+     _vm_snapshot->by_tag(mtThreadStack);
     // report thread count
     out->print_cr("(threads #%zu)", ThreadStackTracker::thread_count());
     out->print("(stack: ");
@@ -249,7 +252,7 @@ void MemSummaryReporter::report_summary_of_type(MemTag mem_tag,
 
    // report malloc'd memory
   if (amount_in_current_scale(MAX2(malloc_memory->malloc_size(), pk_malloc)) > 0) {
-    print_malloc(malloc_memory->malloc_counter());
+    print_malloc(malloc_memory->malloc_counter(), mem_tag);
     out->cr();
   }
 
@@ -380,7 +383,7 @@ int MemDetailReporter::report_virtual_memory_allocation_sites()  {
       print_total(virtual_memory_site->reserved(), virtual_memory_site->committed());
       const MemTag mem_tag = virtual_memory_site->mem_tag();
       if (mem_tag != mtNone) {
-        out->print(" Type=%s", NMTUtil::tag_to_name(mem_tag));
+        out->print(" Tag=%s", NMTUtil::tag_to_name(mem_tag));
       }
       out->print_cr(")");
     )
@@ -419,7 +422,7 @@ void MemDetailReporter::report_virtual_memory_region(const ReservedMemoryRegion*
   outputStream* out = output();
   const char* scale = current_scale();
   const NativeCallStack*  stack = reserved_rgn->call_stack();
-  bool all_committed = reserved_rgn->size() == reserved_rgn->committed_size();
+  bool all_committed = reserved_rgn->size() == VirtualMemoryTracker::Instance::committed_size(reserved_rgn);
   const char* region_type = (all_committed ? "reserved and committed" : "reserved");
   out->cr();
   print_virtual_memory_region(region_type, reserved_rgn->base(), reserved_rgn->size());
@@ -432,34 +435,45 @@ void MemDetailReporter::report_virtual_memory_region(const ReservedMemoryRegion*
   }
 
   if (all_committed) {
-    CommittedRegionIterator itr = reserved_rgn->iterate_committed_regions();
-    const CommittedMemoryRegion* committed_rgn = itr.next();
-    if (committed_rgn->size() == reserved_rgn->size() && committed_rgn->call_stack()->equals(*stack)) {
-      // One region spanning the entire reserved region, with the same stack trace.
-      // Don't print this regions because the "reserved and committed" line above
-      // already indicates that the region is committed.
-      assert(itr.next() == nullptr, "Unexpectedly more than one regions");
+    bool reserved_and_committed = false;
+    VirtualMemoryTracker::Instance::tree()->visit_committed_regions(*reserved_rgn,
+                                                                  [&](CommittedMemoryRegion& committed_rgn) {
+      if (committed_rgn.equals(*reserved_rgn)) {
+        // One region spanning the entire reserved region, with the same stack trace.
+        // Don't print this regions because the "reserved and committed" line above
+        // already indicates that the region is committed.
+        reserved_and_committed = true;
+        return false;
+      }
+      return true;
+    });
+
+    if (reserved_and_committed) {
       return;
     }
   }
 
-  CommittedRegionIterator itr = reserved_rgn->iterate_committed_regions();
-  const CommittedMemoryRegion* committed_rgn;
-  while ((committed_rgn = itr.next()) != nullptr) {
+  auto print_committed_rgn = [&](const CommittedMemoryRegion& crgn) {
     // Don't report if size is too small
-    if (amount_in_current_scale(committed_rgn->size()) == 0) continue;
-    stack = committed_rgn->call_stack();
+    if (amount_in_current_scale(crgn.size()) == 0) return;
+    stack = crgn.call_stack();
     out->cr();
     INDENT_BY(8,
-      print_virtual_memory_region("committed", committed_rgn->base(), committed_rgn->size());
+      print_virtual_memory_region("committed", crgn.base(), crgn.size());
       if (stack->is_empty()) {
         out->cr();
       } else {
         out->print_cr(" from");
-        INDENT_BY(4, stack->print_on(out);)
+        INDENT_BY(4, _stackprinter.print_stack(stack);)
       }
     )
-  }
+  };
+
+  VirtualMemoryTracker::Instance::tree()->visit_committed_regions(*reserved_rgn,
+                                                                  [&](CommittedMemoryRegion& crgn) {
+    print_committed_rgn(crgn);
+    return true;
+  });
 }
 
 void MemDetailReporter::report_memory_file_allocations() {
@@ -524,7 +538,7 @@ void MemSummaryDiffReporter::report_diff() {
     MemTag mem_tag = NMTUtil::index_to_tag(index);
     // thread stack is reported as part of thread category
     if (mem_tag == mtThreadStack) continue;
-    diff_summary_of_type(mem_tag,
+    diff_summary_of_tag(mem_tag,
       _early_baseline.malloc_memory(mem_tag),
       _early_baseline.virtual_memory(mem_tag),
       _early_baseline.metaspace_stats(),
@@ -594,7 +608,7 @@ void MemSummaryDiffReporter::print_virtual_memory_diff(size_t current_reserved,
 }
 
 
-void MemSummaryDiffReporter::diff_summary_of_type(MemTag mem_tag,
+void MemSummaryDiffReporter::diff_summary_of_tag(MemTag mem_tag,
   const MallocMemory* early_malloc, const VirtualMemory* early_vm,
   const MetaspaceCombinedStats& early_ms,
   const MallocMemory* current_malloc, const VirtualMemory* current_vm,
@@ -641,7 +655,7 @@ void MemSummaryDiffReporter::diff_summary_of_type(MemTag mem_tag,
       early_reserved_amount, early_committed_amount);
     out->print_cr(")");
 
-    streamIndentor si(out, indent);
+    StreamIndentor si(out, indent);
 
     // detail lines
     if (mem_tag == mtClass) {
@@ -795,8 +809,8 @@ void MemDetailDiffReporter::report_diff() {
 }
 
 void MemDetailDiffReporter::diff_malloc_sites() const {
-  MallocSiteIterator early_itr = _early_baseline.malloc_sites(MemBaseline::by_site_and_type);
-  MallocSiteIterator current_itr = _current_baseline.malloc_sites(MemBaseline::by_site_and_type);
+  MallocSiteIterator early_itr = _early_baseline.malloc_sites(MemBaseline::by_site_and_tag);
+  MallocSiteIterator current_itr = _current_baseline.malloc_sites(MemBaseline::by_site_and_tag);
 
   const MallocSite* early_site   = early_itr.next();
   const MallocSite* current_site = current_itr.next();
