@@ -22,14 +22,13 @@
  *
  */
 
+#include "memory/arena.hpp"
 #include "nmt/mallocTracker.hpp"
 #include "nmt/memoryFileTracker.hpp"
 #include "nmt/memTracker.hpp"
 #include "nmt/nmtCommon.hpp"
 #include "nmt/nmtUsage.hpp"
 #include "nmt/threadStackTracker.hpp"
-#include "nmt/virtualMemoryTracker.hpp"
-#include "runtime/threadCritical.hpp"
 
 // Enabled all options for snapshot.
 const NMTUsageOptions NMTUsage::OptionsAll = { true, true, true };
@@ -48,14 +47,17 @@ void NMTUsage::walk_thread_stacks() {
   // much memory had been committed if they are backed by virtual memory. This
   // needs to happen before we take the snapshot of the virtual memory since it
   // will update this information.
-  VirtualMemoryTracker::snapshot_thread_stacks();
+  VirtualMemoryTracker::Instance::snapshot_thread_stacks();
 }
 
 void NMTUsage::update_malloc_usage() {
-  // Thread critical needed keep values in sync, total area size
+  MallocMemorySnapshot* ms;
+  // Lock needed to keep values in sync, total area size
   // is deducted from mtChunk in the end to give correct values.
-  ThreadCritical tc;
-  const MallocMemorySnapshot* ms = MallocMemorySummary::as_snapshot();
+  {
+    ChunkPoolLocker lock;
+    ms = MallocMemorySummary::as_snapshot();
+  }
 
   size_t total_arena_size = 0;
   for (int i = 0; i < mt_number_of_tags; i++) {
