@@ -25,8 +25,10 @@
 #ifndef SHARE_OPTO_PREDICATES_HPP
 #define SHARE_OPTO_PREDICATES_HPP
 
+#include "opto/c2_globals.hpp"
 #include "opto/cfgnode.hpp"
 #include "opto/opaquenode.hpp"
+#include "opto/predicates_enums.hpp"
 
 class IdealLoopTree;
 class InitializedAssertionPredicate;
@@ -43,12 +45,15 @@ class TemplateAssertionPredicate;
  * - Parse Predicate: Added during parsing to capture the current JVM state. This predicate represents a "placeholder"
  *                    above which Regular Predicates can be created later after parsing.
  *
- *                    There are initially three Parse Predicates for each loop:
- *                    - Loop Parse Predicate:             The Parse Predicate added for Loop Predicates.
- *                    - Profiled Loop Parse Predicate:    The Parse Predicate added for Profiled Loop Predicates.
- *                    - Loop Limit Check Parse Predicate: The Parse Predicate added for a Loop Limit Check Predicate.
+ *                    There are initially five Parse Predicates for each loop:
+ *                    - Loop Parse Predicate:               The Parse Predicate added for Loop Predicates.
+ *                    - Profiled Loop Parse Predicate:      The Parse Predicate added for Profiled Loop Predicates.
+ *                    - Loop Limit Check Parse Predicate:   The Parse Predicate added for a Loop Limit Check Predicate.
+ *                    - Short Running Loop Parse Predicate: The Parse Predicate added for the short running long loop check.
+ *                    - AutoVectorization Parse Predicate:  The Parse Predicate added for AutoVectorization runtime checks.
  * - Runtime Predicate: This term is used to refer to a Hoisted Check Predicate (either a Loop Predicate or a Profiled
- *                      Loop Predicate) or a Loop Limit Check Predicate. These predicates will be checked at runtime while
+ *                      Loop Predicate), a Loop Limit Check Predicate, a Short Running Long Loop Predicate, or a
+ *                      AutoVectorization Runtime Check Predicate. These predicates will be checked at runtime while
  *                      the Parse and Assertion Predicates are always removed before code generation (except for
  *                      Initialized Assertion Predicates which are kept in debug builds while being removed in product
  *                      builds).
@@ -72,6 +77,29 @@ class TemplateAssertionPredicate;
  *                           counted loop to avoid these overflow problems.
  *                           The predicate does not replace an actual check inside the loop. This predicate can only
  *                           be added once above the Loop Limit Check Parse Predicate for a loop.
+ *     - Short:              This predicate is created when a long counted loop is transformed into an int counted
+ *       Running Long        loop. In general, that transformation requires an outer loop to guarantee that the new
+ *       Loop                loop nest iterates over the entire range of the loop before transformation. However, if the
+ *       Predicate           loop is speculated to run for a small enough number of iterations, the outer loop is not
+ *                           needed. This predicate is added to catch mis-speculation in this case. It also applies to
+ *                           int counted loops with long range checks for which a loop nest also needs to be created
+ *                           in the general case (so the transformation of long range checks to int range checks is
+ *                           legal).
+ *     - AutoVectorization:  This predicate is used for speculative runtime checks required for AutoVectorization.
+ *       Runtime Check       There are multiple reasons why we need a runtime check to allow vectorization:
+ *       Predicate           - Unknown aliasing:
+ *                             An important compoinent of AutoVectorization is proving that memory addresses do not
+ *                             alias, and can therefore be reordered. In some cases, this cannot be done statically
+ *                             and a runtime check is necessary.
+ *                           - Unknown alignment of native memory:
+ *                             While heap objects have 8-byte alignment, off-heap (native) memory often has no alignment
+ *                             guarantees. On platforms that require vectors to be aligned, we need to prove alignment.
+ *                             We cannot do that statically with native memory, hence we need a runtime check.
+ *                           The benefit of using a predicate is that we only have to compile the vectorized loop. If
+ *                           the runtime check fails, we simply deoptimize. Should we eventually recompile, then the
+ *                           predicate is not available any more, and we instead use a multiversioning approach with
+ *                           both a vectorized and a scalar loop, where the runtime determines which loop is taken.
+ *                           See: PhaseIdealLoop::maybe_multiversion_for_auto_vectorization_runtime_checks
  * - Assertion Predicate: An always true predicate which will never fail (its range is already covered by an earlier
  *                        Hoisted Check Predicate or the main-loop entry guard) but is required in order to fold away a
  *                        dead sub loop in which some data could be proven to be dead (by the type system) and replaced
@@ -148,19 +176,27 @@ class TemplateAssertionPredicate;
  *                    Predicates, and the associated Parse Predicate which all share the same uncommon trap. This block
  *                    could be empty if there were no Runtime Predicates created and the Parse Predicate was already
  *                    removed.
- *                    There are three different Predicate Blocks:
+ *                    There are five different Predicate Blocks:
+ *                    - Short Running Long    Groups the Short Running Long Loop Predicate (if created), and the
+ *                      Loop Predicate Block: Short Running Long Loop Parse Predicate together.
  *                    - Loop Predicate Block: Groups the Loop Predicates (if any), including the Assertion Predicates,
  *                                            and the Loop Parse Predicate (if not removed, yet) together.
  *                    - Profiled Loop         Groups the Profiled Loop Predicates (if any), including the Assertion
  *                      Predicate Block:      Predicates, and the Profiled Loop Parse Predicate (if not removed, yet)
  *                                            together.
+ *                    - AutoVectorization     Groups the AutoVectorization Runtime Check Predicates (if any), and the
+ *                      Runtime Check         AutoVectorization Runtime Check Parse Predicate together.
+ *                      Predicate Block:
  *                    - Loop Limit Check      Groups the Loop Limit Check Predicate (if created) and the Loop Limit
  *                      Predicate Block:      Check Parse Predicate (if not removed, yet) together.
  * - Regular Predicate Block: A block that only contains the Regular Predicates of a Predicate Block without the
  *                            Parse Predicate.
  *
  * Initially, before applying any loop-splitting optimizations, we find the following structure after Loop Predication
- * (predicates inside square brackets [] do not need to exist if there are no checks to hoist):
+ * (predicates inside square brackets [] do not need to exist if there are no checks to hoist / insert):
+ *
+ *   [Short Running Long Loop Predicate] (at most one)                 \ Short Running Long
+ * Short Running Long Loop Parse Predicate                             / Loop Predicate Block
  *
  *   [Loop Predicate 1 + two Template Assertion Predicates]            \
  *   [Loop Predicate 2 + two Template Assertion Predicates]            |
@@ -174,6 +210,12 @@ class TemplateAssertionPredicate;
  *   [Profiled Loop Predicate m + two Template Assertion Predicates]   |
  * Profiled Loop Parse Predicate                                       /
  *
+ *   [AutoVectorization Runtime Check Predicate 1]                     \
+ *   [AutoVectorization Runtime Check Predicate 2]                     | AutoVectorization
+ *   ...                                                               | Runtime Check
+ *   [AutoVectorization Runtime Check Predicate l]                     | Predicate Block
+ * AutoVectorization Runtime Check Parse Predicate                     /
+ *
  *   [Loop Limit Check Predicate] (at most one)                        \ Loop Limit Check
  * Loop Limit Check Parse Predicate                                    / Predicate Block
  * Loop Head
@@ -200,16 +242,6 @@ class TemplateAssertionPredicate;
  * Main Loop Head
  */
 
-// Assertion Predicates are either emitted to check the initial value of a range check in the first iteration or the last
-// value of a range check in the last iteration of a loop.
-enum class AssertionPredicateType {
-  None, // Not an Assertion Predicate
-  InitValue,
-  LastValue,
-  // Used for the Initialized Assertion Predicate emitted during Range Check Elimination for the final IV value.
-  FinalIv
-};
-
 // Interface to represent a C2 predicate. A predicate is always represented by two CFG nodes:
 // - An If node (head)
 // - An IfProj node representing the success projection of the If node (tail).
@@ -273,9 +305,10 @@ class AssertionPredicates : public StackObj {
 // - An Initialized Assertion Predicate.
 class AssertionPredicate : public StackObj {
   static bool has_assertion_predicate_opaque(const Node* predicate_proj);
-  static bool has_halt(const Node* success_proj);
+
  public:
   static bool is_predicate(const Node* maybe_success_proj);
+  static bool has_halt(const IfTrueNode* success_proj);
 };
 
 // Utility class representing a Regular Predicate which is either a Runtime Predicate or an Assertion Predicate.
@@ -296,8 +329,6 @@ class ParsePredicate : public Predicate {
   }
 
   static ParsePredicateNode* init_parse_predicate(const Node* parse_predicate_proj, Deoptimization::DeoptReason deopt_reason);
-  NOT_PRODUCT(static void trace_cloned_parse_predicate(bool is_true_path_loop,
-                                                       const ParsePredicateSuccessProj* success_proj);)
 
  public:
   ParsePredicate(Node* parse_predicate_proj, Deoptimization::DeoptReason deopt_reason)
@@ -311,10 +342,11 @@ class ParsePredicate : public Predicate {
     return _entry;
   }
 
-  // This Parse Predicate is valid if the node passed to the constructor is a projection of a ParsePredicateNode and the
-  // deopt_reason of the uncommon trap of the ParsePredicateNode matches the passed deopt_reason to the constructor.
+  // This Parse Predicate is valid if the node passed to the constructor is a projection of a ParsePredicateNode, the
+  // deopt_reason of the uncommon trap of the ParsePredicateNode matches the passed deopt_reason to the constructor and
+  // the ParsePredicateNode is not marked useless.
   bool is_valid() const {
-    return _parse_predicate_node != nullptr;
+    return _parse_predicate_node != nullptr && !_parse_predicate_node->is_useless();
   }
 
   ParsePredicateNode* head() const override {
@@ -327,14 +359,10 @@ class ParsePredicate : public Predicate {
     return _success_proj;
   }
 
-  ParsePredicate clone_to_unswitched_loop(Node* new_control, bool is_true_path_loop,
-                                          PhaseIdealLoop* phase) const;
+  ParsePredicate clone_to_loop(Node* new_control, bool rewire_uncommon_proj_phi_inputs, PhaseIdealLoop* phase) const;
+  NOT_PRODUCT(void trace_cloned_parse_predicate(bool is_false_path_loop) const;)
 
-  // Kills this Parse Predicate by marking it useless. Will be folded away in the next IGVN round.
-  void kill(const PhaseIterGVN& igvn) const {
-    _parse_predicate_node->mark_useless();
-    igvn._worklist.push(_parse_predicate_node);
-  }
+  void kill(PhaseIterGVN& igvn) const;
 };
 
 // Class to represent a Runtime Predicate which always has an associated UCT on the failing path.
@@ -403,15 +431,26 @@ class TemplateAssertionPredicate : public Predicate {
     return _if_node->assertion_predicate_type() == AssertionPredicateType::LastValue;
   }
 
-  TemplateAssertionPredicate clone(Node* new_control, PhaseIdealLoop* phase) const;
+  bool is_useless() const {
+    return opaque_node()->is_useless();
+  }
+
+  TemplateAssertionPredicate clone(Node* new_control, CountedLoopNode* new_loop_node, PhaseIdealLoop* phase) const;
   TemplateAssertionPredicate clone_and_replace_opaque_input(Node* new_control, Node* new_opaque_input,
-                                                            PhaseIdealLoop* phase) const;
+                                                            CountedLoopNode* new_loop_node, PhaseIdealLoop* phase) const;
   void replace_opaque_stride_input(Node* new_stride, PhaseIterGVN& igvn) const;
   InitializedAssertionPredicate initialize(PhaseIdealLoop* phase) const;
   void rewire_loop_data_dependencies(IfTrueNode* target_predicate, const NodeInLoopBody& data_in_loop_body,
                                      const PhaseIdealLoop* phase) const;
-  void kill(PhaseIdealLoop* phase) const;
-  static bool is_predicate(const Node* node);
+  void kill(PhaseIterGVN& igvn) const;
+
+  // Should only be called during Loop Unrolling when we only update the OpaqueLoopStride input but don't require a full
+  // clone of the Template Assertion Expression.
+  void update_associated_loop_node(CountedLoopNode* loop_node) const {
+    opaque_node()->update_loop_node(loop_node);
+  }
+
+  static bool is_predicate(const Node* maybe_success_proj);
 
 #ifdef ASSERT
   static void verify(IfTrueNode* template_assertion_predicate_success_proj) {
@@ -429,6 +468,8 @@ class InitializedAssertionPredicate : public Predicate {
   IfTrueNode* const _success_proj;
   IfNode* const _if_node;
 
+  DEBUG_ONLY(static bool has_halt(const IfTrueNode* success_proj);)
+
  public:
   explicit InitializedAssertionPredicate(IfTrueNode* success_proj)
       : _success_proj(success_proj),
@@ -456,8 +497,12 @@ class InitializedAssertionPredicate : public Predicate {
     return _if_node->assertion_predicate_type() == AssertionPredicateType::LastValue;
   }
 
-  void kill(PhaseIdealLoop* phase) const;
-  static bool is_predicate(const Node* node);
+  bool is_useless() const {
+    return opaque_node()->is_useless();
+  }
+
+  void kill(PhaseIterGVN& igvn) const;
+  static bool is_predicate(const Node* maybe_success_proj);
 
 #ifdef ASSERT
   static void verify(IfTrueNode* initialized_assertion_predicate_success_proj) {
@@ -479,23 +524,25 @@ class TransformStrategyForOpaqueLoopNodes : public StackObj {
 // A Template Assertion Predicate represents the OpaqueTemplateAssertionPredicateNode for the initial value or the last
 // value of a Template Assertion Predicate and all the nodes up to and including the OpaqueLoop* nodes.
 class TemplateAssertionExpression : public StackObj {
-  OpaqueTemplateAssertionPredicateNode* _opaque_node;
-
+  OpaqueTemplateAssertionPredicateNode* const _opaque_node;
+  PhaseIdealLoop* const _phase;
  public:
-  explicit TemplateAssertionExpression(OpaqueTemplateAssertionPredicateNode* opaque_node) : _opaque_node(opaque_node) {}
+  TemplateAssertionExpression(OpaqueTemplateAssertionPredicateNode* opaque_node, PhaseIdealLoop* phase)
+      : _opaque_node(opaque_node),
+        _phase(phase) {}
+  NONCOPYABLE(TemplateAssertionExpression);
 
  private:
   OpaqueTemplateAssertionPredicateNode* clone(const TransformStrategyForOpaqueLoopNodes& transform_strategy,
-                                              Node* new_control, PhaseIdealLoop* phase) const;
+                                              Node* new_control, CountedLoopNode* new_loop_node) const;
 
  public:
-  OpaqueTemplateAssertionPredicateNode* clone(Node* new_control, PhaseIdealLoop* phase) const;
+  OpaqueTemplateAssertionPredicateNode* clone(Node* new_control, CountedLoopNode* new_loop_node) const;
   OpaqueTemplateAssertionPredicateNode* clone_and_replace_init(Node* new_control, Node* new_init,
-                                                               PhaseIdealLoop* phase) const;
+                                                               CountedLoopNode* new_loop_node) const;
   OpaqueTemplateAssertionPredicateNode* clone_and_replace_init_and_stride(Node* new_control, Node* new_init,
-                                                                          Node* new_stride, PhaseIdealLoop* phase) const;
-  void replace_opaque_stride_input(Node* new_stride, PhaseIterGVN& igvn) const;
-  OpaqueInitializedAssertionPredicateNode* clone_and_fold_opaque_loop_nodes(Node* new_control, PhaseIdealLoop* phase) const;
+                                                                          Node* new_stride) const;
+  OpaqueInitializedAssertionPredicateNode* clone_and_fold_opaque_loop_nodes(Node* new_control) const;
 };
 
 // Class to represent a node being part of a Template Assertion Expression. Note that this is not an IR node.
@@ -652,44 +699,78 @@ class InitializedAssertionPredicateCreator : public StackObj {
 
 // This class iterates through all predicates of a Regular Predicate Block and applies the given visitor to each.
 class RegularPredicateBlockIterator : public StackObj {
-  Node* const _start_node;
+  Node* _current_node;
   const Deoptimization::DeoptReason _deopt_reason;
 
  public:
   RegularPredicateBlockIterator(Node* start_node, Deoptimization::DeoptReason deopt_reason)
-      : _start_node(start_node),
+      : _current_node(start_node),
         _deopt_reason(deopt_reason) {}
   NONCOPYABLE(RegularPredicateBlockIterator);
 
   // Skip all predicates by just following the inputs. We do not call any user provided visitor.
-  Node* skip_all() const {
+  Node* skip_all() {
     PredicateVisitor do_nothing; // No real visits, just do nothing.
     return for_each(do_nothing);
   }
 
   // Walk over all predicates of this block (if any) and apply the given 'predicate_visitor' to each predicate.
   // Returns the entry to the earliest predicate.
-  Node* for_each(PredicateVisitor& predicate_visitor) const {
-    Node* current = _start_node;
+  Node* for_each(PredicateVisitor& predicate_visitor) {
     while (predicate_visitor.should_continue()) {
-      if (TemplateAssertionPredicate::is_predicate(current)) {
-        TemplateAssertionPredicate template_assertion_predicate(current->as_IfTrue());
-        predicate_visitor.visit(template_assertion_predicate);
-        current = template_assertion_predicate.entry();
-      } else if (RuntimePredicate::is_predicate(current, _deopt_reason)) {
-        RuntimePredicate runtime_predicate(current->as_IfProj());
-        predicate_visitor.visit(runtime_predicate);
-        current = runtime_predicate.entry();
-      } else if (InitializedAssertionPredicate::is_predicate(current)) {
-        InitializedAssertionPredicate initialized_assertion_predicate(current->as_IfTrue());
-        predicate_visitor.visit(initialized_assertion_predicate);
-        current = initialized_assertion_predicate.entry();
-      } else {
-        // Either a Parse Predicate or not a Regular Predicate. In both cases, the node does not belong to this block.
-        break;
+      if (process_template_assertion_predicate(predicate_visitor)) {
+        continue;
       }
+
+      if (process_runtime_predicate(predicate_visitor)) {
+        continue;
+      }
+
+      if (process_initialized_assertion_predicate(predicate_visitor)) {
+        continue;
+      }
+      // Either a Parse Predicate or not a Regular Predicate. In both cases, the node does not belong to this block.
+      break;
     }
-    return current;
+    return _current_node;
+  }
+
+  bool process_template_assertion_predicate(PredicateVisitor& predicate_visitor) {
+    if (!TemplateAssertionPredicate::is_predicate(_current_node)) {
+      return false;
+    }
+
+    TemplateAssertionPredicate template_assertion_predicate(_current_node->as_IfTrue());
+    if (!template_assertion_predicate.is_useless()) {
+      // Only visit if not useless. Otherwise, just skip over it to possibly process other predicates above.
+      predicate_visitor.visit(template_assertion_predicate);
+    }
+    _current_node = template_assertion_predicate.entry();
+    return true;
+  }
+
+  bool process_runtime_predicate(PredicateVisitor& predicate_visitor) {
+    if (!RuntimePredicate::is_predicate(_current_node, _deopt_reason)) {
+      return false;
+    }
+    RuntimePredicate runtime_predicate(_current_node->as_IfProj());
+    predicate_visitor.visit(runtime_predicate);
+    _current_node = runtime_predicate.entry();
+    return true;
+  }
+
+  bool process_initialized_assertion_predicate(PredicateVisitor& predicate_visitor) {
+    if (!InitializedAssertionPredicate::is_predicate(_current_node)) {
+      return false;
+    }
+
+    InitializedAssertionPredicate initialized_assertion_predicate(_current_node->as_IfTrue());
+    if (!initialized_assertion_predicate.is_useless()) {
+      // Only visit if not useless. Otherwise, just skip over it to possibly process other predicates above.
+      predicate_visitor.visit(initialized_assertion_predicate);
+    }
+    _current_node = initialized_assertion_predicate.entry();
+    return true;
   }
 };
 
@@ -697,7 +778,7 @@ class RegularPredicateBlockIterator : public StackObj {
 class PredicateBlockIterator : public StackObj {
   Node* const _start_node;
   const ParsePredicate _parse_predicate; // Could be missing.
-  const RegularPredicateBlockIterator _regular_predicate_block_iterator;
+  RegularPredicateBlockIterator _regular_predicate_block_iterator;
 
  public:
   PredicateBlockIterator(Node* start_node, Deoptimization::DeoptReason deopt_reason)
@@ -707,7 +788,7 @@ class PredicateBlockIterator : public StackObj {
 
   // Walk over all predicates of this block (if any) and apply the given 'predicate_visitor' to each predicate.
   // Returns the entry to the earliest predicate.
-  Node* for_each(PredicateVisitor& predicate_visitor) const {
+  Node* for_each(PredicateVisitor& predicate_visitor) {
     if (!predicate_visitor.should_continue()) {
       return _start_node;
     }
@@ -731,18 +812,23 @@ class PredicateIterator : public StackObj {
   // Apply the 'predicate_visitor' for each predicate found in the predicate chain started at the provided node.
   // Returns the entry to the earliest predicate.
   Node* for_each(PredicateVisitor& predicate_visitor) const {
-    Node* current = _start_node;
-    PredicateBlockIterator loop_limit_check_predicate_iterator(current, Deoptimization::Reason_loop_limit_check);
-    current = loop_limit_check_predicate_iterator.for_each(predicate_visitor);
+    Node* current_node = _start_node;
+    PredicateBlockIterator loop_limit_check_predicate_iterator(current_node, Deoptimization::Reason_loop_limit_check);
+    current_node = loop_limit_check_predicate_iterator.for_each(predicate_visitor);
+    if (UseAutoVectorizationPredicate) {
+      PredicateBlockIterator auto_vectorization_check_iterator(current_node, Deoptimization::Reason_auto_vectorization_check);
+      current_node = auto_vectorization_check_iterator.for_each(predicate_visitor);
+    }
     if (UseLoopPredicate) {
       if (UseProfiledLoopPredicate) {
-        PredicateBlockIterator profiled_loop_predicate_iterator(current, Deoptimization::Reason_profile_predicate);
-        current = profiled_loop_predicate_iterator.for_each(predicate_visitor);
+        PredicateBlockIterator profiled_loop_predicate_iterator(current_node, Deoptimization::Reason_profile_predicate);
+        current_node = profiled_loop_predicate_iterator.for_each(predicate_visitor);
       }
-      PredicateBlockIterator loop_predicate_iterator(current, Deoptimization::Reason_predicate);
-      current = loop_predicate_iterator.for_each(predicate_visitor);
+      PredicateBlockIterator loop_predicate_iterator(current_node, Deoptimization::Reason_predicate);
+      current_node = loop_predicate_iterator.for_each(predicate_visitor);
     }
-    return current;
+    PredicateBlockIterator short_running_loop_predicate_iterator(current_node, Deoptimization::Reason_short_running_long_loop);
+    return short_running_loop_predicate_iterator.for_each(predicate_visitor);
   }
 };
 
@@ -906,19 +992,25 @@ class PredicateBlock : public StackObj {
 class Predicates : public StackObj {
   Node* const _tail;
   const PredicateBlock _loop_limit_check_predicate_block;
+  const PredicateBlock _auto_vectorization_check_block;
   const PredicateBlock _profiled_loop_predicate_block;
   const PredicateBlock _loop_predicate_block;
+  const PredicateBlock _short_running_long_loop_predicate_block;
   Node* const _entry;
 
  public:
   explicit Predicates(Node* loop_entry)
       : _tail(loop_entry),
         _loop_limit_check_predicate_block(loop_entry, Deoptimization::Reason_loop_limit_check),
-        _profiled_loop_predicate_block(_loop_limit_check_predicate_block.entry(),
+        _auto_vectorization_check_block(_loop_limit_check_predicate_block.entry(),
+                                        Deoptimization::Reason_auto_vectorization_check),
+        _profiled_loop_predicate_block(_auto_vectorization_check_block.entry(),
                                        Deoptimization::Reason_profile_predicate),
         _loop_predicate_block(_profiled_loop_predicate_block.entry(),
                               Deoptimization::Reason_predicate),
-        _entry(_loop_predicate_block.entry()) {}
+        _short_running_long_loop_predicate_block(_loop_predicate_block.entry(),
+                                            Deoptimization::Reason_short_running_long_loop),
+        _entry(_short_running_long_loop_predicate_block.entry()) {}
   NONCOPYABLE(Predicates);
 
   // Returns the control input the first predicate if there are any predicates. If there are no predicates, the same
@@ -935,10 +1027,18 @@ class Predicates : public StackObj {
     return &_profiled_loop_predicate_block;
   }
 
+  const PredicateBlock* auto_vectorization_check_block() const {
+    return &_auto_vectorization_check_block;
+  }
+
   const PredicateBlock* loop_limit_check_predicate_block() const {
     return &_loop_limit_check_predicate_block;
   }
 
+  const PredicateBlock* short_running_long_loop_predicate_block() const {
+    return &_short_running_long_loop_predicate_block;
+  }
+
   bool has_any() const {
     return _entry != _tail;
   }
@@ -1031,33 +1131,45 @@ class NodeInClonedLoopBody : public NodeInLoopBody {
   }
 };
 
+// This class checks whether a node is in the loop body passed to the constructor.
+class NodeInSingleLoopBody : public NodeInLoopBody {
+  PhaseIdealLoop* const _phase;
+  IdealLoopTree* const _ilt;
+
+public:
+  NodeInSingleLoopBody(PhaseIdealLoop* phase, IdealLoopTree* ilt) : _phase(phase), _ilt(ilt) {
+  }
+  NONCOPYABLE(NodeInSingleLoopBody);
+
+  bool check_node_in_loop_body(Node* node) const override;
+};
+
 // Visitor to create Initialized Assertion Predicates at a target loop from Template Assertion Predicates from a source
 // loop. This visitor can be used in combination with a PredicateIterator.
 class CreateAssertionPredicatesVisitor : public PredicateVisitor {
   Node* const _init;
   Node* const _stride;
+  CountedLoopNode* const _target_loop_head;
   Node* const _old_target_loop_entry;
   Node* _current_predicate_chain_head;
   PhaseIdealLoop* const _phase;
-  bool _has_hoisted_check_parse_predicates;
   const NodeInLoopBody& _node_in_loop_body;
-  const bool _clone_template;
+  const bool _kill_old_template;
 
   TemplateAssertionPredicate
   clone_template_and_replace_init_input(const TemplateAssertionPredicate& template_assertion_predicate) const;
 
   InitializedAssertionPredicate initialize_from_template(const TemplateAssertionPredicate& template_assertion_predicate,
-                                                         Node* new_control) const;
+                                                         IfTrueNode* cloned_template_predicate_tail) const;
   void rewire_to_old_predicate_chain_head(Node* initialized_assertion_predicate_success_proj) const;
 
  public:
   CreateAssertionPredicatesVisitor(CountedLoopNode* target_loop_head, PhaseIdealLoop* phase,
-                                   const NodeInLoopBody& node_in_loop_body, bool clone_template);
+                                   const NodeInLoopBody& node_in_loop_body, bool kill_old_template);
   NONCOPYABLE(CreateAssertionPredicatesVisitor);
 
   using PredicateVisitor::visit;
 
-  void visit(const ParsePredicate& parse_predicate) override;
   void visit(const TemplateAssertionPredicate& template_assertion_predicate) override;
 };
 
@@ -1097,6 +1209,8 @@ public:
 //            |                                |                                                        |
 //    source loop head                      target loop head                                       target loop head
 class ClonePredicateToTargetLoop : public StackObj {
+  LoopNode* const _target_loop_head;
+  LoopNode* const _target_outer_loop_head;
   Node* const _old_target_loop_entry; // Used as control for each newly cloned predicate.
   TargetLoopPredicateChain _target_loop_predicate_chain;
   const NodeInLoopBody& _node_in_loop_body;
@@ -1106,20 +1220,14 @@ public:
   ClonePredicateToTargetLoop(LoopNode* target_loop_head, const NodeInLoopBody& node_in_loop_body, PhaseIdealLoop* phase);
 
   // Clones the provided Parse Predicate to the head of the current predicate chain at the target loop.
-  void clone_parse_predicate(const ParsePredicate& parse_predicate, bool is_true_path_loop) {
-    ParsePredicate cloned_parse_predicate = parse_predicate.clone_to_unswitched_loop(_old_target_loop_entry,
-                                                                                     is_true_path_loop, _phase);
+  ParsePredicate clone_parse_predicate(const ParsePredicate& parse_predicate, bool rewire_uncommon_proj_phi_inputs) {
+    ParsePredicate cloned_parse_predicate = parse_predicate.clone_to_loop(_old_target_loop_entry,
+                                                                          rewire_uncommon_proj_phi_inputs, _phase);
     _target_loop_predicate_chain.insert_predicate(cloned_parse_predicate);
+    return cloned_parse_predicate;
   }
 
-  // Clones the provided Template Assertion Predicate to the head of the current predicate chain at the target loop.
-  void clone_template_assertion_predicate(const TemplateAssertionPredicate& template_assertion_predicate) {
-    TemplateAssertionPredicate cloned_template_assertion_predicate =
-        template_assertion_predicate.clone(_old_target_loop_entry, _phase);
-    template_assertion_predicate.rewire_loop_data_dependencies(cloned_template_assertion_predicate.tail(),
-                                                               _node_in_loop_body, _phase);
-    _target_loop_predicate_chain.insert_predicate(cloned_template_assertion_predicate);
-  }
+  void clone_template_assertion_predicate(const TemplateAssertionPredicate& template_assertion_predicate);
 };
 
 // Visitor to clone Parse and Template Assertion Predicates from a loop to its unswitched true and false path loop.
@@ -1131,7 +1239,7 @@ class CloneUnswitchedLoopPredicatesVisitor : public PredicateVisitor {
   ClonePredicateToTargetLoop _clone_predicate_to_false_path_loop;
 
   PhaseIdealLoop* const _phase;
-  bool _has_hoisted_check_parse_predicates;
+  const bool _is_counted_loop;
 
  public:
   CloneUnswitchedLoopPredicatesVisitor(LoopNode* true_path_loop_head,
@@ -1144,6 +1252,9 @@ class CloneUnswitchedLoopPredicatesVisitor : public PredicateVisitor {
   using PredicateVisitor::visit;
 
   void visit(const ParsePredicate& parse_predicate) override;
+
+  void clone_parse_predicate(const ParsePredicate &parse_predicate,
+                             bool is_false_path_loop);
   void visit(const TemplateAssertionPredicate& template_assertion_predicate) override;
 };
 
@@ -1167,6 +1278,7 @@ class OpaqueTemplateAssertionPredicateCollector : public PredicateVisitor {
 // from the updated templates. The old Initialized Assertion Predicates are killed.
 class UpdateStrideForAssertionPredicates : public PredicateVisitor {
   Node* const _new_stride;
+  CountedLoopNode* const _loop_node;
   PhaseIdealLoop* const _phase;
 
   void replace_opaque_stride_input(const TemplateAssertionPredicate& template_assertion_predicate) const;
@@ -1176,23 +1288,52 @@ class UpdateStrideForAssertionPredicates : public PredicateVisitor {
   void connect_initialized_assertion_predicate(Node* new_control_out, const InitializedAssertionPredicate& initialized_assertion_predicate) const;
 
  public:
-  UpdateStrideForAssertionPredicates(Node* const new_stride, PhaseIdealLoop* phase)
+  UpdateStrideForAssertionPredicates(Node* const new_stride, CountedLoopNode* loop_node, PhaseIdealLoop* phase)
       : _new_stride(new_stride),
+        _loop_node(loop_node),
         _phase(phase) {}
   NONCOPYABLE(UpdateStrideForAssertionPredicates);
 
   using PredicateVisitor::visit;
 
   void visit(const TemplateAssertionPredicate& template_assertion_predicate) override;
-
-  // Kill the old Initialized Assertion Predicates with old strides before unrolling. The new Initialized Assertion
-  // Predicates are inserted after the Template Assertion Predicate which ensures that we are not accidentally visiting
-  // and killing a newly created Initialized Assertion Predicate here.
-  void visit(const InitializedAssertionPredicate& initialized_assertion_predicate) override {
-    if (initialized_assertion_predicate.is_last_value()) {
-      // Only Last Value Initialized Assertion Predicates need to be killed and updated.
-      initialized_assertion_predicate.kill(_phase);
-    }
-  }
+  void visit(const InitializedAssertionPredicate& initialized_assertion_predicate) override;
 };
+
+// Eliminate all useless Parse and Template Assertion Predicates. They become useless when they can no longer be found
+// from a loop head. We mark these useless to clean them up later during IGVN. A Predicate that is marked useless will
+// no longer be visited by a PredicateVisitor.
+class EliminateUselessPredicates : public StackObj {
+  Compile* const C;
+  const GrowableArray<ParsePredicateNode*>& _parse_predicates;
+  const GrowableArray<OpaqueTemplateAssertionPredicateNode*>& _template_assertion_predicate_opaques;
+  PhaseIterGVN& _igvn;
+  IdealLoopTree* const _ltree_root;
+
+  void mark_all_predicates_maybe_useful() const;
+  template <class PredicateList>
+  static void mark_predicates_on_list_maybe_useful(const PredicateList& predicate_list);
+
+  void mark_loop_associated_predicates_useful() const;
+  static void mark_useful_predicates_for_loop(IdealLoopTree* loop);
+
+  void mark_maybe_useful_predicates_useless() const;
+  template <class PredicateList>
+  void mark_maybe_useful_predicates_on_list_useless(const PredicateList& predicate_list) const;
+
+#ifdef ASSERT
+  void verify_loop_nodes_of_useless_templates_assertion_predicates_are_dead() const;
+  Unique_Node_List collect_loop_nodes_of_useless_template_assertion_predicates() const;
+  void verify_associated_loop_nodes_are_dead(const Unique_Node_List& loop_nodes_of_useless_template_assertion_predicates) const;
+#endif // ASSERT
+ public:
+  EliminateUselessPredicates(PhaseIterGVN& igvn, IdealLoopTree* ltree_root)
+      : C(igvn.C), _parse_predicates(igvn.C->parse_predicates()),
+        _template_assertion_predicate_opaques(igvn.C->template_assertion_predicate_opaques()),
+        _igvn(igvn),
+        _ltree_root(ltree_root) {}
+
+  void eliminate() const;
+};
+
 #endif // SHARE_OPTO_PREDICATES_HPP
