@@ -21,10 +21,11 @@
  * questions.
  */
 
-#include "opto/vtransform.hpp"
-#include "opto/vectornode.hpp"
 #include "opto/castnode.hpp"
 #include "opto/convertnode.hpp"
+#include "opto/vectorization.hpp"
+#include "opto/vectornode.hpp"
+#include "opto/vtransform.hpp"
 
 void VTransformGraph::add_vtnode(VTransformNode* vtnode) {
   assert(vtnode->_idx == _vtnodes.length(), "position must match idx");
@@ -57,7 +58,7 @@ bool VTransformGraph::schedule() {
   VectorSet pre_visited;
   VectorSet post_visited;
 
-  collect_nodes_without_req_or_dependency(stack);
+  collect_nodes_without_strong_in_edges(stack);
 
   // We create a reverse-post-visit order. This gives us a linearization, if there are
   // no cycles. Then, we simply reverse the order, and we have a schedule.
@@ -72,8 +73,11 @@ bool VTransformGraph::schedule() {
       //   No  -> we are mid-visit.
       bool all_uses_already_visited = true;
 
-      for (int i = 0; i < vtn->outs(); i++) {
-        VTransformNode* use = vtn->out(i);
+      // We only need to respect the strong edges (data edges and strong memory edges).
+      // Violated weak memory edges are allowed, but require a speculative aliasing
+      // runtime check, see VTransform::apply_speculative_aliasing_runtime_checks.
+      for (uint i = 0; i < vtn->out_strong_edges(); i++) {
+        VTransformNode* use = vtn->out_strong_edge(i);
         if (post_visited.test(use->_idx)) { continue; }
         if (pre_visited.test(use->_idx)) {
           // Cycle detected!
@@ -100,7 +104,7 @@ bool VTransformGraph::schedule() {
   }
 
 #ifndef PRODUCT
-  if (_trace._verbose) {
+  if (_trace._info) {
     print_schedule();
   }
 #endif
@@ -109,11 +113,11 @@ bool VTransformGraph::schedule() {
   return true;
 }
 
-// Push all "root" nodes, i.e. those that have no inputs (req or dependency):
-void VTransformGraph::collect_nodes_without_req_or_dependency(GrowableArray<VTransformNode*>& stack) const {
+// Push all "root" nodes, i.e. those that have no strong input edges (data edges and strong memory edges):
+void VTransformGraph::collect_nodes_without_strong_in_edges(GrowableArray<VTransformNode*>& stack) const {
   for (int i = 0; i < _vtnodes.length(); i++) {
     VTransformNode* vtn = _vtnodes.at(i);
-    if (!vtn->has_req_or_dependency()) {
+    if (!vtn->has_strong_in_edge()) {
       stack.push(vtn);
     }
   }
@@ -144,21 +148,19 @@ void VTransformApplyResult::trace(VTransformNode* vtnode) const {
 }
 #endif
 
-void VTransform::apply_speculative_runtime_checks() {
+void VTransform::apply_speculative_alignment_runtime_checks() {
   if (VLoop::vectors_should_be_aligned()) {
 #ifdef ASSERT
     if (_trace._align_vector || _trace._speculative_runtime_checks) {
-      tty->print_cr("\nVTransform::apply_speculative_runtime_checks: native memory alignment");
+      tty->print_cr("\nVTransform::apply_speculative_alignment_runtime_checks: native memory alignment");
     }
 #endif
 
     const GrowableArray<VTransformNode*>& vtnodes = _graph.vtnodes();
     for (int i = 0; i < vtnodes.length(); i++) {
-      VTransformVectorNode* vtn = vtnodes.at(i)->isa_Vector();
+      VTransformMemVectorNode* vtn = vtnodes.at(i)->isa_MemVector();
       if (vtn == nullptr) { continue; }
-      MemNode* p0 = vtn->nodes().at(0)->isa_Mem();
-      if (p0 == nullptr) { continue; }
-      const VPointer& vp = vpointer(p0);
+      const VPointer& vp = vtn->vpointer();
       if (vp.mem_pointer().base().is_object()) { continue; }
       assert(vp.mem_pointer().base().is_native(), "VPointer base must be object or native");
 
@@ -199,13 +201,13 @@ void VTransform::add_speculative_alignment_check(Node* node, juint alignment) {
     TRACE_SPECULATIVE_ALIGNMENT_CHECK(node);
   }
 
-  Node* mask_alignment = igvn().intcon(alignment-1);
+  Node* mask_alignment = phase()->intcon(alignment-1);
   Node* base_alignment = new AndINode(node, mask_alignment);
   phase()->register_new_node(base_alignment, ctrl);
   TRACE_SPECULATIVE_ALIGNMENT_CHECK(mask_alignment);
   TRACE_SPECULATIVE_ALIGNMENT_CHECK(base_alignment);
 
-  Node* zero = igvn().intcon(0);
+  Node* zero = phase()->intcon(0);
   Node* cmp_alignment = CmpNode::make(base_alignment, zero, T_INT, false);
   BoolNode* bol_alignment = new BoolNode(cmp_alignment, BoolTest::eq);
   phase()->register_new_node(cmp_alignment, ctrl);
@@ -213,10 +215,217 @@ void VTransform::add_speculative_alignment_check(Node* node, juint alignment) {
   TRACE_SPECULATIVE_ALIGNMENT_CHECK(cmp_alignment);
   TRACE_SPECULATIVE_ALIGNMENT_CHECK(bol_alignment);
 
-  add_speculative_check(bol_alignment);
+  add_speculative_check([&] (Node* ctrl) { return bol_alignment; });
 }
 
-void VTransform::add_speculative_check(BoolNode* bol) {
+class VPointerWeakAliasingPair : public StackObj {
+private:
+  // Using references instead of pointers would be preferrable, but GrowableArray
+  // requires a default constructor, and we do not have a default constructor for
+  // VPointer.
+  const VPointer* _vp1 = nullptr;
+  const VPointer* _vp2 = nullptr;
+
+  VPointerWeakAliasingPair(const VPointer& vp1, const VPointer& vp2) : _vp1(&vp1), _vp2(&vp2) {
+    assert(vp1.is_valid(), "sanity");
+    assert(vp2.is_valid(), "sanity");
+    assert(!vp1.never_overlaps_with(vp2), "otherwise no aliasing");
+    assert(!vp1.always_overlaps_with(vp2), "otherwise must be strong");
+    assert(VPointer::cmp_summands_and_con(vp1, vp2) <= 0, "must be sorted");
+  }
+
+public:
+  // Default constructor to make GrowableArray happy.
+  VPointerWeakAliasingPair() : _vp1(nullptr), _vp2(nullptr) {}
+
+  static VPointerWeakAliasingPair make(const VPointer& vp1, const VPointer& vp2) {
+    if (VPointer::cmp_summands_and_con(vp1, vp2) <= 0) {
+      return VPointerWeakAliasingPair(vp1, vp2);
+    } else {
+      return VPointerWeakAliasingPair(vp2, vp1);
+    }
+  }
+
+  const VPointer& vp1() const { return *_vp1; }
+  const VPointer& vp2() const { return *_vp2; }
+
+  // Sort by summands, so that pairs with same summands (summand1, summands2) are adjacent.
+  static int cmp_for_sort(VPointerWeakAliasingPair* pair1, VPointerWeakAliasingPair* pair2) {
+    int cmp_summands1 = VPointer::cmp_summands(pair1->vp1(), pair2->vp1());
+    if (cmp_summands1 != 0) { return cmp_summands1; }
+    return VPointer::cmp_summands(pair1->vp2(), pair2->vp2());
+  }
+};
+
+void VTransform::apply_speculative_aliasing_runtime_checks() {
+
+  if (_vloop.use_speculative_aliasing_checks()) {
+
+#ifdef ASSERT
+    if (_trace._speculative_aliasing_analysis || _trace._speculative_runtime_checks) {
+      tty->print_cr("\nVTransform::apply_speculative_aliasing_runtime_checks: speculative aliasing analysis runtime checks");
+    }
+#endif
+
+    // It would be nice to add a ResourceMark here. But it would collide with resource allocation
+    // in PhaseIdealLoop::set_idom for _idom and _dom_depth. See also JDK-8337015.
+    VectorSet visited;
+    GrowableArray<VPointerWeakAliasingPair> weak_aliasing_pairs;
+
+    const GrowableArray<VTransformNode*>& schedule = _graph.get_schedule();
+    for (int i = 0; i < schedule.length(); i++) {
+      VTransformNode* vtn = schedule.at(i);
+      for (uint i = 0; i < vtn->out_weak_edges(); i++) {
+        VTransformNode* use = vtn->out_weak_edge(i);
+        if (visited.test(use->_idx)) {
+          // The use node was already visited, i.e. is higher up in the schedule.
+          // The "out" edge thus points backward, i.e. it is violated.
+          const VPointer& vp1 = vtn->vpointer();
+          const VPointer& vp2 = use->vpointer();
+#ifdef ASSERT
+          if (_trace._speculative_aliasing_analysis || _trace._speculative_runtime_checks) {
+            tty->print_cr("\nViolated Weak Edge:");
+            vtn->print();
+            vp1.print_on(tty);
+            use->print();
+            vp2.print_on(tty);
+          }
+#endif
+
+          // We could generate checks for the pair (vp1, vp2) directly. But in
+          // some graphs, this generates quadratically many checks. Example:
+          //
+          //   set1: a[i+0] a[i+1] a[i+2] a[i+3]
+          //   set2: b[i+0] b[i+1] b[i+2] b[i+3]
+          //
+          // We may have a weak memory edge between every memory access from
+          // set1 to every memory access from set2. In this example, this would
+          // be 4 * 4 = 16 checks. But instead, we can create a union VPointer
+          // for set1 and set2 each, and only create a single check.
+          //
+          //   set1: a[i+0, size = 4]
+          //   set1: b[i+0, size = 4]
+          //
+          // For this, we add all pairs to an array, and process it below.
+          weak_aliasing_pairs.push(VPointerWeakAliasingPair::make(vp1, vp2));
+        }
+      }
+      visited.set(vtn->_idx);
+    }
+
+    // Sort so that all pairs with the same summands (summands1, summands2)
+    // are consecutive, i.e. in the same group. This allows us to do a linear
+    // walk over all pairs of a group and create the union VPointers.
+    weak_aliasing_pairs.sort(VPointerWeakAliasingPair::cmp_for_sort);
+
+    int group_start = 0;
+    while (group_start < weak_aliasing_pairs.length()) {
+      // New group: pick the first pair as the reference.
+      const VPointer* vp1 = &weak_aliasing_pairs.at(group_start).vp1();
+      const VPointer* vp2 = &weak_aliasing_pairs.at(group_start).vp2();
+      jint size1 = vp1->size();
+      jint size2 = vp2->size();
+      int group_end = group_start + 1;
+      while (group_end < weak_aliasing_pairs.length()) {
+        const VPointer* vp1_next = &weak_aliasing_pairs.at(group_end).vp1();
+        const VPointer* vp2_next = &weak_aliasing_pairs.at(group_end).vp2();
+        jint size1_next = vp1_next->size();
+        jint size2_next = vp2_next->size();
+
+        // Different summands -> different group.
+        if (VPointer::cmp_summands(*vp1, *vp1_next) != 0) { break; }
+        if (VPointer::cmp_summands(*vp2, *vp2_next) != 0) { break; }
+
+        // Pick the one with the lower con as the reference.
+        if (vp1->con() > vp1_next->con()) {
+          swap(vp1, vp1_next);
+          swap(size1, size1_next);
+        }
+        if (vp2->con() > vp2_next->con()) {
+          swap(vp2, vp2_next);
+          swap(size2, size2_next);
+        }
+
+        // Compute the distance from vp1 to vp1_next + size, to get a size that would include vp1_next.
+        NoOverflowInt new_size1 = NoOverflowInt(vp1_next->con()) + NoOverflowInt(size1_next) - NoOverflowInt(vp1->con());
+        NoOverflowInt new_size2 = NoOverflowInt(vp2_next->con()) + NoOverflowInt(size2_next) - NoOverflowInt(vp2->con());
+        if (new_size1.is_NaN() || new_size2.is_NaN()) { break; /* overflow -> new group */ }
+
+        // The "next" VPointer indeed belong to the group.
+        //
+        // vp1:       |-------------->
+        // vp1_next:            |---------------->
+        // result:    |-------------------------->
+        //
+        // vp1:       |-------------------------->
+        // vp1_next:            |------->
+        // result:    |-------------------------->
+        //
+        size1 = MAX2(size1, new_size1.value());
+        size2 = MAX2(size2, new_size2.value());
+        group_end++;
+      }
+      // Create "union" VPointer that cover all VPointer from the group.
+      const VPointer vp1_union = vp1->make_with_size(size1);
+      const VPointer vp2_union = vp2->make_with_size(size2);
+
+#ifdef ASSERT
+      if (_trace._speculative_aliasing_analysis || _trace._speculative_runtime_checks) {
+        tty->print_cr("\nUnion of %d weak aliasing edges:", group_end - group_start);
+        vp1_union.print_on(tty);
+        vp2_union.print_on(tty);
+      }
+
+      // Verification - union must contain all VPointer of the group.
+      for (int i = group_start; i < group_end; i++) {
+        const VPointer& vp1_i = weak_aliasing_pairs.at(i).vp1();
+        const VPointer& vp2_i = weak_aliasing_pairs.at(i).vp2();
+        assert(vp1_union.con() <= vp1_i.con(), "must start before");
+        assert(vp2_union.con() <= vp2_i.con(), "must start before");
+        assert(vp1_union.size() >= vp1_i.size(), "must end after");
+        assert(vp2_union.size() >= vp2_i.size(), "must end after");
+      }
+#endif
+
+      add_speculative_check([&] (Node* ctrl) {
+        return vp1_union.make_speculative_aliasing_check_with(vp2_union, ctrl);
+      });
+
+      group_start = group_end;
+    }
+  }
+}
+
+// Runtime Checks:
+//   Some required properties cannot be proven statically, and require a
+//   runtime check:
+//   - Alignment:
+//       See VTransform::add_speculative_alignment_check
+//   - Aliasing:
+//       See VTransform::apply_speculative_aliasing_runtime_checks
+//   There is a two staged approach for compilation:
+//   - AutoVectorization Predicate:
+//       See VM flag UseAutoVectorizationPredicate and documentation in predicates.hpp
+//       We speculate that the checks pass, and only compile a vectorized  loop.
+//       We expect the checks to pass in almost all cases, and so we only need
+//       to compile and cache the vectorized loop.
+//       If the predicate ever fails, we deoptimize, and eventually compile
+//       without predicate. This means we will recompile with multiversioning.
+//    - Multiversioning:
+//       See VM Flag LoopMultiversioning and documentaiton in loopUnswitch.cpp
+//       If the predicate is not available or previously failed, then we compile
+//       a vectorized and a scalar loop. If the runtime check passes we take the
+//       vectorized loop, else the scalar loop.
+//       Multiversioning takes more compile time and code cache, but it also
+//       produces fast code for when the runtime check passes (vectorized) and
+//       when it fails (scalar performance).
+//
+// Callback:
+//   In some cases, we require the ctrl just before the check iff_speculate to
+//   generate the values required in the check. We pass this ctrl into the
+//   callback, which is expected to produce the check, i.e. a BoolNode.
+template<typename Callback>
+void VTransform::add_speculative_check(Callback callback) {
   assert(_vloop.are_speculative_checks_possible(), "otherwise we cannot make speculative assumptions");
   ParsePredicateSuccessProj* parse_predicate_proj = _vloop.auto_vectorization_parse_predicate_proj();
   IfTrueNode* new_check_proj = nullptr;
@@ -228,6 +437,10 @@ void VTransform::add_speculative_check(BoolNode* bol) {
     new_check_proj = phase()->create_new_if_for_multiversion(_vloop.multiversioning_fast_proj());
   }
   Node* iff_speculate = new_check_proj->in(0);
+
+  // Create the check, given the ctrl just before the iff.
+  BoolNode* bol = callback(iff_speculate->in(0));
+
   igvn().replace_input_of(iff_speculate, 1, bol);
   TRACE_SPECULATIVE_ALIGNMENT_CHECK(iff_speculate);
 }
@@ -426,7 +639,7 @@ bool VTransformGraph::has_store_to_load_forwarding_failure(const VLoopAnalyzer&
     for (int i = 0; i < _schedule.length(); i++) {
       VTransformNode* vtn = _schedule.at(i);
       if (vtn->is_load_or_store_in_loop()) {
-        const VPointer& p = vtn->vpointer(vloop_analyzer);
+        const VPointer& p = vtn->vpointer();
         if (p.is_valid()) {
           VTransformVectorNode* vector = vtn->isa_Vector();
           bool is_load = vtn->is_load_in_loop();
@@ -493,172 +706,173 @@ bool VTransformGraph::has_store_to_load_forwarding_failure(const VLoopAnalyzer&
   return false;
 }
 
-Node* VTransformNode::find_transformed_input(int i, const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
-  Node* n = vnode_idx_to_transformed_node.at(in(i)->_idx);
-  assert(n != nullptr, "must find input IR node");
+void VTransformApplyState::set_transformed_node(VTransformNode* vtn, Node* n) {
+  assert(_vtnode_idx_to_transformed_node.at(vtn->_idx) == nullptr, "only set once");
+  _vtnode_idx_to_transformed_node.at_put(vtn->_idx, n);
+}
+
+Node* VTransformApplyState::transformed_node(const VTransformNode* vtn) const {
+  Node* n = _vtnode_idx_to_transformed_node.at(vtn->_idx);
+  assert(n != nullptr, "must find IR node for vtnode");
   return n;
 }
 
-VTransformApplyResult VTransformScalarNode::apply(const VLoopAnalyzer& vloop_analyzer,
-                                                  const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
-  // This was just wrapped. Now we simply unwap without touching the inputs.
+VTransformApplyResult VTransformMemopScalarNode::apply(VTransformApplyState& apply_state) const {
+  // This was just wrapped. Now we simply unwrap without touching the inputs.
   return VTransformApplyResult::make_scalar(_node);
 }
 
-VTransformApplyResult VTransformReplicateNode::apply(const VLoopAnalyzer& vloop_analyzer,
-                                                     const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
-  Node* val = find_transformed_input(1, vnode_idx_to_transformed_node);
-  VectorNode* vn = VectorNode::scalar2vector(val, _vlen, _element_type);
-  register_new_node_from_vectorization(vloop_analyzer, vn, val);
-  return VTransformApplyResult::make_vector(vn, _vlen, vn->length_in_bytes());
+VTransformApplyResult VTransformDataScalarNode::apply(VTransformApplyState& apply_state) const {
+  // This was just wrapped. Now we simply unwrap without touching the inputs.
+  return VTransformApplyResult::make_scalar(_node);
 }
 
-VTransformApplyResult VTransformConvI2LNode::apply(const VLoopAnalyzer& vloop_analyzer,
-                                                   const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
-  Node* val = find_transformed_input(1, vnode_idx_to_transformed_node);
+VTransformApplyResult VTransformLoopPhiNode::apply(VTransformApplyState& apply_state) const {
+  // This was just wrapped. Now we simply unwrap without touching the inputs.
+  return VTransformApplyResult::make_scalar(_node);
+}
+
+VTransformApplyResult VTransformCFGNode::apply(VTransformApplyState& apply_state) const {
+  // This was just wrapped. Now we simply unwrap without touching the inputs.
+  return VTransformApplyResult::make_scalar(_node);
+}
+
+VTransformApplyResult VTransformOuterNode::apply(VTransformApplyState& apply_state) const {
+  // This was just wrapped. Now we simply unwrap without touching the inputs.
+  return VTransformApplyResult::make_scalar(_node);
+}
+
+VTransformApplyResult VTransformReplicateNode::apply(VTransformApplyState& apply_state) const {
+  Node* val = apply_state.transformed_node(in_req(1));
+  VectorNode* vn = VectorNode::scalar2vector(val, _vlen, _element_type);
+  register_new_node_from_vectorization(apply_state, vn);
+  return VTransformApplyResult::make_vector(vn);
+}
+
+VTransformApplyResult VTransformConvI2LNode::apply(VTransformApplyState& apply_state) const {
+  Node* val = apply_state.transformed_node(in_req(1));
   Node* n = new ConvI2LNode(val);
-  register_new_node_from_vectorization(vloop_analyzer, n, val);
+  register_new_node_from_vectorization(apply_state, n);
   return VTransformApplyResult::make_scalar(n);
 }
 
-VTransformApplyResult VTransformShiftCountNode::apply(const VLoopAnalyzer& vloop_analyzer,
-                                                      const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
-  PhaseIdealLoop* phase = vloop_analyzer.vloop().phase();
-  Node* shift_count_in = find_transformed_input(1, vnode_idx_to_transformed_node);
+VTransformApplyResult VTransformShiftCountNode::apply(VTransformApplyState& apply_state) const {
+  PhaseIdealLoop* phase = apply_state.phase();
+  Node* shift_count_in = apply_state.transformed_node(in_req(1));
   assert(shift_count_in->bottom_type()->isa_int(), "int type only for shift count");
   // The shift_count_in would be automatically truncated to the lowest _mask
   // bits in a scalar shift operation. But vector shift does not truncate, so
   // we must apply the mask now.
-  Node* shift_count_masked = new AndINode(shift_count_in, phase->igvn().intcon(_mask));
-  register_new_node_from_vectorization(vloop_analyzer, shift_count_masked, shift_count_in);
+  Node* shift_count_masked = new AndINode(shift_count_in, phase->intcon(_mask));
+  register_new_node_from_vectorization(apply_state, shift_count_masked);
   // Now that masked value is "boadcast" (some platforms only set the lowest element).
   VectorNode* vn = VectorNode::shift_count(_shift_opcode, shift_count_masked, _vlen, _element_bt);
-  register_new_node_from_vectorization(vloop_analyzer, vn, shift_count_in);
-  return VTransformApplyResult::make_vector(vn, _vlen, vn->length_in_bytes());
+  register_new_node_from_vectorization(apply_state, vn);
+  return VTransformApplyResult::make_vector(vn);
 }
 
 
-VTransformApplyResult VTransformPopulateIndexNode::apply(const VLoopAnalyzer& vloop_analyzer,
-                                                         const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
-  PhaseIdealLoop* phase = vloop_analyzer.vloop().phase();
-  Node* val = find_transformed_input(1, vnode_idx_to_transformed_node);
+VTransformApplyResult VTransformPopulateIndexNode::apply(VTransformApplyState& apply_state) const {
+  PhaseIdealLoop* phase = apply_state.phase();
+  Node* val = apply_state.transformed_node(in_req(1));
   assert(val->is_Phi(), "expected to be iv");
   assert(VectorNode::is_populate_index_supported(_element_bt), "should support");
   const TypeVect* vt = TypeVect::make(_element_bt, _vlen);
-  VectorNode* vn = new PopulateIndexNode(val, phase->igvn().intcon(1), vt);
-  register_new_node_from_vectorization(vloop_analyzer, vn, val);
-  return VTransformApplyResult::make_vector(vn, _vlen, vn->length_in_bytes());
+  VectorNode* vn = new PopulateIndexNode(val, phase->intcon(1), vt);
+  register_new_node_from_vectorization(apply_state, vn);
+  return VTransformApplyResult::make_vector(vn);
 }
 
-VTransformApplyResult VTransformElementWiseVectorNode::apply(const VLoopAnalyzer& vloop_analyzer,
-                                                             const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
-  Node* first = nodes().at(0);
-  uint  vlen = nodes().length();
-  int   opc  = first->Opcode();
-  BasicType bt = vloop_analyzer.types().velt_basic_type(first);
-
-  if (first->is_Cmp()) {
-    // Cmp + Bool -> VectorMaskCmp
-    // Handled by Bool / VTransformBoolVectorNode, so we do not generate any nodes here.
-    return VTransformApplyResult::make_empty();
-  }
-
+VTransformApplyResult VTransformElementWiseVectorNode::apply(VTransformApplyState& apply_state) const {
   assert(2 <= req() && req() <= 4, "Must have 1-3 inputs");
-  VectorNode* vn = nullptr;
-  Node* in1 =                find_transformed_input(1, vnode_idx_to_transformed_node);
-  Node* in2 = (req() >= 3) ? find_transformed_input(2, vnode_idx_to_transformed_node) : nullptr;
-  Node* in3 = (req() >= 4) ? find_transformed_input(3, vnode_idx_to_transformed_node) : nullptr;
+  const TypeVect* vt = TypeVect::make(element_basic_type(), vector_length());
+  Node* in1 =                apply_state.transformed_node(in_req(1));
+  Node* in2 = (req() >= 3) ? apply_state.transformed_node(in_req(2)) : nullptr;
 
-  if (first->is_CMove()) {
-    assert(req() == 4, "three inputs expected: mask, blend1, blend2");
-    vn = new VectorBlendNode(/* blend1 */ in2, /* blend2 */ in3, /* mask */ in1);
-  } else if (VectorNode::is_convert_opcode(opc)) {
-    assert(first->req() == 2 && req() == 2, "only one input expected");
-    int vopc = VectorCastNode::opcode(opc, in1->bottom_type()->is_vect()->element_basic_type());
-    vn = VectorCastNode::make(vopc, in1, bt, vlen);
-  } else if (VectorNode::can_use_RShiftI_instead_of_URShiftI(first, bt)) {
-    opc = Op_RShiftI;
-    vn = VectorNode::make(opc, in1, in2, vlen, bt);
-  } else if (VectorNode::is_scalar_op_that_returns_int_but_vector_op_returns_long(opc)) {
-    // The scalar operation was a long -> int operation.
-    // However, the vector operation is long -> long.
-    VectorNode* long_vn = VectorNode::make(opc, in1, nullptr, vlen, T_LONG);
-    register_new_node_from_vectorization(vloop_analyzer, long_vn, first);
-    // Cast long -> int, to mimic the scalar long -> int operation.
-    vn = VectorCastNode::make(Op_VectorCastL2X, long_vn, T_INT, vlen);
-  } else if (req() == 3 ||
-             VectorNode::is_scalar_unary_op_with_equal_input_and_output_types(opc)) {
-    assert(!VectorNode::is_roundopD(first) || in2->is_Con(), "rounding mode must be constant");
-    vn = VectorNode::make(opc, in1, in2, vlen, bt); // unary and binary
+  VectorNode* vn = nullptr;
+  if (req() <= 3) {
+    vn = VectorNode::make(_vector_opcode, in1, in2, vt); // unary and binary
   } else {
-    assert(req() == 4, "three inputs expected");
-    assert(opc == Op_FmaD ||
-           opc == Op_FmaF ||
-           opc == Op_SignumF ||
-           opc == Op_SignumD,
-           "element wise operation must be from this list");
-    vn = VectorNode::make(opc, in1, in2, in3, vlen, bt); // ternary
+    Node* in3 = apply_state.transformed_node(in_req(3));
+    vn = VectorNode::make(_vector_opcode, in1, in2, in3, vt); // ternary
   }
 
-  register_new_node_from_vectorization_and_replace_scalar_nodes(vloop_analyzer, vn);
-  return VTransformApplyResult::make_vector(vn, vlen, vn->length_in_bytes());
+  register_new_node_from_vectorization_and_replace_scalar_nodes(apply_state, vn);
+  return VTransformApplyResult::make_vector(vn);
 }
 
-VTransformApplyResult VTransformBoolVectorNode::apply(const VLoopAnalyzer& vloop_analyzer,
-                                                      const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
-  BoolNode* first = nodes().at(0)->as_Bool();
-  uint  vlen = nodes().length();
-  BasicType bt = vloop_analyzer.types().velt_basic_type(first);
+VTransformApplyResult VTransformElementWiseLongOpWithCastToIntVectorNode::apply(VTransformApplyState& apply_state) const {
+  uint vlen = vector_length();
+  int sopc  = scalar_opcode();
+  Node* in1 = apply_state.transformed_node(in_req(1));
+
+  // The scalar operation was a long -> int operation.
+  // However, the vector operation is long -> long.
+  VectorNode* long_vn = VectorNode::make(sopc, in1, nullptr, vlen, T_LONG);
+  register_new_node_from_vectorization(apply_state, long_vn);
+  // Cast long -> int, to mimic the scalar long -> int operation.
+  VectorNode* vn = VectorCastNode::make(Op_VectorCastL2X, long_vn, T_INT, vlen);
+  register_new_node_from_vectorization_and_replace_scalar_nodes(apply_state, vn);
+  return VTransformApplyResult::make_vector(vn);
+}
+
+VTransformApplyResult VTransformReinterpretVectorNode::apply(VTransformApplyState& apply_state) const {
+  const TypeVect* dst_vt = TypeVect::make(element_basic_type(), vector_length());
+  const TypeVect* src_vt = TypeVect::make(_src_bt,              vector_length());
+  assert(VectorNode::is_reinterpret_opcode(scalar_opcode()), "scalar opcode must be reinterpret");
+
+  Node* in1 = apply_state.transformed_node(in_req(1));
+  VectorNode* vn = new VectorReinterpretNode(in1, src_vt, dst_vt);
+
+  register_new_node_from_vectorization_and_replace_scalar_nodes(apply_state, vn);
+  return VTransformApplyResult::make_vector(vn);
+}
+
+VTransformApplyResult VTransformBoolVectorNode::apply(VTransformApplyState& apply_state) const {
+  const TypeVect* vt = TypeVect::make(element_basic_type(), vector_length());
+  assert(scalar_opcode() == Op_Bool, "");
 
   // Cmp + Bool -> VectorMaskCmp
-  VTransformElementWiseVectorNode* vtn_cmp = in(1)->isa_ElementWiseVector();
-  assert(vtn_cmp != nullptr && vtn_cmp->nodes().at(0)->is_Cmp(),
-         "bool vtn expects cmp vtn as input");
+  VTransformCmpVectorNode* vtn_cmp = in_req(1)->isa_CmpVector();
+  assert(vtn_cmp != nullptr, "bool vtn expects cmp vtn as input");
 
-  Node* cmp_in1 = vtn_cmp->find_transformed_input(1, vnode_idx_to_transformed_node);
-  Node* cmp_in2 = vtn_cmp->find_transformed_input(2, vnode_idx_to_transformed_node);
+  Node* cmp_in1 = apply_state.transformed_node(vtn_cmp->in_req(1));
+  Node* cmp_in2 = apply_state.transformed_node(vtn_cmp->in_req(2));
   BoolTest::mask mask = test()._mask;
 
-  PhaseIdealLoop* phase = vloop_analyzer.vloop().phase();
-  ConINode* mask_node  = phase->igvn().intcon((int)mask);
-  const TypeVect* vt = TypeVect::make(bt, vlen);
+  PhaseIdealLoop* phase = apply_state.phase();
+  ConINode* mask_node  = phase->intcon((int)mask);
   VectorNode* vn = new VectorMaskCmpNode(mask, cmp_in1, cmp_in2, mask_node, vt);
-  register_new_node_from_vectorization_and_replace_scalar_nodes(vloop_analyzer, vn);
-  return VTransformApplyResult::make_vector(vn, vlen, vn->vect_type()->length_in_bytes());
+  register_new_node_from_vectorization_and_replace_scalar_nodes(apply_state, vn);
+  return VTransformApplyResult::make_vector(vn);
 }
 
-VTransformApplyResult VTransformReductionVectorNode::apply(const VLoopAnalyzer& vloop_analyzer,
-                                                           const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
-  Node* first = nodes().at(0);
-  uint  vlen = nodes().length();
-  int   opc  = first->Opcode();
-  BasicType bt = first->bottom_type()->basic_type();
+VTransformApplyResult VTransformReductionVectorNode::apply(VTransformApplyState& apply_state) const {
+  Node* init = apply_state.transformed_node(in_req(1));
+  Node* vec  = apply_state.transformed_node(in_req(2));
 
-  Node* init = find_transformed_input(1, vnode_idx_to_transformed_node);
-  Node* vec  = find_transformed_input(2, vnode_idx_to_transformed_node);
-
-  ReductionNode* vn = ReductionNode::make(opc, nullptr, init, vec, bt);
-  register_new_node_from_vectorization_and_replace_scalar_nodes(vloop_analyzer, vn);
-  return VTransformApplyResult::make_vector(vn, vlen, vn->vect_type()->length_in_bytes());
+  ReductionNode* vn = ReductionNode::make(scalar_opcode(), nullptr, init, vec, element_basic_type());
+  register_new_node_from_vectorization_and_replace_scalar_nodes(apply_state, vn);
+  return VTransformApplyResult::make_vector(vn, vn->vect_type());
 }
 
-VTransformApplyResult VTransformLoadVectorNode::apply(const VLoopAnalyzer& vloop_analyzer,
-                                                      const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
+VTransformApplyResult VTransformLoadVectorNode::apply(VTransformApplyState& apply_state) const {
+  int sopc     = scalar_opcode();
+  uint vlen    = vector_length();
+  BasicType bt = element_basic_type();
+
   LoadNode* first = nodes().at(0)->as_Load();
-  uint  vlen = nodes().length();
-  Node* ctrl = first->in(MemNode::Control);
+  Node* ctrl = apply_state.transformed_node(in_req(MemNode::Control));
+  // first has the correct memory state, determined by VTransformGraph::apply_memops_reordering_with_schedule
   Node* mem  = first->in(MemNode::Memory);
-  Node* adr  = first->in(MemNode::Address);
-  int   opc  = first->Opcode();
-  const TypePtr* adr_type = first->adr_type();
-  BasicType bt = vloop_analyzer.types().velt_basic_type(first);
+  Node* adr  = apply_state.transformed_node(in_req(MemNode::Address));
 
   // Set the memory dependency of the LoadVector as early as possible.
   // Walk up the memory chain, and ignore any StoreVector that provably
   // does not have any memory dependency.
-  const VPointer& load_p = vpointer(vloop_analyzer);
+  const VPointer& load_p = vpointer();
   while (mem->is_StoreVector()) {
-    VPointer store_p(mem->as_Mem(), vloop_analyzer.vloop());
+    VPointer store_p(mem->as_Mem(), apply_state.vloop());
     if (store_p.never_overlaps_with(load_p)) {
       mem = mem->in(MemNode::Memory);
     } else {
@@ -666,35 +880,33 @@ VTransformApplyResult VTransformLoadVectorNode::apply(const VLoopAnalyzer& vloop
     }
   }
 
-  LoadVectorNode* vn = LoadVectorNode::make(opc, ctrl, mem, adr, adr_type, vlen, bt,
+  LoadVectorNode* vn = LoadVectorNode::make(sopc, ctrl, mem, adr, _adr_type, vlen, bt,
                                             control_dependency());
   DEBUG_ONLY( if (VerifyAlignVector) { vn->set_must_verify_alignment(); } )
-  register_new_node_from_vectorization_and_replace_scalar_nodes(vloop_analyzer, vn);
-  return VTransformApplyResult::make_vector(vn, vlen, vn->memory_size());
+  register_new_node_from_vectorization_and_replace_scalar_nodes(apply_state, vn);
+  return VTransformApplyResult::make_vector(vn, vn->vect_type());
 }
 
-VTransformApplyResult VTransformStoreVectorNode::apply(const VLoopAnalyzer& vloop_analyzer,
-                                                       const GrowableArray<Node*>& vnode_idx_to_transformed_node) const {
+VTransformApplyResult VTransformStoreVectorNode::apply(VTransformApplyState& apply_state) const {
+  int sopc  = scalar_opcode();
+  uint vlen = vector_length();
+
   StoreNode* first = nodes().at(0)->as_Store();
-  uint  vlen = nodes().length();
-  Node* ctrl = first->in(MemNode::Control);
+  Node* ctrl = apply_state.transformed_node(in_req(MemNode::Control));
+  // first has the correct memory state, determined by VTransformGraph::apply_memops_reordering_with_schedule
   Node* mem  = first->in(MemNode::Memory);
-  Node* adr  = first->in(MemNode::Address);
-  int   opc  = first->Opcode();
-  const TypePtr* adr_type = first->adr_type();
+  Node* adr  = apply_state.transformed_node(in_req(MemNode::Address));
 
-  Node* value = find_transformed_input(MemNode::ValueIn, vnode_idx_to_transformed_node);
-  StoreVectorNode* vn = StoreVectorNode::make(opc, ctrl, mem, adr, adr_type, value, vlen);
+  Node* value = apply_state.transformed_node(in_req(MemNode::ValueIn));
+  StoreVectorNode* vn = StoreVectorNode::make(sopc, ctrl, mem, adr, _adr_type, value, vlen);
   DEBUG_ONLY( if (VerifyAlignVector) { vn->set_must_verify_alignment(); } )
-  register_new_node_from_vectorization_and_replace_scalar_nodes(vloop_analyzer, vn);
-  return VTransformApplyResult::make_vector(vn, vlen, vn->memory_size());
+  register_new_node_from_vectorization_and_replace_scalar_nodes(apply_state, vn);
+  return VTransformApplyResult::make_vector(vn, vn->vect_type());
 }
 
-void VTransformVectorNode::register_new_node_from_vectorization_and_replace_scalar_nodes(const VLoopAnalyzer& vloop_analyzer, Node* vn) const {
-  PhaseIdealLoop* phase = vloop_analyzer.vloop().phase();
-  Node* first = nodes().at(0);
-
-  register_new_node_from_vectorization(vloop_analyzer, vn, first);
+void VTransformVectorNode::register_new_node_from_vectorization_and_replace_scalar_nodes(VTransformApplyState& apply_state, Node* vn) const {
+  PhaseIdealLoop* phase = apply_state.phase();
+  register_new_node_from_vectorization(apply_state, vn);
 
   for (int i = 0; i < _nodes.length(); i++) {
     Node* n = _nodes.at(i);
@@ -702,9 +914,11 @@ void VTransformVectorNode::register_new_node_from_vectorization_and_replace_scal
   }
 }
 
-void VTransformNode::register_new_node_from_vectorization(const VLoopAnalyzer& vloop_analyzer, Node* vn, Node* old_node) const {
-  PhaseIdealLoop* phase = vloop_analyzer.vloop().phase();
-  phase->register_new_node_with_ctrl_of(vn, old_node);
+void VTransformNode::register_new_node_from_vectorization(VTransformApplyState& apply_state, Node* vn) const {
+  PhaseIdealLoop* phase = apply_state.phase();
+  // Using the cl is sometimes not the most accurate, but still correct. We do not have to be
+  // perfectly accurate, because we will set major_progress anyway.
+  phase->register_new_node(vn, apply_state.vloop().cl());
   phase->igvn()._worklist.push(vn);
   VectorNode::trace_new_vector(vn, "AutoVectorization");
 }
@@ -745,15 +959,27 @@ void VTransformNode::print() const {
     print_node_idx(_in.at(i));
   }
   if ((uint)_in.length() > _req) {
-    tty->print(" |");
-    for (int i = _req; i < _in.length(); i++) {
+    tty->print(" | strong:");
+    for (uint i = _req; i < _in_end_strong_memory_edges; i++) {
+      print_node_idx(_in.at(i));
+    }
+  }
+  if ((uint)_in.length() > _in_end_strong_memory_edges) {
+    tty->print(" | weak:");
+    for (uint i = _in_end_strong_memory_edges; i < (uint)_in.length(); i++) {
       print_node_idx(_in.at(i));
     }
   }
   tty->print(") [");
-  for (int i = 0; i < _out.length(); i++) {
+  for (uint i = 0; i < _out_end_strong_edges; i++) {
     print_node_idx(_out.at(i));
   }
+  if ((uint)_out.length() > _out_end_strong_edges) {
+    tty->print(" | weak:");
+    for (uint i = _out_end_strong_edges; i < (uint)_out.length(); i++) {
+      print_node_idx(_out.at(i));
+    }
+  }
   tty->print("] ");
   print_spec();
   tty->cr();
@@ -767,7 +993,24 @@ void VTransformNode::print_node_idx(const VTransformNode* vtn) {
   }
 }
 
-void VTransformScalarNode::print_spec() const {
+void VTransformMemopScalarNode::print_spec() const {
+  tty->print("node[%d %s] ", _node->_idx, _node->Name());
+  _vpointer.print_on(tty, false);
+}
+
+void VTransformDataScalarNode::print_spec() const {
+  tty->print("node[%d %s]", _node->_idx, _node->Name());
+}
+
+void VTransformLoopPhiNode::print_spec() const {
+  tty->print("node[%d %s]", _node->_idx, _node->Name());
+}
+
+void VTransformCFGNode::print_spec() const {
+  tty->print("node[%d %s]", _node->_idx, _node->Name());
+}
+
+void VTransformOuterNode::print_spec() const {
   tty->print("node[%d %s]", _node->_idx, _node->Name());
 }
 
@@ -786,14 +1029,32 @@ void VTransformPopulateIndexNode::print_spec() const {
 }
 
 void VTransformVectorNode::print_spec() const {
-  tty->print("%d-pack[", _nodes.length());
-  for (int i = 0; i < _nodes.length(); i++) {
-    Node* n = _nodes.at(i);
-    if (i > 0) {
-      tty->print(", ");
-    }
-    tty->print("%d %s", n->_idx, n->Name());
+  tty->print("Properties[orig=[%d %s] sopc=%s vlen=%d element_bt=%s]",
+             approximate_origin()->_idx,
+             approximate_origin()->Name(),
+             NodeClassNames[scalar_opcode()],
+             vector_length(),
+             type2name(element_basic_type()));
+  if (is_load_or_store_in_loop()) {
+    tty->print(" ");
+    vpointer().print_on(tty, false);
   }
-  tty->print("]");
+}
+
+void VTransformElementWiseVectorNode::print_spec() const {
+  VTransformVectorNode::print_spec();
+  tty->print(" vopc=%s", NodeClassNames[_vector_opcode]);
+}
+
+void VTransformReinterpretVectorNode::print_spec() const {
+  VTransformVectorNode::print_spec();
+  tty->print(" src_bt=%s", type2name(_src_bt));
+}
+
+void VTransformBoolVectorNode::print_spec() const {
+  VTransformVectorNode::print_spec();
+  const BoolTest bt(_test._mask);
+  tty->print(" test=");
+  bt.dump_on(tty);
 }
 #endif
