@@ -29,7 +29,7 @@
 #include "memory/resourceArea.hpp"
 #include "nmt/memTag.hpp"
 #include "oops/oop.inline.hpp"
-#include "runtime/atomic.hpp"
+#include "runtime/atomicAccess.hpp"
 #include "runtime/basicLock.inline.hpp"
 #include "runtime/globals_extension.hpp"
 #include "runtime/interfaceSupport.inline.hpp"
@@ -115,15 +115,16 @@ class ObjectMonitorTable : AllStatic {
   };
 
   static void inc_items_count() {
-    Atomic::inc(&_items_count);
+    AtomicAccess::inc(&_items_count, memory_order_relaxed);
   }
 
   static void dec_items_count() {
-    Atomic::dec(&_items_count);
+    AtomicAccess::dec(&_items_count, memory_order_relaxed);
   }
 
   static double get_load_factor() {
-    return (double)_items_count / (double)_table_size;
+    size_t count = AtomicAccess::load(&_items_count);
+    return (double)count / (double)_table_size;
   }
 
   static size_t table_size(Thread* current = Thread::current()) {
@@ -194,8 +195,8 @@ class ObjectMonitorTable : AllStatic {
   }
 
   static void try_notify_grow() {
-    if (!_table->is_max_size_reached() && !Atomic::load(&_resize)) {
-      Atomic::store(&_resize, true);
+    if (!_table->is_max_size_reached() && !AtomicAccess::load(&_resize)) {
+      AtomicAccess::store(&_resize, true);
       if (Service_lock->try_lock()) {
         Service_lock->notify();
         Service_lock->unlock();
@@ -215,7 +216,7 @@ class ObjectMonitorTable : AllStatic {
   }
 
   static bool should_resize() {
-    return should_grow() || should_shrink() || Atomic::load(&_resize);
+    return should_grow() || should_shrink() || AtomicAccess::load(&_resize);
   }
 
   template<typename Task, typename... Args>
@@ -264,14 +265,14 @@ class ObjectMonitorTable : AllStatic {
       lt.print("Start growing with load factor %f", get_load_factor());
       success = grow(current);
     } else {
-      if (!_table->is_max_size_reached() && Atomic::load(&_resize)) {
+      if (!_table->is_max_size_reached() && AtomicAccess::load(&_resize)) {
         lt.print("WARNING: Getting resize hints with load factor %f", get_load_factor());
       }
       lt.print("Start cleaning with load factor %f", get_load_factor());
       success = clean(current);
     }
 
-    Atomic::store(&_resize, false);
+    AtomicAccess::store(&_resize, false);
 
     return success;
   }
@@ -332,8 +333,6 @@ size_t ObjectMonitorTable::_table_size = 0;
 volatile bool ObjectMonitorTable::_resize = false;
 
 ObjectMonitor* LightweightSynchronizer::get_or_insert_monitor_from_table(oop object, JavaThread* current, bool* inserted) {
-  assert(LockingMode == LM_LIGHTWEIGHT, "must be");
-
   ObjectMonitor* monitor = get_monitor_from_table(current, object);
   if (monitor != nullptr) {
     *inserted = false;
@@ -518,8 +517,12 @@ class LightweightSynchronizer::CacheSetter : StackObj {
     // Only use the cache if using the table.
     if (UseObjectMonitorTable) {
       if (_monitor != nullptr) {
-        _thread->om_set_monitor_cache(_monitor);
-        _lock->set_object_monitor_cache(_monitor);
+        // If the monitor is already in the BasicLock cache then it is most
+        // likely in the thread cache, do not set it again to avoid reordering.
+        if (_monitor != _lock->object_monitor_cache()) {
+          _thread->om_set_monitor_cache(_monitor);
+          _lock->set_object_monitor_cache(_monitor);
+        }
       } else {
         _lock->clear_object_monitor_cache();
       }
@@ -533,6 +536,16 @@ class LightweightSynchronizer::CacheSetter : StackObj {
 
 };
 
+// Reads first from the BasicLock cache then from the OMCache in the current thread.
+// C2 fast-path may have put the monitor in the cache in the BasicLock.
+inline static ObjectMonitor* read_caches(JavaThread* current, BasicLock* lock, oop object) {
+  ObjectMonitor* monitor = lock->object_monitor_cache();
+  if (monitor == nullptr) {
+    monitor = current->om_get_from_monitor_cache(object);
+  }
+  return monitor;
+}
+
 class LightweightSynchronizer::VerifyThreadState {
   bool _no_safepoint;
 
@@ -613,7 +626,7 @@ bool LightweightSynchronizer::fast_lock_spin_enter(oop obj, LockStack& lock_stac
 }
 
 void LightweightSynchronizer::enter_for(Handle obj, BasicLock* lock, JavaThread* locking_thread) {
-  assert(LockingMode == LM_LIGHTWEIGHT, "must be");
+  assert(!UseObjectMonitorTable || lock->object_monitor_cache() == nullptr, "must be cleared");
   JavaThread* current = JavaThread::current();
   VerifyThreadState vts(locking_thread, current);
 
@@ -621,8 +634,6 @@ void LightweightSynchronizer::enter_for(Handle obj, BasicLock* lock, JavaThread*
     ObjectSynchronizer::handle_sync_on_value_based_class(obj, locking_thread);
   }
 
-  CacheSetter cache_setter(locking_thread, lock);
-
   LockStack& lock_stack = locking_thread->lock_stack();
 
   ObjectMonitor* monitor = nullptr;
@@ -633,17 +644,16 @@ void LightweightSynchronizer::enter_for(Handle obj, BasicLock* lock, JavaThread*
   } else {
     do {
       // It is assumed that enter_for must enter on an object without contention.
-      monitor = inflate_and_enter(obj(), ObjectSynchronizer::inflate_cause_monitor_enter, locking_thread, current);
+      monitor = inflate_and_enter(obj(), lock, ObjectSynchronizer::inflate_cause_monitor_enter, locking_thread, current);
       // But there may still be a race with deflation.
     } while (monitor == nullptr);
   }
 
   assert(monitor != nullptr, "LightweightSynchronizer::enter_for must succeed");
-  cache_setter.set_monitor(monitor);
+  assert(!UseObjectMonitorTable || lock->object_monitor_cache() == nullptr, "unused. already cleared");
 }
 
 void LightweightSynchronizer::enter(Handle obj, BasicLock* lock, JavaThread* current) {
-  assert(LockingMode == LM_LIGHTWEIGHT, "must be");
   assert(current == JavaThread::current(), "must be");
 
   if (obj->klass()->is_value_based()) {
@@ -689,7 +699,7 @@ void LightweightSynchronizer::enter(Handle obj, BasicLock* lock, JavaThread* cur
       spin_yield.wait();
     }
 
-    ObjectMonitor* monitor = inflate_and_enter(obj(), ObjectSynchronizer::inflate_cause_monitor_enter, current, current);
+    ObjectMonitor* monitor = inflate_and_enter(obj(), lock, ObjectSynchronizer::inflate_cause_monitor_enter, current, current);
     if (monitor != nullptr) {
       cache_setter.set_monitor(monitor);
       return;
@@ -703,8 +713,7 @@ void LightweightSynchronizer::enter(Handle obj, BasicLock* lock, JavaThread* cur
   }
 }
 
-void LightweightSynchronizer::exit(oop object, JavaThread* current) {
-  assert(LockingMode == LM_LIGHTWEIGHT, "must be");
+void LightweightSynchronizer::exit(oop object, BasicLock* lock, JavaThread* current) {
   assert(current == Thread::current(), "must be");
 
   markWord mark = object->mark();
@@ -738,7 +747,15 @@ void LightweightSynchronizer::exit(oop object, JavaThread* current) {
 
   assert(mark.has_monitor(), "must be");
   // The monitor exists
-  ObjectMonitor* monitor = ObjectSynchronizer::read_monitor(current, object, mark);
+  ObjectMonitor* monitor;
+  if (UseObjectMonitorTable) {
+    monitor = read_caches(current, lock, object);
+    if (monitor == nullptr) {
+      monitor = get_monitor_from_table(current, object);
+    }
+  } else {
+    monitor = ObjectSynchronizer::read_monitor(mark);
+  }
   if (monitor->has_anonymous_owner()) {
     assert(current->lock_stack().contains(object), "current must have object on its lock stack");
     monitor->set_owner_from_anonymous(current);
@@ -748,18 +765,14 @@ void LightweightSynchronizer::exit(oop object, JavaThread* current) {
   monitor->exit(current);
 }
 
-// LightweightSynchronizer::inflate_locked_or_imse is used to to get an inflated
-// ObjectMonitor* with LM_LIGHTWEIGHT. It is used from contexts which require
-// an inflated ObjectMonitor* for a monitor, and expects to throw a
-// java.lang.IllegalMonitorStateException if it is not held by the current
-// thread. Such as notify/wait and jni_exit. LM_LIGHTWEIGHT keeps it invariant
-// that it only inflates if it is already locked by the current thread or the
-// current thread is in the process of entering. To maintain this invariant we
-// need to throw a java.lang.IllegalMonitorStateException before inflating if
-// the current thread is not the owner.
-// LightweightSynchronizer::inflate_locked_or_imse facilitates this.
+// LightweightSynchronizer::inflate_locked_or_imse is used to get an
+// inflated ObjectMonitor* from contexts which require that, such as
+// notify/wait and jni_exit. Lightweight locking keeps the invariant that it
+// only inflates if it is already locked by the current thread or the current
+// thread is in the process of entering. To maintain this invariant we need to
+// throw a java.lang.IllegalMonitorStateException before inflating if the
+// current thread is not the owner.
 ObjectMonitor* LightweightSynchronizer::inflate_locked_or_imse(oop obj, ObjectSynchronizer::InflateCause cause, TRAPS) {
-  assert(LockingMode == LM_LIGHTWEIGHT, "must be");
   JavaThread* current = THREAD;
 
   for (;;) {
@@ -804,12 +817,11 @@ ObjectMonitor* LightweightSynchronizer::inflate_locked_or_imse(oop obj, ObjectSy
 
 ObjectMonitor* LightweightSynchronizer::inflate_into_object_header(oop object, ObjectSynchronizer::InflateCause cause, JavaThread* locking_thread, Thread* current) {
 
-  // The JavaThread* locking_thread parameter is only used by LM_LIGHTWEIGHT and requires
-  // that the locking_thread == Thread::current() or is suspended throughout the call by
-  // some other mechanism.
-  // Even with LM_LIGHTWEIGHT the thread might be nullptr when called from a non
+  // The JavaThread* locking parameter requires that the locking_thread == JavaThread::current,
+  // or is suspended throughout the call by some other mechanism.
+  // Even with lightweight locking the thread might be nullptr when called from a non
   // JavaThread. (As may still be the case from FastHashCode). However it is only
-  // important for the correctness of the LM_LIGHTWEIGHT algorithm that the thread
+  // important for the correctness of the lightweight locking algorithm that the thread
   // is set when called from ObjectSynchronizer::enter from the owning thread,
   // ObjectSynchronizer::enter_for from any thread, or ObjectSynchronizer::exit.
   EventJavaMonitorInflate event;
@@ -921,7 +933,6 @@ ObjectMonitor* LightweightSynchronizer::inflate_into_object_header(oop object, O
 }
 
 ObjectMonitor* LightweightSynchronizer::inflate_fast_locked_object(oop object, ObjectSynchronizer::InflateCause cause, JavaThread* locking_thread, JavaThread* current) {
-  assert(LockingMode == LM_LIGHTWEIGHT, "only used for lightweight");
   VerifyThreadState vts(locking_thread, current);
   assert(locking_thread->lock_stack().contains(object), "locking_thread must have object on its lock stack");
 
@@ -977,8 +988,7 @@ ObjectMonitor* LightweightSynchronizer::inflate_fast_locked_object(oop object, O
   return monitor;
 }
 
-ObjectMonitor* LightweightSynchronizer::inflate_and_enter(oop object, ObjectSynchronizer::InflateCause cause, JavaThread* locking_thread, JavaThread* current) {
-  assert(LockingMode == LM_LIGHTWEIGHT, "only used for lightweight");
+ObjectMonitor* LightweightSynchronizer::inflate_and_enter(oop object, BasicLock* lock, ObjectSynchronizer::InflateCause cause, JavaThread* locking_thread, JavaThread* current) {
   VerifyThreadState vts(locking_thread, current);
 
   // Note: In some paths (deoptimization) the 'current' thread inflates and
@@ -1003,18 +1013,17 @@ ObjectMonitor* LightweightSynchronizer::inflate_and_enter(oop object, ObjectSync
 
   NoSafepointVerifier nsv;
 
-  // Lightweight monitors require that hash codes are installed first
-  ObjectSynchronizer::FastHashCode(locking_thread, object);
-
   // Try to get the monitor from the thread-local cache.
   // There's no need to use the cache if we are locking
   // on behalf of another thread.
   if (current == locking_thread) {
-    monitor = current->om_get_from_monitor_cache(object);
+    monitor = read_caches(current, lock, object);
   }
 
   // Get or create the monitor
   if (monitor == nullptr) {
+    // Lightweight monitors require that hash codes are installed first
+    ObjectSynchronizer::FastHashCode(locking_thread, object);
     monitor = get_or_insert_monitor(object, current, cause);
   }
 
@@ -1030,6 +1039,9 @@ ObjectMonitor* LightweightSynchronizer::inflate_and_enter(oop object, ObjectSync
     // The MonitorDeflation thread is deflating the monitor. The locking thread
     // must spin until further progress has been made.
 
+    // Clear the BasicLock cache as it may contain this monitor.
+    lock->clear_object_monitor_cache();
+
     const markWord mark = object->mark_acquire();
 
     if (mark.has_monitor()) {
@@ -1162,9 +1174,6 @@ bool LightweightSynchronizer::quick_enter(oop obj, BasicLock* lock, JavaThread*
   assert(obj != nullptr, "must be");
   NoSafepointVerifier nsv;
 
-  // If quick_enter succeeds with entering, the cache should be in a valid initialized state.
-  CacheSetter cache_setter(current, lock);
-
   LockStack& lock_stack = current->lock_stack();
   if (lock_stack.is_full()) {
     // Always go into runtime if the lock stack is full.
@@ -1191,17 +1200,28 @@ bool LightweightSynchronizer::quick_enter(oop obj, BasicLock* lock, JavaThread*
 #endif
 
   if (mark.has_monitor()) {
-    ObjectMonitor* const monitor = UseObjectMonitorTable ? current->om_get_from_monitor_cache(obj) :
-                                                           ObjectSynchronizer::read_monitor(mark);
+    ObjectMonitor* monitor;
+    if (UseObjectMonitorTable) {
+      monitor = read_caches(current, lock, obj);
+    } else {
+      monitor = ObjectSynchronizer::read_monitor(mark);
+    }
 
     if (monitor == nullptr) {
       // Take the slow-path on a cache miss.
       return false;
     }
 
-    if (monitor->try_enter(current)) {
-      // ObjectMonitor enter successful.
-      cache_setter.set_monitor(monitor);
+    if (UseObjectMonitorTable) {
+      // Set the monitor regardless of success.
+      // Either we successfully lock on the monitor, or we retry with the
+      // monitor in the slow path. If the monitor gets deflated, it will be
+      // cleared, either by the CacheSetter if we fast lock in enter or in
+      // inflate_and_enter when we see that the monitor is deflated.
+      lock->set_object_monitor_cache(monitor);
+    }
+
+    if (monitor->spin_enter(current)) {
       return true;
     }
   }
