@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2020, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
  *
  */
 
-#include "precompiled.hpp"
 #include "logging/log.hpp"
 #include "runtime/javaThread.hpp"
 #include "runtime/os.inline.hpp"
@@ -81,8 +80,7 @@ void StackOverflow::create_stack_guard_pages() {
   if (!os::uses_stack_guard_pages() ||
       _stack_guard_state != stack_guard_unused ||
       (DisablePrimordialThreadGuardPages && os::is_primordial_thread())) {
-      log_info(os, thread)("Stack guard page creation for thread "
-                           UINTX_FORMAT " disabled", os::current_thread_id());
+      log_info(os, thread)("Stack guard page creation for thread %zu disabled", os::current_thread_id());
     return;
   }
   address low_addr = stack_end();
@@ -91,8 +89,8 @@ void StackOverflow::create_stack_guard_pages() {
   assert(is_aligned(low_addr, os::vm_page_size()), "Stack base should be the start of a page");
   assert(is_aligned(len, os::vm_page_size()), "Stack size should be a multiple of page size");
 
-  int must_commit = os::must_commit_stack_guard_pages();
-  // warning("Guarding at " PTR_FORMAT " for len " SIZE_FORMAT "\n", low_addr, len);
+  bool must_commit = os::must_commit_stack_guard_pages();
+  // warning("Guarding at " PTR_FORMAT " for len %zu\n", low_addr, len);
 
   if (must_commit && !os::create_stack_guard_pages((char *) low_addr, len)) {
     log_warning(os, thread)("Attempt to allocate stack guard pages failed.");
@@ -107,7 +105,7 @@ void StackOverflow::create_stack_guard_pages() {
     vm_exit_out_of_memory(len, OOM_MPROTECT_ERROR, "memory to guard stack pages");
   }
 
-  log_debug(os, thread)("Thread " UINTX_FORMAT " stack guard pages activated: "
+  log_debug(os, thread)("Thread %zu stack guard pages activated: "
     PTR_FORMAT "-" PTR_FORMAT ".",
     os::current_thread_id(), p2i(low_addr), p2i(low_addr + len));
 }
@@ -136,7 +134,7 @@ void StackOverflow::remove_stack_guard_pages() {
     }
   }
 
-  log_debug(os, thread)("Thread " UINTX_FORMAT " stack guard pages removed: "
+  log_debug(os, thread)("Thread %zu stack guard pages removed: "
     PTR_FORMAT "-" PTR_FORMAT ".",
     os::current_thread_id(), p2i(low_addr), p2i(low_addr + len));
 }
@@ -214,20 +212,6 @@ void StackOverflow::disable_stack_yellow_reserved_zone() {
   }
 }
 
-void StackOverflow::enable_stack_red_zone() {
-  // The base notation is from the stacks point of view, growing downward.
-  // We need to adjust it to work correctly with guard_memory()
-  assert(_stack_guard_state != stack_guard_unused, "must be using guard pages.");
-  address base = stack_red_zone_base() - stack_red_zone_size();
-
-  guarantee(base < stack_base(), "Error calculating stack red zone");
-  guarantee(base < os::current_stack_pointer(), "Error calculating stack red zone");
-
-  if (!os::guard_memory((char *) base, stack_red_zone_size())) {
-    warning("Attempt to guard stack red zone failed.");
-  }
-}
-
 void StackOverflow::disable_stack_red_zone() {
   // The base notation is from the stacks point of view, growing downward.
   // We need to adjust it to work correctly with guard_memory()
