@@ -24,7 +24,7 @@
 
 #include "memory/allocation.inline.hpp"
 #include "memory/resourceArea.hpp"
-#include "runtime/atomic.hpp"
+#include "runtime/atomicAccess.hpp"
 #include "utilities/bitMap.inline.hpp"
 #include "utilities/copy.hpp"
 #include "utilities/debug.hpp"
@@ -243,11 +243,11 @@ void BitMap::par_put_range_within_word(idx_t beg, idx_t end, bool value) {
   // required by inverted_bit_mask_for_range.  Also avoids an unnecessary write.
   if (beg != end) {
     volatile bm_word_t* pw = word_addr(beg);
-    bm_word_t w = Atomic::load(pw);
+    bm_word_t w = AtomicAccess::load(pw);
     bm_word_t mr = inverted_bit_mask_for_range(beg, end);
     bm_word_t nw = value ? (w | ~mr) : (w & mr);
     while (true) {
-      bm_word_t res = Atomic::cmpxchg(pw, w, nw);
+      bm_word_t res = AtomicAccess::cmpxchg(pw, w, nw);
       if (res == w) break;
       w  = res;
       nw = value ? (w | ~mr) : (w & mr);
@@ -688,7 +688,7 @@ BitMap::idx_t BitMap::count_one_bits(idx_t beg, idx_t end) const {
 
 }
 
-void BitMap::print_on_error(outputStream* st, const char* prefix) const {
+void BitMap::print_range_on(outputStream* st, const char* prefix) const {
   st->print_cr("%s[" PTR_FORMAT ", " PTR_FORMAT ")",
       prefix, p2i(map()), p2i((char*)map() + (size() >> LogBitsPerByte)));
 }
