@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -93,12 +93,6 @@ public:
   bool  is_empty() const        { return _len == 0; }
   bool  is_nonempty() const     { return _len != 0; }
   bool  is_full() const         { return _len == _capacity; }
-
-  void  clear()                 { _len = 0; }
-  void  trunc_to(int length)    {
-    assert(length <= _len,"cannot increase length");
-    _len = length;
-  }
 };
 
 template <typename E> class GrowableArrayIterator;
@@ -189,11 +183,6 @@ public:
     return GrowableArrayIterator<E>(this, length());
   }
 
-  E pop() {
-    assert(_len > 0, "empty list");
-    return _data[--_len];
-  }
-
   void at_put(int i, const E& elem) {
     assert(0 <= i && i < _len, "illegal index %d for length %d", i, _len);
     _data[i] = elem;
@@ -247,59 +236,6 @@ public:
     return -1;
   }
 
-  // Order preserving remove operations.
-
-  void remove(const E& elem) {
-    // Assuming that element does exist.
-    bool removed = remove_if_existing(elem);
-    if (removed) return;
-    ShouldNotReachHere();
-  }
-
-  bool remove_if_existing(const E& elem) {
-    // Returns TRUE if elem is removed.
-    for (int i = 0; i < _len; i++) {
-      if (_data[i] == elem) {
-        remove_at(i);
-        return true;
-      }
-    }
-    return false;
-  }
-
-  void remove_at(int index) {
-    assert(0 <= index && index < _len, "illegal index %d for length %d", index, _len);
-    for (int j = index + 1; j < _len; j++) {
-      _data[j-1] = _data[j];
-    }
-    _len--;
-  }
-
-  // Remove all elements up to the index (exclusive). The order is preserved.
-  void remove_till(int idx) {
-    remove_range(0, idx);
-  }
-
-  // Remove all elements in the range [start - end). The order is preserved.
-  void remove_range(int start, int end) {
-    assert(0 <= start, "illegal start index %d", start);
-    assert(start < end && end <= _len, "erase called with invalid range (%d, %d) for length %d", start, end, _len);
-
-    for (int i = start, j = end; j < length(); i++, j++) {
-      at_put(i, at(j));
-    }
-    trunc_to(length() - (end - start));
-  }
-
-  // The order is changed.
-  void delete_at(int index) {
-    assert(0 <= index && index < _len, "illegal index %d for length %d", index, _len);
-    if (index < --_len) {
-      // Replace removed element with last one.
-      _data[index] = _data[_len];
-    }
-  }
-
   void sort(int f(E*, E*)) {
     if (_data == nullptr) return;
     qsort(_data, length(), sizeof(E), (_sort_Fn)f);
@@ -405,6 +341,9 @@ protected:
     }
   }
 
+  GrowableArrayWithAllocator(E* data, int capacity, int initial_len) :
+    GrowableArrayView<E>(data, capacity, initial_len) {}
+
   ~GrowableArrayWithAllocator() {}
 
 public:
@@ -424,6 +363,11 @@ public:
 
   void push(const E& elem) { append(elem); }
 
+  E pop() {
+    assert(this->_len > 0, "empty list");
+    return this->_data[--this->_len];
+  }
+
   E& at_grow(int i, const E& fill = E()) {
     assert(0 <= i, "negative index %d", i);
     if (i >= this->_len) {
@@ -511,9 +455,71 @@ public:
   // Ensure capacity is at least new_capacity.
   void reserve(int new_capacity);
 
+  void trunc_to(int length) {
+    assert(length <= this->_len,"cannot increase length");
+    this->_len = length;
+  }
+
+  // Order preserving remove operations.
+
+  void remove_at(int index) {
+    assert(0 <= index && index < this->_len,
+           "illegal index %d for length %d", index, this->_len);
+    for (int j = index + 1; j < this->_len; j++) {
+      this->_data[j-1] = this->_data[j];
+    }
+    this->_len--;
+  }
+
+  void remove(const E& elem) {
+    // Assuming that element does exist.
+    bool removed = this->remove_if_existing(elem);
+    if (removed) return;
+    ShouldNotReachHere();
+  }
+
+  bool remove_if_existing(const E& elem) {
+    // Returns TRUE if elem is removed.
+    for (int i = 0; i < this->_len; i++) {
+      if (this->_data[i] == elem) {
+        this->remove_at(i);
+        return true;
+      }
+    }
+    return false;
+  }
+
+  // Remove all elements up to the index (exclusive). The order is preserved.
+  void remove_till(int idx) {
+    remove_range(0, idx);
+  }
+
+  // Remove all elements in the range [start - end). The order is preserved.
+  void remove_range(int start, int end) {
+    assert(0 <= start, "illegal start index %d", start);
+    assert(start < end && end <= this->_len,
+           "erase called with invalid range (%d, %d) for length %d",
+           start, end, this->_len);
+
+    for (int i = start, j = end; j < this->length(); i++, j++) {
+      this->at_put(i, this->at(j));
+    }
+    this->_len -= (end - start);
+  }
+
+  // Replaces the designated element with the last element and shrinks by 1.
+  void delete_at(int index) {
+    assert(0 <= index && index < this->_len, "illegal index %d for length %d", index, this->_len);
+    if (index < --this->_len) {
+      // Replace removed element with last one.
+      this->_data[index] = this->_data[this->_len];
+    }
+  }
+
   // Reduce capacity to length.
   void shrink_to_fit();
 
+  void clear() { this->_len = 0; }
   void clear_and_deallocate();
 };
 
@@ -622,7 +628,7 @@ class GrowableArrayMetadata {
   uintptr_t _bits;
 
   // resource area nesting at creation
-  debug_only(GrowableArrayNestingCheck _nesting_check;)
+  DEBUG_ONLY(GrowableArrayNestingCheck _nesting_check;)
 
   // Resource allocation
   static uintptr_t bits() {
@@ -645,19 +651,19 @@ public:
   // Resource allocation
   GrowableArrayMetadata() :
       _bits(bits())
-      debug_only(COMMA _nesting_check(true)) {
+      DEBUG_ONLY(COMMA _nesting_check(true)) {
   }
 
   // Arena allocation
   GrowableArrayMetadata(Arena* arena) :
       _bits(bits(arena))
-      debug_only(COMMA _nesting_check(arena)) {
+      DEBUG_ONLY(COMMA _nesting_check(arena)) {
   }
 
   // CHeap allocation
   GrowableArrayMetadata(MemTag mem_tag) :
       _bits(bits(mem_tag))
-      debug_only(COMMA _nesting_check(false)) {
+      DEBUG_ONLY(COMMA _nesting_check(false)) {
   }
 
 #ifdef ASSERT
@@ -725,7 +731,7 @@ class GrowableArray : public GrowableArrayWithAllocator<E, GrowableArray<E>> {
 
   GrowableArrayMetadata _metadata;
 
-  void init_checks() const { debug_only(_metadata.init_checks(this);) }
+  void init_checks() const { DEBUG_ONLY(_metadata.init_checks(this);) }
 
   // Where are we going to allocate memory?
   bool on_C_heap() const        { return _metadata.on_C_heap(); }
@@ -734,7 +740,7 @@ class GrowableArray : public GrowableArrayWithAllocator<E, GrowableArray<E>> {
 
   E* allocate() {
     if (on_resource_area()) {
-      debug_only(_metadata.on_resource_area_alloc_check());
+      DEBUG_ONLY(_metadata.on_resource_area_alloc_check());
       return allocate(this->_capacity);
     }
 
@@ -743,7 +749,7 @@ class GrowableArray : public GrowableArrayWithAllocator<E, GrowableArray<E>> {
     }
 
     assert(on_arena(), "Sanity");
-    debug_only(_metadata.on_arena_alloc_check());
+    DEBUG_ONLY(_metadata.on_arena_alloc_check());
     return allocate(this->_capacity, _metadata.arena());
   }
 
@@ -780,6 +786,15 @@ public:
     init_checks();
   }
 
+  // This constructor performs no default initialization, so be careful.
+  GrowableArray(int initial_capacity, int initial_len, MemTag mem_tag) :
+    GrowableArrayWithAllocator<E, GrowableArray>(
+      allocate(initial_capacity, mem_tag),
+      initial_capacity, initial_len),
+    _metadata(mem_tag) {
+    init_checks();
+  }
+
   GrowableArray(int initial_capacity, int initial_len, const E& filler, MemTag mem_tag) :
       GrowableArrayWithAllocator<E, GrowableArray>(
           allocate(initial_capacity, mem_tag),
@@ -811,10 +826,6 @@ class GrowableArrayCHeap : public GrowableArrayWithAllocator<E, GrowableArrayCHe
   STATIC_ASSERT(MT != mtNone);
 
   static E* allocate(int max, MemTag mem_tag) {
-    if (max == 0) {
-      return nullptr;
-    }
-
     return (E*)GrowableArrayCHeapAllocator::allocate(max, sizeof(E), mem_tag);
   }
 
