@@ -25,16 +25,15 @@
 #ifndef SHARE_UTILITIES_RBTREE_INLINE_HPP
 #define SHARE_UTILITIES_RBTREE_INLINE_HPP
 
+#include "utilities/rbTree.hpp"
+
 #include "metaprogramming/enableIf.hpp"
 #include "utilities/debug.hpp"
 #include "utilities/globalDefinitions.hpp"
 #include "utilities/ostream.hpp"
 #include "utilities/powerOfTwo.hpp"
-#include "utilities/rbTree.hpp"
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::replace_child(
-    RBNode* old_child, RBNode* new_child) {
+inline void IntrusiveRBNode::replace_child(IntrusiveRBNode* old_child, IntrusiveRBNode* new_child) {
   if (_left == old_child) {
     _left = new_child;
   } else if (_right == old_child) {
@@ -44,11 +43,9 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::replace_child(
   }
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline typename RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode*
-RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::rotate_left() {
+inline IntrusiveRBNode* IntrusiveRBNode::rotate_left() {
   // This node down, right child up
-  RBNode* old_right = _right;
+  IntrusiveRBNode* old_right = _right;
 
   _right = old_right->_left;
   if (_right != nullptr) {
@@ -66,11 +63,9 @@ RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::rotate_left() {
   return old_right;
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline typename RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode*
-RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::rotate_right() {
+inline IntrusiveRBNode* IntrusiveRBNode::rotate_right() {
   // This node down, left child up
-  RBNode* old_left = _left;
+  IntrusiveRBNode* old_left = _left;
 
   _left = old_left->_right;
   if (_left != nullptr) {
@@ -88,11 +83,9 @@ RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::rotate_right() {
   return old_left;
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline const typename RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode*
-RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::prev() const {
-  const RBNode* node = this;
-  if (_left != nullptr) { // right subtree exists
+inline const IntrusiveRBNode* IntrusiveRBNode::prev() const {
+  const IntrusiveRBNode* node = this;
+  if (_left != nullptr) { // left subtree exists
     node = _left;
     while (node->_right != nullptr) {
       node = node->_right;
@@ -106,10 +99,8 @@ RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::prev() const {
   return node->parent();
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline const typename RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode*
-RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::next() const {
-  const RBNode* node = this;
+inline const IntrusiveRBNode* IntrusiveRBNode::next() const {
+  const IntrusiveRBNode* node = this;
   if (_right != nullptr) { // right subtree exists
     node = _right;
     while (node->_left != nullptr) {
@@ -124,13 +115,20 @@ RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::next() const {
   return node->parent();
 }
 
-#ifdef ASSERT
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::verify(
+inline IntrusiveRBNode* IntrusiveRBNode::prev() {
+  return const_cast<IntrusiveRBNode*>(static_cast<const IntrusiveRBNode*>(this)->prev());
+}
+
+inline IntrusiveRBNode* IntrusiveRBNode::next() {
+  return const_cast<IntrusiveRBNode*>(static_cast<const IntrusiveRBNode*>(this)->next());
+}
+
+template <typename NodeType, typename NodeVerifier>
+inline void IntrusiveRBNode::verify(
     size_t& num_nodes, size_t& black_nodes_until_leaf, size_t& shortest_leaf_path, size_t& longest_leaf_path,
-    size_t& tree_depth, bool expect_visited) const {
+    size_t& tree_depth, bool expect_visited, NodeVerifier verifier) const {
   assert(expect_visited != _visited, "node already visited");
-  _visited = !_visited;
+  DEBUG_ONLY(_visited = !_visited);
 
   size_t num_black_nodes_left = 0;
   size_t shortest_leaf_path_left = 0;
@@ -138,14 +136,14 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::verify(
   size_t tree_depth_left = 0;
 
   if (_left != nullptr) {
+    assert(verifier((NodeType*)_left, (NodeType*)this), "left child must compare strictly less than parent");
     if (_right == nullptr) {
       assert(is_black() && _left->is_red(), "if one child it must be red and node black");
     }
-    assert(COMPARATOR::cmp(_left->key(), _key) < 0, "left node must be less than parent");
     assert(is_black() || _left->is_black(), "2 red nodes in a row");
     assert(_left->parent() == this, "pointer mismatch");
-    _left->verify(num_nodes, num_black_nodes_left, shortest_leaf_path_left,
-                  longest_leaf_path_left, tree_depth_left, expect_visited);
+    _left->verify<NodeType>(num_nodes, num_black_nodes_left, shortest_leaf_path_left,
+                  longest_leaf_path_left, tree_depth_left, expect_visited, verifier);
   }
 
   size_t num_black_nodes_right = 0;
@@ -154,14 +152,14 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::verify(
   size_t tree_depth_right = 0;
 
   if (_right != nullptr) {
+    assert(verifier((NodeType*)this, (NodeType*)_right), "right child must compare strictly greater than parent");
     if (_left == nullptr) {
       assert(is_black() && _right->is_red(), "if one child it must be red and node black");
     }
-    assert(COMPARATOR::cmp(_right->key(), _key) > 0, "right node must be greater than parent");
     assert(is_black() || _left->is_black(), "2 red nodes in a row");
     assert(_right->parent() == this, "pointer mismatch");
-    _right->verify(num_nodes, num_black_nodes_right, shortest_leaf_path_right,
-                   longest_leaf_path_right, tree_depth_right, expect_visited);
+    _right->verify<NodeType>(num_nodes, num_black_nodes_right, shortest_leaf_path_right,
+                   longest_leaf_path_right, tree_depth_right, expect_visited, verifier);
   }
 
   shortest_leaf_path = MAX2(longest_leaf_path_left, longest_leaf_path_right);
@@ -185,85 +183,91 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode::verify(
 
 }
 
-#endif // ASSERT
+template <typename K, typename NodeType, typename COMPARATOR>
+inline const typename AbstractRBTree<K, NodeType, COMPARATOR>::Cursor
+AbstractRBTree<K, NodeType, COMPARATOR>::cursor(const K& key, const NodeType* hint_node) const {
+  IntrusiveRBNode* parent = nullptr;
+  IntrusiveRBNode* const* insert_location = &_root;
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline const typename RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode*
-RBTree<K, V, COMPARATOR, ALLOCATOR>::find_node(const K& key) const {
-  RBNode* curr = _root;
-  while (curr != nullptr) {
-    const int key_cmp_k = COMPARATOR::cmp(key, curr->key());
+  if (hint_node != nullptr) {
+    const int hint_cmp = cmp(key, hint_node);
+    while (hint_node->parent() != nullptr) {
+      const int parent_cmp = cmp(key, (NodeType*)hint_node->parent());
+      // Move up until the parent would put us on the other side of the key.
+      // Meaning we are in the correct subtree.
+      if ((parent_cmp <= 0 && hint_cmp < 0) ||
+          (parent_cmp >= 0 && hint_cmp > 0)) {
+        hint_node = (NodeType*)hint_node->parent();
+      } else {
+        break;
+      }
+    }
 
-    if (key_cmp_k == 0) {
-      return curr;
-    } else if (key_cmp_k < 0) {
-      curr = curr->_left;
-    } else {
-      curr = curr->_right;
+    if (hint_node->is_left_child()) {
+      insert_location = &hint_node->parent()->_left;
+    } else if (hint_node->is_right_child()) {
+      insert_location = &hint_node->parent()->_right;
     }
   }
 
-  return nullptr;
-}
-
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline typename RBTree<K, V, COMPARATOR, ALLOCATOR>::RBNode*
-RBTree<K, V, COMPARATOR, ALLOCATOR>::insert_node(const K& key, const V& val) {
-  RBNode* curr = _root;
-  if (curr == nullptr) { // Tree is empty
-    _root = allocate_node(key, val);
-    return _root;
-  }
-
-  RBNode* parent = nullptr;
-  while (curr != nullptr) {
-    const int key_cmp_k = COMPARATOR::cmp(key, curr->key());
+  while (*insert_location != nullptr) {
+    NodeType* curr = (NodeType*)*insert_location;
+    const int key_cmp_k = cmp(key, curr);
 
     if (key_cmp_k == 0) {
-      curr->_value = val;
-      return curr;
+      break;
     }
 
-    parent = curr;
+    parent = *insert_location;
     if (key_cmp_k < 0) {
-      curr = curr->_left;
+      insert_location = &curr->_left;
     } else {
-      curr = curr->_right;
+      insert_location = &curr->_right;
     }
   }
 
-  // Create and insert new node
-  RBNode* node = allocate_node(key, val);
-  node->set_parent(parent);
-
-  const int key_cmp_k = COMPARATOR::cmp(key, parent->key());
-  if (key_cmp_k < 0) {
-    parent->_left = node;
-  } else {
-    parent->_right = node;
-  }
-
-  return node;
+  return Cursor((NodeType**)insert_location, (NodeType*)parent);
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::fix_insert_violations(RBNode* node) {
+template <typename K, typename NodeType, typename COMPARATOR>
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::insert_at_cursor(NodeType* node, const Cursor& node_cursor) {
+  assert(node_cursor.valid() && !node_cursor.found(), "must be");
+  _num_nodes++;
+
+  *node_cursor._insert_location = node;
+
+  node->set_parent(node_cursor._parent);
+  node->set_red();
+  node->_left = nullptr;
+  node->_right = nullptr;
+
+  DEBUG_ONLY(node->_visited = _expected_visited);
+
+  if (node_cursor._parent == nullptr) {
+    return;
+  }
+
+  fix_insert_violations(node);
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::fix_insert_violations(IntrusiveRBNode* node) {
   if (node->is_black()) { // node's value was updated
     return;               // Tree is already correct
   }
 
-  RBNode* parent = node->parent();
+  IntrusiveRBNode* parent = node->parent();
   while (parent != nullptr && parent->is_red()) {
     // Node and parent are both red, creating a red-violation
 
-    RBNode* grandparent = parent->parent();
+    IntrusiveRBNode* grandparent = parent->parent();
     if (grandparent == nullptr) { // Parent is the tree root
       assert(parent == _root, "parent must be root");
       parent->set_black(); // Color parent black to eliminate the red-violation
       return;
     }
 
-    RBNode* uncle = parent->is_left_child() ? grandparent->_right : grandparent->_left;
+    IntrusiveRBNode* uncle = parent->is_left_child() ? grandparent->_right : grandparent->_left;
     if (is_black(uncle)) { // Parent is red, uncle is black
       // Rotate the parent to the position of the grandparent
       if (parent->is_left_child()) {
@@ -305,14 +309,14 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::fix_insert_violations(RBNode* n
   }
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::remove_black_leaf(RBNode* node) {
+template <typename K, typename NodeType, typename COMPARATOR>
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::remove_black_leaf(IntrusiveRBNode* node) {
   // Black node removed, balancing needed
-  RBNode* parent = node->parent();
+  IntrusiveRBNode* parent = node->parent();
   while (parent != nullptr) {
     // Sibling must exist. If it did not, node would need to be red to not break
     // tree properties, and could be trivially removed before reaching here
-    RBNode* sibling = node->is_left_child() ? parent->_right : parent->_left;
+    IntrusiveRBNode* sibling = node->is_left_child() ? parent->_right : parent->_left;
     if (is_red(sibling)) { // Sibling red, parent and nephews must be black
       assert(is_black(parent), "parent must be black");
       assert(is_black(sibling->_left), "nephew must be black");
@@ -336,8 +340,8 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::remove_black_leaf(RBNode* node)
       // Further balancing needed
     }
 
-    RBNode* close_nephew = node->is_left_child() ? sibling->_left : sibling->_right;
-    RBNode* distant_nephew = node->is_left_child() ? sibling->_right : sibling->_left;
+    IntrusiveRBNode* close_nephew = node->is_left_child() ? sibling->_left : sibling->_right;
+    IntrusiveRBNode* distant_nephew = node->is_left_child() ? sibling->_right : sibling->_left;
     if (is_red(distant_nephew) || is_red(close_nephew)) {
       if (is_black(distant_nephew)) { // close red, distant black
         // Rotate sibling down and inner nephew up
@@ -393,11 +397,11 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::remove_black_leaf(RBNode* node)
   }
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::remove_from_tree(RBNode* node) {
-  RBNode* parent = node->parent();
-  RBNode* left = node->_left;
-  RBNode* right = node->_right;
+template <typename K, typename NodeType, typename COMPARATOR>
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::remove_from_tree(IntrusiveRBNode* node) {
+  IntrusiveRBNode* parent = node->parent();
+  IntrusiveRBNode* left = node->_left;
+  IntrusiveRBNode* right = node->_right;
   if (left != nullptr) { // node has a left only-child
     // node must be black, and child red, otherwise a black-violation would
     // exist Remove node and color the child black.
@@ -439,13 +443,16 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::remove_from_tree(RBNode* node)
   }
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::remove(RBNode* node) {
-  assert(node != nullptr, "must be");
+template <typename K, typename NodeType, typename COMPARATOR>
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::remove_at_cursor(const Cursor& node_cursor) {
+  assert(node_cursor.valid() && node_cursor.found(), "must be");
+  _num_nodes--;
+
+  IntrusiveRBNode* node = node_cursor.node();
 
   if (node->_left != nullptr && node->_right != nullptr) { // node has two children
     // Swap place with the in-order successor and delete there instead
-    RBNode* curr = node->_right;
+    IntrusiveRBNode* curr = node->_right;
     while (curr->_left != nullptr) {
       curr = curr->_left;
     }
@@ -475,46 +482,189 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::remove(RBNode* node) {
   }
 
   remove_from_tree(node);
-  free_node(node);
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
+template <typename K, typename NodeType, typename COMPARATOR>
+inline const typename AbstractRBTree<K, NodeType, COMPARATOR>::Cursor
+AbstractRBTree<K, NodeType, COMPARATOR>::cursor(const NodeType* node) const {
+  if (node == nullptr) {
+    return Cursor();
+  }
+
+  if (node->parent() == nullptr) {
+    return Cursor((NodeType**)&_root, nullptr);
+  }
+
+  IntrusiveRBNode* parent = node->parent();
+  IntrusiveRBNode** insert_location =
+      node->is_left_child() ? &parent->_left : &parent->_right;
+  return Cursor((NodeType**)insert_location, (NodeType*)parent);
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+inline const typename AbstractRBTree<K, NodeType, COMPARATOR>::Cursor
+AbstractRBTree<K, NodeType, COMPARATOR>::next(const Cursor& node_cursor) const {
+  if (node_cursor.found()) {
+    return cursor(node_cursor.node()->next());
+  }
+
+  if (node_cursor._parent == nullptr) { // Tree is empty
+    return Cursor();
+  }
+
+  // Pointing to non-existant node
+  if ((NodeType**)&node_cursor._parent->_left == node_cursor._insert_location) { // Left child, parent is next
+    return cursor(node_cursor._parent);
+  }
+
+  return cursor(node_cursor._parent->next()); // Right child, parent's next is also node's next
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+inline const typename AbstractRBTree<K, NodeType, COMPARATOR>::Cursor
+AbstractRBTree<K, NodeType, COMPARATOR>::prev(const Cursor& node_cursor) const {
+  if (node_cursor.found()) {
+    return cursor(node_cursor.node()->prev());
+  }
+
+  if (node_cursor._parent == nullptr) { // Tree is empty
+    return Cursor();
+  }
+
+  // Pointing to non-existant node
+  if ((NodeType**)&node_cursor._parent->_right == node_cursor._insert_location) { // Right child, parent is prev
+    return cursor(node_cursor._parent);
+  }
+
+  return cursor(node_cursor._parent->prev()); // Left child, parent's prev is also node's prev
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::replace_at_cursor(NodeType* new_node, const Cursor& node_cursor) {
+  assert(node_cursor.valid() && node_cursor.found(), "must be");
+  NodeType* old_node = node_cursor.node();
+  if (old_node == new_node) {
+    return;
+  }
+
+  *node_cursor._insert_location = new_node;
+  new_node->_parent = old_node->_parent;
+
+  if (new_node->is_left_child()) {
+    assert(cmp(static_cast<const NodeType*>(new_node), static_cast<const NodeType*>(new_node->parent())), "new node not < parent");
+  } else if (new_node->is_right_child()) {
+    assert(cmp(static_cast<const NodeType*>(new_node->parent()), static_cast<const NodeType*>(new_node)), "new node not > parent");
+  }
+
+  new_node->_left = old_node->_left;
+  new_node->_right = old_node->_right;
+  if (new_node->_left != nullptr) {
+    assert(cmp(static_cast<const NodeType*>(new_node->_left), static_cast<const NodeType*>(new_node)), "left child not < new node");
+    new_node->_left->set_parent(new_node);
+  }
+  if (new_node->_right != nullptr) {
+    assert(cmp(static_cast<const NodeType*>(new_node), static_cast<const NodeType*>(new_node->_right)), "right child not > new node");
+    new_node->_right->set_parent(new_node);
+  }
+
+  DEBUG_ONLY(new_node->_visited = old_node->_visited);
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+inline typename AbstractRBTree<K, NodeType, COMPARATOR>::Cursor
+AbstractRBTree<K, NodeType, COMPARATOR>::cursor(const K& key, const NodeType* hint_node) {
+  return static_cast<const AbstractRBTree<K, NodeType, COMPARATOR>*>(this)->cursor(key, hint_node);
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+inline typename AbstractRBTree<K, NodeType, COMPARATOR>::Cursor
+AbstractRBTree<K, NodeType, COMPARATOR>::cursor(const NodeType* node) {
+  return static_cast<const AbstractRBTree<K, NodeType, COMPARATOR>*>(this)->cursor(node);
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+inline typename AbstractRBTree<K, NodeType, COMPARATOR>::Cursor
+AbstractRBTree<K, NodeType, COMPARATOR>::next(const Cursor& node_cursor) {
+  return static_cast<const AbstractRBTree<K, NodeType, COMPARATOR>*>(this)->next(node_cursor);
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+inline typename AbstractRBTree<K, NodeType, COMPARATOR>::Cursor
+AbstractRBTree<K, NodeType, COMPARATOR>::prev(const Cursor& node_cursor) {
+  return static_cast<const AbstractRBTree<K, NodeType, COMPARATOR>*>(this)->prev(node_cursor);
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
 template <typename F>
-inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::visit_in_order(F f) const {
-  const RBNode* to_visit[64];
-  int stack_idx = 0;
-  const RBNode* head = _root;
-  while (stack_idx > 0 || head != nullptr) {
-    while (head != nullptr) {
-      to_visit[stack_idx++] = head;
-      assert(stack_idx <= (int)(sizeof(to_visit)/sizeof(to_visit[0])), "stack too deep");
-      head = head->_left;
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::visit_in_order(F f) const {
+  const NodeType* node = leftmost();
+  while (node != nullptr) {
+    if (!f(node)) {
+      return;
     }
-    head = to_visit[--stack_idx];
-    f(head);
-    head = head->_right;
+    node = node->next();
   }
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
+template <typename K, typename NodeType, typename COMPARATOR>
 template <typename F>
-inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::visit_range_in_order(const K& from, const K& to, F f) const {
-  assert(COMPARATOR::cmp(from, to) <= 0, "from must be less or equal to to");
-  const RBNode* curr = closest_geq(from);
-  if (curr == nullptr) return;
-  const RBNode* const end = closest_geq(to);
-
-  while (curr != nullptr && curr != end) {
-    f(curr);
-    curr = curr->next();
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::visit_in_order(F f) {
+  NodeType* node = leftmost();
+  while (node != nullptr) {
+    if (!f(node)) {
+      return;
+    }
+    node = node->next();
   }
 }
 
-#ifdef ASSERT
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::verify_self() const {
+template <typename K, typename NodeType, typename COMPARATOR>
+template <typename F>
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::visit_range_in_order(const K& from, const K& to, F f) const {
+  assert_key_leq(from, to);
   if (_root == nullptr) {
-    assert(_num_nodes == 0, "rbtree has nodes but no root");
+    return;
+  }
+
+  Cursor cursor_start = cursor(from);
+  Cursor cursor_end = cursor(to);
+  const NodeType* start = cursor_start.found() ? cursor_start.node() : next(cursor_start).node();
+  const NodeType* end = next(cursor_end).node();
+
+  while (start != end) {
+    if (!f(start)) {
+      return;
+    }
+    start = start->next();
+  }
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+template <typename F>
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::visit_range_in_order(const K& from, const K& to, F f) {
+  assert_key_leq(from, to);
+  if (_root == nullptr) {
+    return;
+  }
+
+  Cursor cursor_start = cursor(from);
+  Cursor cursor_end = cursor(to);
+  NodeType* start = cursor_start.found() ? cursor_start.node() : next(cursor_start).node();
+  NodeType* end = next(cursor_end).node();
+
+  while (start != end) {
+    if (!f(start)) {
+      return;
+    }
+    start = start->next();
+  }
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+template <typename NodeVerifier>
+inline void AbstractRBTree<K, NodeType, COMPARATOR>::verify_self(NodeVerifier verifier) const {
+  if (_root == nullptr) {
+    assert(_num_nodes == 0, "rbtree has %zu nodes but no root", _num_nodes);
     return;
   }
 
@@ -525,9 +675,11 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::verify_self() const {
   size_t tree_depth = 0;
   size_t shortest_leaf_path = 0;
   size_t longest_leaf_path = 0;
-  _expected_visited = !_expected_visited;
+  DEBUG_ONLY(_expected_visited = !_expected_visited);
+  bool expected_visited = DEBUG_ONLY(_expected_visited) NOT_DEBUG(false);
 
-  _root->verify(num_nodes, black_depth, shortest_leaf_path, longest_leaf_path, tree_depth, _expected_visited);
+  _root->verify<NodeType>(num_nodes, black_depth, shortest_leaf_path, longest_leaf_path,
+                tree_depth, expected_visited, verifier);
 
   const unsigned int maximum_depth = log2i(size() + 1) * 2;
 
@@ -539,7 +691,6 @@ inline void RBTree<K, V, COMPARATOR, ALLOCATOR>::verify_self() const {
          "unexpected number of nodes in rbtree. expected: %zu"
          ", actual: %zu", size(), num_nodes);
 }
-#endif // ASSERT
 
 template <typename T,
           ENABLE_IF(std::is_integral<T>::value),
@@ -561,28 +712,40 @@ void print_T(outputStream* st, T x) {
   st->print(PTR_FORMAT, p2i(x));
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-void RBTree<K, V, COMPARATOR, ALLOCATOR>::print_node_on(outputStream* st, int depth, const NodeType* n) const {
+inline void IntrusiveRBNode::print_on(outputStream* st, int depth) const {
   st->print("(%d)", depth);
   st->sp(1 + depth * 2);
-  st->print("@" PTR_FORMAT ": [", p2i(n));
-  print_T<K>(st, n->key());
-  st->print("] = ");
-  print_T<V>(st, n->val());
+  st->print("@" PTR_FORMAT, p2i(this));
   st->cr();
+}
+
+template <typename K, typename V>
+inline void RBNode<K, V>::print_on(outputStream* st, int depth) const {
+  st->print("(%d)", depth);
+  st->sp(1 + depth * 2);
+  st->print("@" PTR_FORMAT ": [", p2i(this));
+  print_T<K>(st, key());
+  st->print("] = ");
+  print_T<V>(st, val());
+  st->cr();
+}
+
+template <typename K, typename NodeType, typename COMPARATOR>
+void AbstractRBTree<K, NodeType, COMPARATOR>::print_node_on(outputStream* st, int depth, const NodeType* n) const {
+  n->print_on(st, depth);
   depth++;
   if (n->_right != nullptr) {
-    print_node_on(st, depth, n->_right);
+    print_node_on(st, depth, (NodeType*)n->_right);
   }
   if (n->_left != nullptr) {
-    print_node_on(st, depth, n->_left);
+    print_node_on(st, depth, (NodeType*)n->_left);
   }
 }
 
-template <typename K, typename V, typename COMPARATOR, typename ALLOCATOR>
-void RBTree<K, V, COMPARATOR, ALLOCATOR>::print_on(outputStream* st) const {
+template <typename K, typename NodeType, typename COMPARATOR>
+void AbstractRBTree<K, NodeType, COMPARATOR>::print_on(outputStream* st) const {
   if (_root != nullptr) {
-    print_node_on(st, 0, _root);
+    print_node_on(st, 0, (NodeType*)_root);
   }
 }
 
