@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1996, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -534,7 +534,7 @@ public abstract class HttpURLConnection extends URLConnection {
 
                 try {
                     responseCode = Integer.parseInt
-                            (statusLine.substring(codePos+1, phrasePos));
+                            (statusLine, codePos+1, phrasePos, 10);
                     return responseCode;
                 } catch (NumberFormatException e) { }
             }
@@ -556,6 +556,10 @@ public abstract class HttpURLConnection extends URLConnection {
      * @return the HTTP response message, or {@code null}
      */
     public String getResponseMessage() throws IOException {
+        // If the responseMessage is already set then return it
+        if (responseMessage != null) {
+            return responseMessage;
+        }
         getResponseCode();
         return responseMessage;
     }
@@ -606,7 +610,13 @@ public abstract class HttpURLConnection extends URLConnection {
      * @return a {@code SocketPermission} object representing the
      *         permission necessary to connect to the destination
      *         host and port.
+     *
+     * @deprecated
+     * Permissions can no longer be used for controlling access to resources
+     * as the Security Manager is no longer supported.
      */
+    @Deprecated(since = "25", forRemoval = true)
+    @SuppressWarnings("removal")
     public Permission getPermission() throws IOException {
         int port = url.getPort();
         port = port < 0 ? 80 : port;
