@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1995, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -30,8 +30,6 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.net.spi.URLStreamHandlerProvider;
 import java.nio.file.Path;
-import java.security.AccessController;
-import java.security.PrivilegedAction;
 import java.util.Hashtable;
 import java.io.InvalidObjectException;
 import java.io.ObjectStreamException;
@@ -39,17 +37,17 @@ import java.io.ObjectStreamField;
 import java.io.ObjectInputStream.GetField;
 import java.util.Iterator;
 import java.util.Locale;
-import java.util.NoSuchElementException;
-import java.util.ServiceConfigurationError;
 import java.util.ServiceLoader;
 
 import jdk.internal.access.JavaNetURLAccess;
 import jdk.internal.access.SharedSecrets;
 import jdk.internal.misc.ThreadTracker;
 import jdk.internal.misc.VM;
+import jdk.internal.vm.annotation.AOTRuntimeSetup;
+import jdk.internal.vm.annotation.AOTSafeClassInitializer;
 import sun.net.util.IPAddressUtil;
-import sun.security.util.SecurityConstants;
-import sun.security.action.GetPropertyAction;
+import static jdk.internal.util.Exceptions.filterNonSocketInfo;
+import static jdk.internal.util.Exceptions.formatMsg;
 
 /**
  * Class {@code URL} represents a Uniform Resource
@@ -129,6 +127,26 @@ import sun.security.action.GetPropertyAction;
  * the protocol, host name, or port number is missing, the value is
  * inherited from the fully specified URL. The file component must be
  * specified. The optional fragment is not inherited.
+ *
+ * <h2><a id="constructor-deprecation"></a>Constructing instances of {@code URL}</h2>
+ *
+ * The {@code java.net.URL} constructors are deprecated.
+ * Developers are encouraged to use {@link URI java.net.URI} to parse
+ * or construct a {@code URL}. In cases where an instance of {@code
+ * java.net.URL} is needed to open a connection, {@link URI} can be used
+ * to construct or parse the URL string, possibly calling {@link
+ * URI#parseServerAuthority()} to validate that the authority component
+ * can be parsed as a server-based authority, and then calling
+ * {@link URI#toURL()} to create the {@code URL} instance.
+ * <p>
+ * The URL constructors are specified to throw
+ * {@link MalformedURLException} but the actual parsing/validation
+ * that is performed is implementation dependent. Some parsing/validation
+ * may be delayed until later, when the underlying {@linkplain
+ * URLStreamHandler stream handler's implementation} is called.
+ * Being able to construct an instance of {@code URL} doesn't
+ * provide any guarantee about its conformance to the URL
+ * syntax specification.
  * <p>
  * The URL class does not itself encode or decode any URL components
  * according to the escaping mechanism defined in RFC2396. It is the
@@ -152,6 +170,7 @@ import sun.security.action.GetPropertyAction;
  *
  * @apiNote
  *
+ * <a id="integrity"></a>
  * Applications working with file paths and file URIs should take great
  * care to use the appropriate methods to convert between the two.
  * The {@link Path#of(URI)} factory method and the {@link File#File(URI)}
@@ -164,16 +183,40 @@ import sun.security.action.GetPropertyAction;
  * from the direct string representation of a {@code File} or {@code Path}
  * instance.
  * <p>
+ * Before constructing a {@code URL} from a {@code URI}, and depending
+ * on the protocol involved, applications should consider validating
+ * whether the URI authority {@linkplain URI#parseServerAuthority()
+ * can be parsed as server-based}.
+ * <p>
  * Some components of a URL or URI, such as <i>userinfo</i>, may
  * be abused to construct misleading URLs or URIs. Applications
  * that deal with URLs or URIs should take into account
  * the recommendations advised in <a
  * href="https://tools.ietf.org/html/rfc3986#section-7">RFC3986,
  * Section 7, Security Considerations</a>.
+ * <p>
+ * All {@code URL} constructors may throw {@link MalformedURLException}.
+ * In particular, if the underlying {@link URLStreamHandler}
+ * implementation rejects, or is known to reject, any of the parameters,
+ * {@link MalformedURLException} may be thrown.
+ * Typically, a constructor that calls the stream handler's {@linkplain
+ * URLStreamHandler#parseURL(URL, String, int, int) parseURL method} may
+ * throw {@code MalformedURLException} if the underlying stream handler
+ * implementation of that method throws {@code IllegalArgumentException}.
+ * However, which checks are performed, or not, by the stream handlers
+ * is implementation dependent, and callers should not rely on such
+ * checks for full URL validation.
  *
+ * @spec https://www.rfc-editor.org/info/rfc2396
+ *      RFC 2396: Uniform Resource Identifiers (URI): Generic Syntax
+ * @spec https://www.rfc-editor.org/info/rfc2732
+ *      RFC 2732: Format for Literal IPv6 Addresses in URL's
+ * @spec https://www.rfc-editor.org/info/rfc3986
+ *      RFC 3986: Uniform Resource Identifier (URI): Generic Syntax
  * @author  James Gosling
  * @since 1.0
  */
+@AOTSafeClassInitializer
 public final class URL implements java.io.Serializable {
 
     static final String BUILTIN_HANDLERS_PREFIX = "sun.net.www.protocol";
@@ -352,14 +395,22 @@ public final class URL implements java.io.Serializable {
      * @param      port       the port number on the host.
      * @param      file       the file on the host
      * @throws     MalformedURLException  if an unknown protocol or the port
-     *                  is a negative number other than -1
+     *                  is a negative number other than -1, or if the
+     *                  underlying stream handler implementation rejects,
+     *                  or is known to reject, the {@code URL}
+     * @spec https://www.rfc-editor.org/info/rfc2373 RFC 2373: IP Version 6 Addressing Architecture
+     * @spec https://www.rfc-editor.org/info/rfc2732 RFC 2732: Format for Literal IPv6 Addresses in URL's
      * @see        java.lang.System#getProperty(java.lang.String)
      * @see        java.net.URL#setURLStreamHandlerFactory(
      *                  java.net.URLStreamHandlerFactory)
      * @see        java.net.URLStreamHandler
      * @see        java.net.URLStreamHandlerFactory#createURLStreamHandler(
      *                  java.lang.String)
+     * @deprecated Use {@link URI#toURL()} to construct an instance of URL. See the note on
+     * <a href="#constructor-deprecation">constructor deprecation</a> for more
+     * details.
      */
+    @Deprecated(since = "20")
     public URL(String protocol, String host, int port, String file)
         throws MalformedURLException
     {
@@ -380,10 +431,16 @@ public final class URL implements java.io.Serializable {
      * @param      protocol   the name of the protocol to use.
      * @param      host       the name of the host.
      * @param      file       the file on the host.
-     * @throws     MalformedURLException  if an unknown protocol is specified.
+     * @throws     MalformedURLException  if an unknown protocol is specified,
+     *                        or if the underlying stream handler implementation
+     *                        rejects, or is known to reject, the {@code URL}
      * @see        java.net.URL#URL(java.lang.String, java.lang.String,
      *                  int, java.lang.String)
+     * @deprecated Use {@link URI#toURL()} to construct an instance of URL. See the note on
+     * <a href="#constructor-deprecation">constructor deprecation</a> for more
+     * details.
      */
+    @Deprecated(since = "20")
     public URL(String protocol, String host, String file)
             throws MalformedURLException {
         this(protocol, host, -1, file);
@@ -401,12 +458,6 @@ public final class URL implements java.io.Serializable {
      *     {@link java.net.URL#URL(java.lang.String, java.lang.String, int,
      *                      java.lang.String)}
      *
-     * <p>If the handler is not null and there is a security manager,
-     * the security manager's {@code checkPermission}
-     * method is called with a
-     * {@code NetPermission("specifyStreamHandler")} permission.
-     * This may result in a SecurityException.
-     *
      * No validation of the inputs is performed by this constructor.
      *
      * @param      protocol   the name of the protocol to use.
@@ -415,32 +466,26 @@ public final class URL implements java.io.Serializable {
      * @param      file       the file on the host
      * @param      handler    the stream handler for the URL.
      * @throws     MalformedURLException  if an unknown protocol or the port
-     *                    is a negative number other than -1
-     * @throws     SecurityException
-     *        if a security manager exists and its
-     *        {@code checkPermission} method doesn't allow
-     *        specifying a stream handler explicitly.
+     *                    is a negative number other than -1,
+     *                    or if the underlying stream handler implementation
+     *                    rejects, or is known to reject, the {@code URL}
      * @see        java.lang.System#getProperty(java.lang.String)
      * @see        java.net.URL#setURLStreamHandlerFactory(
      *                  java.net.URLStreamHandlerFactory)
      * @see        java.net.URLStreamHandler
      * @see        java.net.URLStreamHandlerFactory#createURLStreamHandler(
      *                  java.lang.String)
-     * @see        SecurityManager#checkPermission
-     * @see        java.net.NetPermission
+     * @deprecated
+     * Use {@link #of(URI, URLStreamHandler)} to construct an instance of URL
+     * associated with a custom protocol handler.
+     * See the note on <a href="#constructor-deprecation">constructor deprecation</a>
+     * for more details.
      */
+    @Deprecated(since = "20")
     public URL(String protocol, String host, int port, String file,
                URLStreamHandler handler) throws MalformedURLException {
-        if (handler != null) {
-            @SuppressWarnings("removal")
-            SecurityManager sm = System.getSecurityManager();
-            if (sm != null) {
-                // check for permission to specify a handler
-                checkSpecifyHandler(sm);
-            }
-        }
 
-        protocol = toLowerCase(protocol);
+        protocol = lowerCaseProtocol(protocol);
         this.protocol = protocol;
         if (host != null) {
 
@@ -510,9 +555,16 @@ public final class URL implements java.io.Serializable {
      * @throws     MalformedURLException  if no protocol is specified, or an
      *               unknown protocol is found, or {@code spec} is {@code null},
      *               or the parsed URL fails to comply with the specific syntax
-     *               of the associated protocol.
+     *               of the associated protocol, or the
+     *               underlying stream handler's {@linkplain
+     *               URLStreamHandler#parseURL parseURL method} throws
+     *               {@code IllegalArgumentException}
      * @see        java.net.URL#URL(java.net.URL, java.lang.String)
+     * @deprecated Use {@link URI#toURL()} to construct an instance of URL. See the note on
+     * <a href="#constructor-deprecation">constructor deprecation</a> for more
+     * details.
      */
+    @Deprecated(since = "20")
     public URL(String spec) throws MalformedURLException {
         this(null, spec);
     }
@@ -522,7 +574,7 @@ public final class URL implements java.io.Serializable {
      *
      * The new URL is created from the given context URL and the spec
      * argument as described in
-     * RFC2396 &quot;Uniform Resource Identifiers : Generic * Syntax&quot; :
+     * RFC2396 &quot;Uniform Resource Identifiers : Generic Syntax&quot; :
      * <blockquote><pre>
      *          &lt;scheme&gt;://&lt;authority&gt;&lt;path&gt;?&lt;query&gt;#&lt;fragment&gt;
      * </pre></blockquote>
@@ -554,18 +606,29 @@ public final class URL implements java.io.Serializable {
      * <p>
      * For a more detailed description of URL parsing, refer to RFC2396.
      *
+     * @implSpec Parsing the URL includes calling the {@link
+     * URLStreamHandler#parseURL(URL, String, int, int) parseURL} method on the
+     * selected handler.
+     *
      * @param      context   the context in which to parse the specification.
      * @param      spec      the {@code String} to parse as a URL.
      * @throws     MalformedURLException  if no protocol is specified, or an
      *               unknown protocol is found, or {@code spec} is {@code null},
      *               or the parsed URL fails to comply with the specific syntax
-     *               of the associated protocol.
+     *               of the associated protocol, or the
+     *               underlying stream handler's {@linkplain
+     *               URLStreamHandler#parseURL parseURL method} throws
+     *               {@code IllegalArgumentException}
      * @see        java.net.URL#URL(java.lang.String, java.lang.String,
      *                  int, java.lang.String)
      * @see        java.net.URLStreamHandler
      * @see        java.net.URLStreamHandler#parseURL(java.net.URL,
      *                  java.lang.String, int, int)
+     * @deprecated Use {@link URI#toURL()} to construct an instance of URL. See the note on
+     * <a href="#constructor-deprecation">constructor deprecation</a> for more
+     * details.
      */
+    @Deprecated(since = "20")
     public URL(URL context, String spec) throws MalformedURLException {
         this(context, spec, null);
     }
@@ -575,23 +638,32 @@ public final class URL implements java.io.Serializable {
      * within a specified context. If the handler is null, the parsing
      * occurs as with the two argument constructor.
      *
+     * @implSpec Parsing the URL includes calling the {@link
+     * URLStreamHandler#parseURL(URL, String, int, int) parseURL} method on the
+     * selected handler.
+     *
      * @param      context   the context in which to parse the specification.
      * @param      spec      the {@code String} to parse as a URL.
      * @param      handler   the stream handler for the URL.
      * @throws     MalformedURLException  if no protocol is specified, or an
      *               unknown protocol is found, or {@code spec} is {@code null},
      *               or the parsed URL fails to comply with the specific syntax
-     *               of the associated protocol.
-     * @throws     SecurityException
-     *        if a security manager exists and its
-     *        {@code checkPermission} method doesn't allow
-     *        specifying a stream handler.
+     *               of the associated protocol, or the
+     *               underlying stream handler's {@linkplain
+     *               URLStreamHandler#parseURL(URL, String, int, int)
+     *               parseURL method} throws {@code IllegalArgumentException}
      * @see        java.net.URL#URL(java.lang.String, java.lang.String,
      *                  int, java.lang.String)
      * @see        java.net.URLStreamHandler
      * @see        java.net.URLStreamHandler#parseURL(java.net.URL,
      *                  java.lang.String, int, int)
+     * @deprecated
+     * Use {@link #of(URI, URLStreamHandler)} to construct an instance of URL
+     * associated with a custom protocol handler.
+     * See the note on <a href="#constructor-deprecation">constructor deprecation</a>
+     * for more details.
      */
+    @Deprecated(since = "20")
     public URL(URL context, String spec, URLStreamHandler handler)
         throws MalformedURLException
     {
@@ -602,15 +674,6 @@ public final class URL implements java.io.Serializable {
         boolean aRef=false;
         boolean isRelative = false;
 
-        // Check for permission to specify a handler
-        if (handler != null) {
-            @SuppressWarnings("removal")
-            SecurityManager sm = System.getSecurityManager();
-            if (sm != null) {
-                checkSpecifyHandler(sm);
-            }
-        }
-
         try {
             limit = spec.length();
             while ((limit > 0) && (spec.charAt(limit - 1) <= ' ')) {
@@ -633,7 +696,7 @@ public final class URL implements java.io.Serializable {
             for (i = start ; !aRef && (i < limit) &&
                      ((c = spec.charAt(i)) != '/') ; i++) {
                 if (c == ':') {
-                    String s = toLowerCase(spec.substring(start, i));
+                    String s = lowerCaseProtocol(spec.substring(start, i));
                     if (isValidProtocol(s)) {
                         newProtocol = s;
                         start = i + 1;
@@ -713,23 +776,65 @@ public final class URL implements java.io.Serializable {
     }
 
     /**
-     * Creates a URL from a URI, as if by invoking {@code uri.toURL()}.
+     * Creates a URL from a URI, as if by invoking {@code uri.toURL()}, but
+     * associating it with the given {@code URLStreamHandler}, if allowed.
+     *
+     * @apiNote
+     * Applications should consider performing additional integrity
+     * checks before constructing a {@code URL} and opening a connection.
+     * See the <a href=#integrity>API note</a> in the class level API
+     * documentation.
+     *
+     * @implSpec The implementation of this method includes calling the {@link
+     * URLStreamHandler#parseURL(URL, String, int, int) parseURL} method on the
+     * selected handler.
+     *
+     * @param uri the {@code URI} from which the returned {@code URL} should
+     *           be built
+     * @param handler a custom protocol stream handler for
+     *                      the returned {@code URL}. Can be {@code null},
+     *                      in which case the default stream handler for
+     *                      the protocol if any, will be used.
+     *
+     * @return a new {@code URL} instance created from the given {@code URI}
+     *   and associated with the given {@code URLStreamHandler}, if any
+     *
+     * @throws NullPointerException if {@code uri} is {@code null}
+     *
+     * @throws IllegalArgumentException if no protocol is specified
+     *         (the {@linkplain URI#getScheme() uri scheme} is {@code null}), or
+     *         if the {@code URLStreamHandler} is not {@code null} and can not be
+     *         set for the given protocol
+     *
+     * @throws  MalformedURLException if an unknown protocol is found,
+     *          or the given URI fails to comply with the specific
+     *          syntax of the associated protocol, or the
+     *          underlying stream handler's {@linkplain
+     *          URLStreamHandler#parseURL(URL, String, int, int)
+     *          parseURL method} throws {@code IllegalArgumentException}
      *
      * @see java.net.URI#toURL()
+     *
+     * @since 20
      */
-    static URL fromURI(URI uri) throws MalformedURLException {
+    public static URL of(URI uri, URLStreamHandler handler)
+        throws MalformedURLException {
         if (!uri.isAbsolute()) {
             throw new IllegalArgumentException("URI is not absolute");
         }
+
         String protocol = uri.getScheme();
 
+        // fast path for canonical jrt:/... URLs
+        //
         // In general we need to go via Handler.parseURL, but for the jrt
         // protocol we enforce that the Handler is not overridable and can
         // optimize URI to URL conversion.
         //
         // Case-sensitive comparison for performance; malformed protocols will
         // be handled correctly by the slow path.
-        if (protocol.equals("jrt") && !uri.isOpaque()
+        if (handler == null && protocol.equals("jrt") && !uri.isOpaque()
+                && uri.getRawAuthority() == null
                 && uri.getRawFragment() == null) {
 
             String query = uri.getRawQuery();
@@ -745,9 +850,28 @@ public final class URL implements java.io.Serializable {
             int port = uri.getPort();
 
             return new URL("jrt", host, port, file, null);
-        } else {
-            return new URL((URL)null, uri.toString(), null);
         }
+
+        // slow path (will work for non-canonical forms of jrt: too)
+
+        if ("url".equalsIgnoreCase(protocol)) {;
+            String uristr = uri.toString();
+            try {
+                URI inner = new URI(uristr.substring(4));
+                if (inner.isAbsolute()) {
+                    protocol = inner.getScheme();
+                }
+            } catch (URISyntaxException use) {
+                throw new MalformedURLException(use.getMessage());
+            }
+        }
+
+        if (handler != null && !isOverrideable(protocol)) {
+            throw new IllegalArgumentException("Can't override URLStreamHandler for protocol "
+                    + protocol);
+        }
+
+        return new URL((URL)null, uri.toString(), handler);
     }
 
     /*
@@ -770,13 +894,6 @@ public final class URL implements java.io.Serializable {
         return true;
     }
 
-    /*
-     * Checks for permission to specify a stream handler.
-     */
-    private void checkSpecifyHandler(@SuppressWarnings("removal") SecurityManager sm) {
-        sm.checkPermission(SecurityConstants.SPECIFY_HANDLER_PERMISSION);
-    }
-
     /**
      * Sets the specified 8 fields of the URL. This is not a public method so
      * that only URLStreamHandlers can modify URL fields. URLs are otherwise
@@ -814,9 +931,8 @@ public final class URL implements java.io.Serializable {
 
     /**
      * Returns the address of the host represented by this URL.
-     * A {@link SecurityException} or an {@link UnknownHostException}
-     * while getting the host address will result in this method returning
-     * {@code null}
+     * An {@link UnknownHostException} while getting the host address
+     * will result in this method returning {@code null}.
      *
      * @return an {@link InetAddress} representing the host
      */
@@ -830,7 +946,7 @@ public final class URL implements java.io.Serializable {
         }
         try {
             hostAddress = InetAddress.getByName(host);
-        } catch (UnknownHostException | SecurityException ex) {
+        } catch (UnknownHostException e) {
             return null;
         }
         return hostAddress;
@@ -1057,7 +1173,8 @@ public final class URL implements java.io.Serializable {
         URI uri = new URI(toString());
         if (authority != null && isBuiltinStreamHandler(handler)) {
             String s = IPAddressUtil.checkAuthority(this);
-            if (s != null) throw new URISyntaxException(authority, s);
+            if (s != null)
+                throw new URISyntaxException(formatMsg("%s", filterNonSocketInfo(authority)), s);
         }
         return uri;
     }
@@ -1109,9 +1226,6 @@ public final class URL implements java.io.Serializable {
      *             Proxy.NO_PROXY should be specified.
      * @return     a {@code URLConnection} to the URL.
      * @throws     IOException  if an I/O exception occurs.
-     * @throws     SecurityException if a security manager is present
-     *             and the caller doesn't have permission to connect
-     *             to the proxy.
      * @throws     IllegalArgumentException will be thrown if proxy is null,
      *             or proxy has the wrong type
      * @throws     UnsupportedOperationException if the subclass that
@@ -1132,16 +1246,6 @@ public final class URL implements java.io.Serializable {
 
         // Create a copy of Proxy as a security measure
         Proxy p = proxy == Proxy.NO_PROXY ? Proxy.NO_PROXY : sun.net.ApplicationProxy.create(proxy);
-        @SuppressWarnings("removal")
-        SecurityManager sm = System.getSecurityManager();
-        if (p.type() != Proxy.Type.DIRECT && sm != null) {
-            InetSocketAddress epoint = (InetSocketAddress) p.address();
-            if (epoint.isUnresolved())
-                sm.checkConnect(epoint.getHostName(), epoint.getPort());
-            else
-                sm.checkConnect(epoint.getAddress().getHostAddress(),
-                                epoint.getPort());
-        }
         return handler.openConnection(this, p);
     }
 
@@ -1208,31 +1312,17 @@ public final class URL implements java.io.Serializable {
      *<p> The {@code URLStreamHandlerFactory} instance is used to
      *construct a stream protocol handler from a protocol name.
      *
-     * <p> If there is a security manager, this method first calls
-     * the security manager's {@code checkSetFactory} method
-     * to ensure the operation is allowed.
-     * This could result in a SecurityException.
-     *
      * @param      fac   the desired factory.
      * @throws     Error  if the application has already set a factory.
-     * @throws     SecurityException  if a security manager exists and its
-     *             {@code checkSetFactory} method doesn't allow
-     *             the operation.
      * @see        java.net.URL#URL(java.lang.String, java.lang.String,
      *             int, java.lang.String)
      * @see        java.net.URLStreamHandlerFactory
-     * @see        SecurityManager#checkSetFactory
      */
     public static void setURLStreamHandlerFactory(URLStreamHandlerFactory fac) {
         synchronized (streamHandlerLock) {
             if (factory != null) {
                 throw new Error("factory already defined");
             }
-            @SuppressWarnings("removal")
-            SecurityManager security = System.getSecurityManager();
-            if (security != null) {
-                security.checkSetFactory();
-            }
             handlers.clear();
 
             // safe publication of URLStreamHandlerFactory with volatile write
@@ -1268,9 +1358,8 @@ public final class URL implements java.io.Serializable {
     }
 
     private static URLStreamHandler lookupViaProperty(String protocol) {
-        String packagePrefixList =
-                GetPropertyAction.privilegedGetProperty(protocolPathProp);
-        if (packagePrefixList == null) {
+        String packagePrefixList = System.getProperty(protocolPathProp);
+        if (packagePrefixList == null || packagePrefixList.isEmpty()) {
             // not set
             return null;
         }
@@ -1279,6 +1368,9 @@ public final class URL implements java.io.Serializable {
         URLStreamHandler handler = null;
         for (int i=0; handler == null && i<packagePrefixes.length; i++) {
             String packagePrefix = packagePrefixes[i].trim();
+            if (packagePrefix.isEmpty()) {
+                continue;
+            }
             try {
                 String clsName = packagePrefix + "." + protocol + ".Handler";
                 Class<?> cls = null;
@@ -1302,47 +1394,6 @@ public final class URL implements java.io.Serializable {
         return handler;
     }
 
-    private static Iterator<URLStreamHandlerProvider> providers() {
-        return new Iterator<>() {
-
-            ClassLoader cl = ClassLoader.getSystemClassLoader();
-            ServiceLoader<URLStreamHandlerProvider> sl =
-                    ServiceLoader.load(URLStreamHandlerProvider.class, cl);
-            Iterator<URLStreamHandlerProvider> i = sl.iterator();
-
-            URLStreamHandlerProvider next = null;
-
-            private boolean getNext() {
-                while (next == null) {
-                    try {
-                        if (!i.hasNext())
-                            return false;
-                        next = i.next();
-                    } catch (ServiceConfigurationError sce) {
-                        if (sce.getCause() instanceof SecurityException) {
-                            // Ignore security exceptions
-                            continue;
-                        }
-                        throw sce;
-                    }
-                }
-                return true;
-            }
-
-            public boolean hasNext() {
-                return getNext();
-            }
-
-            public URLStreamHandlerProvider next() {
-                if (!getNext())
-                    throw new NoSuchElementException();
-                URLStreamHandlerProvider n = next;
-                next = null;
-                return n;
-            }
-        };
-    }
-
     private static class ThreadTrackHolder {
         static final ThreadTracker TRACKER = new ThreadTracker();
     }
@@ -1355,26 +1406,23 @@ public final class URL implements java.io.Serializable {
         ThreadTrackHolder.TRACKER.end(key);
     }
 
-    @SuppressWarnings("removal")
     private static URLStreamHandler lookupViaProviders(final String protocol) {
         Object key = tryBeginLookup();
         if (key == null) {
             throw new Error("Circular loading of URL stream handler providers detected");
         }
         try {
-            return AccessController.doPrivileged(
-                new PrivilegedAction<>() {
-                    public URLStreamHandler run() {
-                        Iterator<URLStreamHandlerProvider> itr = providers();
-                        while (itr.hasNext()) {
-                            URLStreamHandlerProvider f = itr.next();
-                            URLStreamHandler h = f.createURLStreamHandler(protocol);
-                            if (h != null)
-                                return h;
-                        }
-                        return null;
-                    }
-                });
+            final ClassLoader cl = ClassLoader.getSystemClassLoader();
+            final ServiceLoader<URLStreamHandlerProvider> sl =
+                    ServiceLoader.load(URLStreamHandlerProvider.class, cl);
+            final Iterator<URLStreamHandlerProvider> itr = sl.iterator();
+            while (itr.hasNext()) {
+                URLStreamHandlerProvider f = itr.next();
+                URLStreamHandler h = f.createURLStreamHandler(protocol);
+                if (h != null)
+                    return h;
+            }
+            return null;
         } finally {
             endLookup(key);
         }
@@ -1384,9 +1432,13 @@ public final class URL implements java.io.Serializable {
      * Returns the protocol in lower case. Special cases known protocols
      * to avoid loading locale classes during startup.
      */
-    static String toLowerCase(String protocol) {
-        if (protocol.equals("jrt") || protocol.equals("file") || protocol.equals("jar")) {
-            return protocol;
+    static String lowerCaseProtocol(String protocol) {
+        if (protocol.equals("jrt")) {
+            return "jrt";
+        } else if (protocol.equals("file")) {
+            return "file";
+        } else if (protocol.equals("jar")) {
+            return "jar";
         } else {
             return protocol.toLowerCase(Locale.ROOT);
         }
@@ -1419,7 +1471,7 @@ public final class URL implements java.io.Serializable {
     /**
      * A table of protocol handlers.
      */
-    static Hashtable<String,URLStreamHandler> handlers = new Hashtable<>();
+    private static final Hashtable<String, URLStreamHandler> handlers = new Hashtable<>();
     private static final Object streamHandlerLock = new Object();
 
     /**
@@ -1709,6 +1761,11 @@ public final class URL implements java.io.Serializable {
     }
 
     static {
+        runtimeSetup();
+    }
+
+    @AOTRuntimeSetup
+    private static void runtimeSetup() {
         SharedSecrets.setJavaNetURLAccess(
                 new JavaNetURLAccess() {
                     @Override
