@@ -40,7 +40,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import sun.security.ssl.NamedGroup.NamedGroupSpec;
-import sun.security.ssl.SupportedGroupsExtension.SupportedGroups;
 import sun.security.ssl.X509Authentication.X509Possession;
 import sun.security.util.KeyUtil;
 import sun.security.util.SignatureUtil;
@@ -147,12 +146,12 @@ enum SignatureScheme {
                                     "RSA", 511,
                                     ProtocolVersion.PROTOCOLS_TO_12);
 
-    final int id;                       // hash + signature
-    final String name;                  // literal name
-    private final String algorithm;     // signature algorithm
-    final String keyAlgorithm;          // signature key algorithm
-    private final SigAlgParamSpec signAlgParams;    // signature parameters
-    private final NamedGroup namedGroup;    // associated named group
+    final int id;                         // hash + signature
+    final String name;                    // literal name
+    final String algorithm;               // signature algorithm
+    final String keyAlgorithm;            // signature key algorithm
+    final SigAlgParamSpec signAlgParams;  // signature parameters
+    private final NamedGroup namedGroup;  // associated named group
 
     // The minimal required key size in bits.
     //
@@ -181,12 +180,12 @@ enum SignatureScheme {
             "anonymous",    "rsa",      "dsa",      "ecdsa",
         };
 
-    static enum SigAlgParamSpec {   // support RSASSA-PSS only now
+    enum SigAlgParamSpec {   // support RSASSA-PSS only now
         RSA_PSS_SHA256 ("SHA-256", 32),
         RSA_PSS_SHA384 ("SHA-384", 48),
         RSA_PSS_SHA512 ("SHA-512", 64);
 
-        private final AlgorithmParameterSpec parameterSpec;
+        final AlgorithmParameterSpec parameterSpec;
         private final AlgorithmParameters parameters;
         private final boolean isAvailable;
 
@@ -231,13 +230,13 @@ enum SignatureScheme {
     private static final Set<CryptoPrimitive> SIGNATURE_PRIMITIVE_SET =
             Set.of(CryptoPrimitive.SIGNATURE);
 
-    private SignatureScheme(int id, String name,
+    SignatureScheme(int id, String name,
             String algorithm, String keyAlgorithm,
             ProtocolVersion[] supportedProtocols) {
         this(id, name, algorithm, keyAlgorithm, -1, supportedProtocols);
     }
 
-    private SignatureScheme(int id, String name,
+    SignatureScheme(int id, String name,
             String algorithm, String keyAlgorithm,
             int minimalKeySize,
             ProtocolVersion[] supportedProtocols) {
@@ -245,7 +244,7 @@ enum SignatureScheme {
                 null, minimalKeySize, supportedProtocols);
     }
 
-    private SignatureScheme(int id, String name,
+    SignatureScheme(int id, String name,
             String algorithm, String keyAlgorithm,
             SigAlgParamSpec signAlgParamSpec, int minimalKeySize,
             ProtocolVersion[] supportedProtocols) {
@@ -254,7 +253,7 @@ enum SignatureScheme {
                 supportedProtocols, supportedProtocols);
     }
 
-    private SignatureScheme(int id, String name,
+    SignatureScheme(int id, String name,
             String algorithm, String keyAlgorithm,
             NamedGroup namedGroup,
             ProtocolVersion[] supportedProtocols) {
@@ -263,7 +262,7 @@ enum SignatureScheme {
                 supportedProtocols, supportedProtocols);
     }
 
-    private SignatureScheme(int id, String name,
+    SignatureScheme(int id, String name,
             String algorithm, String keyAlgorithm,
             SigAlgParamSpec signAlgParams,
             NamedGroup namedGroup, int minimalKeySize,
@@ -415,15 +414,10 @@ enum SignatureScheme {
             Set<SSLScope> scopes) {
         List<SignatureScheme> supported = new LinkedList<>();
 
-        // If config.signatureSchemes is non-empty then it means that
-        // it was defined by a System property.  Per
-        // SSLConfiguration.getCustomizedSignatureScheme() the list will
-        // only contain schemes that are in the enum.
-        // Otherwise, use the enum constants (converted to a List).
         List<SignatureScheme> schemesToCheck =
-                config.signatureSchemes.isEmpty() ?
+                config.signatureSchemes == null ?
                     Arrays.asList(SignatureScheme.values()) :
-                    config.signatureSchemes;
+                    namesOfAvailable(config.signatureSchemes);
 
         for (SignatureScheme ss: schemesToCheck) {
             if (!ss.isAvailable) {
@@ -476,8 +470,8 @@ enum SignatureScheme {
                             "Unsupported signature scheme: " +
                             SignatureScheme.nameOf(ssid));
                 }
-            } else if ((config.signatureSchemes.isEmpty()
-                        || config.signatureSchemes.contains(ss))
+            } else if ((config.signatureSchemes == null
+                        || Utilities.contains(config.signatureSchemes, ss.name))
                     && ss.isAllowed(constraints, protocolVersion, scopes)) {
                 supported.add(ss);
             } else {
@@ -508,6 +502,7 @@ enum SignatureScheme {
     }
 
     static Map.Entry<SignatureScheme, Signature> getSignerOfPreferableAlgorithm(
+            SSLConfiguration sslConfig,
             SSLAlgorithmConstraints constraints,
             List<SignatureScheme> schemes,
             X509Possession x509Possession,
@@ -560,7 +555,7 @@ enum SignatureScheme {
                     if (params != null) {
                         NamedGroup keyGroup = NamedGroup.valueOf(params);
                         if (keyGroup != null &&
-                                SupportedGroups.isSupported(keyGroup)) {
+                                NamedGroup.isEnabled(sslConfig, keyGroup)) {
                             Signature signer = ss.getSigner(signingKey);
                             if (signer != null) {
                                 return new SimpleImmutableEntry<>(ss, signer);
@@ -619,6 +614,33 @@ enum SignatureScheme {
         return new String[0];
     }
 
+    private static List<SignatureScheme> namesOfAvailable(
+                String[] signatureSchemes) {
+
+        if (signatureSchemes == null || signatureSchemes.length == 0) {
+            return Collections.emptyList();
+        }
+
+        List<SignatureScheme> sss = new ArrayList<>(signatureSchemes.length);
+        for (String ss : signatureSchemes) {
+            SignatureScheme scheme = SignatureScheme.nameOf(ss);
+            if (scheme == null || !scheme.isAvailable) {
+                if (SSLLogger.isOn &&
+                        SSLLogger.isOn("ssl,handshake,verbose")) {
+                    SSLLogger.finest(
+                            "Ignore the signature algorithm (" + ss
+                          + "), unsupported or unavailable");
+                }
+
+                continue;
+            }
+
+            sss.add(scheme);
+        }
+
+        return sss;
+    }
+
     // This method is used to get the signature instance of this signature
     // scheme for the specific public key.  Unlike getSigner(), the exception
     // is bubbled up.  If the public key does not support this signature
