@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -48,18 +48,6 @@ jint mtlPreviousOp = MTL_OP_INIT;
 
 extern void MTLGC_DestroyMTLGraphicsConfig(jlong pConfigInfo);
 
-bool isDrawOp (jint op) {
-     switch(op) {
-        case MTL_OP_DRAW_LINE:
-        case MTL_OP_DRAW_RECT:
-        case MTL_OP_DRAW_PARALLELOGRAM:
-        case MTL_OP_FILL_RECT:
-        case MTL_OP_FILL_PARALLELOGRAM:
-             return true;
-        default: return false;
-     }
-}
-
 void MTLRenderQueue_CheckPreviousOp(jint op) {
 
     if (mtlPreviousOp == op) {
@@ -67,16 +55,6 @@ void MTLRenderQueue_CheckPreviousOp(jint op) {
         return;
     }
 
-    if (isDrawOp(mtlPreviousOp)) {
-        // submit the vertex batch
-        MTLRenderer_SubmitVertexBatch(mtlc, dstOps);
-        if (isDrawOp(op)) {
-            // Do not cause endEncoder if we continue with Draw operations
-            mtlPreviousOp = op;
-            return;
-        }
-    }
-
     if (op == MTL_OP_SET_COLOR) {
         if (mtlPreviousOp != MTL_OP_MASK_OP) {
             return; // SET_COLOR should not cause endEncoder
@@ -87,8 +65,8 @@ void MTLRenderQueue_CheckPreviousOp(jint op) {
         return;
     }
 
-    J2dTraceLn1(J2D_TRACE_VERBOSE,
-                "MTLRenderQueue_CheckPreviousOp: new op=%d", op);
+    J2dTraceLn(J2D_TRACE_VERBOSE,
+               "MTLRenderQueue_CheckPreviousOp: new op=%d", op);
 
     switch (mtlPreviousOp) {
         case MTL_OP_INIT :
@@ -126,8 +104,8 @@ Java_sun_java2d_metal_MTLRenderQueue_flushBuffer
 {
     unsigned char *b, *end;
 
-    J2dTraceLn1(J2D_TRACE_INFO,
-                "MTLRenderQueue_flushBuffer: limit=%d", limit);
+    J2dTraceLn(J2D_TRACE_INFO,
+               "MTLRenderQueue_flushBuffer: limit=%d", limit);
 
     b = (unsigned char *)jlong_to_ptr(buf);
     if (b == NULL) {
@@ -141,16 +119,16 @@ Java_sun_java2d_metal_MTLRenderQueue_flushBuffer
         while (b < end) {
             jint opcode = NEXT_INT(b);
 
-            J2dTraceLn2(J2D_TRACE_VERBOSE,
-                    "MTLRenderQueue_flushBuffer: opcode=%d, rem=%d",
-                    opcode, (end-b));
+            J2dTraceLn(J2D_TRACE_VERBOSE,
+                       "MTLRenderQueue_flushBuffer: opcode=%d, rem=%d",
+                       opcode, (end-b));
 
             switch (opcode) {
 
                 // draw ops
                 case sun_java2d_pipe_BufferedOpCodes_DRAW_LINE:
                 {
-                    CHECK_PREVIOUS_OP(MTL_OP_DRAW_LINE);
+                    CHECK_PREVIOUS_OP(MTL_OP_OTHER);
 
                     if ([mtlc useXORComposite]) {
                         commitEncodedCommands();
@@ -166,10 +144,9 @@ Java_sun_java2d_metal_MTLRenderQueue_flushBuffer
                 }
                 case sun_java2d_pipe_BufferedOpCodes_DRAW_RECT:
                 {
-                    CHECK_PREVIOUS_OP(MTL_OP_DRAW_RECT);
+                    CHECK_PREVIOUS_OP(MTL_OP_OTHER);
 
                     if ([mtlc useXORComposite]) {
-
                         commitEncodedCommands();
                         J2dTraceLn(J2D_TRACE_VERBOSE,
                                    "DRAW_RECT in XOR mode - Force commit earlier draw calls before DRAW_RECT.");
@@ -251,7 +228,7 @@ Java_sun_java2d_metal_MTLRenderQueue_flushBuffer
                 }
                 case sun_java2d_pipe_BufferedOpCodes_DRAW_PARALLELOGRAM:
                 {
-                    CHECK_PREVIOUS_OP(MTL_OP_DRAW_PARALLELOGRAM);
+                    CHECK_PREVIOUS_OP(MTL_OP_OTHER);
 
                     if ([mtlc useXORComposite]) {
                         commitEncodedCommands();
@@ -299,7 +276,7 @@ Java_sun_java2d_metal_MTLRenderQueue_flushBuffer
                 // fill ops
                 case sun_java2d_pipe_BufferedOpCodes_FILL_RECT:
                 {
-                    CHECK_PREVIOUS_OP(MTL_OP_FILL_RECT);
+                    CHECK_PREVIOUS_OP(MTL_OP_OTHER);
 
                     if ([mtlc useXORComposite]) {
                         commitEncodedCommands();
@@ -331,7 +308,7 @@ Java_sun_java2d_metal_MTLRenderQueue_flushBuffer
                 }
                 case sun_java2d_pipe_BufferedOpCodes_FILL_PARALLELOGRAM:
                 {
-                    CHECK_PREVIOUS_OP(MTL_OP_FILL_PARALLELOGRAM);
+                    CHECK_PREVIOUS_OP(MTL_OP_OTHER);
 
                     if ([mtlc useXORComposite]) {
                         commitEncodedCommands();
@@ -605,7 +582,6 @@ Java_sun_java2d_metal_MTLRenderQueue_flushBuffer
                     jlong pDst = NEXT_LONG(b);
 
                     if (mtlc != NULL) {
-                        MTLRenderer_SubmitVertexBatch(mtlc, dstOps);
                         [mtlc.encoderManager endEncoder];
                         MTLCommandBufferWrapper * cbwrapper = [mtlc pullCommandBufferWrapper];
                         id<MTLCommandBuffer> commandbuf = [cbwrapper getCommandBuffer];
@@ -633,7 +609,6 @@ Java_sun_java2d_metal_MTLRenderQueue_flushBuffer
 
                         } else {
                             if (mtlc != NULL) {
-                                MTLRenderer_SubmitVertexBatch(mtlc, dstOps);
                                 [mtlc.encoderManager endEncoder];
                                 MTLCommandBufferWrapper * cbwrapper = [mtlc pullCommandBufferWrapper];
                                 id<MTLCommandBuffer> commandbuf = [cbwrapper getCommandBuffer];
@@ -893,8 +868,9 @@ Java_sun_java2d_metal_MTLRenderQueue_flushBuffer
                 }
 
                 default:
-                    J2dRlsTraceLn1(J2D_TRACE_ERROR,
-                        "MTLRenderQueue_flushBuffer: invalid opcode=%d", opcode);
+                    J2dRlsTraceLn(J2D_TRACE_ERROR,
+                                  "MTLRenderQueue_flushBuffer: invalid opcode=%d",
+                                  opcode);
                     return;
             }
         }
@@ -903,8 +879,6 @@ Java_sun_java2d_metal_MTLRenderQueue_flushBuffer
             if (mtlPreviousOp == MTL_OP_MASK_OP) {
                 MTLVertexCache_DisableMaskCache(mtlc);
             }
-            MTLRenderer_SubmitVertexBatch(mtlc, dstOps);
-
             [mtlc.encoderManager endEncoder];
             MTLCommandBufferWrapper * cbwrapper = [mtlc pullCommandBufferWrapper];
             id<MTLCommandBuffer> commandbuf = [cbwrapper getCommandBuffer];
@@ -946,13 +920,10 @@ MTLRenderQueue_GetCurrentDestination()
 }
 
 /**
- * commit earlier encoded commmands
+ * commit earlier encoded commands
  * these would be rendered to the back-buffer - which is read in shader while rendering in XOR mode
  */
 void commitEncodedCommands() {
-
-    MTLRenderer_SubmitVertexBatch(mtlc, dstOps);
-
     [mtlc.encoderManager endEncoder];
 
     MTLCommandBufferWrapper *cbwrapper = [mtlc pullCommandBufferWrapper];
