@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -26,15 +26,10 @@
 package sun.print;
 
 import java.io.BufferedReader;
-import java.io.FileInputStream;
-import java.io.InputStream;
-import java.io.InputStreamReader;
 import java.io.IOException;
+import java.net.MalformedURLException;
 import java.util.ArrayList;
 import java.util.Vector;
-import java.security.AccessController;
-import java.security.PrivilegedActionException;
-import java.security.PrivilegedExceptionAction;
 import javax.print.DocFlavor;
 import javax.print.MultiDocPrintService;
 import javax.print.PrintService;
@@ -54,11 +49,14 @@ import java.io.FileReader;
 import java.net.URL;
 import java.nio.file.Files;
 
+import sun.awt.OSInfo;
+import sun.awt.util.ThreadGroupUtils;
+
 /*
  * Remind: This class uses solaris commands. We also need a linux
  * version
  */
-public class PrintServiceLookupProvider extends PrintServiceLookup
+public final class PrintServiceLookupProvider extends PrintServiceLookup
     implements BackgroundServiceLookup, Runnable {
 
     /* Remind: the current implementation is static, as its assumed
@@ -75,10 +73,6 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
     private static final int DEFAULT_MINREFRESH = 120;  // 2 minutes
     private static int minRefreshTime = DEFAULT_MINREFRESH;
 
-    @SuppressWarnings("removal")
-    static String osname = java.security.AccessController.doPrivileged(
-            new sun.security.action.GetPropertyAction("os.name"));
-
     // List of commands used to deal with the printer queues on AIX
     String[] lpNameComAix = {
       "/usr/bin/lsallq",
@@ -97,9 +91,7 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
          * can be used to force the printing code to poll or not poll
          * for PrintServices.
          */
-        @SuppressWarnings("removal")
-        String pollStr = java.security.AccessController.doPrivileged(
-            new sun.security.action.GetPropertyAction("sun.java2d.print.polling"));
+        String pollStr = System.getProperty("sun.java2d.print.polling");
 
         if (pollStr != null) {
             if (pollStr.equalsIgnoreCase("true")) {
@@ -113,10 +105,7 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
          * can be used to specify minimum refresh time (in seconds)
          * for polling PrintServices.  The default is 120.
          */
-        @SuppressWarnings("removal")
-        String refreshTimeStr = java.security.AccessController.doPrivileged(
-            new sun.security.action.GetPropertyAction(
-                "sun.java2d.print.minRefreshTime"));
+        String refreshTimeStr = System.getProperty("sun.java2d.print.minRefreshTime");
 
         if (refreshTimeStr != null) {
             try {
@@ -134,9 +123,7 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
          * take lots of time if thousands of printers are attached to a server.
          */
         if (isAIX()) {
-            @SuppressWarnings("removal")
-            String aixPrinterEnumerator = java.security.AccessController.doPrivileged(
-                new sun.security.action.GetPropertyAction("sun.java2d.print.aix.lpstat"));
+            String aixPrinterEnumerator = System.getProperty("sun.java2d.print.aix.lpstat");
 
             if (aixPrinterEnumerator != null) {
                 if (aixPrinterEnumerator.equalsIgnoreCase("lpstat")) {
@@ -149,20 +136,20 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
     }
 
     static boolean isMac() {
-        return osname.startsWith("Mac");
+        return OSInfo.getOSType() == OSInfo.OSType.MACOSX;
     }
 
     static boolean isLinux() {
-        return (osname.equals("Linux"));
+        return OSInfo.getOSType() == OSInfo.OSType.LINUX;
     }
 
     static boolean isBSD() {
-        return (osname.equals("Linux") ||
-                osname.contains("OS X"));
+        return (OSInfo.getOSType() == OSInfo.OSType.LINUX ||
+                OSInfo.getOSType() == OSInfo.OSType.MACOSX);
     }
 
     static boolean isAIX() {
-        return osname.equals("AIX");
+        return OSInfo.getOSType() == OSInfo.OSType.AIX;
     }
 
     static final int UNINITIALIZED = -1;
@@ -204,12 +191,13 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
         return BSD_LPD;
     }
 
-
     public PrintServiceLookupProvider() {
         // start the printer listener thread
         if (pollServices) {
-            Thread thr = new Thread(null, new PrinterChangeListener(),
+            Thread thr = new Thread(ThreadGroupUtils.getRootThreadGroup(),
+                                    new PrinterChangeListener(),
                                     "PrinterListener", 0, false);
+            thr.setContextClassLoader(null);
             thr.setDaemon(true);
             thr.start();
             IPPPrintService.debug_println(debugPrefix+"polling turned on");
@@ -221,13 +209,8 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
      * This isn't required by the API and there's a risk doing this will
      * lead people to assume its guaranteed.
      */
+    @Override
     public synchronized PrintService[] getPrintServices() {
-        @SuppressWarnings("removal")
-        SecurityManager security = System.getSecurityManager();
-        if (security != null) {
-            security.checkPrintJobAccess();
-        }
-
         if (printServices == null || !pollServices) {
             refreshServices();
         }
@@ -471,7 +454,7 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
         if (CUPSPrinter.isCupsRunning()) {
             try {
                 return new IPPPrintService(name,
-                                           new URL("http://"+
+                                           newURL("http://"+
                                                    CUPSPrinter.getServer()+":"+
                                                    CUPSPrinter.getPort()+"/"+
                                                    name));
@@ -505,13 +488,13 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
          * Directly retrieve that service and confirm
          * that it meets the other requirements.
          * If printer name isn't mentioned then go a slow path checking
-         * all printers if they meet the reqiremements.
+         * all printers if they meet the requirements.
          */
         PrintService[] services;
         PrinterName name = (PrinterName)serviceSet.get(PrinterName.class);
         PrintService defService;
         if (name != null && (defService = getDefaultPrintService()) != null) {
-            /* To avoid execing a unix command  see if the client is asking
+            /* To avoid executing a unix command, see if the client is asking
              * for the default printer by name, since we already have that
              * initialised.
              */
@@ -559,13 +542,9 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
      * If service attributes are specified then there must be additional
      * filtering.
      */
+    @Override
     public PrintService[] getPrintServices(DocFlavor flavor,
                                            AttributeSet attributes) {
-        @SuppressWarnings("removal")
-        SecurityManager security = System.getSecurityManager();
-        if (security != null) {
-          security.checkPrintJobAccess();
-        }
         PrintRequestAttributeSet requestSet = null;
         PrintServiceAttributeSet serviceSet = null;
 
@@ -622,25 +601,16 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
     /*
      * return empty array as don't support multi docs
      */
+    @Override
     public MultiDocPrintService[]
         getMultiDocPrintServices(DocFlavor[] flavors,
                                  AttributeSet attributes) {
-        @SuppressWarnings("removal")
-        SecurityManager security = System.getSecurityManager();
-        if (security != null) {
-          security.checkPrintJobAccess();
-        }
         return new MultiDocPrintService[0];
     }
 
 
+    @Override
     public synchronized PrintService getDefaultPrintService() {
-        @SuppressWarnings("removal")
-        SecurityManager security = System.getSecurityManager();
-        if (security != null) {
-          security.checkPrintJobAccess();
-        }
-
         // clear defaultPrintService
         defaultPrintService = null;
         String psuri = null;
@@ -683,7 +653,7 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
                                                         psuri, true);
                     } else {
                         defaultPS = new IPPPrintService(defaultPrinter,
-                                            new URL("http://"+
+                                            newURL("http://"+
                                                     CUPSPrinter.getServer()+":"+
                                                     CUPSPrinter.getPort()+"/"+
                                                     defaultPrinter));
@@ -699,6 +669,7 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
         return defaultPrintService;
     }
 
+    @Override
     public synchronized void
         getServicesInbackground(BackgroundLookupListener listener) {
         if (printServices != null) {
@@ -729,6 +700,7 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
         }
     }
 
+    @Override
     public void run() {
         PrintService[] services = getPrintServices();
         synchronized (this) {
@@ -869,7 +841,6 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
         return printerNames.toArray(new String[printerNames.size()]);
     }
 
-    @SuppressWarnings("removal")
     static String[] execCmd(final String command) {
         ArrayList<String> results = null;
         try {
@@ -884,51 +855,46 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
                 cmd[2] = "LC_ALL=C " + command;
             }
 
-            results = AccessController.doPrivileged(
-                new PrivilegedExceptionAction<ArrayList<String>>() {
-                    public ArrayList<String> run() throws IOException {
+            Process proc;
+            BufferedReader bufferedReader = null;
+            File f = Files.createTempFile("prn", "xc")
+                          .toFile();
+            cmd[2] = cmd[2] + ">" + f.getAbsolutePath();
 
-                        Process proc;
-                        BufferedReader bufferedReader = null;
-                        File f = Files.createTempFile("prn","xc").toFile();
-                        cmd[2] = cmd[2]+">"+f.getAbsolutePath();
-
-                        proc = Runtime.getRuntime().exec(cmd);
-                        try {
-                            boolean done = false; // in case of interrupt.
-                            while (!done) {
-                                try {
-                                    proc.waitFor();
-                                    done = true;
-                                } catch (InterruptedException e) {
-                                }
-                            }
-
-                            if (proc.exitValue() == 0) {
-                                FileReader reader = new FileReader(f);
-                                bufferedReader = new BufferedReader(reader);
-                                String line;
-                                ArrayList<String> results = new ArrayList<>();
-                                while ((line = bufferedReader.readLine())
-                                       != null) {
-                                    results.add(line);
-                                }
-                                return results;
-                            }
-                        } finally {
-                            f.delete();
-                            // promptly close all streams.
-                            if (bufferedReader != null) {
-                                bufferedReader.close();
-                            }
-                            proc.getInputStream().close();
-                            proc.getErrorStream().close();
-                            proc.getOutputStream().close();
-                        }
-                        return null;
+            proc = Runtime.getRuntime().exec(cmd);
+            try {
+                boolean done = false; // in case of interrupt.
+                while (!done) {
+                    try {
+                        proc.waitFor();
+                        done = true;
+                    } catch (InterruptedException ignored) {
                     }
-                });
-        } catch (PrivilegedActionException e) {
+                }
+
+                if (proc.exitValue() == 0) {
+                    FileReader reader = new FileReader(f);
+                    bufferedReader = new BufferedReader(reader);
+                    String line;
+                    while ((line = bufferedReader.readLine())
+                           != null) {
+                        results.add(line);
+                    }
+                }
+            } finally {
+                f.delete();
+                // promptly close all streams.
+                if (bufferedReader != null) {
+                    bufferedReader.close();
+                }
+                proc.getInputStream()
+                    .close();
+                proc.getErrorStream()
+                    .close();
+                proc.getOutputStream()
+                    .close();
+            }
+        } catch (IOException ignored) {
         }
         if (results == null) {
             return new String[0];
@@ -937,7 +903,7 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
         }
     }
 
-    private class PrinterChangeListener implements Runnable {
+    private final class PrinterChangeListener implements Runnable {
 
         @Override
         public void run() {
@@ -965,4 +931,9 @@ public class PrintServiceLookupProvider extends PrintServiceLookup
             }
         }
     }
+
+    @SuppressWarnings("deprecation")
+    private static URL newURL(String spec) throws MalformedURLException {
+        return new URL(spec);
+    }
 }
