@@ -43,7 +43,6 @@ import javax.swing.Icon;
 import javax.swing.JComponent;
 import javax.swing.JMenu;
 import javax.swing.JMenuItem;
-import javax.swing.SwingConstants;
 import javax.swing.UIManager;
 import javax.swing.plaf.ComponentUI;
 import javax.swing.plaf.UIResource;
@@ -55,6 +54,7 @@ import com.sun.java.swing.plaf.windows.TMSchema.State;
 import com.sun.java.swing.plaf.windows.XPStyle.Skin;
 import sun.swing.MenuItemCheckIconFactory;
 import sun.swing.MenuItemLayoutHelper;
+import sun.swing.MnemonicHandler;
 import sun.swing.SwingUtilities2;
 
 /**
@@ -62,23 +62,26 @@ import sun.swing.SwingUtilities2;
  *
  * @author Igor Kushnirskiy
  */
-public class WindowsMenuItemUI extends BasicMenuItemUI {
+public final class WindowsMenuItemUI extends BasicMenuItemUI {
     /**
      * The instance of {@code PropertyChangeListener}.
      */
     private PropertyChangeListener changeListener;
 
     final WindowsMenuItemUIAccessor accessor =
-        new  WindowsMenuItemUIAccessor() {
+        new WindowsMenuItemUIAccessor() {
 
+            @Override
             public JMenuItem getMenuItem() {
                 return menuItem;
             }
 
+            @Override
             public State getState(JMenuItem menuItem) {
                 return WindowsMenuItemUI.getState(this, menuItem);
             }
 
+            @Override
             public Part getPart(JMenuItem menuItem) {
                 return WindowsMenuItemUI.getPart(this, menuItem);
             }
@@ -216,17 +219,8 @@ public class WindowsMenuItemUI extends BasicMenuItemUI {
 
         if (lh.getCheckIcon() != null && lh.useCheckAndArrow()) {
             Rectangle rect = lr.getTextRect();
-            if (menuItem.getComponentOrientation().isLeftToRight()) {
-                if (menuItem.getHorizontalTextPosition() != SwingConstants.LEADING
-                    && menuItem.getHorizontalTextPosition() != SwingConstants.LEFT) {
-                    rect.x += lh.getAfterCheckIconGap();
-                }
-            } else {
-                if (menuItem.getHorizontalTextPosition() != SwingConstants.LEADING
-                    && menuItem.getHorizontalTextPosition() != SwingConstants.RIGHT) {
-                    rect.x -= lh.getAfterCheckIconGap();
-                }
-            }
+
+            rect.x += lh.getAfterCheckIconGap();
 
             lr.setTextRect(rect);
         }
@@ -242,11 +236,7 @@ public class WindowsMenuItemUI extends BasicMenuItemUI {
         }
         if (lh.getCheckIcon() != null && lh.useCheckAndArrow()) {
             Rectangle rect = lr.getAccRect();
-            if (menuItem.getComponentOrientation().isLeftToRight()) {
-                rect.x += lh.getAfterCheckIconGap();
-            } else {
-                rect.x -= lh.getAfterCheckIconGap();
-            }
+            rect.x += lh.getAfterCheckIconGap();
             lr.setAccRect(rect);
         }
         SwingUtilities3.paintAccText(g, lh, lr, disabledForeground,
@@ -267,6 +257,7 @@ public class WindowsMenuItemUI extends BasicMenuItemUI {
      * @param textRect Bounding rectangle to render the text.
      * @param text String to render
      */
+    @Override
     protected void paintText(Graphics g, JMenuItem menuItem,
                              Rectangle textRect, String text) {
         if (WindowsMenuItemUI.isVistaPainting()) {
@@ -330,7 +321,7 @@ public class WindowsMenuItemUI extends BasicMenuItemUI {
             FontMetrics fm = SwingUtilities2.getFontMetrics(menuItem, g);
             int mnemIndex = menuItem.getDisplayedMnemonicIndex();
             // W2K Feature: Check to see if the Underscore should be rendered.
-            if (WindowsLookAndFeel.isMnemonicHidden() == true) {
+            if (MnemonicHandler.isMnemonicHidden()) {
                 mnemIndex = -1;
             }
             WindowsGraphicsUtils.paintXPText(menuItem,
