@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2000, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,16 +33,19 @@ import sun.security.jgss.spi.*;
 import sun.security.jgss.TokenTracker;
 import sun.security.krb5.*;
 import java.io.InputStream;
-import java.io.OutputStream;
+import java.io.InvalidObjectException;
 import java.io.IOException;
+import java.io.ObjectInputStream;
+import java.io.OutputStream;
 import java.security.*;
 import javax.security.auth.Subject;
-import javax.security.auth.kerberos.ServicePermission;
 import javax.security.auth.kerberos.KerberosCredMessage;
 import javax.security.auth.kerberos.KerberosPrincipal;
 import javax.security.auth.kerberos.KerberosTicket;
 import sun.security.krb5.internal.AuthorizationData;
 
+import static sun.security.krb5.internal.Krb5.DEBUG;
+
 /**
  * Implements the mechanism specific context class for the Kerberos v5
  * GSS-API mechanism.
@@ -121,7 +124,6 @@ class Krb5Context implements GSSContextSpi {
     private Credentials serviceCreds;
     private KrbApReq apReq;
     private final GSSCaller caller;
-    private static final boolean DEBUG = Krb5Util.DEBUG;
 
     /**
      * Constructor for Krb5Context to be called on the context initiator's
@@ -379,8 +381,8 @@ class Krb5Context implements GSSContextSpi {
     }
 
     final void resetMySequenceNumber(int seqNumber) {
-        if (DEBUG) {
-            System.out.println("Krb5Context setting mySeqNumber to: "
+        if (DEBUG != null) {
+            DEBUG.println("Krb5Context setting mySeqNumber to: "
                                + seqNumber);
         }
         synchronized (mySeqNumberLock) {
@@ -389,8 +391,8 @@ class Krb5Context implements GSSContextSpi {
     }
 
     final void resetPeerSequenceNumber(int seqNumber) {
-        if (DEBUG) {
-            System.out.println("Krb5Context setting peerSeqNumber to: "
+        if (DEBUG != null) {
+            DEBUG.println("Krb5Context setting peerSeqNumber to: "
                                + seqNumber);
         }
         synchronized (peerSeqNumberLock) {
@@ -534,8 +536,8 @@ class Krb5Context implements GSSContextSpi {
         // We will only try constrained delegation once (if necessary).
         if (!isConstrainedDelegationTried) {
             if (delegatedCred == null) {
-                if (DEBUG) {
-                    System.out.println(">>> Constrained deleg from " + caller);
+                if (DEBUG != null) {
+                    DEBUG.println(">>> Constrained deleg from " + caller);
                 }
                 // The constrained delegation part. The acceptor needs to have
                 // isInitiator=true in order to get a TGT, either earlier at
@@ -593,8 +595,8 @@ class Krb5Context implements GSSContextSpi {
             byte[] retVal = null;
             InitialToken token = null;
             int errorCode = GSSException.FAILURE;
-            if (DEBUG) {
-                System.out.println("Entered Krb5Context.initSecContext with " +
+            if (DEBUG != null) {
+                DEBUG.println("Entered Krb5Context.initSecContext with " +
                                    "state=" + printState(state));
             }
             if (!isInitiator()) {
@@ -628,8 +630,6 @@ class Krb5Context implements GSSContextSpi {
                         tgt = proxyCreds.self.getKrb5Credentials();
                     }
 
-                    checkPermission(peerName.getKrb5PrincipalName().getName(),
-                                    "initiate");
                     /*
                      * If useSubjectCredsonly is true then
                      * we check whether we already have the ticket
@@ -638,36 +638,24 @@ class Krb5Context implements GSSContextSpi {
 
                     if (GSSUtil.useSubjectCredsOnly(caller)) {
                         KerberosTicket kerbTicket = null;
-                        try {
-                           // get service ticket from caller's subject
-                           @SuppressWarnings("removal")
-                           var tmp = AccessController.doPrivilegedWithCombiner(
-                                new PrivilegedExceptionAction<KerberosTicket>() {
-                                public KerberosTicket run() throws Exception {
-                                    // XXX to be cleaned
-                                    // highly consider just calling:
-                                    // Subject.getSubject
-                                    // SubjectComber.find
-                                    // instead of Krb5Util.getServiceTicket
-                                    return Krb5Util.getServiceTicket(
-                                        GSSCaller.CALLER_UNKNOWN,
-                                        // since it's useSubjectCredsOnly here,
-                                        // don't worry about the null
-                                        proxyCreds == null ?
-                                            myName.getKrb5PrincipalName().getName():
-                                            proxyCreds.getName().getKrb5PrincipalName().getName(),
-                                        peerName.getKrb5PrincipalName().getName());
-                                }});
-                            kerbTicket = tmp;
-                        } catch (PrivilegedActionException e) {
-                            if (DEBUG) {
-                                System.out.println("Attempt to obtain service"
-                                        + " ticket from the subject failed!");
-                            }
-                        }
+                        // get service ticket from caller's subject
+                        // XXX to be cleaned
+                        // highly consider just calling:
+                        // Subject.getSubject
+                        // SubjectComber.find
+                        // instead of Krb5Util.getServiceTicket
+                        kerbTicket = Krb5Util.getServiceTicket(
+                            GSSCaller.CALLER_UNKNOWN,
+                            // since it's useSubjectCredsOnly here,
+                            // don't worry about the null
+                            proxyCreds == null ?
+                                myName.getKrb5PrincipalName().getName():
+                                proxyCreds.getName().getKrb5PrincipalName().getName(),
+                            peerName.getKrb5PrincipalName().getName());
+
                         if (kerbTicket != null) {
-                            if (DEBUG) {
-                                System.out.println("Found service ticket in " +
+                            if (DEBUG != null) {
+                                DEBUG.println("Found service ticket in " +
                                                    "the subject" +
                                                    kerbTicket);
                             }
@@ -681,8 +669,8 @@ class Krb5Context implements GSSContextSpi {
                     if (serviceCreds == null) {
                         // either we did not find the serviceCreds in the
                         // Subject or useSubjectCreds is false
-                        if (DEBUG) {
-                            System.out.println("Service ticket not found in " +
+                        if (DEBUG != null) {
+                            DEBUG.println("Service ticket not found in " +
                                                "the subject");
                         }
                         // Get Service ticket using the Kerberos protocols
@@ -698,10 +686,7 @@ class Krb5Context implements GSSContextSpi {
                                     tgt);
                         }
                         if (GSSUtil.useSubjectCredsOnly(caller)) {
-                            @SuppressWarnings("removal")
-                            final Subject subject =
-                                AccessController.doPrivilegedWithCombiner(
-                                        (PrivilegedAction<Subject>) Subject::current);
+                            Subject subject = Subject.current();
                             if (subject != null &&
                                 !subject.isReadOnly()) {
                                 /*
@@ -711,18 +696,13 @@ class Krb5Context implements GSSContextSpi {
                                  * successfully established; however it is easier
                                  * to do it here and there is no harm.
                                  */
-                                final KerberosTicket kt =
+                                KerberosTicket kt =
                                         Krb5Util.credsToTicket(serviceCreds);
-                                @SuppressWarnings("removal")
-                                var dummy = AccessController.doPrivileged (
-                                        (PrivilegedAction<Void>) () -> {
-                                          subject.getPrivateCredentials().add(kt);
-                                          return null;
-                                        });
+                                subject.getPrivateCredentials().add(kt);
                             } else {
                                 // log it for debugging purpose
-                                if (DEBUG) {
-                                    System.out.println("Subject is " +
+                                if (DEBUG != null) {
+                                    DEBUG.println("Subject is " +
                                         "readOnly;Kerberos Service "+
                                         "ticket not stored");
                                 }
@@ -738,8 +718,8 @@ class Krb5Context implements GSSContextSpi {
                     if (!getMutualAuthState()) {
                         state = STATE_DONE;
                     }
-                    if (DEBUG) {
-                        System.out.println("Created InitSecContextToken:\n"+
+                    if (DEBUG != null) {
+                        DEBUG.println("Created InitSecContextToken:\n"+
                             new HexDumpEncoder().encodeBuffer(retVal));
                     }
                 } else if (state == STATE_IN_PROCESS) {
@@ -750,12 +730,12 @@ class Krb5Context implements GSSContextSpi {
                     state = STATE_DONE;
                 } else {
                     // XXX Use logging API?
-                    if (DEBUG) {
-                        System.out.println(state);
+                    if (DEBUG != null) {
+                        DEBUG.println("state is " + state);
                     }
                 }
             } catch (KrbException e) {
-                if (DEBUG) {
+                if (DEBUG != null) {
                     e.printStackTrace();
                 }
                 GSSException gssException =
@@ -792,8 +772,8 @@ class Krb5Context implements GSSContextSpi {
 
         byte[] retVal = null;
 
-        if (DEBUG) {
-            System.out.println("Entered Krb5Context.acceptSecContext with " +
+        if (DEBUG != null) {
+            DEBUG.println("Entered Krb5Context.acceptSecContext with " +
                                "state=" +  printState(state));
         }
 
@@ -813,11 +793,6 @@ class Krb5Context implements GSSContextSpi {
                 }
                 myName = (Krb5NameElement) myCred.getName();
 
-                // If there is already a bound name, check now
-                if (myName != null) {
-                    Krb5MechFactory.checkAcceptCredPermission(myName, myName);
-                }
-
                 InitSecContextToken token = new InitSecContextToken(this,
                                                     (Krb5AcceptCredential) myCred, is);
                 PrincipalName clientName = token.getKrbApReq().getClient();
@@ -827,7 +802,6 @@ class Krb5Context implements GSSContextSpi {
                 if (myName == null) {
                     myName = Krb5NameElement.getInstance(
                         token.getKrbApReq().getCreds().getServer());
-                    Krb5MechFactory.checkAcceptCredPermission(myName, myName);
                 }
 
                 if (getMutualAuthState()) {
@@ -839,8 +813,8 @@ class Krb5Context implements GSSContextSpi {
                 state = STATE_DONE;
             } else  {
                 // XXX Use logging API?
-                if (DEBUG) {
-                    System.out.println(state);
+                if (DEBUG != null) {
+                    DEBUG.println("state is " + state);
                 }
             }
         } catch (KrbException e) {
@@ -849,8 +823,8 @@ class Krb5Context implements GSSContextSpi {
             gssException.initCause(e);
             throw gssException;
         } catch (IOException e) {
-            if (DEBUG) {
-                e.printStackTrace();
+            if (DEBUG != null) {
+                e.printStackTrace(DEBUG.getPrintStream());
             }
             GSSException gssException =
                 new GSSException(GSSException.FAILURE, -1, e.getMessage());
@@ -898,15 +872,11 @@ class Krb5Context implements GSSContextSpi {
 
     public final byte[] wrap(byte[] inBuf, int offset, int len,
                              MessageProp msgProp) throws GSSException {
-        if (DEBUG) {
-            System.out.println("Krb5Context.wrap: data=["
-                               + getHexBytes(inBuf, offset, len)
-                               + "]");
-        }
 
-        if (state != STATE_DONE)
-        throw new GSSException(GSSException.NO_CONTEXT, -1,
-                               "Wrap called in invalid state!");
+        if (state != STATE_DONE) {
+            throw new GSSException(GSSException.NO_CONTEXT, -1,
+                    "Wrap called in invalid state!");
+        }
 
         byte[] encToken = null;
         try {
@@ -919,8 +889,8 @@ class Krb5Context implements GSSContextSpi {
                         new WrapToken_v2(this, msgProp, inBuf, offset, len);
                 encToken = token.encode();
             }
-            if (DEBUG) {
-                System.out.println("Krb5Context.wrap: token=["
+            if (DEBUG != null) {
+                DEBUG.println("Krb5Context.wrap: token=["
                                    + getHexBytes(encToken, 0, encToken.length)
                                    + "]");
             }
@@ -952,8 +922,8 @@ class Krb5Context implements GSSContextSpi {
                         new WrapToken_v2(this, msgProp, inBuf, inOffset, len);
                 retVal = token.encode(outBuf, outOffset);
             }
-            if (DEBUG) {
-                System.out.println("Krb5Context.wrap: token=["
+            if (DEBUG != null) {
+                DEBUG.println("Krb5Context.wrap: token=["
                                    + getHexBytes(outBuf, outOffset, retVal)
                                    + "]");
             }
@@ -980,14 +950,14 @@ class Krb5Context implements GSSContextSpi {
                 WrapToken token =
                         new WrapToken(this, msgProp, inBuf, offset, len);
                 token.encode(os);
-                if (DEBUG) {
+                if (DEBUG != null) {
                     encToken = token.encode();
                 }
             } else if (cipherHelper.getProto() == 1) {
                 WrapToken_v2 token =
                         new WrapToken_v2(this, msgProp, inBuf, offset, len);
                 token.encode(os);
-                if (DEBUG) {
+                if (DEBUG != null) {
                     encToken = token.encode();
                 }
             }
@@ -998,8 +968,8 @@ class Krb5Context implements GSSContextSpi {
             throw gssException;
         }
 
-        if (DEBUG) {
-            System.out.println("Krb5Context.wrap: token=["
+        if (DEBUG != null) {
+            DEBUG.println("Krb5Context.wrap: token=["
                         + getHexBytes(encToken, 0, encToken.length)
                         + "]");
         }
@@ -1025,8 +995,8 @@ class Krb5Context implements GSSContextSpi {
                                MessageProp msgProp)
         throws GSSException {
 
-            if (DEBUG) {
-                System.out.println("Krb5Context.unwrap: token=["
+            if (DEBUG != null) {
+                DEBUG.println("Krb5Context.unwrap: token=["
                                    + getHexBytes(inBuf, offset, len)
                                    + "]");
             }
@@ -1049,12 +1019,6 @@ class Krb5Context implements GSSContextSpi {
                 setSequencingAndReplayProps(token, msgProp);
             }
 
-            if (DEBUG) {
-                System.out.println("Krb5Context.unwrap: data=["
-                                   + getHexBytes(data, 0, data.length)
-                                   + "]");
-            }
-
             return data;
         }
 
@@ -1329,16 +1293,6 @@ class Krb5Context implements GSSContextSpi {
         }
     }
 
-    private void checkPermission(String principal, String action) {
-        @SuppressWarnings("removal")
-        SecurityManager sm = System.getSecurityManager();
-        if (sm != null) {
-            ServicePermission perm =
-                new ServicePermission(principal, action);
-            sm.checkPermission(perm);
-        }
-    }
-
     private static String getHexBytes(byte[] bytes, int pos, int len) {
 
         StringBuilder sb = new StringBuilder();
@@ -1404,8 +1358,22 @@ class Krb5Context implements GSSContextSpi {
 
         @Override
         public String toString() {
-            return "Kerberos session key: etype: " + key.getEType() + "\n" +
-                    new HexDumpEncoder().encodeBuffer(key.getBytes());
+            return "Kerberos session key: etype=" + key.getEType()
+                    + ", " + Krb5Util.keyInfo(key.getBytes());
+        }
+
+        /**
+         * Restores the state of this object from the stream.
+         *
+         * @param  stream the {@code ObjectInputStream} from which data is read
+         * @throws IOException if an I/O error occurs
+         * @throws ClassNotFoundException if a serialized class cannot be loaded
+         */
+        @java.io.Serial
+        private void readObject(ObjectInputStream stream)
+                throws IOException, ClassNotFoundException {
+            throw new InvalidObjectException
+                    ("KerberosSessionKey not directly deserializable");
         }
     }
 
@@ -1476,5 +1444,4 @@ class Krb5Context implements GSSContextSpi {
     public void setAuthzData(AuthorizationData authzData) {
         this.authzData = authzData;
     }
-
 }
