@@ -34,6 +34,7 @@ import com.sun.tools.javac.tree.*;
 import com.sun.tools.javac.util.*;
 import com.sun.tools.javac.util.JCDiagnostic.DiagnosticPosition;
 import com.sun.tools.javac.util.JCDiagnostic.Error;
+import com.sun.tools.javac.code.Source.Feature;
 
 import com.sun.tools.javac.code.Symbol.*;
 import com.sun.tools.javac.code.Type.*;
@@ -55,6 +56,8 @@ import static com.sun.tools.javac.code.TypeTag.TYPEVAR;
 public class MemberEnter extends JCTree.Visitor {
     protected static final Context.Key<MemberEnter> memberEnterKey = new Context.Key<>();
 
+    /** The Source language setting. */
+    private final Source source;
     private final Enter enter;
     private final Log log;
     private final Check chk;
@@ -62,7 +65,7 @@ public class MemberEnter extends JCTree.Visitor {
     private final Symtab syms;
     private final Annotate annotate;
     private final Types types;
-    private final DeferredLintHandler deferredLintHandler;
+    private final Names names;
 
     public static MemberEnter instance(Context context) {
         MemberEnter instance = context.get(memberEnterKey);
@@ -71,6 +74,7 @@ public class MemberEnter extends JCTree.Visitor {
         return instance;
     }
 
+    @SuppressWarnings("this-escape")
     protected MemberEnter(Context context) {
         context.put(memberEnterKey, this);
         enter = Enter.instance(context);
@@ -80,7 +84,8 @@ public class MemberEnter extends JCTree.Visitor {
         syms = Symtab.instance(context);
         annotate = Annotate.instance(context);
         types = Types.instance(context);
-        deferredLintHandler = DeferredLintHandler.instance(context);
+        source = Source.instance(context);
+        names = Names.instance(context);
     }
 
     /** Construct method type from method signature.
@@ -178,7 +183,7 @@ public class MemberEnter extends JCTree.Visitor {
     public void visitMethodDef(JCMethodDecl tree) {
         WriteableScope enclScope = enter.enterScope(env);
         MethodSymbol m = new MethodSymbol(0, tree.name, null, enclScope.owner);
-        m.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, m, tree);
+        m.flags_field = chk.checkFlags(tree.mods.flags, m, tree);
         tree.sym = m;
 
         //if this is a default method, add the DEFAULT flag to the enclosing interface
@@ -187,16 +192,11 @@ public class MemberEnter extends JCTree.Visitor {
         }
 
         Env<AttrContext> localEnv = methodEnv(tree, env);
-        DiagnosticPosition prevLintPos = deferredLintHandler.setPos(tree.pos());
-        try {
-            // Compute the method type
-            m.type = signature(m, tree.typarams, tree.params,
-                               tree.restype, tree.recvparam,
-                               tree.thrown,
-                               localEnv);
-        } finally {
-            deferredLintHandler.setPos(prevLintPos);
-        }
+        // Compute the method type
+        m.type = signature(m, tree.typarams, tree.params,
+                           tree.restype, tree.recvparam,
+                           tree.thrown,
+                           localEnv);
 
         if (types.isSignaturePolymorphic(m)) {
             m.flags_field |= SIGNATURE_POLYMORPHIC;
@@ -220,14 +220,14 @@ public class MemberEnter extends JCTree.Visitor {
         enclScope.enter(m);
         }
 
-        annotate.annotateLater(tree.mods.annotations, localEnv, m, tree.pos());
+        annotate.annotateLater(tree.mods.annotations, localEnv, m);
         // Visit the signature of the method. Note that
         // TypeAnnotate doesn't descend into the body.
-        annotate.queueScanTreeAndTypeAnnotate(tree, localEnv, m, tree.pos());
+        annotate.queueScanTreeAndTypeAnnotate(tree, localEnv, m);
 
         if (tree.defaultValue != null) {
             m.defaultValue = annotate.unfinishedDefaultValue(); // set it to temporary sentinel for now
-            annotate.annotateDefaultValueLater(tree.defaultValue, localEnv, m, tree.pos());
+            annotate.annotateDefaultValueLater(tree.defaultValue, localEnv, m);
         }
     }
 
@@ -256,18 +256,13 @@ public class MemberEnter extends JCTree.Visitor {
             localEnv = env.dup(tree, env.info.dup());
             localEnv.info.staticLevel++;
         }
-        DiagnosticPosition prevLintPos = deferredLintHandler.setPos(tree.pos());
 
-        try {
-            if (TreeInfo.isEnumInit(tree)) {
-                attr.attribIdentAsEnumType(localEnv, (JCIdent)tree.vartype);
-            } else if (!tree.isImplicitlyTyped()) {
-                attr.attribType(tree.vartype, localEnv);
-                if (TreeInfo.isReceiverParam(tree))
-                    checkReceiver(tree, localEnv);
-            }
-        } finally {
-            deferredLintHandler.setPos(prevLintPos);
+        if (TreeInfo.isEnumInit(tree)) {
+            attr.attribIdentAsEnumType(localEnv, (JCIdent)tree.vartype);
+        } else if (!tree.isImplicitlyTyped()) {
+            attr.attribType(tree.vartype, localEnv);
+            if (TreeInfo.isReceiverParam(tree))
+                checkReceiver(tree, localEnv);
         }
 
         if ((tree.mods.flags & VARARGS) != 0) {
@@ -284,8 +279,9 @@ public class MemberEnter extends JCTree.Visitor {
         Type vartype = tree.isImplicitlyTyped()
                 ? env.info.scope.owner.kind == MTH ? Type.noType : syms.errType
                 : tree.vartype.type;
-        VarSymbol v = new VarSymbol(0, tree.name, vartype, enclScope.owner);
-        v.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, v, tree);
+        Name name = tree.name;
+        VarSymbol v = new VarSymbol(0, name, vartype, enclScope.owner);
+        v.flags_field = chk.checkFlags(tree.mods.flags, v, tree);
         tree.sym = v;
         if (tree.init != null) {
             v.flags_field |= HASINIT;
@@ -296,17 +292,20 @@ public class MemberEnter extends JCTree.Visitor {
                 v.setLazyConstValue(initEnv(tree, initEnv), env, attr, tree);
             }
         }
-        if (chk.checkUnique(tree.pos(), v, enclScope)) {
-            chk.checkTransparentVar(tree.pos(), v, enclScope);
-            enclScope.enter(v);
-        } else if (v.owner.kind == MTH || (v.flags_field & (Flags.PRIVATE | Flags.FINAL | Flags.GENERATED_MEMBER | Flags.RECORD)) != 0) {
-            // if this is a parameter or a field obtained from a record component, enter it
-            enclScope.enter(v);
+
+        if(!(Feature.UNNAMED_VARIABLES.allowedInSource(source) && tree.sym.isUnnamedVariable())) {
+            if (chk.checkUnique(tree.pos(), v, enclScope)) {
+                chk.checkTransparentVar(tree.pos(), v, enclScope);
+                enclScope.enter(v);
+            } else if (v.owner.kind == MTH || (v.flags_field & (Flags.PRIVATE | Flags.FINAL | Flags.GENERATED_MEMBER | Flags.RECORD)) != 0) {
+                // if this is a parameter or a field obtained from a record component, enter it
+                enclScope.enter(v);
+            }
         }
 
-        annotate.annotateLater(tree.mods.annotations, localEnv, v, tree.pos());
+        annotate.annotateLater(tree.mods.annotations, localEnv, v);
         if (!tree.isImplicitlyTyped()) {
-            annotate.queueScanTreeAndTypeAnnotate(tree.vartype, localEnv, v, tree.pos());
+            annotate.queueScanTreeAndTypeAnnotate(tree.vartype, localEnv, v);
         }
 
         v.pos = tree.pos;
