@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2024, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -57,7 +57,6 @@ import jdk.javadoc.internal.doclets.toolkit.util.DocPath;
 import jdk.javadoc.internal.html.Content;
 import jdk.javadoc.internal.html.ContentBuilder;
 import jdk.javadoc.internal.html.HtmlAttr;
-import jdk.javadoc.internal.html.HtmlTag;
 import jdk.javadoc.internal.html.HtmlTree;
 import jdk.javadoc.internal.html.Text;
 
@@ -78,6 +77,9 @@ public class ClassWriter extends SubWriterHolderWriter {
                      "java.lang.constant.ConstantDesc",
                      "java.io.Serializable");
 
+    /* Length threshold to determine whether to insert whitespace between type parameters */
+    protected static final int LONG_TYPE_PARAM = 8;
+
     protected final TypeElement typeElement;
 
     protected final ClassTree classTree;
@@ -92,10 +94,9 @@ public class ClassWriter extends SubWriterHolderWriter {
                        ClassTree classTree) {
         super(configuration, configuration.docPaths.forClass(typeElement));
         this.typeElement = typeElement;
-        configuration.currentTypeElement = typeElement;
         this.classTree = classTree;
 
-        pHelper = new PropertyUtils.PropertyHelper(configuration, typeElement);
+        pHelper = configuration.propertyUtils.getPropertyHelper(typeElement);
 
         switch (typeElement.getKind()) {
             case ENUM   -> setEnumDocumentation(typeElement);
@@ -459,9 +460,16 @@ public class ClassWriter extends SubWriterHolderWriter {
                     .linkToSelf(false);  // Let's not link to ourselves in the header
             content.add("<");
             var first = true;
+            boolean longTypeParams = typeParams.stream()
+                    .map(t -> getLink(linkInfo.forType(t.asType())))
+                    .anyMatch(t -> t.charCount() > ClassWriter.LONG_TYPE_PARAM);
             for (TypeParameterElement t : typeParams) {
                 if (!first) {
-                    content.add(",").add(HtmlTree.WBR());
+                    if (longTypeParams) {
+                        content.add(", ");
+                    } else {
+                        content.add(",").add(HtmlTree.WBR());
+                    }
                 }
                 var typeParamLink = getLink(linkInfo.forType(t.asType()));
                 content.add(needsId
@@ -497,7 +505,7 @@ public class ClassWriter extends SubWriterHolderWriter {
     }
 
     @Override
-    public TypeElement getCurrentPageElement() {
+    public TypeElement getCurrentTypeElement() {
         return typeElement;
     }
 
@@ -508,13 +516,12 @@ public class ClassWriter extends SubWriterHolderWriter {
 
     protected void addClassDescription(Content classInfo) {
         addPreviewInfo(classInfo);
-        tableOfContents.addLink(HtmlIds.TOP_OF_PAGE, contents.descriptionLabel);
+        tableOfContents.addLink(HtmlIds.TOP_OF_PAGE, contents.descriptionLabel,
+                TableOfContents.Level.FIRST);
         if (!options.noComment()) {
             // generate documentation for the class.
             if (!utils.getFullBody(typeElement).isEmpty()) {
-                tableOfContents.pushNestedList();
                 addInlineComment(typeElement, classInfo);
-                tableOfContents.popNestedList();
             }
         }
     }
@@ -526,9 +533,7 @@ public class ClassWriter extends SubWriterHolderWriter {
     protected void addClassTagInfo(Content classInfo) {
         if (!options.noComment()) {
             // Print Information about all the tags here
-            tableOfContents.pushNestedList();
             addTagsInfo(typeElement, classInfo);
-            tableOfContents.popNestedList();
         }
     }
 
@@ -731,16 +736,14 @@ public class ClassWriter extends SubWriterHolderWriter {
             }
             // TODO: should we simply split this method up to avoid instanceof ?
             if (type instanceof TypeElement te) {
-                Content link = getLink(
-                        new HtmlLinkInfo(configuration, context, te));
-                content.add(HtmlTree.CODE(link));
+                content.add(getLink(
+                        new HtmlLinkInfo(configuration, context, te)));
             } else {
-                Content link = getLink(
-                        new HtmlLinkInfo(configuration, context, ((TypeMirror)type)));
-                content.add(HtmlTree.CODE(link));
+                content.add(getLink(
+                        new HtmlLinkInfo(configuration, context, ((TypeMirror)type))));
             }
         }
-        return content;
+        return HtmlTree.CODE(content);
     }
 
     /**
