@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -33,26 +33,26 @@ import java.util.function.Predicate;
 import java.util.regex.Pattern;
 import java.util.stream.Collectors;
 
-import javax.lang.model.element.Element;
 import javax.lang.model.element.ModuleElement;
 import javax.lang.model.element.PackageElement;
 import javax.lang.model.element.TypeElement;
 
 import com.sun.source.doctree.DeprecatedTree;
 import com.sun.source.doctree.DocTree;
-import jdk.javadoc.internal.doclets.formats.html.markup.BodyContents;
-import jdk.javadoc.internal.doclets.formats.html.markup.ContentBuilder;
-import jdk.javadoc.internal.doclets.formats.html.markup.Entity;
-import jdk.javadoc.internal.doclets.formats.html.markup.HtmlStyle;
-import jdk.javadoc.internal.doclets.formats.html.markup.TagName;
-import jdk.javadoc.internal.doclets.formats.html.markup.HtmlTree;
+
 import jdk.javadoc.internal.doclets.formats.html.Navigation.PageMode;
-import jdk.javadoc.internal.doclets.formats.html.markup.Text;
+import jdk.javadoc.internal.doclets.formats.html.markup.BodyContents;
+import jdk.javadoc.internal.doclets.formats.html.markup.HtmlStyles;
 import jdk.javadoc.internal.doclets.toolkit.DocletException;
 import jdk.javadoc.internal.doclets.toolkit.util.CommentHelper;
 import jdk.javadoc.internal.doclets.toolkit.util.DocFileIOException;
 import jdk.javadoc.internal.doclets.toolkit.util.DocPath;
 import jdk.javadoc.internal.doclets.toolkit.util.DocPaths;
+import jdk.javadoc.internal.html.Content;
+import jdk.javadoc.internal.html.ContentBuilder;
+import jdk.javadoc.internal.html.HtmlTag;
+import jdk.javadoc.internal.html.HtmlTree;
+import jdk.javadoc.internal.html.Text;
 
 /**
  * Class to generate file for each package contents in the right-hand
@@ -72,7 +72,7 @@ public class PackageWriter extends HtmlDocletWriter {
     /**
      * The HTML element for the section tag being written.
      */
-    private final HtmlTree section = HtmlTree.SECTION(HtmlStyle.packageDescription, new ContentBuilder());
+    private final HtmlTree section = HtmlTree.SECTION(HtmlStyles.packageDescription, new ContentBuilder());
 
     private final BodyContents bodyContents = new BodyContents();
 
@@ -128,10 +128,12 @@ public class PackageWriter extends HtmlDocletWriter {
      */
     protected void buildContent() {
         Content packageContent = getContentHeader();
-
-        addPackageSignature(packageContent);
-        buildPackageDescription(packageContent);
-        buildPackageTags(packageContent);
+        packageContent.add(HtmlTree.HR());
+        Content div = HtmlTree.DIV(HtmlStyles.horizontalScroll);
+        addPackageSignature(div);
+        buildPackageDescription(div);
+        buildPackageTags(div);
+        packageContent.add(div);
         buildSummary(packageContent);
 
         addPackageContent(packageContent);
@@ -177,6 +179,8 @@ public class PackageWriter extends HtmlDocletWriter {
      *                       be added
      */
     protected void buildPackageDescription(Content packageContent) {
+        tableOfContents.addLink(HtmlIds.TOP_OF_PAGE, contents.navDescription,
+                TableOfContents.Level.FIRST);
         if (options.noComment()) {
             return;
         }
@@ -198,23 +202,14 @@ public class PackageWriter extends HtmlDocletWriter {
     protected Content getPackageHeader() {
         String packageName = getLocalizedPackageName(packageElement).toString();
         HtmlTree body = getBody(getWindowTitle(packageName));
-        var div = HtmlTree.DIV(HtmlStyle.header);
-        if (configuration.showModules) {
-            ModuleElement mdle = configuration.docEnv.getElementUtils().getModuleOf(packageElement);
-            var classModuleLabel = HtmlTree.SPAN(HtmlStyle.moduleLabelInPackage, contents.moduleLabel);
-            var moduleNameDiv = HtmlTree.DIV(HtmlStyle.subTitle, classModuleLabel);
-            moduleNameDiv.add(Entity.NO_BREAK_SPACE);
-            moduleNameDiv.add(getModuleLink(mdle,
-                    Text.of(mdle.getQualifiedName().toString())));
-            div.add(moduleNameDiv);
-        }
+        var div = HtmlTree.DIV(HtmlStyles.header);
         Content packageHead = new ContentBuilder();
         if (!packageElement.isUnnamed()) {
             packageHead.add(contents.packageLabel).add(" ");
         }
         packageHead.add(packageName);
         var tHeading = HtmlTree.HEADING_TITLE(Headings.PAGE_TITLE_HEADING,
-                HtmlStyle.title, packageHead);
+                HtmlStyles.title, packageHead);
         div.add(tHeading);
         bodyContents.setHeader(getHeader(PageMode.PACKAGE, packageElement))
                 .addMainContent(div);
@@ -273,21 +268,6 @@ public class PackageWriter extends HtmlDocletWriter {
         return packages;
     }
 
-    @Override
-    protected Navigation getNavBar(PageMode pageMode, Element element) {
-        Content linkContent = getModuleLink(utils.elementUtils.getModuleOf(packageElement),
-                contents.moduleLabel);
-        return super.getNavBar(pageMode, element)
-                .setNavLinkModule(linkContent)
-                .setSubNavLinks(() -> List.of(
-                        links.createLink(HtmlIds.PACKAGE_DESCRIPTION, contents.navDescription,
-                                !utils.getFullBody(packageElement).isEmpty() && !options.noComment()),
-                        links.createLink(HtmlIds.RELATED_PACKAGE_SUMMARY, contents.relatedPackages,
-                                relatedPackages != null && !relatedPackages.isEmpty()),
-                        links.createLink(HtmlIds.CLASS_SUMMARY, contents.navClassesAndInterfaces,
-                                allClasses != null && !allClasses.isEmpty())));
-    }
-
     /**
      * Add the package deprecation information to the documentation tree.
      *
@@ -297,8 +277,8 @@ public class PackageWriter extends HtmlDocletWriter {
         List<? extends DeprecatedTree> deprs = utils.getDeprecatedTrees(packageElement);
         if (utils.isDeprecated(packageElement)) {
             CommentHelper ch = utils.getCommentHelper(packageElement);
-            var deprDiv = HtmlTree.DIV(HtmlStyle.deprecationBlock);
-            var deprPhrase = HtmlTree.SPAN(HtmlStyle.deprecatedLabel, getDeprecatedPhrase(packageElement));
+            var deprDiv = HtmlTree.DIV(HtmlStyles.deprecationBlock);
+            var deprPhrase = HtmlTree.SPAN(HtmlStyles.deprecatedLabel, getDeprecatedPhrase(packageElement));
             deprDiv.add(deprPhrase);
             if (!deprs.isEmpty()) {
                 List<? extends DocTree> commentTags = ch.getDescription(deprs.get(0));
@@ -311,7 +291,7 @@ public class PackageWriter extends HtmlDocletWriter {
     }
 
     protected Content getSummariesList() {
-        return HtmlTree.UL(HtmlStyle.summaryList);
+        return HtmlTree.UL(HtmlStyles.summaryList);
     }
 
     protected void addRelatedPackagesSummary(Content summaryContent) {
@@ -319,8 +299,7 @@ public class PackageWriter extends HtmlDocletWriter {
         TableHeader tableHeader= showModules
                 ? new TableHeader(contents.moduleLabel, contents.packageLabel, contents.descriptionLabel)
                 : new TableHeader(contents.packageLabel, contents.descriptionLabel);
-        addPackageSummary(relatedPackages, contents.relatedPackages, tableHeader,
-                summaryContent, showModules);
+        addRelatedPackageSummary(tableHeader, summaryContent, showModules);
     }
 
     /**
@@ -329,9 +308,9 @@ public class PackageWriter extends HtmlDocletWriter {
      * @param target the content to which the links will be added
      */
     public void addAllClassesAndInterfacesSummary(Content target) {
-        var table = new Table<TypeElement>(HtmlStyle.summaryTable)
+        var table = new Table<TypeElement>(HtmlStyles.summaryTable)
                 .setHeader(new TableHeader(contents.classLabel, contents.descriptionLabel))
-                .setColumnStyles(HtmlStyle.colFirst, HtmlStyle.colLast)
+                .setColumnStyles(HtmlStyles.colFirst, HtmlStyles.colLast)
                 .setId(HtmlIds.CLASS_SUMMARY)
                 .setDefaultTab(contents.allClassesAndInterfacesLabel)
                 .addTab(contents.interfaces, utils::isPlainInterface)
@@ -359,25 +338,28 @@ public class PackageWriter extends HtmlDocletWriter {
             }
         }
         if (!table.isEmpty()) {
+            tableOfContents.addLink(HtmlIds.CLASS_SUMMARY, contents.navClassesAndInterfaces,
+                    TableOfContents.Level.FIRST);
             target.add(HtmlTree.LI(table));
         }
     }
 
-    protected void addPackageSummary(List<PackageElement> packages, Content label,
-                                  TableHeader tableHeader, Content summaryContent,
-                                  boolean showModules) {
-        if (!packages.isEmpty()) {
-            var table = new Table<Void>(HtmlStyle.summaryTable)
+    protected void addRelatedPackageSummary(TableHeader tableHeader, Content summaryContent,
+                                     boolean showModules) {
+        if (!relatedPackages.isEmpty()) {
+            tableOfContents.addLink(HtmlIds.RELATED_PACKAGE_SUMMARY, contents.relatedPackages,
+                    TableOfContents.Level.FIRST);
+            var table = new Table<Void>(HtmlStyles.summaryTable)
                     .setId(HtmlIds.RELATED_PACKAGE_SUMMARY)
-                    .setCaption(label)
+                    .setCaption(contents.relatedPackages)
                     .setHeader(tableHeader);
             if (showModules) {
-                table.setColumnStyles(HtmlStyle.colPlain, HtmlStyle.colFirst, HtmlStyle.colLast);
+                table.setColumnStyles(HtmlStyles.colPlain, HtmlStyles.colFirst, HtmlStyles.colLast);
             } else {
-                table.setColumnStyles(HtmlStyle.colFirst, HtmlStyle.colLast);
+                table.setColumnStyles(HtmlStyles.colFirst, HtmlStyles.colLast);
             }
 
-            for (PackageElement pkg : packages) {
+            for (PackageElement pkg : relatedPackages) {
                 Content packageLink = getPackageLink(pkg, Text.of(pkg.getQualifiedName()));
                 Content moduleLink = Text.EMPTY;
                 if (showModules) {
@@ -422,12 +404,12 @@ public class PackageWriter extends HtmlDocletWriter {
     }
 
     protected void addPackageSignature(Content packageContent) {
-        packageContent.add(new HtmlTree(TagName.HR));
         packageContent.add(Signatures.getPackageSignature(packageElement, this));
     }
 
     protected void addPackageContent(Content packageContent) {
         bodyContents.addMainContent(packageContent);
+        bodyContents.setSideContent(tableOfContents.toContent(false));
     }
 
     protected void addPackageFooter() {
@@ -443,7 +425,7 @@ public class PackageWriter extends HtmlDocletWriter {
     }
 
     protected Content getPackageSummary(Content summaryContent) {
-        return HtmlTree.SECTION(HtmlStyle.summary, summaryContent);
+        return HtmlTree.SECTION(HtmlStyles.summary, summaryContent);
     }
 
     private boolean hasRelatedPackagesInOtherModules(List<PackageElement> relatedPackages) {
@@ -456,4 +438,9 @@ public class PackageWriter extends HtmlDocletWriter {
                 .filter(p -> p != packageElement && filter.test(p))
                 .collect(Collectors.toList());
     }
+
+    @Override
+    public boolean isIndexable() {
+        return true;
+    }
 }
