@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -75,8 +75,10 @@ import com.sun.source.doctree.LinkTree;
 import com.sun.source.doctree.LiteralTree;
 import com.sun.source.doctree.ParamTree;
 import com.sun.source.doctree.ProvidesTree;
+import com.sun.source.doctree.RawTextTree;
 import com.sun.source.doctree.ReferenceTree;
 import com.sun.source.doctree.ReturnTree;
+import com.sun.source.doctree.SeeTree;
 import com.sun.source.doctree.SerialDataTree;
 import com.sun.source.doctree.SerialFieldTree;
 import com.sun.source.doctree.SinceTree;
@@ -99,9 +101,16 @@ import com.sun.tools.javac.util.Assert;
 import com.sun.tools.javac.util.DefinedBy;
 import com.sun.tools.javac.util.DefinedBy.Api;
 
-import jdk.javadoc.internal.doclint.HtmlTag.AttrKind;
-import jdk.javadoc.internal.doclint.HtmlTag.ElemKind;
-import static jdk.javadoc.internal.doclint.Messages.Group.*;
+import jdk.javadoc.internal.html.HtmlAttr;
+import jdk.javadoc.internal.html.HtmlAttr.AttrKind;
+import jdk.javadoc.internal.html.HtmlTag;
+import jdk.javadoc.internal.html.HtmlTag.ElemKind;
+
+import static jdk.javadoc.internal.doclint.Messages.Group.ACCESSIBILITY;
+import static jdk.javadoc.internal.doclint.Messages.Group.HTML;
+import static jdk.javadoc.internal.doclint.Messages.Group.MISSING;
+import static jdk.javadoc.internal.doclint.Messages.Group.REFERENCE;
+import static jdk.javadoc.internal.doclint.Messages.Group.SYNTAX;
 
 
 /**
@@ -130,12 +139,12 @@ public class Checker extends DocTreePathScanner<Void, Void> {
     static class TagStackItem {
         final DocTree tree; // typically, but not always, StartElementTree
         final HtmlTag tag;
-        final Set<HtmlTag.Attr> attrs;
+        final Set<HtmlAttr> attrs;
         final Set<Flag> flags;
         TagStackItem(DocTree tree, HtmlTag tag) {
             this.tree = tree;
             this.tag = tag;
-            attrs = EnumSet.noneOf(HtmlTag.Attr.class);
+            attrs = EnumSet.noneOf(HtmlAttr.class);
             flags = EnumSet.noneOf(Flag.class);
         }
         @Override
@@ -150,6 +159,7 @@ public class Checker extends DocTreePathScanner<Void, Void> {
     private int implicitHeadingRank;
     private boolean inIndex;
     private boolean inLink;
+    private boolean inSee;
     private boolean inSummary;
 
     // <editor-fold defaultstate="collapsed" desc="Top level">
@@ -192,7 +202,8 @@ public class Checker extends DocTreePathScanner<Void, Void> {
                     if (isNormalClass(p.getParentPath())) {
                         reportMissing("dc.default.constructor");
                     }
-                } else if (!isOverridingMethod && !isSynthetic() && !isAnonymous() && !isRecordComponentOrField()) {
+                } else if (!isOverridingMethod && !isSynthetic() && !isAnonymous() && !isRecordComponentOrField()
+                        && !isImplicitlyDeclaredClass(env.currPath.getLeaf())) {
                     reportMissing("dc.missing.comment");
                 }
                 return null;
@@ -310,6 +321,12 @@ public class Checker extends DocTreePathScanner<Void, Void> {
         env.messages.report(REFERENCE, Kind.WARNING, env.currPath.getLeaf(), code, args);
     }
 
+    @Override
+    public Void scan(DocTreePath path, Void unused) {
+        // interposition point for all scans
+        return super.scan(path, unused);
+    }
+
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitDocComment(DocCommentTree tree, Void ignore) {
         scan(tree.getFirstSentence(), ignore);
@@ -389,7 +406,7 @@ public class Checker extends DocTreePathScanner<Void, Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitStartElement(StartElementTree tree, Void ignore) {
         final Name treeName = tree.getName();
-        final HtmlTag t = HtmlTag.get(treeName);
+        final HtmlTag t = HtmlTag.of(treeName);
         if (t == null) {
             env.messages.error(HTML, tree, "dc.tag.unknown", treeName);
         } else if (t.elemKind == ElemKind.HTML4) {
@@ -462,7 +479,7 @@ public class Checker extends DocTreePathScanner<Void, Void> {
                     }
 
                     case IMG -> {
-                        if (!top.attrs.contains(HtmlTag.Attr.ALT))
+                        if (!top.attrs.contains(HtmlAttr.ALT))
                             env.messages.error(ACCESSIBILITY, tree, "dc.no.alt.attr.for.image");
                     }
                 }
@@ -582,7 +599,7 @@ public class Checker extends DocTreePathScanner<Void, Void> {
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitEndElement(EndElementTree tree, Void ignore) {
         final Name treeName = tree.getName();
-        final HtmlTag t = HtmlTag.get(treeName);
+        final HtmlTag t = HtmlTag.of(treeName);
         if (t == null) {
             env.messages.error(HTML, tree, "dc.tag.unknown", treeName);
         } else if (t.endKind == HtmlTag.EndKind.NONE) {
@@ -595,7 +612,7 @@ public class Checker extends DocTreePathScanner<Void, Void> {
                     switch (t) {
                         case TABLE -> {
                             if (!top.flags.contains(Flag.TABLE_IS_PRESENTATION)
-                                    && !top.attrs.contains(HtmlTag.Attr.SUMMARY)
+                                    && !top.attrs.contains(HtmlAttr.SUMMARY)
                                     && !top.flags.contains(Flag.TABLE_HAS_CAPTION)) {
                                 env.messages.error(ACCESSIBILITY, tree,
                                         "dc.no.summary.or.caption.for.table");
@@ -661,7 +678,7 @@ public class Checker extends DocTreePathScanner<Void, Void> {
 
     // <editor-fold defaultstate="collapsed" desc="HTML attributes">
 
-    @Override @DefinedBy(Api.COMPILER_TREE) @SuppressWarnings("fallthrough")
+    @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitAttribute(AttributeTree tree, Void ignore) {
         // for now, ensure we're in an HTML StartElementTree;
         // in time, we might check uses of attributes in other tree nodes
@@ -672,7 +689,7 @@ public class Checker extends DocTreePathScanner<Void, Void> {
         HtmlTag currTag = tagStack.peek().tag;
         if (currTag != null && currTag.elemKind != ElemKind.HTML4) {
             Name name = tree.getName();
-            HtmlTag.Attr attr = currTag.getAttr(name);
+            HtmlAttr attr = currTag.getAttr(name);
             if (attr != null) {
                 boolean first = tagStack.peek().attrs.add(attr);
                 if (!first)
@@ -680,7 +697,9 @@ public class Checker extends DocTreePathScanner<Void, Void> {
             }
             // for now, doclint allows all attribute names beginning with "on" as event handler names,
             // without checking the validity or applicability of the name
-            if (!name.toString().startsWith("on")) {
+            // custom "data-*" attributes are also accepted
+            var attrName = name.toString();
+            if (!attrName.startsWith("on") && !attrName.startsWith("data-")) {
                 AttrKind k = currTag.getAttrKind(name);
                 switch (k) {
                     case OK -> { }
@@ -746,19 +765,21 @@ public class Checker extends DocTreePathScanner<Void, Void> {
                             String v = getAttrValue(tree);
                             try {
                                 if (v == null || (!v.isEmpty() && Integer.parseInt(v) != 1)) {
-                                    env.messages.error(HTML, tree, "dc.attr.table.border.not.valid", attr);
+                                    env.messages.error(HTML, tree, "dc.attr.table.border.not.valid",
+                                            (v == null ? tree : v));
                                 }
                             } catch (NumberFormatException ex) {
-                                env.messages.error(HTML, tree, "dc.attr.table.border.not.number", attr);
+                                env.messages.error(HTML, tree, "dc.attr.table.border.not.number", v);
                             }
                         } else if (currTag == HtmlTag.IMG) {
                             String v = getAttrValue(tree);
                             try {
                                 if (v == null || (!v.isEmpty() && Integer.parseInt(v) != 0)) {
-                                    env.messages.error(HTML, tree, "dc.attr.img.border.not.valid", attr);
+                                    env.messages.error(HTML, tree, "dc.attr.img.border.not.valid",
+                                            (v == null ? tree : v));
                                 }
                             } catch (NumberFormatException ex) {
-                                env.messages.error(HTML, tree, "dc.attr.img.border.not.number", attr);
+                                env.messages.error(HTML, tree, "dc.attr.img.border.not.number", v);
                             }
                         }
                     }
@@ -895,6 +916,16 @@ public class Checker extends DocTreePathScanner<Void, Void> {
         }
     }
 
+    @Override
+    public Void visitSee(SeeTree node, Void unused) {
+        try {
+            inSee = true;
+            return super.visitSee(node, unused);
+        } finally {
+            inSee = false;
+        }
+    }
+
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitLiteral(LiteralTree tree, Void ignore) {
         markEnclosingTag(Flag.HAS_INLINE_TAG);
@@ -976,6 +1007,9 @@ public class Checker extends DocTreePathScanner<Void, Void> {
         Element e = env.trees.getElement(getCurrentPath());
         if (e == null) {
             reportBadReference(tree);
+        } else if ((inLink || inSee)
+                && e.getKind() == ElementKind.CLASS && e.asType().getKind() != TypeKind.DECLARED) {
+            reportBadReference(tree);
         }
         return super.visitReference(tree, ignore);
     }
@@ -1005,8 +1039,11 @@ public class Checker extends DocTreePathScanner<Void, Void> {
             env.messages.warning(REFERENCE, tree, "dc.exists.return");
         }
         if (tree.isInline()) {
-            DocCommentTree dct = getCurrentPath().getDocComment();
-            if (dct.getFirstSentence().isEmpty() || tree != dct.getFirstSentence().get(0)) {
+            var dct = getCurrentPath().getDocComment();
+            var first = dct.getFirstSentence().stream()
+                    .filter(t -> !isBlank(t))
+                    .findFirst();
+            if (first.isEmpty() || first.get() != tree) {
                 env.messages.warning(SYNTAX, tree, "dc.return.not.first");
             }
         }
@@ -1020,6 +1057,14 @@ public class Checker extends DocTreePathScanner<Void, Void> {
         return super.visitReturn(tree, ignore);
     }
 
+    private static boolean isBlank(DocTree t) {
+        return switch (t.getKind()) {
+            case TEXT -> ((TextTree) t).getBody().isBlank();
+            case MARKDOWN -> ((RawTextTree) t).getContent().isBlank();
+            default -> false;
+        };
+    }
+
     @Override @DefinedBy(Api.COMPILER_TREE)
     public Void visitSerialData(SerialDataTree tree, Void ignore) {
         warnIfEmpty(tree, tree.getDescription());
@@ -1248,14 +1293,8 @@ public class Checker extends DocTreePathScanner<Void, Void> {
     }
 
     private boolean isDefaultConstructor() {
-        if (env.currElement.getKind() == ElementKind.CONSTRUCTOR) {
-            // A synthetic default constructor has the same pos as the
-            // enclosing class
-            TreePath p = env.currPath;
-            return env.getPos(p) == env.getPos(p.getParentPath());
-        } else {
-            return false;
-        }
+        return env.currElement.getKind() == ElementKind.CONSTRUCTOR
+                && env.elements.getOrigin(env.currElement) == Elements.Origin.MANDATED;
     }
 
     private boolean isDeclaredType() {
@@ -1281,11 +1320,20 @@ public class Checker extends DocTreePathScanner<Void, Void> {
     private boolean isNormalClass(TreePath p) {
         return switch (p.getLeaf().getKind()) {
             case ENUM, RECORD -> false;
-            case CLASS -> true;
+            case CLASS -> !isImplicitlyDeclaredClass(p.getLeaf());
             default -> throw new IllegalArgumentException(p.getLeaf().getKind().name());
         };
     }
 
+    /*
+     * If a similar query is ever added to com.sun.source.tree, use that instead.
+     */
+    private boolean isImplicitlyDeclaredClass(Tree t) {
+        return t.getKind() == Tree.Kind.CLASS
+                && t instanceof com.sun.tools.javac.tree.JCTree.JCClassDecl classDecl
+                && (classDecl.mods.flags & com.sun.tools.javac.code.Flags.IMPLICIT_CLASS) != 0;
+    }
+
     void markEnclosingTag(Flag flag) {
         TagStackItem top = tagStack.peek();
         if (top != null)
