@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -25,6 +25,7 @@
 
 package jdk.jpackage.internal;
 
+
 import java.io.File;
 import java.io.IOException;
 import java.nio.file.Files;
@@ -38,23 +39,22 @@ import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
-import java.util.function.BiFunction;
-import java.util.function.Function;
-import java.util.stream.Collectors;
 import java.util.stream.Stream;
+import jdk.jpackage.internal.model.ConfigException;
+import jdk.jpackage.internal.model.ExternalApplication;
+import static jdk.jpackage.internal.ApplicationLayoutUtils.PLATFORM_APPLICATION_LAYOUT;
+import static jdk.jpackage.internal.model.RuntimeBuilder.getDefaultModulePath;
 
 /**
- * StandardBundlerParam
+ * Standard bundler parameters.
  *
- * A parameter to a bundler.
- *
- * Also contains static definitions of all of the common bundler parameters.
+ * Contains static definitions of all of the common bundler parameters.
  * (additional platform specific and mode specific bundler parameters
  * are defined in each of the specific bundlers)
  *
  * Also contains static methods that operate on maps of parameters.
  */
-class StandardBundlerParam<T> extends BundlerParamInfo<T> {
+final class StandardBundlerParam {
 
     private static final String JAVABASEJMOD = "java.base.jmod";
     private static final String DEFAULT_VERSION = "1.0";
@@ -65,79 +65,78 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
             "--no-man-pages",
             "--no-header-files"};
 
-    StandardBundlerParam(String id, Class<T> valueType,
-            Function<Map<String, ? super Object>, T> defaultValueFunction,
-            BiFunction<String, Map<String, ? super Object>, T> stringConverter)
-    {
-        this.id = id;
-        this.valueType = valueType;
-        this.defaultValueFunction = defaultValueFunction;
-        this.stringConverter = stringConverter;
-    }
+    static final BundlerParamInfo<LauncherData> LAUNCHER_DATA = BundlerParamInfo.createBundlerParam(
+            LauncherData.class, LauncherData::create);
 
-    static final StandardBundlerParam<LauncherData> LAUNCHER_DATA =
-            new StandardBundlerParam<>(
-                    "launcherData",
-                    LauncherData.class,
-                    params -> {
-                        try {
-                            return LauncherData.create(params);
-                        } catch (ConfigException | IOException ex) {
-                            throw new RuntimeException(ex);
-                        }
-                    },
-                    null
-            );
-
-    static final StandardBundlerParam<Path> SOURCE_DIR =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<Path> SOURCE_DIR =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.INPUT.getId(),
                     Path.class,
                     p -> null,
                     (s, p) -> Path.of(s)
             );
 
+    static final BundlerParamInfo<Path> OUTPUT_DIR =
+            new BundlerParamInfo<>(
+                    Arguments.CLIOptions.OUTPUT.getId(),
+                    Path.class,
+                    p -> Path.of("").toAbsolutePath(),
+                    (s, p) -> Path.of(s)
+            );
+
     // note that each bundler is likely to replace this one with
     // their own converter
-    static final StandardBundlerParam<Path> MAIN_JAR =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<Path> MAIN_JAR =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.MAIN_JAR.getId(),
                     Path.class,
                     params -> LAUNCHER_DATA.fetchFrom(params).mainJarName(),
                     null
             );
 
-    static final StandardBundlerParam<String> MAIN_CLASS =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<Path> PREDEFINED_APP_IMAGE =
+            new BundlerParamInfo<>(
+            Arguments.CLIOptions.PREDEFINED_APP_IMAGE.getId(),
+            Path.class,
+            params -> null,
+            (s, p) -> Path.of(s));
+
+    static final BundlerParamInfo<ExternalApplication> PREDEFINED_APP_IMAGE_FILE = BundlerParamInfo.createBundlerParam(
+            ExternalApplication.class, params -> {
+                if (hasPredefinedAppImage(params)) {
+                    var appImage = PREDEFINED_APP_IMAGE.fetchFrom(params);
+                    return AppImageFile.load(appImage, PLATFORM_APPLICATION_LAYOUT);
+                } else {
+                    return null;
+                }
+            });
+
+    static final BundlerParamInfo<String> MAIN_CLASS =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.APPCLASS.getId(),
                     String.class,
                     params -> {
                         if (isRuntimeInstaller(params)) {
                             return null;
+                        } else if (hasPredefinedAppImage(params)) {
+                            PREDEFINED_APP_IMAGE_FILE.fetchFrom(params).getMainClass();
                         }
                         return LAUNCHER_DATA.fetchFrom(params).qualifiedClassName();
                     },
                     (s, p) -> s
             );
 
-    static final StandardBundlerParam<Path> PREDEFINED_RUNTIME_IMAGE =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<Path> PREDEFINED_RUNTIME_IMAGE =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.PREDEFINED_RUNTIME_IMAGE.getId(),
                     Path.class,
                     params -> null,
                     (s, p) -> Path.of(s)
             );
 
-    static final StandardBundlerParam<Path> PREDEFINED_APP_IMAGE =
-            new StandardBundlerParam<>(
-            Arguments.CLIOptions.PREDEFINED_APP_IMAGE.getId(),
-            Path.class,
-            params -> null,
-            (s, p) -> Path.of(s));
-
     // this is the raw --app-name arg - used in APP_NAME and INSTALLER_NAME
-    static final StandardBundlerParam<String> NAME =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<String> NAME =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.NAME.getId(),
                     String.class,
                     params -> null,
@@ -146,16 +145,14 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
 
     // this is the application name, either from the app-image (if given),
     // the name (if given) derived from the main-class, or the runtime image
-    static final StandardBundlerParam<String> APP_NAME =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<String> APP_NAME =
+            new BundlerParamInfo<>(
                     "application-name",
                     String.class,
                     params -> {
-                        Path appImage = PREDEFINED_APP_IMAGE.fetchFrom(params);
                         String appName = NAME.fetchFrom(params);
-                        if (appImage != null) {
-                            String name = AppImageFile.extractAppName(appImage);
-                            appName  = (name != null) ? name : appName;
+                        if (hasPredefinedAppImage(params)) {
+                            appName = PREDEFINED_APP_IMAGE_FILE.fetchFrom(params).getLauncherName();
                         } else if (appName == null) {
                             String s = MAIN_CLASS.fetchFrom(params);
                             if (s != null) {
@@ -173,8 +170,8 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
                     (s, p) -> s
             );
 
-    static final StandardBundlerParam<String> INSTALLER_NAME =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<String> INSTALLER_NAME =
+            new BundlerParamInfo<>(
                     "installer-name",
                     String.class,
                     params -> {
@@ -185,32 +182,32 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
                     (s, p) -> s
             );
 
-    static final StandardBundlerParam<Path> ICON =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<Path> ICON =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.ICON.getId(),
                     Path.class,
                     params -> null,
                     (s, p) -> Path.of(s)
             );
 
-    static final StandardBundlerParam<String> ABOUT_URL =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<String> ABOUT_URL =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.ABOUT_URL.getId(),
                     String.class,
                     params -> null,
                     (s, p) -> s
             );
 
-    static final StandardBundlerParam<String> VENDOR =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<String> VENDOR =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.VENDOR.getId(),
                     String.class,
                     params -> I18N.getString("param.vendor.default"),
                     (s, p) -> s
             );
 
-    static final StandardBundlerParam<String> DESCRIPTION =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<String> DESCRIPTION =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.DESCRIPTION.getId(),
                     String.class,
                     params -> params.containsKey(APP_NAME.getID())
@@ -219,8 +216,8 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
                     (s, p) -> s
             );
 
-    static final StandardBundlerParam<String> COPYRIGHT =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<String> COPYRIGHT =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.COPYRIGHT.getId(),
                     String.class,
                     params -> MessageFormat.format(I18N.getString(
@@ -229,8 +226,8 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
             );
 
     @SuppressWarnings("unchecked")
-    static final StandardBundlerParam<List<String>> ARGUMENTS =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<List<String>> ARGUMENTS =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.ARGUMENTS.getId(),
                     (Class<List<String>>) (Object) List.class,
                     params -> Collections.emptyList(),
@@ -238,41 +235,40 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
             );
 
     @SuppressWarnings("unchecked")
-    static final StandardBundlerParam<List<String>> JAVA_OPTIONS =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<List<String>> JAVA_OPTIONS =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.JAVA_OPTIONS.getId(),
                     (Class<List<String>>) (Object) List.class,
                     params -> Collections.emptyList(),
                     (s, p) -> Arrays.asList(s.split("\n\n"))
             );
 
-    static final StandardBundlerParam<String> VERSION =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<String> VERSION =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.VERSION.getId(),
                     String.class,
                     StandardBundlerParam::getDefaultAppVersion,
                     (s, p) -> s
             );
 
-    static final StandardBundlerParam<String> RELEASE =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<String> RELEASE =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.RELEASE.getId(),
                     String.class,
                     params -> DEFAULT_RELEASE,
                     (s, p) -> s
             );
 
-    @SuppressWarnings("unchecked")
-    public static final StandardBundlerParam<String> LICENSE_FILE =
-            new StandardBundlerParam<>(
+    public static final BundlerParamInfo<String> LICENSE_FILE =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.LICENSE_FILE.getId(),
                     String.class,
                     params -> null,
                     (s, p) -> s
             );
 
-    static final StandardBundlerParam<Path> TEMP_ROOT =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<Path> TEMP_ROOT =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.TEMP_ROOT.getId(),
                     Path.class,
                     params -> {
@@ -285,8 +281,8 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
                     (s, p) -> Path.of(s)
             );
 
-    public static final StandardBundlerParam<Path> CONFIG_ROOT =
-            new StandardBundlerParam<>(
+    public static final BundlerParamInfo<Path> CONFIG_ROOT =
+            new BundlerParamInfo<>(
                 "configRoot",
                 Path.class,
                 params -> {
@@ -301,8 +297,8 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
                 (s, p) -> null
             );
 
-    static final StandardBundlerParam<Boolean> VERBOSE  =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<Boolean> VERBOSE  =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.VERBOSE.getId(),
                     Boolean.class,
                     params -> false,
@@ -311,26 +307,8 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
                             true : Boolean.valueOf(s)
             );
 
-    static final StandardBundlerParam<Boolean> SHORTCUT_HINT  =
-            new StandardBundlerParam<>(
-                    "shortcut-hint", // not directly related to a CLI option
-                    Boolean.class,
-                    params -> true,  // defaults to true
-                    (s, p) -> (s == null || "null".equalsIgnoreCase(s)) ?
-                            true : Boolean.valueOf(s)
-            );
-
-    static final StandardBundlerParam<Boolean> MENU_HINT  =
-            new StandardBundlerParam<>(
-                    "menu-hint", // not directly related to a CLI option
-                    Boolean.class,
-                    params -> true,  // defaults to true
-                    (s, p) -> (s == null || "null".equalsIgnoreCase(s)) ?
-                            true : Boolean.valueOf(s)
-            );
-
-    static final StandardBundlerParam<Path> RESOURCE_DIR =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<Path> RESOURCE_DIR =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.RESOURCE_DIR.getId(),
                     Path.class,
                     params -> null,
@@ -338,17 +316,27 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
             );
 
     static final BundlerParamInfo<String> INSTALL_DIR =
-            new StandardBundlerParam<>(
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.INSTALL_DIR.getId(),
                     String.class,
                      params -> null,
                     (s, p) -> s
     );
 
+    static final BundlerParamInfo<Boolean> LAUNCHER_AS_SERVICE =
+            new BundlerParamInfo<>(
+                    Arguments.CLIOptions.LAUNCHER_AS_SERVICE.getId(),
+                    Boolean.class,
+                    params -> false,
+                    // valueOf(null) is false, and we actually do want null
+                    (s, p) -> (s == null || "null".equalsIgnoreCase(s)) ?
+                            true : Boolean.valueOf(s)
+            );
+
 
     @SuppressWarnings("unchecked")
-    static final StandardBundlerParam<List<Map<String, ? super Object>>> ADD_LAUNCHERS =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<List<Map<String, ? super Object>>> ADD_LAUNCHERS =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.ADD_LAUNCHER.getId(),
                     (Class<List<Map<String, ? super Object>>>) (Object)
                             List.class,
@@ -358,9 +346,9 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
             );
 
     @SuppressWarnings("unchecked")
-    static final StandardBundlerParam
+    static final BundlerParamInfo
             <List<Map<String, ? super Object>>> FILE_ASSOCIATIONS =
-            new StandardBundlerParam<>(
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.FILE_ASSOCIATIONS.getId(),
                     (Class<List<Map<String, ? super Object>>>) (Object)
                             List.class,
@@ -370,8 +358,8 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
             );
 
     @SuppressWarnings("unchecked")
-    static final StandardBundlerParam<List<String>> FA_EXTENSIONS =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<List<String>> FA_EXTENSIONS =
+            new BundlerParamInfo<>(
                     "fileAssociation.extension",
                     (Class<List<String>>) (Object) List.class,
                     params -> null, // null means not matched to an extension
@@ -379,8 +367,8 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
             );
 
     @SuppressWarnings("unchecked")
-    static final StandardBundlerParam<List<String>> FA_CONTENT_TYPE =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<List<String>> FA_CONTENT_TYPE =
+            new BundlerParamInfo<>(
                     "fileAssociation.contentType",
                     (Class<List<String>>) (Object) List.class,
                     params -> null,
@@ -388,25 +376,43 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
                     (s, p) -> Arrays.asList(s.split("(,|\\s)+"))
             );
 
-    static final StandardBundlerParam<String> FA_DESCRIPTION =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<String> FA_DESCRIPTION =
+            new BundlerParamInfo<>(
                     "fileAssociation.description",
                     String.class,
                     p -> null,
                     (s, p) -> s
             );
 
-    static final StandardBundlerParam<Path> FA_ICON =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<Path> FA_ICON =
+            new BundlerParamInfo<>(
                     "fileAssociation.icon",
                     Path.class,
                     ICON::fetchFrom,
                     (s, p) -> Path.of(s)
             );
 
+    @SuppressWarnings("unchecked")
+    static final BundlerParamInfo<List<Path>> DMG_CONTENT =
+            new BundlerParamInfo<>(
+                    Arguments.CLIOptions.DMG_CONTENT.getId(),
+                    (Class<List<Path>>) (Object)List.class,
+                    p -> Collections.emptyList(),
+                    (s, p) -> Stream.of(s.split(",")).map(Path::of).toList()
+            );
+
+    @SuppressWarnings("unchecked")
+    static final BundlerParamInfo<List<Path>> APP_CONTENT =
+            new BundlerParamInfo<>(
+                    Arguments.CLIOptions.APP_CONTENT.getId(),
+                    (Class<List<Path>>) (Object)List.class,
+                    p->Collections.emptyList(),
+                    (s, p) -> Stream.of(s.split(",")).map(Path::of).toList()
+            );
+
     @SuppressWarnings("unchecked")
     static final BundlerParamInfo<List<Path>> MODULE_PATH =
-            new StandardBundlerParam<>(
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.MODULE_PATH.getId(),
                     (Class<List<Path>>) (Object)List.class,
                     p -> getDefaultModulePath(),
@@ -451,7 +457,7 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
     }
 
     static final BundlerParamInfo<String> MODULE =
-            new StandardBundlerParam<>(
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.MODULE.getId(),
                     String.class,
                     p -> null,
@@ -461,7 +467,7 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
 
     @SuppressWarnings("unchecked")
     static final BundlerParamInfo<Set<String>> ADD_MODULES =
-            new StandardBundlerParam<>(
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.ADD_MODULES.getId(),
                     (Class<Set<String>>) (Object) Set.class,
                     p -> new LinkedHashSet<String>(),
@@ -469,8 +475,8 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
             );
 
     @SuppressWarnings("unchecked")
-    static final StandardBundlerParam<List<String>> JLINK_OPTIONS =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<List<String>> JLINK_OPTIONS =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.JLINK_OPTIONS.getId(),
                     (Class<List<String>>) (Object) List.class,
                     p -> Arrays.asList(DEFAULT_JLINK_OPTIONS),
@@ -478,15 +484,15 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
 
     @SuppressWarnings("unchecked")
     static final BundlerParamInfo<Set<String>> LIMIT_MODULES =
-            new StandardBundlerParam<>(
+            new BundlerParamInfo<>(
                     "limit-modules",
                     (Class<Set<String>>) (Object) Set.class,
                     p -> new LinkedHashSet<String>(),
                     (s, p) -> new LinkedHashSet<>(Arrays.asList(s.split(",")))
             );
 
-    static final StandardBundlerParam<Boolean> SIGN_BUNDLE =
-            new StandardBundlerParam<>(
+    static final BundlerParamInfo<Boolean> SIGN_BUNDLE =
+            new BundlerParamInfo<>(
                     Arguments.CLIOptions.MAC_SIGN.getId(),
                     Boolean.class,
                     params -> false,
@@ -494,7 +500,6 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
                     null : Boolean.valueOf(s)
         );
 
-
     static boolean isRuntimeInstaller(Map<String, ? super Object> params) {
         if (params.containsKey(MODULE.getID()) ||
                 params.containsKey(MAIN_JAR.getID()) ||
@@ -506,65 +511,8 @@ class StandardBundlerParam<T> extends BundlerParamInfo<T> {
         return params.containsKey(PREDEFINED_RUNTIME_IMAGE.getID());
     }
 
-    static Path getPredefinedAppImage(Map<String, ? super Object> params) {
-        Path applicationImage = PREDEFINED_APP_IMAGE.fetchFrom(params);
-        if (applicationImage != null && !IOUtils.exists(applicationImage)) {
-            throw new RuntimeException(
-                    MessageFormat.format(I18N.getString(
-                            "message.app-image-dir-does-not-exist"),
-                            PREDEFINED_APP_IMAGE.getID(),
-                            applicationImage.toString()));
-        }
-        return applicationImage;
-    }
-
-    static void copyPredefinedRuntimeImage(Map<String, ? super Object> params,
-            ApplicationLayout appLayout) throws IOException, ConfigException {
-        Path topImage = PREDEFINED_RUNTIME_IMAGE.fetchFrom(params);
-        if (!IOUtils.exists(topImage)) {
-            throw new ConfigException(
-                    MessageFormat.format(I18N.getString(
-                    "message.runtime-image-dir-does-not-exist"),
-                    PREDEFINED_RUNTIME_IMAGE.getID(),
-                    topImage.toString()),
-                    MessageFormat.format(I18N.getString(
-                    "message.runtime-image-dir-does-not-exist.advice"),
-                    PREDEFINED_RUNTIME_IMAGE.getID()));
-        }
-
-        if (Platform.isMac()) {
-            // On Mac topImage can be runtime root or runtime home.
-            Path runtimeHome = topImage.resolve("Contents/Home");
-            if (Files.isDirectory(runtimeHome)) {
-                // topImage references runtime root, adjust it to pick data from
-                // runtime home
-                topImage = runtimeHome;
-            }
-        }
-
-        // copy whole runtime, need to skip jmods and src.zip
-        final List<String> excludes = Arrays.asList("jmods", "src.zip");
-        IOUtils.copyRecursive(topImage,
-                appLayout.runtimeHomeDirectory(), excludes);
-
-        // if module-path given - copy modules to appDir/mods
-        List<Path> modulePath = MODULE_PATH.fetchFrom(params);
-        List<Path> defaultModulePath = getDefaultModulePath();
-        Path dest = appLayout.appModsDirectory();
-
-        if (dest != null) {
-            for (Path mp : modulePath) {
-                if (!defaultModulePath.contains(mp)) {
-                    Files.createDirectories(dest);
-                    IOUtils.copyRecursive(mp, dest);
-                }
-            }
-        }
-    }
-
-    private static List<Path> getDefaultModulePath() {
-        return List.of(
-                Path.of(System.getProperty("java.home"), "jmods").toAbsolutePath());
+    static boolean hasPredefinedAppImage(Map<String, ? super Object> params) {
+        return params.containsKey(PREDEFINED_APP_IMAGE.getID());
     }
 
     private static String getDefaultAppVersion(Map<String, ? super Object> params) {
