@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -37,6 +37,7 @@ import java.io.PrintStream;
 import java.io.Writer;
 import java.net.URI;
 import java.nio.charset.Charset;
+import java.nio.charset.StandardCharsets;
 import java.time.Instant;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -121,7 +122,7 @@ class ConsoleIOContext extends IOContext {
     String prefix = "";
 
     ConsoleIOContext(JShellTool repl, InputStream cmdin, PrintStream cmdout,
-                     boolean interactive) throws Exception {
+                     boolean interactive, Size size) throws Exception {
         this.repl = repl;
         Map<String, Object> variables = new HashMap<>();
         this.input = new StopDetectingInputStream(() -> repl.stop(),
@@ -143,7 +144,6 @@ class ConsoleIOContext extends IOContext {
                 terminal = new TestTerminal(nonBlockingInput, cmdout);
                 enableHighlighter = Boolean.getBoolean("test.enable.highlighter");
             } else {
-                Size size = null;
                 terminal = new ProgrammaticInTerminal(nonBlockingInput, cmdout, interactive,
                                                       size);
                 if (!interactive) {
@@ -155,10 +155,20 @@ class ConsoleIOContext extends IOContext {
             setupReader = setupReader.andThen(r -> r.option(Option.DISABLE_HIGHLIGHTER, !enableHighlighter));
             input.setInputStream(cmdin);
         } else {
-            terminal = TerminalBuilder.builder().inputStreamWrapper(in -> {
-                input.setInputStream(in);
-                return nonBlockingInput;
-            }).nativeSignals(false).build();
+            //on platforms which are known to be fully supported by
+            //the FFMTerminalProvider, do not permit the ExecTerminalProvider:
+            boolean allowExecTerminal = !OSUtils.IS_WINDOWS &&
+                                        !OSUtils.IS_LINUX &&
+                                        !OSUtils.IS_OSX;
+            terminal = TerminalBuilder.builder()
+                                      .exec(allowExecTerminal)
+                                      .inputStreamWrapper(in -> {
+                                          input.setInputStream(in);
+                                          return nonBlockingInput;
+                                      })
+                                      .nativeSignals(false)
+                                      .encoding(System.getProperty("stdin.encoding"))
+                                      .build();
             useComplexDeprecationHighlight = !OSUtils.IS_WINDOWS;
         }
         this.allowIncompleteInputs = allowIncompleteInputs;
@@ -384,9 +394,9 @@ class ConsoleIOContext extends IOContext {
                                        .reduce(ConsoleIOContext::commonPrefix);
 
                     String prefix =
-                            prefixOpt.orElse("").substring(cursor - anchor[0]);
+                            prefixOpt.orElse("");
 
-                    if (!prefix.isEmpty() && !command) {
+                    if (prefix.length() > cursor - anchor[0] && !command) {
                         //the completion will fill in the prefix, which will invalidate
                         //the documentation, avoid adding documentation tasks into the
                         //todo list:
@@ -395,6 +405,7 @@ class ConsoleIOContext extends IOContext {
 
                     ordinaryCompletion =
                             new OrdinaryCompletionTask(ordinaryCompletionToShow,
+                                                       anchor[0],
                                                        prefix,
                                                        !command && !doc.isEmpty(),
                                                        hasBoth);
@@ -599,15 +610,18 @@ class ConsoleIOContext extends IOContext {
 
     private final class OrdinaryCompletionTask implements CompletionTask {
         private final List<? extends CharSequence> toShow;
+        private final int anchor;
         private final String prefix;
         private final boolean cont;
         private final boolean showSmart;
 
         public OrdinaryCompletionTask(List<? extends CharSequence> toShow,
+                                      int anchor,
                                       String prefix,
                                       boolean cont,
                                       boolean showSmart) {
             this.toShow = toShow;
+            this.anchor = anchor;
             this.prefix = prefix;
             this.cont = cont;
             this.showSmart = showSmart;
@@ -620,7 +634,14 @@ class ConsoleIOContext extends IOContext {
 
         @Override
         public Result perform(String text, int cursor) throws IOException {
-            in.putString(prefix);
+            String existingPrefix = in.getBuffer().substring(anchor, cursor);
+
+            if (prefix.startsWith(existingPrefix)) {
+                in.putString(prefix.substring(existingPrefix.length()));
+            } else {
+                in.getBuffer().backspace(existingPrefix.length());
+                in.putString(prefix);
+            }
 
             boolean showItems = toShow.size() > 1 || showSmart;
 
@@ -629,7 +650,7 @@ class ConsoleIOContext extends IOContext {
                 printColumns(toShow);
             }
 
-            if (!prefix.isEmpty())
+            if (prefix.length() > existingPrefix.length())
                 return showItems ? Result.FINISH : Result.SKIP_NOREPAINT;
 
             return cont ? Result.CONTINUE : Result.FINISH;
@@ -833,7 +854,7 @@ class ConsoleIOContext extends IOContext {
     public void beforeUserCode() {
         synchronized (this) {
             pendingBytes = null;
-            pendingLine = null;
+            pendingLineCharacters = null;
         }
         input.setState(State.BUFFER);
     }
@@ -964,39 +985,79 @@ class ConsoleIOContext extends IOContext {
         }
     }
 
-    private String pendingLine;
-    private int pendingLinePointer;
+    private static final Charset stdinCharset =
+            Charset.forName(System.getProperty("stdin.encoding"),
+                            Charset.defaultCharset());
+    private char[] pendingLineCharacters;
+    private int pendingLineCharactersPointer;
     private byte[] pendingBytes;
     private int pendingBytesPointer;
 
     @Override
     public synchronized int readUserInput() throws IOException {
         if (pendingBytes == null || pendingBytes.length <= pendingBytesPointer) {
-            char userChar = readUserInputChar();
-            pendingBytes = String.valueOf(userChar).getBytes();
+            int userCharInput = readUserInputChar();
+            if (userCharInput == (-1)) {
+                return -1;
+            }
+            char userChar = (char) userCharInput;
+            StringBuilder dataToConvert = new StringBuilder();
+            dataToConvert.append(userChar);
+            if (Character.isHighSurrogate(userChar)) {
+                //surrogates cannot be converted independently,
+                //read the low surrogate and append it to dataToConvert:
+                int lowSurrogateInput = readUserInputChar();
+                if (lowSurrogateInput == (-1)) {
+                    //end of input, ignore at this stage
+                } else if (Character.isLowSurrogate((char) lowSurrogateInput)) {
+                    dataToConvert.append((char) lowSurrogateInput);
+                } else {
+                    //if not the low surrogate, rollback the reading of the character:
+                    pendingLineCharactersPointer--;
+                }
+            }
+            pendingBytes = dataToConvert.toString().getBytes(stdinCharset);
             pendingBytesPointer = 0;
         }
         return pendingBytes[pendingBytesPointer++];
     }
 
     @Override
-    public synchronized char readUserInputChar() throws IOException {
-        while (pendingLine == null || pendingLine.length() <= pendingLinePointer) {
-            pendingLine = doReadUserLine("", null) + System.getProperty("line.separator");
-            pendingLinePointer = 0;
+    public synchronized int readUserInputChar() throws IOException {
+        if (pendingLineCharacters != null && pendingLineCharacters.length == 0) {
+            return -1;
         }
-        return pendingLine.charAt(pendingLinePointer++);
+        while (pendingLineCharacters == null || pendingLineCharacters.length <= pendingLineCharactersPointer) {
+            String readLine = doReadUserLine("", null);
+            if (readLine == null) {
+                pendingLineCharacters = new char[0];
+                return -1;
+            } else {
+                pendingLineCharacters = (readLine + System.getProperty("line.separator")).toCharArray();
+            }
+            pendingLineCharactersPointer = 0;
+        }
+        return pendingLineCharacters[pendingLineCharactersPointer++];
     }
 
     @Override
     public synchronized String readUserLine(String prompt) throws IOException {
         //TODO: correct behavior w.r.t. pre-read stuff?
-        if (pendingLine != null && pendingLine.length() > pendingLinePointer) {
-            return pendingLine.substring(pendingLinePointer);
+        if (pendingLineCharacters != null && pendingLineCharacters.length > pendingLineCharactersPointer) {
+            String result = new String(pendingLineCharacters,
+                                       pendingLineCharactersPointer,
+                                       pendingLineCharacters.length - pendingLineCharactersPointer);
+            pendingLineCharacters = null;
+            return result;
         }
         return doReadUserLine(prompt, null);
     }
 
+    @Override
+    public String readUserLine() throws IOException {
+        return readUserLine("");
+    }
+
     private synchronized String doReadUserLine(String prompt, Character mask) throws IOException {
         History prevHistory = in.getHistory();
         boolean prevDisableCr = Display.DISABLE_CR;
@@ -1010,6 +1071,8 @@ class ConsoleIOContext extends IOContext {
             return in.readLine(prompt.replace("%", "%%"), mask);
         } catch (UserInterruptException ex) {
             throw new InterruptedIOException();
+        } catch (EndOfFileException ex) {
+            return null; // Signal that Ctrl+D or similar happened
         } finally {
             in.setParser(prevParser);
             in.setHistory(prevHistory);
@@ -1020,7 +1083,11 @@ class ConsoleIOContext extends IOContext {
 
     public char[] readPassword(String prompt) throws IOException {
         //TODO: correct behavior w.r.t. pre-read stuff?
-        return doReadUserLine(prompt, '\0').toCharArray();
+        String line = doReadUserLine(prompt, '\0');
+        if (line == null) {
+            return null;
+        }
+        return line.toCharArray();
     }
 
     @Override
@@ -1312,6 +1379,7 @@ class ConsoleIOContext extends IOContext {
 
     private static class ProgrammaticInTerminal extends LineDisciplineTerminal {
 
+        protected static final int DEFAULT_WIDTH = 80;
         protected static final int DEFAULT_HEIGHT = 24;
 
         private final NonBlockingReader inputReader;
@@ -1320,9 +1388,9 @@ class ConsoleIOContext extends IOContext {
         public ProgrammaticInTerminal(InputStream input, OutputStream output,
                                        boolean interactive, Size size) throws Exception {
             this(input, output, interactive ? "ansi" : "dumb",
-                 size != null ? size : new Size(80, DEFAULT_HEIGHT),
+                 size != null ? size : new Size(DEFAULT_WIDTH, DEFAULT_HEIGHT),
                  size != null ? size
-                              : interactive ? new Size(80, DEFAULT_HEIGHT)
+                              : interactive ? new Size(DEFAULT_WIDTH, DEFAULT_HEIGHT)
                                             : new Size(Integer.MAX_VALUE - 1, DEFAULT_HEIGHT));
         }
 
@@ -1366,7 +1434,7 @@ class ConsoleIOContext extends IOContext {
             } catch (Throwable ex) {
                 // ignore
             }
-            return new Size(80, h);
+            return new Size(DEFAULT_WIDTH, h);
         }
         public TestTerminal(InputStream input, OutputStream output) throws Exception {
             this(input, output, computeSize());
@@ -1374,6 +1442,14 @@ class ConsoleIOContext extends IOContext {
         private TestTerminal(InputStream input, OutputStream output, Size size) throws Exception {
             super(input, output, "ansi", size, size);
         }
+
+        @Override
+        public Attributes enterRawMode() {
+            Attributes res = super.enterRawMode();
+            res.setControlChar(ControlChar.VEOF, 4);
+            return res;
+        }
+
     }
 
     private static final class CompletionState {
