@@ -24,6 +24,7 @@
  */
 package jdk.jshell.execution;
 
+import java.io.ByteArrayInputStream;
 import java.lang.constant.ClassDesc;
 import java.lang.constant.ConstantDescs;
 import java.lang.reflect.Field;
@@ -34,7 +35,10 @@ import java.util.Set;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.stream.Stream;
 import java.lang.classfile.ClassFile;
+import java.lang.classfile.ClassHierarchyResolver;
 import java.lang.classfile.ClassTransform;
+import java.lang.classfile.CodeBuilder;
+import java.lang.classfile.CodeElement;
 import java.lang.classfile.CodeTransform;
 import java.lang.classfile.Label;
 import java.lang.classfile.instruction.BranchInstruction;
@@ -83,38 +87,44 @@ public class LocalExecutionControl extends DirectExecutionControl {
     @Override
     public void load(ClassBytecodes[] cbcs)
             throws ClassInstallException, NotImplementedException, EngineTerminationException {
-        super.load(Stream.of(cbcs)
-                .map(cbc -> new ClassBytecodes(cbc.name(), instrument(cbc.bytecodes())))
-                .toArray(ClassBytecodes[]::new));
+        super.load(instrument(cbcs));
     }
 
     private static final String CANCEL_CLASS = "REPL.$Cancel$";
     private static final ClassDesc CD_Cancel = ClassDesc.of(CANCEL_CLASS);
+    private static final String STOP_CHECK = "stopCheck";
     private static final ClassDesc CD_ThreadDeath = ClassDesc.of("java.lang.ThreadDeath");
 
-    private static byte[] instrument(byte[] classFile) {
-        var cc = ClassFile.of();
+    private static ClassBytecodes[] instrument(ClassBytecodes[] cbcs) {
+        var cc = ClassFile.of(ClassFile.ClassHierarchyResolverOption.of(
+                ClassHierarchyResolver.defaultResolver().orElse(
+                        ClassHierarchyResolver.ofResourceParsing(cd -> {
+                            String cName = cd.descriptorString();
+                            cName = cName.substring(1, cName.length() - 1).replace('/', '.');
+                            for (ClassBytecodes cbc : cbcs) {
+                                if (cName.equals(cbc.name())) {
+                                    return new ByteArrayInputStream(cbc.bytecodes());
+                                }
+                            }
+                            return null;
+                        }))));
+
+        return Stream.of(cbcs)
+                .map(cbc -> new ClassBytecodes(cbc.name(), instrument(cc, cbc.bytecodes())))
+                .toArray(ClassBytecodes[]::new);
+    }
+
+    private static byte[] instrument(ClassFile cc, byte[] classFile) {
         return cc.transformClass(cc.parse(classFile),
                         ClassTransform.transformingMethodBodies(
-                            CodeTransform.ofStateful(() -> {
-                                Set<Label> priorLabels = new HashSet<>();
-                                return (builder, element) -> {
-                                    switch (element) {
-                                        case LabelTarget target -> priorLabels.add(target.label());
-                                        case BranchInstruction branch when priorLabels.contains(branch.target())
-                                            -> builder.invokestatic(CD_Cancel, "stopCheck", ConstantDescs.MTD_void);
-                                        default -> { }
-                                    }
-                                    builder.with(element);
-                                };
-                            })));
+                            CodeTransform.ofStateful(StopCheckWeaver::new)));
     }
 
     private static ClassBytecodes genCancelClass() {
         return new ClassBytecodes(CANCEL_CLASS, ClassFile.of().build(CD_Cancel, clb ->
              clb.withFlags(ClassFile.ACC_PUBLIC)
                 .withField("allStop", ConstantDescs.CD_boolean, ClassFile.ACC_PUBLIC | ClassFile.ACC_STATIC | ClassFile.ACC_VOLATILE)
-                .withMethodBody("stopCheck", ConstantDescs.MTD_void, ClassFile.ACC_PUBLIC | ClassFile.ACC_STATIC, cob ->
+                .withMethodBody(STOP_CHECK, ConstantDescs.MTD_void, ClassFile.ACC_PUBLIC | ClassFile.ACC_STATIC, cob ->
                         cob.getstatic(CD_Cancel, "allStop", ConstantDescs.CD_boolean)
                            .ifThenElse(tb -> tb.new_(CD_ThreadDeath)
                                                .dup()
@@ -123,6 +133,31 @@ public class LocalExecutionControl extends DirectExecutionControl {
                                        eb -> eb.return_()))));
     }
 
+    // This inserts calls to REPL.$Cancel$.stopCheck() at method start and prior to any backward branch
+    private static class StopCheckWeaver implements CodeTransform {
+
+        private final Set<Label> priorLabels = new HashSet<>();
+
+        @Override
+        public void atStart(CodeBuilder builder) {
+            stopCheck(builder);
+        }
+
+        @Override
+        public void accept(CodeBuilder builder, CodeElement element) {
+            switch (element) {
+                case LabelTarget target -> priorLabels.add(target.label());
+                case BranchInstruction branch when priorLabels.contains(branch.target()) -> stopCheck(builder);
+                default -> { }
+            }
+            builder.with(element);
+        };
+
+        private void stopCheck(CodeBuilder builder) {
+            builder.invokestatic(CD_Cancel, STOP_CHECK, ConstantDescs.MTD_void);
+        }
+    }
+
     @Override
     @SuppressWarnings("removal")
     protected String invoke(Method doitMethod) throws Exception {
