@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * The Universal Permissive License (UPL), Version 1.0
@@ -49,25 +49,22 @@
    HotSpot PrintAssembly option.
 */
 
-#ifndef SYSTEM_BINUTILS
-#include <config.h> /* required by bfd.h */
-#endif
-
 #include <errno.h>
 #include <inttypes.h>
 #include <string.h>
 
-#include <libiberty.h>
+#ifndef SYSTEM_BINUTILS
+/* defines for bfd.h */
+#define PACKAGE "hsdis"
+#define PACKAGE_VERSION 1
+#endif
+
 #include <bfd.h>
 #include <dis-asm.h>
+#include <stdbool.h>
 
 #include "hsdis.h"
 
-#ifndef bool
-#define bool int
-#define true 1
-#define false 0
-#endif /*bool*/
 
 /* short names for stuff in hsdis.h */
 typedef decode_instructions_event_callback_ftype  event_callback_t;
@@ -119,10 +116,8 @@ static const char* format_insn_close(const char* close,
                                      disassemble_info* dinfo,
                                      char* buf, size_t bufsize);
 
+JNIEXPORT
 void*
-#ifdef DLL_ENTRY
-  DLL_ENTRY
-#endif
 decode_instructions_virtual(uintptr_t start_va, uintptr_t end_va,
                             unsigned char* buffer, uintptr_t length,
                             event_callback_t  event_callback_arg,  void* event_stream_arg,
@@ -144,10 +139,8 @@ decode_instructions_virtual(uintptr_t start_va, uintptr_t end_va,
 }
 
 /* This is the compatability interface for older version of hotspot */
+JNIEXPORT
 void*
-#ifdef DLL_ENTRY
-  DLL_ENTRY
-#endif
 decode_instructions(void* start_pv, void* end_pv,
                     event_callback_t  event_callback_arg,  void* event_stream_arg,
                     printf_callback_t printf_callback_arg, void* printf_stream_arg,
@@ -335,8 +328,9 @@ static void setup_app_data(struct hsdis_app_data* app_data,
                                  app_data->printf_stream,
                                  app_data->printf_callback,
                                  native_bfd,
-                                 /* On PowerPC we get warnings, if we pass empty options */
-                                 (caller_options == NULL) ? NULL : app_data->insn_options);
+                                 /* On some archs we get warnings, if we pass empty options */
+                                 ((caller_options == NULL) || (app_data->insn_options[0] == '\0'))
+                                   ? NULL : app_data->insn_options);
 
   /* Finish linking together the various callback blocks. */
   app_data->dinfo.application_data = (void*) app_data;
@@ -556,12 +550,34 @@ static void parse_fake_insn(disassembler_ftype dfn,
   dinfo->fprintf_func     = fprintf_func;
 }
 
+static fprintf_ftype target_fprintf_func = NULL;
+
+#ifdef BINUTILS_NEW_API
+static int wrapper_fprintf_styled_ftype(void *v, enum disassembler_style style_unused, const char* fmt, ...) {
+  char buffer[1024] = {};
+  va_list args;
+  int r;
+  va_start(args, fmt);
+  r = vsnprintf(buffer, sizeof(buffer), fmt, args);
+  va_end(args);
+  if (target_fprintf_func != NULL) {
+    return target_fprintf_func(v, "%s", buffer);
+  }
+  return r;
+}
+#endif
+
 static void init_disassemble_info_from_bfd(struct disassemble_info* dinfo,
                                            void *stream,
                                            fprintf_ftype fprintf_func,
                                            bfd* abfd,
                                            char* disassembler_options) {
+  target_fprintf_func = fprintf_func;
+#ifdef BINUTILS_NEW_API
+  init_disassemble_info(dinfo, stream, fprintf_func, wrapper_fprintf_styled_ftype);
+#else
   init_disassemble_info(dinfo, stream, fprintf_func);
+#endif
 
   dinfo->flavour = bfd_get_flavour(abfd);
   dinfo->arch = bfd_get_arch(abfd);
