@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -28,7 +28,7 @@
  *
  * @build jdk.test.whitebox.WhiteBox
  * @run driver jdk.test.lib.helpers.ClassFileInstaller jdk.test.whitebox.WhiteBox
- * @run driver/timeout=180 compiler.jsr292.ContinuousCallSiteTargetChange
+ * @run driver/timeout=720 compiler.jsr292.ContinuousCallSiteTargetChange
  */
 
 package compiler.jsr292;
@@ -171,7 +171,7 @@ public class ContinuousCallSiteTargetChange {
             WhiteBox whiteBox = WhiteBox.getWhiteBox();
             for (int i = 0; i < iterations; i++) {
                 iteration();
-                whiteBox.forceNMethodSweep();
+                whiteBox.fullGC();
             }
         }
     }
