@@ -26,6 +26,7 @@ package compiler.lib.generators;
 import java.lang.foreign.MemorySegment;
 import java.lang.foreign.ValueLayout;
 import java.util.*;
+import static java.lang.Float.floatToFloat16;
 
 import jdk.test.lib.Utils;
 
@@ -100,7 +101,7 @@ import jdk.test.lib.Utils;
  * <p>
  * Unless you have reasons to pick a specific distribution, you are encouraged to rely on {@link #ints()},
  * {@link #longs()}, {@link #doubles()} and {@link #floats()}, which will randomly pick an interesting distribution.
- * This is best practice, because that allows the test to be run under different conditions – maybe only a single
+ * This is best practice, because that allows the test to be run under different conditions - maybe only a single
  * distribution can trigger a bug.
  */
 public final class Generators {
@@ -165,6 +166,20 @@ public final class Generators {
         return new AnyBitsDoubleGenerator(this);
     }
 
+    /**
+     * Generates uniform float16s in the range of [lo, hi) (inclusive of lo, exclusive of hi).
+     */
+    public RestrictableGenerator<Short> uniformFloat16s(short lo, short hi) {
+        return new UniformFloat16Generator(this, lo, hi);
+    }
+
+    /**
+     * Generates uniform float16s in the range of [0, 1) (inclusive of 0, exclusive of 1).
+     */
+    public RestrictableGenerator<Short> uniformFloat16s() {
+        return uniformFloat16s(floatToFloat16(0.0f), floatToFloat16(1.0f));
+    }
+
     /**
      * Generates uniform doubles in the range of [lo, hi) (inclusive of lo, exclusive of hi).
      */
@@ -172,6 +187,14 @@ public final class Generators {
         return new UniformFloatGenerator(this, lo, hi);
     }
 
+    /**
+     * Provides an any-bits float16 distribution random generator, i.e. the bits are uniformly sampled,
+     * thus creating any possible float16 value, including the multiple different NaN representations.
+     */
+    public Generator<Short> anyBitsFloat16s() {
+        return new AnyBitsFloat16Generator(this);
+    }
+
     /**
      * Generates uniform floats in the range of [0, 1) (inclusive of 0, exclusive of 1).
      */
@@ -344,6 +367,25 @@ public final class Generators {
         return mixed(uniformLongs(), powerOfTwoLongs(rangeSpecial), weightUniform, weightSpecial);
     }
 
+    /**
+     * Randomly pick a float16 generator.
+     *
+     * @return Random float16 generator.
+     */
+    public Generator<Short> float16s() {
+        switch(random.nextInt(0, 5)) {
+            case 0  -> { return uniformFloat16s(floatToFloat16(-1.0f), floatToFloat16(1.0f)); }
+            // Well-balanced, so that multiplication reduction never explodes or collapses to zero:
+            case 1  -> { return uniformFloat16s(floatToFloat16(0.999f), floatToFloat16(1.001f)); }
+            case 2  -> { return anyBitsFloat16s(); }
+            // A tame distribution, mixed in with the occasional special float value:
+            case 3  -> { return mixedWithSpecialFloat16s(uniformFloat16s(floatToFloat16(0.999f), floatToFloat16(1.001f)), 10, 1000); }
+            // Generating any bits, but special values are more frequent.
+            case 4  -> { return mixedWithSpecialFloat16s(anyBitsFloat16s(), 100, 200); }
+            default -> { throw new RuntimeException("impossible"); }
+        }
+    }
+
     /**
      * Randomly pick a float generator.
      *
@@ -388,6 +430,7 @@ public final class Generators {
      */
     public final RestrictableGenerator<Double> SPECIAL_DOUBLES = orderedRandomElement(List.of(
         0d,
+        -0d,
         1d,
         -1d,
         Double.POSITIVE_INFINITY,
@@ -415,11 +458,12 @@ public final class Generators {
     }
 
     /**
-     * Generates interesting double values, which often are corner cases such as, 0, 1, -1, NaN, +/- Infinity, Min,
+     * Generates interesting float values, which often are corner cases such as, 0, 1, -1, NaN, +/- Infinity, Min,
      * Max.
      */
     public final RestrictableGenerator<Float> SPECIAL_FLOATS = orderedRandomElement(List.of(
         0f,
+        -0f,
         1f,
         -1f,
         Float.POSITIVE_INFINITY,
@@ -430,6 +474,29 @@ public final class Generators {
         Float.MIN_VALUE
     ));
 
+    /**
+     * Generates interesting float16 values, which often are corner cases such as, +/- 0, NaN, +/- Infinity, Min,
+     * Max.
+     */
+    public final RestrictableGenerator<Short> SPECIAL_FLOAT16S = orderedRandomElement(List.of(
+        floatToFloat16(0.0f),
+        floatToFloat16(-0.0f),
+        floatToFloat16(Float.POSITIVE_INFINITY),
+        floatToFloat16(Float.NEGATIVE_INFINITY),
+        floatToFloat16(Float.NaN),
+        floatToFloat16(0x1.ffcP+15f), // MAX_VALUE
+        floatToFloat16(0x1.0P-14f),   // MIN_NORMAL
+        floatToFloat16(0x1.0P-24f)    // MIN_VALUE
+    ));
+
+    /**
+     * Returns a mixed generator that mixes the provided background generator and {@link #SPECIAL_FLOAT16S} with the provided
+     * weights.
+     */
+    public Generator<Short> mixedWithSpecialFloat16s(Generator<Short> background, int weightNormal, int weightSpecial) {
+        return mixed(background, SPECIAL_FLOAT16S, weightNormal, weightSpecial);
+    }
+
     /**
      * Returns a mixed generator that mixes the provided background generator and {@link #SPECIAL_FLOATS} with the provided
      * weights.
@@ -538,6 +605,29 @@ public final class Generators {
         fillFloat(generator, MemorySegment.ofArray(a));
     }
 
+    /**
+     * Fills the memory segments with shorts obtained by calling next on the generator.
+     *
+     * @param generator The generator from which to source the values.
+     * @param ms Memory segment to be filled with random values.
+     */
+    public void fillShort(Generator<Short> generator, MemorySegment ms) {
+        var layout = ValueLayout.JAVA_SHORT_UNALIGNED;
+        for (long i = 0; i < ms.byteSize() / layout.byteSize(); i++) {
+            ms.setAtIndex(layout, i, generator.next());
+        }
+    }
+
+    /**
+     * Fill the array with shorts using the distribution of the generator.
+     *
+     * @param a Array to be filled with random values.
+     */
+    public void fill(Generator<Short> generator, short[] a) {
+        fillShort(generator, MemorySegment.ofArray(a));
+    }
+
+
     /**
      * Fills the memory segments with ints obtained by calling next on the generator.
      *
