@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -65,11 +65,9 @@ public class TestAggressiveHeap {
         " *bool +UseParallelGC *= *true +\\{product\\} *\\{command line\\}";
 
     private static void testFlag() throws Exception {
-        ProcessBuilder pb = GCArguments.createTestJvm(
+        OutputAnalyzer output = GCArguments.executeTestJava(
             option, heapSizeOption, "-XX:+PrintFlagsFinal", "-version");
 
-        OutputAnalyzer output = new OutputAnalyzer(pb.start());
-
         output.shouldHaveExitValue(0);
 
         String value = output.firstMatch(parallelGCPattern);
