@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2023, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -157,7 +157,6 @@ public class TestNewSizeFlags {
                 (maxNewSize >= 0 ? "-XX:MaxNewSize=" + maxNewSize : ""),
                 "-Xmx" + maxHeapSize,
                 "-Xms" + heapSize,
-                "-XX:GCLockerEdenExpansionPercent=0",
                 "-XX:-UseLargePages",
                 NewSizeVerifier.class.getName(),
                 Long.toString(expectedNewSize),
@@ -183,8 +182,6 @@ public class TestNewSizeFlags {
         private static final GCTypes.YoungGCType YOUNG_GC_TYPE = GCTypes.YoungGCType.getYoungGCType();
         private static final long HEAP_SPACE_ALIGNMENT = WB.getHeapSpaceAlignment();
         private static final long HEAP_ALIGNMENT = WB.getHeapAlignment();
-        private static final long PS_VIRTUAL_SPACE_ALIGNMENT =
-                (YOUNG_GC_TYPE == GCTypes.YoungGCType.PSNew) ? WB.psVirtualSpaceAlignment() : 0;
 
         public static final int ARRAY_LENGTH = 100;
         public static final int CHUNK_SIZE = 1024;
@@ -304,9 +301,7 @@ public class TestNewSizeFlags {
                 case DefNew:
                     return HeapRegionUsageTool.alignDown(value, HEAP_SPACE_ALIGNMENT);
                 case PSNew:
-                    return HeapRegionUsageTool.alignUp(HeapRegionUsageTool.alignDown(value,
-                            HEAP_SPACE_ALIGNMENT),
-                            PS_VIRTUAL_SPACE_ALIGNMENT);
+                    return HeapRegionUsageTool.alignDown(value, HEAP_SPACE_ALIGNMENT);
                 case G1:
                     return HeapRegionUsageTool.alignUp(value, WB.g1RegionSize());
                 default:
