@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -92,7 +92,7 @@ class TestEagerReclaimHumongousRegionsWithRefsReclaimRegionFast {
 public class TestEagerReclaimHumongousRegionsWithRefs {
 
     public static void main(String[] args) throws Exception {
-        ProcessBuilder pb = ProcessTools.createJavaProcessBuilder(
+        OutputAnalyzer output = ProcessTools.executeLimitedTestJava(
             "-XX:+UseG1GC",
             "-Xms128M",
             "-Xmx128M",
@@ -102,8 +102,6 @@ public class TestEagerReclaimHumongousRegionsWithRefs {
 
         Pattern p = Pattern.compile("Full GC");
 
-        OutputAnalyzer output = new OutputAnalyzer(pb.start());
-
         int found = 0;
         Matcher m = p.matcher(output.getStdout());
         while (m.find()) {
