@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021, 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2021, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -76,6 +76,8 @@ public class SimpleFileServerTest {
     static final boolean ENABLE_LOGGING = true;
     static final Logger LOGGER = Logger.getLogger("com.sun.net.httpserver");
 
+    static final String EXPECTED_LAST_MODIFIED_OF_FAVICON = "Mon, 23 May 1995 11:11:11 GMT";
+
     @BeforeTest
     public void setup() throws IOException {
         if (ENABLE_LOGGING) {
@@ -142,6 +144,65 @@ public class SimpleFileServerTest {
         }
     }
 
+    @Test
+    public void testFavIconGET() throws Exception {
+        var root = Files.createDirectory(TEST_DIR.resolve("testFavIconGET"));
+
+        var server = SimpleFileServer.createFileServer(LOOPBACK_ADDR, root, OutputLevel.VERBOSE);
+        server.start();
+        try {
+            // expect built-in icon
+            var client = HttpClient.newBuilder().proxy(NO_PROXY).build();
+            var request = HttpRequest.newBuilder(uri(server, "favicon.ico")).build();
+            var response = client.send(request, BodyHandlers.ofString());
+            assertEquals(response.statusCode(), 200);
+            assertEquals(response.headers().firstValue("content-type").get(), "image/x-icon");
+            assertEquals(response.headers().firstValue("last-modified").get(), EXPECTED_LAST_MODIFIED_OF_FAVICON);
+
+            // expect custom (and broken) icon
+            var file = Files.writeString(root.resolve("favicon.ico"), "broken icon", CREATE);
+            try {
+                var lastModified = getLastModified(file);
+                var expectedLength = Long.toString(Files.size(file));
+                response = client.send(request, BodyHandlers.ofString());
+                assertEquals(response.statusCode(), 200);
+                assertEquals(response.headers().firstValue("content-type").get(), "application/octet-stream");
+                assertEquals(response.headers().firstValue("content-length").get(), expectedLength);
+                assertEquals(response.headers().firstValue("last-modified").get(), lastModified);
+            } finally {
+                Files.delete(file);
+            }
+
+            // expect built-in icon
+            response = client.send(request, BodyHandlers.ofString());
+            assertEquals(response.statusCode(), 200);
+            assertEquals(response.headers().firstValue("content-type").get(), "image/x-icon");
+            assertEquals(response.headers().firstValue("last-modified").get(), EXPECTED_LAST_MODIFIED_OF_FAVICON);
+        } finally {
+            server.stop(0);
+        }
+    }
+
+    @Test
+    public void testFavIconHEAD() throws Exception {
+        var root = Files.createDirectory(TEST_DIR.resolve("testFavIconHEAD"));
+
+        var server = SimpleFileServer.createFileServer(LOOPBACK_ADDR, root, OutputLevel.VERBOSE);
+        server.start();
+        try {
+            var client = HttpClient.newBuilder().proxy(NO_PROXY).build();
+            var request = HttpRequest.newBuilder(uri(server, "favicon.ico"))
+                    .method("HEAD", BodyPublishers.noBody()).build();
+            var response = client.send(request, BodyHandlers.ofString());
+            assertEquals(response.statusCode(), 200);
+            assertEquals(response.headers().firstValue("content-type").get(), "image/x-icon");
+            assertEquals(response.headers().firstValue("last-modified").get(), EXPECTED_LAST_MODIFIED_OF_FAVICON);
+            assertEquals(response.body(), "");
+        } finally {
+            server.stop(0);
+        }
+    }
+
     @Test
     public void testFileHEAD() throws Exception {
         var root = Files.createDirectory(TEST_DIR.resolve("testFileHEAD"));
@@ -248,63 +309,65 @@ public class SimpleFileServerTest {
 
     @Test
     public void testNotReadableFileGET() throws Exception {
-        if (!Platform.isWindows()) {  // not applicable on Windows
-            var expectedBody = openHTML + """
-                <h1>File not found</h1>
-                <p>&#x2F;aFile.txt</p>
-                """ + closeHTML;
-            var expectedLength = Integer.toString(expectedBody.getBytes(UTF_8).length);
-            var root = Files.createDirectory(TEST_DIR.resolve("testNotReadableFileGET"));
-            var file = Files.writeString(root.resolve("aFile.txt"), "some text", CREATE);
+        if (Platform.isWindows()) {
+            throw new SkipException("Not applicable on Windows");
+        }
+        var expectedBody = openHTML + """
+            <h1>File not found</h1>
+            <p>&#x2F;aFile.txt</p>
+            """ + closeHTML;
+        var expectedLength = Integer.toString(expectedBody.getBytes(UTF_8).length);
+        var root = Files.createDirectory(TEST_DIR.resolve("testNotReadableFileGET"));
+        var file = Files.writeString(root.resolve("aFile.txt"), "some text", CREATE);
 
-            file.toFile().setReadable(false, false);
-            assert !Files.isReadable(file);
+        file.toFile().setReadable(false, false);
+        assert !Files.isReadable(file);
 
-            var server = SimpleFileServer.createFileServer(LOOPBACK_ADDR, root, OutputLevel.VERBOSE);
-            server.start();
-            try {
-                var client = HttpClient.newBuilder().proxy(NO_PROXY).build();
-                var request = HttpRequest.newBuilder(uri(server, "aFile.txt")).build();
-                var response = client.send(request, BodyHandlers.ofString());
-                assertEquals(response.statusCode(), 404);
-                assertEquals(response.headers().firstValue("content-length").get(), expectedLength);
-                assertEquals(response.body(), expectedBody);
-            } finally {
-                server.stop(0);
-                file.toFile().setReadable(true, false);
-            }
+        var server = SimpleFileServer.createFileServer(LOOPBACK_ADDR, root, OutputLevel.VERBOSE);
+        server.start();
+        try {
+            var client = HttpClient.newBuilder().proxy(NO_PROXY).build();
+            var request = HttpRequest.newBuilder(uri(server, "aFile.txt")).build();
+            var response = client.send(request, BodyHandlers.ofString());
+            assertEquals(response.statusCode(), 404);
+            assertEquals(response.headers().firstValue("content-length").get(), expectedLength);
+            assertEquals(response.body(), expectedBody);
+        } finally {
+            server.stop(0);
+            file.toFile().setReadable(true, false);
         }
     }
 
     @Test
     public void testNotReadableSegmentGET() throws Exception {
-        if (!Platform.isWindows()) {  // not applicable on Windows
-            var expectedBody = openHTML + """
-                <h1>File not found</h1>
-                <p>&#x2F;dir&#x2F;aFile.txt</p>
-                """ + closeHTML;
-            var expectedLength = Integer.toString(expectedBody.getBytes(UTF_8).length);
-            var root = Files.createDirectory(TEST_DIR.resolve("testNotReadableSegmentGET"));
-            var dir = Files.createDirectory(root.resolve("dir"));
-            var file = Files.writeString(dir.resolve("aFile.txt"), "some text", CREATE);
+        if (Platform.isWindows()) {
+            throw new SkipException("Not applicable on Windows");
+        }
+        var expectedBody = openHTML + """
+            <h1>File not found</h1>
+            <p>&#x2F;dir&#x2F;aFile.txt</p>
+            """ + closeHTML;
+        var expectedLength = Integer.toString(expectedBody.getBytes(UTF_8).length);
+        var root = Files.createDirectory(TEST_DIR.resolve("testNotReadableSegmentGET"));
+        var dir = Files.createDirectory(root.resolve("dir"));
+        var file = Files.writeString(dir.resolve("aFile.txt"), "some text", CREATE);
 
-            dir.toFile().setReadable(false, false);
-            assert !Files.isReadable(dir);
-            assert Files.isReadable(file);
+        dir.toFile().setReadable(false, false);
+        assert !Files.isReadable(dir);
+        assert Files.isReadable(file);
 
-            var server = SimpleFileServer.createFileServer(LOOPBACK_ADDR, root, OutputLevel.VERBOSE);
-            server.start();
-            try {
-                var client = HttpClient.newBuilder().proxy(NO_PROXY).build();
-                var request = HttpRequest.newBuilder(uri(server, "dir/aFile.txt")).build();
-                var response = client.send(request, BodyHandlers.ofString());
-                assertEquals(response.statusCode(), 404);
-                assertEquals(response.headers().firstValue("content-length").get(), expectedLength);
-                assertEquals(response.body(), expectedBody);
-            } finally {
-                server.stop(0);
-                dir.toFile().setReadable(true, false);
-            }
+        var server = SimpleFileServer.createFileServer(LOOPBACK_ADDR, root, OutputLevel.VERBOSE);
+        server.start();
+        try {
+            var client = HttpClient.newBuilder().proxy(NO_PROXY).build();
+            var request = HttpRequest.newBuilder(uri(server, "dir/aFile.txt")).build();
+            var response = client.send(request, BodyHandlers.ofString());
+            assertEquals(response.statusCode(), 404);
+            assertEquals(response.headers().firstValue("content-length").get(), expectedLength);
+            assertEquals(response.body(), expectedBody);
+        } finally {
+            server.stop(0);
+            dir.toFile().setReadable(true, false);
         }
     }
 
@@ -607,14 +670,6 @@ public class SimpleFileServerTest {
     @Test
     public void testIllegalPath() throws Exception {
         var addr = LOOPBACK_ADDR;
-        {   // not absolute
-            Path p = Path.of(".");
-            assert Files.isDirectory(p);
-            assert Files.exists(p);
-            assert !p.isAbsolute();
-            var iae = expectThrows(IAE, () -> SimpleFileServer.createFileServer(addr, p, OutputLevel.INFO));
-            assertTrue(iae.getMessage().contains("is not absolute"));
-        }
         {   // not a directory
             Path p = Files.createFile(TEST_DIR.resolve("aFile"));
             assert !Files.isDirectory(p);
@@ -627,18 +682,22 @@ public class SimpleFileServerTest {
             var iae = expectThrows(IAE, () -> SimpleFileServer.createFileServer(addr, p, OutputLevel.INFO));
             assertTrue(iae.getMessage().contains("does not exist"));
         }
-        {   // not readable
-            if (!Platform.isWindows()) {  // not applicable on Windows
-                Path p = Files.createDirectory(TEST_DIR.resolve("aDir"));
-                p.toFile().setReadable(false, false);
-                assert !Files.isReadable(p);
-                try {
-                    var iae = expectThrows(IAE, () -> SimpleFileServer.createFileServer(addr, p, OutputLevel.INFO));
-                    assertTrue(iae.getMessage().contains("not readable"));
-                } finally {
-                    p.toFile().setReadable(true, false);
-                }
-            }
+    }
+
+    @Test
+    public void testNonReadablePath() throws Exception {
+        if (Platform.isWindows()) {
+            throw new SkipException("Not applicable on Windows");
+        }
+        var addr = LOOPBACK_ADDR;
+        Path p = Files.createDirectory(TEST_DIR.resolve("aDir"));
+        p.toFile().setReadable(false, false);
+        assert !Files.isReadable(p);
+        try {
+            var iae = expectThrows(IAE, () -> SimpleFileServer.createFileServer(addr, p, OutputLevel.INFO));
+            assertTrue(iae.getMessage().contains("not readable"));
+        } finally {
+            p.toFile().setReadable(true, false);
         }
     }
 
