@@ -1,24 +1,24 @@
 /*
- *  Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
- *  DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
+ * Copyright (c) 2019, 2023, Oracle and/or its affiliates. All rights reserved.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *  This code is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License version 2 only, as
- *  published by the Free Software Foundation.
+ * This code is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License version 2 only, as
+ * published by the Free Software Foundation.
  *
- *  This code is distributed in the hope that it will be useful, but WITHOUT
- *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  version 2 for more details (a copy is included in the LICENSE file that
- *  accompanied this code).
+ * This code is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * version 2 for more details (a copy is included in the LICENSE file that
+ * accompanied this code).
  *
- *  You should have received a copy of the GNU General Public License version
- *  2 along with this work; if not, write to the Free Software Foundation,
- *  Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU General Public License version
+ * 2 along with this work; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- *  Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
- *  or visit www.oracle.com if you need additional information or have any
- *  questions.
+ * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
+ * or visit www.oracle.com if you need additional information or have any
+ * questions.
  */
 
 /*
@@ -29,14 +29,8 @@
  * @run testng/othervm -Djava.lang.invoke.VarHandle.VAR_HANDLE_GUARDS=false -Djava.lang.invoke.VarHandle.VAR_HANDLE_IDENTITY_ADAPT=true -Xverify:all TestMemoryAccess
  */
 
-import jdk.incubator.foreign.GroupLayout;
-import jdk.incubator.foreign.MemoryAddress;
-import jdk.incubator.foreign.MemoryLayout;
-import jdk.incubator.foreign.MemoryLayout.PathElement;
-import jdk.incubator.foreign.MemorySegment;
-import jdk.incubator.foreign.ResourceScope;
-import jdk.incubator.foreign.SequenceLayout;
-import jdk.incubator.foreign.ValueLayout;
+import java.lang.foreign.*;
+import java.lang.foreign.MemoryLayout.PathElement;
 
 import java.lang.invoke.VarHandle;
 import java.nio.ByteOrder;
@@ -55,7 +49,7 @@ public class TestMemoryAccess {
 
     @Test(dataProvider = "elements")
     public void testPaddedAccessByName(Function<MemorySegment, MemorySegment> viewFactory, MemoryLayout elemLayout, Checker checker) {
-        GroupLayout layout = MemoryLayout.structLayout(MemoryLayout.paddingLayout(elemLayout.bitSize()), elemLayout.withName("elem"));
+        GroupLayout layout = MemoryLayout.structLayout(MemoryLayout.paddingLayout(elemLayout.byteSize()), elemLayout.withName("elem"));
         testAccessInternal(viewFactory, layout, layout.varHandle(PathElement.groupElement("elem")), checker);
     }
 
@@ -73,7 +67,7 @@ public class TestMemoryAccess {
 
     @Test(dataProvider = "arrayElements")
     public void testPaddedArrayAccessByName(Function<MemorySegment, MemorySegment> viewFactory, MemoryLayout elemLayout, ArrayChecker checker) {
-        SequenceLayout seq = MemoryLayout.sequenceLayout(10, MemoryLayout.structLayout(MemoryLayout.paddingLayout(elemLayout.bitSize()), elemLayout.withName("elem")));
+        SequenceLayout seq = MemoryLayout.sequenceLayout(10, MemoryLayout.structLayout(MemoryLayout.paddingLayout(elemLayout.byteSize()), elemLayout.withName("elem")));
         testArrayAccessInternal(viewFactory, seq, seq.varHandle(MemoryLayout.PathElement.sequenceElement(), MemoryLayout.PathElement.groupElement("elem")), checker);
     }
 
@@ -85,15 +79,15 @@ public class TestMemoryAccess {
 
     private void testAccessInternal(Function<MemorySegment, MemorySegment> viewFactory, MemoryLayout layout, VarHandle handle, Checker checker) {
         MemorySegment outer_segment;
-        try (ResourceScope scope = ResourceScope.newConfinedScope()) {
-            MemorySegment segment = viewFactory.apply(MemorySegment.allocateNative(layout, scope));
+        try (Arena arena = Arena.ofConfined()) {
+            MemorySegment segment = viewFactory.apply(arena.allocate(layout));
             boolean isRO = segment.isReadOnly();
             try {
                 checker.check(handle, segment);
                 if (isRO) {
                     throw new AssertionError(); //not ok, memory should be immutable
                 }
-            } catch (UnsupportedOperationException ex) {
+            } catch (IllegalArgumentException ex) {
                 if (!isRO) {
                     throw new AssertionError(); //we should not have failed!
                 }
@@ -109,32 +103,32 @@ public class TestMemoryAccess {
         }
         try {
             checker.check(handle, outer_segment);
-            throw new AssertionError(); //not ok, scope is closed
+            throw new AssertionError(); //not ok, session is closed
         } catch (IllegalStateException ex) {
-            //ok, should fail (scope is closed)
+            //ok, should fail (session is closed)
         }
     }
 
     private void testArrayAccessInternal(Function<MemorySegment, MemorySegment> viewFactory, SequenceLayout seq, VarHandle handle, ArrayChecker checker) {
         MemorySegment outer_segment;
-        try (ResourceScope scope = ResourceScope.newConfinedScope()) {
-            MemorySegment segment = viewFactory.apply(MemorySegment.allocateNative(seq, scope));
+        try (Arena arena = Arena.ofConfined()) {
+            MemorySegment segment = viewFactory.apply(arena.allocate(seq));
             boolean isRO = segment.isReadOnly();
             try {
-                for (int i = 0; i < seq.elementCount().getAsLong(); i++) {
+                for (int i = 0; i < seq.elementCount(); i++) {
                     checker.check(handle, segment, i);
                 }
                 if (isRO) {
                     throw new AssertionError(); //not ok, memory should be immutable
                 }
-            } catch (UnsupportedOperationException ex) {
+            } catch (IllegalArgumentException ex) {
                 if (!isRO) {
                     throw new AssertionError(); //we should not have failed!
                 }
                 return;
             }
             try {
-                checker.check(handle, segment, seq.elementCount().getAsLong());
+                checker.check(handle, segment, seq.elementCount());
                 throw new AssertionError(); //not ok, out of bounds
             } catch (IndexOutOfBoundsException ex) {
                 //ok, should fail (out of bounds)
@@ -143,9 +137,9 @@ public class TestMemoryAccess {
         }
         try {
             checker.check(handle, outer_segment, 0);
-            throw new AssertionError(); //not ok, scope is closed
+            throw new AssertionError(); //not ok, session is closed
         } catch (IllegalStateException ex) {
-            //ok, should fail (scope is closed)
+            //ok, should fail (session is closed)
         }
     }
 
@@ -160,7 +154,7 @@ public class TestMemoryAccess {
     @Test(dataProvider = "matrixElements")
     public void testPaddedMatrixAccessByName(Function<MemorySegment, MemorySegment> viewFactory, MemoryLayout elemLayout, MatrixChecker checker) {
         SequenceLayout seq = MemoryLayout.sequenceLayout(20,
-                MemoryLayout.sequenceLayout(10, MemoryLayout.structLayout(MemoryLayout.paddingLayout(elemLayout.bitSize()), elemLayout.withName("elem"))));
+                MemoryLayout.sequenceLayout(10, MemoryLayout.structLayout(MemoryLayout.paddingLayout(elemLayout.byteSize()), elemLayout.withName("elem"))));
         testMatrixAccessInternal(viewFactory, seq,
                 seq.varHandle(
                         PathElement.sequenceElement(), PathElement.sequenceElement(), PathElement.groupElement("elem")),
@@ -179,27 +173,27 @@ public class TestMemoryAccess {
 
     private void testMatrixAccessInternal(Function<MemorySegment, MemorySegment> viewFactory, SequenceLayout seq, VarHandle handle, MatrixChecker checker) {
         MemorySegment outer_segment;
-        try (ResourceScope scope = ResourceScope.newConfinedScope()) {
-            MemorySegment segment = viewFactory.apply(MemorySegment.allocateNative(seq, scope));
+        try (Arena arena = Arena.ofConfined()) {
+            MemorySegment segment = viewFactory.apply(arena.allocate(seq));
             boolean isRO = segment.isReadOnly();
             try {
-                for (int i = 0; i < seq.elementCount().getAsLong(); i++) {
-                    for (int j = 0; j < ((SequenceLayout) seq.elementLayout()).elementCount().getAsLong(); j++) {
+                for (int i = 0; i < seq.elementCount(); i++) {
+                    for (int j = 0; j < ((SequenceLayout) seq.elementLayout()).elementCount(); j++) {
                         checker.check(handle, segment, i, j);
                     }
                 }
                 if (isRO) {
                     throw new AssertionError(); //not ok, memory should be immutable
                 }
-            } catch (UnsupportedOperationException ex) {
+            } catch (IllegalArgumentException ex) {
                 if (!isRO) {
                     throw new AssertionError(); //we should not have failed!
                 }
                 return;
             }
             try {
-                checker.check(handle, segment, seq.elementCount().getAsLong(),
-                        ((SequenceLayout)seq.elementLayout()).elementCount().getAsLong());
+                checker.check(handle, segment, seq.elementCount(),
+                        ((SequenceLayout)seq.elementLayout()).elementCount());
                 throw new AssertionError(); //not ok, out of bounds
             } catch (IndexOutOfBoundsException ex) {
                 //ok, should fail (out of bounds)
@@ -208,9 +202,9 @@ public class TestMemoryAccess {
         }
         try {
             checker.check(handle, outer_segment, 0, 0);
-            throw new AssertionError(); //not ok, scope is closed
+            throw new AssertionError(); //not ok, session is closed
         } catch (IllegalStateException ex) {
-            //ok, should fail (scope is closed)
+            //ok, should fail (session is closed)
         }
     }
 
@@ -259,38 +253,38 @@ public class TestMemoryAccess {
         void check(VarHandle handle, MemorySegment segment);
 
         Checker BYTE = (handle, segment) -> {
-            handle.set(segment, (byte)42);
-            assertEquals(42, (byte)handle.get(segment));
+            handle.set(segment, 0L, (byte)42);
+            assertEquals(42, (byte)handle.get(segment, 0L));
         };
 
         Checker SHORT = (handle, segment) -> {
-            handle.set(segment, (short)42);
-            assertEquals(42, (short)handle.get(segment));
+            handle.set(segment, 0L, (short)42);
+            assertEquals(42, (short)handle.get(segment, 0L));
         };
 
         Checker CHAR = (handle, segment) -> {
-            handle.set(segment, (char)42);
-            assertEquals(42, (char)handle.get(segment));
+            handle.set(segment, 0L, (char)42);
+            assertEquals(42, (char)handle.get(segment, 0L));
         };
 
         Checker INT = (handle, segment) -> {
-            handle.set(segment, 42);
-            assertEquals(42, (int)handle.get(segment));
+            handle.set(segment, 0L, 42);
+            assertEquals(42, (int)handle.get(segment, 0L));
         };
 
         Checker LONG = (handle, segment) -> {
-            handle.set(segment, (long)42);
-            assertEquals(42, (long)handle.get(segment));
+            handle.set(segment, 0L, (long)42);
+            assertEquals(42, (long)handle.get(segment, 0L));
         };
 
         Checker FLOAT = (handle, segment) -> {
-            handle.set(segment, (float)42);
-            assertEquals((float)42, (float)handle.get(segment));
+            handle.set(segment, 0L, (float)42);
+            assertEquals((float)42, (float)handle.get(segment, 0L));
         };
 
         Checker DOUBLE = (handle, segment) -> {
-            handle.set(segment, (double)42);
-            assertEquals((double)42, (double)handle.get(segment));
+            handle.set(segment, 0L, (double)42);
+            assertEquals((double)42, (double)handle.get(segment, 0L));
         };
     }
 
@@ -336,38 +330,38 @@ public class TestMemoryAccess {
         void check(VarHandle handle, MemorySegment segment, long index);
 
         ArrayChecker BYTE = (handle, segment, i) -> {
-            handle.set(segment, i, (byte)i);
-            assertEquals(i, (byte)handle.get(segment, i));
+            handle.set(segment, 0L, i, (byte)i);
+            assertEquals(i, (byte)handle.get(segment, 0L, i));
         };
 
         ArrayChecker SHORT = (handle, segment, i) -> {
-            handle.set(segment, i, (short)i);
-            assertEquals(i, (short)handle.get(segment, i));
+            handle.set(segment, 0L, i, (short)i);
+            assertEquals(i, (short)handle.get(segment, 0L, i));
         };
 
         ArrayChecker CHAR = (handle, segment, i) -> {
-            handle.set(segment, i, (char)i);
-            assertEquals(i, (char)handle.get(segment, i));
+            handle.set(segment, 0L, i, (char)i);
+            assertEquals(i, (char)handle.get(segment, 0L, i));
         };
 
         ArrayChecker INT = (handle, segment, i) -> {
-            handle.set(segment, i, (int)i);
-            assertEquals(i, (int)handle.get(segment, i));
+            handle.set(segment, 0L, i, (int)i);
+            assertEquals(i, (int)handle.get(segment, 0L, i));
         };
 
         ArrayChecker LONG = (handle, segment, i) -> {
-            handle.set(segment, i, (long)i);
-            assertEquals(i, (long)handle.get(segment, i));
+            handle.set(segment, 0L, i, (long)i);
+            assertEquals(i, (long)handle.get(segment, 0L, i));
         };
 
         ArrayChecker FLOAT = (handle, segment, i) -> {
-            handle.set(segment, i, (float)i);
-            assertEquals((float)i, (float)handle.get(segment, i));
+            handle.set(segment, 0L, i, (float)i);
+            assertEquals((float)i, (float)handle.get(segment, 0L, i));
         };
 
         ArrayChecker DOUBLE = (handle, segment, i) -> {
-            handle.set(segment, i, (double)i);
-            assertEquals((double)i, (double)handle.get(segment, i));
+            handle.set(segment, 0L, i, (double)i);
+            assertEquals((double)i, (double)handle.get(segment, 0L, i));
         };
     }
 
@@ -421,48 +415,48 @@ public class TestMemoryAccess {
         void check(VarHandle handle, MemorySegment segment, long row, long col);
 
         MatrixChecker BYTE = (handle, segment, r, c) -> {
-            handle.set(segment, r, c, (byte)(r + c));
-            assertEquals(r + c, (byte)handle.get(segment, r, c));
+            handle.set(segment, 0L, r, c, (byte)(r + c));
+            assertEquals(r + c, (byte)handle.get(segment, 0L, r, c));
         };
 
         MatrixChecker BOOLEAN = (handle, segment, r, c) -> {
-            handle.set(segment, r, c, (r + c) != 0);
-            assertEquals((r + c) != 0, (boolean)handle.get(segment, r, c));
+            handle.set(segment, 0L, r, c, (r + c) != 0);
+            assertEquals((r + c) != 0, (boolean)handle.get(segment, 0L, r, c));
         };
 
         MatrixChecker SHORT = (handle, segment, r, c) -> {
-            handle.set(segment, r, c, (short)(r + c));
-            assertEquals(r + c, (short)handle.get(segment, r, c));
+            handle.set(segment, 0L, r, c, (short)(r + c));
+            assertEquals(r + c, (short)handle.get(segment, 0L, r, c));
         };
 
         MatrixChecker CHAR = (handle, segment, r, c) -> {
-            handle.set(segment, r, c, (char)(r + c));
-            assertEquals(r + c, (char)handle.get(segment, r, c));
+            handle.set(segment, 0L, r, c, (char)(r + c));
+            assertEquals(r + c, (char)handle.get(segment, 0L, r, c));
         };
 
         MatrixChecker INT = (handle, segment, r, c) -> {
-            handle.set(segment, r, c, (int)(r + c));
-            assertEquals(r + c, (int)handle.get(segment, r, c));
+            handle.set(segment, 0L, r, c, (int)(r + c));
+            assertEquals(r + c, (int)handle.get(segment, 0L, r, c));
         };
 
         MatrixChecker LONG = (handle, segment, r, c) -> {
-            handle.set(segment, r, c, r + c);
-            assertEquals(r + c, (long)handle.get(segment, r, c));
+            handle.set(segment, 0L, r, c, r + c);
+            assertEquals(r + c, (long)handle.get(segment, 0L, r, c));
         };
 
         MatrixChecker ADDR = (handle, segment, r, c) -> {
-            handle.set(segment, r, c, MemoryAddress.ofLong(r + c));
-            assertEquals(MemoryAddress.ofLong(r + c), (MemoryAddress)handle.get(segment, r, c));
+            handle.set(segment, 0L, r, c, MemorySegment.ofAddress(r + c));
+            assertEquals(MemorySegment.ofAddress(r + c), (MemorySegment) handle.get(segment, 0L, r, c));
         };
 
         MatrixChecker FLOAT = (handle, segment, r, c) -> {
-            handle.set(segment, r, c, (float)(r + c));
-            assertEquals((float)(r + c), (float)handle.get(segment, r, c));
+            handle.set(segment, 0L, r, c, (float)(r + c));
+            assertEquals((float)(r + c), (float)handle.get(segment, 0L, r, c));
         };
 
         MatrixChecker DOUBLE = (handle, segment, r, c) -> {
-            handle.set(segment, r, c, (double)(r + c));
-            assertEquals((double)(r + c), (double)handle.get(segment, r, c));
+            handle.set(segment, 0L, r, c, (double)(r + c));
+            assertEquals((double)(r + c), (double)handle.get(segment, 0L, r, c));
         };
     }
 }
