@@ -33,6 +33,7 @@ import java.net.http.HttpClient.Version;
 import java.net.http.HttpRequest;
 import java.net.http.HttpRequest.BodyPublisher;
 import java.net.http.HttpRequest.BodyPublishers;
+import java.net.http.HttpOption.Http3DiscoveryMode;
 import java.net.http.HttpResponse;
 import java.net.http.HttpResponse.BodyHandlers;
 import java.nio.charset.StandardCharsets;
@@ -44,7 +45,6 @@ import java.util.List;
 import java.util.Optional;
 import java.util.Random;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.CompletionException;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.atomic.AtomicInteger;
@@ -52,7 +52,9 @@ import java.util.concurrent.atomic.AtomicLong;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 import javax.net.ssl.SSLContext;
+
 import jdk.httpclient.test.lib.common.HttpServerAdapters;
+import jdk.test.lib.Utils;
 import jdk.test.lib.net.SimpleSSLContext;
 import sun.net.NetProperties;
 import sun.net.www.HeaderParser;
@@ -60,8 +62,9 @@ import sun.net.www.HeaderParser;
 import static java.lang.System.out;
 import static java.lang.System.err;
 import static java.lang.String.format;
+import static java.net.http.HttpOption.H3_DISCOVERY;
 
-/**
+/*
  * @test
  * @summary this test verifies that a client may provides authorization
  *          headers directly when connecting with a server.
@@ -189,7 +192,11 @@ public class DigestEchoClient {
     static final ReferenceTracker TRACKER = ReferenceTracker.INSTANCE;
     public HttpClient newHttpClient(DigestEchoServer server) {
         clientCount.incrementAndGet();
-        HttpClient.Builder builder = HttpClient.newBuilder();
+        HttpClient.Builder builder = switch (server.version()) {
+            case HTTP_3 -> HttpServerAdapters.createClientBuilderForH3()
+                    .version(Version.HTTP_3);
+            default -> HttpClient.newBuilder();
+        };
         builder = builder.proxy(ProxySelector.of(null));
         if (useSSL) {
             builder.sslContext(context);
@@ -223,11 +230,11 @@ public class DigestEchoClient {
     }
 
     public static List<Version> serverVersions(Version clientVersion) {
-        if (clientVersion == Version.HTTP_1_1) {
-            return List.of(clientVersion);
-        } else {
-            return List.of(Version.values());
-        }
+        return switch (clientVersion) {
+            case HTTP_1_1 -> List.of(clientVersion);
+            case HTTP_2 -> List.of(Version.HTTP_1_1, Version.HTTP_2);
+            case HTTP_3 -> List.of(Version.HTTP_2, Version.HTTP_3);
+        };
     }
 
     public static List<Version> clientVersions() {
@@ -261,8 +268,9 @@ public class DigestEchoClient {
         }
         try {
             for (DigestEchoServer.HttpAuthType authType : types) {
-                // The test server does not support PROXY305 properly
-                if (authType == DigestEchoServer.HttpAuthType.PROXY305) continue;
+                // The test server does not support PROXY305 or SERVER307 properly
+                if (authType == DigestEchoServer.HttpAuthType.PROXY305 ||
+                    authType == DigestEchoServer.HttpAuthType.SERVER307) continue;
                 EnumSet<DigestEchoServer.HttpAuthSchemeType> basics =
                         EnumSet.of(DigestEchoServer.HttpAuthSchemeType.BASICSERVER,
                                 DigestEchoServer.HttpAuthSchemeType.BASIC);
@@ -272,6 +280,13 @@ public class DigestEchoClient {
                             authType);
                     for (Version clientVersion : clientVersions()) {
                         for (Version serverVersion : serverVersions(clientVersion)) {
+                            if (serverVersion == Version.HTTP_3) {
+                                if (!useSSL) continue;
+                                switch (authType) {
+                                    case PROXY, PROXY305: continue;
+                                    default: break;
+                                }
+                            }
                             for (boolean expectContinue : expectContinue(serverVersion)) {
                                 for (boolean async : BOOLEANS) {
                                     for (boolean preemptive : BOOLEANS) {
@@ -292,6 +307,13 @@ public class DigestEchoClient {
                             authType);
                     for (Version clientVersion : clientVersions()) {
                         for (Version serverVersion : serverVersions(clientVersion)) {
+                            if (serverVersion == Version.HTTP_3) {
+                                if (!useSSL) continue;
+                                switch (authType) {
+                                    case PROXY, PROXY305: continue;
+                                    default: break;
+                                }
+                            }
                             for (boolean expectContinue : expectContinue(serverVersion)) {
                                 for (boolean async : BOOLEANS) {
                                     dec.testDigest(clientVersion, serverVersion,
@@ -313,7 +335,7 @@ public class DigestEchoClient {
             throw t;
         } finally {
             Thread.sleep(100);
-            AssertionError trackFailed = TRACKER.check(500);
+            AssertionError trackFailed = TRACKER.check(Utils.adjustTimeout(1000));
             EchoServers.stop();
             System.out.println(" ---------------------------------------------------------- ");
             System.out.println(String.format("DigestEchoClient %s %s", useSSL ? "SSL" : "CLEAR", types));
@@ -369,6 +391,14 @@ public class DigestEchoClient {
                 .isPresent();
     }
 
+    static Http3DiscoveryMode serverConfig(int step, DigestEchoServer server) {
+        var config = server.serverConfig();
+        return switch (config) {
+            case HTTP_3_URI_ONLY -> config;
+            default -> Http3DiscoveryMode.ALT_SVC;
+        };
+    }
+
     final static AtomicLong basics = new AtomicLong();
     final static AtomicLong basicCount = new AtomicLong();
     // @Test
@@ -412,6 +442,7 @@ public class DigestEchoClient {
                         + ",expectContinue=" + expectContinue + ",version=" + clientVersion);
                 reqURI = URI.create(baseReq + ",basicCount=" + basicCount.get());
                 HttpRequest.Builder builder = HttpRequest.newBuilder(reqURI).version(clientVersion)
+                        .setOption(H3_DISCOVERY, serverConfig(i, server))
                         .POST(reqBody).expectContinue(expectContinue);
                 boolean isTunnel = isProxy(authType) && useSSL;
                 if (addHeaders) {
@@ -449,6 +480,16 @@ public class DigestEchoClient {
                         out.printf("%s client.send(%s)%n", DigestEchoServer.now(), request);
                         resp = client.send(request, BodyHandlers.ofLines());
                     }
+                    if (serverVersion == Version.HTTP_3 && clientVersion == Version.HTTP_3) {
+                        out.println("Response version [" + i + "]: " + resp.version());
+                        int required = isRedirecting(authType) ? 1 : 0;
+                        if (i > required) {
+                            if (resp.version() != serverVersion) {
+                                throw new AssertionError("Expected HTTP/3, but got: "
+                                        + resp.version());
+                            }
+                        }
+                    }
                 } catch (Throwable t) {
                     long stop = System.nanoTime();
                     synchronized (basicCount) {
@@ -470,6 +511,7 @@ public class DigestEchoClient {
                     reqURI = URI.create(baseReq + ",withAuthorization="
                             + authType + ",basicCount=" + basicCount.get());
                     request = HttpRequest.newBuilder(reqURI).version(clientVersion)
+                            .setOption(H3_DISCOVERY, server.serverConfig())
                             .POST(reqBody).header(authorizationKey(authType), auth).build();
                     if (async) {
                         out.printf("%s client.sendAsync(%s)%n", DigestEchoServer.now(), request);
@@ -478,6 +520,16 @@ public class DigestEchoClient {
                         out.printf("%s client.send(%s)%n", DigestEchoServer.now(), request);
                         resp = client.send(request, BodyHandlers.ofLines());
                     }
+                    if (serverVersion == Version.HTTP_3 && clientVersion == Version.HTTP_3) {
+                        out.println("Response version [" + i + "]: " + resp.version());
+                        int required = isRedirecting(authType) ? 1 : 0;
+                        if (i > required) {
+                            if (resp.version() != serverVersion) {
+                                throw new AssertionError("Expected HTTP/3, but got: "
+                                        + resp.version());
+                            }
+                        }
+                    }
                 }
                 final List<String> respLines;
                 try {
@@ -519,16 +571,19 @@ public class DigestEchoClient {
             failed = decorated;
             throw decorated;
         } finally {
-            client = null;
-            System.gc();
-            while (!ref.refersTo(null)) {
+            if (client != null) {
+                var tracker = TRACKER.getTracker(client);
+                client = null;
                 System.gc();
-                if (queue.remove(100) == ref) break;
-            }
-            var error = TRACKER.checkShutdown(900);
-            if (error != null) {
-                if (failed != null) error.addSuppressed(failed);
-                throw error;
+                while (!ref.refersTo(null)) {
+                    System.gc();
+                    if (queue.remove(100) == ref) break;
+                }
+                var error = TRACKER.checkShutdown(tracker, Utils.adjustTimeout(900), false);
+                if (error != null) {
+                    if (failed != null) error.addSuppressed(failed);
+                    throw error;
+                }
             }
         }
         System.out.println("OK");
@@ -562,13 +617,13 @@ public class DigestEchoClient {
                 server.getServerAddress(), "/foo/");
 
         HttpClient client = newHttpClient(server);
+        ReferenceQueue<HttpClient> queue = new ReferenceQueue<>();
+        WeakReference<HttpClient> ref = new WeakReference<>(client, queue);
         HttpResponse<String> r;
         CompletableFuture<HttpResponse<String>> cf1;
         byte[] cnonce = new byte[16];
         String cnonceStr = null;
         DigestEchoServer.DigestResponse challenge = null;
-        ReferenceQueue<HttpClient> queue = new ReferenceQueue<>();
-        WeakReference<HttpClient> ref = new WeakReference<>(client, queue);
         URI reqURI = null;
         Throwable failed = null;
         try {
@@ -583,6 +638,7 @@ public class DigestEchoClient {
                 reqURI = URI.create(baseReq + ",digestCount=" + digestCount.get());
                 HttpRequest.Builder reqBuilder = HttpRequest
                         .newBuilder(reqURI).version(clientVersion).POST(reqBody)
+                        .setOption(H3_DISCOVERY, serverConfig(i, server))
                         .expectContinue(expectContinue);
 
                 boolean isTunnel = isProxy(authType) && useSSL;
@@ -611,6 +667,16 @@ public class DigestEchoClient {
                     out.printf("%s client.send(%s)%n", DigestEchoServer.now(), request);
                     resp = client.send(request, BodyHandlers.ofLines());
                 }
+                if (serverVersion == Version.HTTP_3 && clientVersion == Version.HTTP_3) {
+                    out.println("Response version [" + i + "]: " + resp.version());
+                    int required = isRedirecting(authType) ? 1 : 0;
+                    if (i > required) {
+                        if (resp.version() != serverVersion) {
+                            throw new AssertionError("Expected HTTP/3, but got: "
+                                    + resp.version());
+                        }
+                    }
+                }
                 System.out.println(resp);
                 assert challenge != null || resp.statusCode() == 401 || resp.statusCode() == 407
                         : "challenge=" + challenge + ", resp=" + resp + ", test=[" + test + "]";
@@ -641,6 +707,7 @@ public class DigestEchoClient {
                     reqURI = URI.create(baseReq + ",withAuth=" + authType + ",digestCount=" + digestCount.get());
                     try {
                         request = HttpRequest.newBuilder(reqURI).version(clientVersion)
+                                .setOption(H3_DISCOVERY, serverConfig(i, server))
                                 .POST(reqBody).header(authorizationKey(authType), auth).build();
                     } catch (IllegalArgumentException x) {
                         throw x;
@@ -653,6 +720,16 @@ public class DigestEchoClient {
                         resp = client.send(request, BodyHandlers.ofLines());
                     }
                     System.out.println(resp);
+                    if (serverVersion == Version.HTTP_3 && clientVersion == Version.HTTP_3) {
+                        out.println("Response version [" + i + "]: " + resp.version());
+                        int required = isRedirecting(authType) ? 1 : 0;
+                        if (i > required) {
+                            if (resp.version() != serverVersion) {
+                                throw new AssertionError("Expected HTTP/3, but got: "
+                                        + resp.version());
+                            }
+                        }
+                    }
                 }
                 final List<String> respLines;
                 try {
@@ -690,24 +767,27 @@ public class DigestEchoClient {
             failed = decorated;
             throw decorated;
         } finally {
-            client = null;
-            System.gc();
-            while (!ref.refersTo(null)) {
+            if (client != null) {
+                var tracker = TRACKER.getTracker(client);
+                client = null;
                 System.gc();
-                if (queue.remove(100) == ref) break;
-            }
-            var error = TRACKER.checkShutdown(900);
-            if (error != null) {
-                if (failed != null) {
-                    error.addSuppressed(failed);
+                while (!ref.refersTo(null)) {
+                    System.gc();
+                    if (queue.remove(100) == ref) break;
+                }
+                var error = TRACKER.checkShutdown(tracker, Utils.adjustTimeout(900), false);
+                if (error != null) {
+                    if (failed != null) {
+                        error.addSuppressed(failed);
+                    }
+                    throw error;
                 }
-                throw error;
             }
         }
         System.out.println("OK");
     }
 
-    // WARNING: This is not a full fledged implementation of DIGEST.
+    // WARNING: This is not a full-fledged implementation of DIGEST.
     // It does contain bugs and inaccuracy.
     static String digestResponse(URI uri, String method, DigestEchoServer.DigestResponse challenge, String cnonce)
             throws NoSuchAlgorithmException {
@@ -728,32 +808,34 @@ public class DigestEchoClient {
     }
 
     static String authenticateKey(DigestEchoServer.HttpAuthType authType) {
-        switch (authType) {
-            case SERVER: return "www-authenticate";
-            case SERVER307: return "www-authenticate";
-            case PROXY: return "proxy-authenticate";
-            case PROXY305: return "proxy-authenticate";
-            default: throw new InternalError("authType: " + authType);
-        }
+        return switch (authType) {
+            case SERVER    -> "www-authenticate";
+            case SERVER307 -> "www-authenticate";
+            case PROXY     -> "proxy-authenticate";
+            case PROXY305  -> "proxy-authenticate";
+        };
     }
 
     static String authorizationKey(DigestEchoServer.HttpAuthType authType) {
-        switch (authType) {
-            case SERVER: return "authorization";
-            case SERVER307: return "Authorization";
-            case PROXY: return "Proxy-Authorization";
-            case PROXY305: return "proxy-Authorization";
-            default: throw new InternalError("authType: " + authType);
-        }
+        return switch (authType) {
+            case SERVER    -> "authorization";
+            case SERVER307 -> "Authorization";
+            case PROXY     -> "Proxy-Authorization";
+            case PROXY305  -> "proxy-Authorization";
+        };
     }
 
     static boolean isProxy(DigestEchoServer.HttpAuthType authType) {
-        switch (authType) {
-            case SERVER: return false;
-            case SERVER307: return false;
-            case PROXY: return true;
-            case PROXY305: return true;
-            default: throw new InternalError("authType: " + authType);
-        }
+        return switch (authType) {
+            case SERVER, SERVER307 -> false;
+            case PROXY,  PROXY305  -> true;
+        };
+    }
+
+    static boolean isRedirecting(DigestEchoServer.HttpAuthType authType) {
+        return switch (authType) {
+            case SERVER307, PROXY305 -> true;
+            case SERVER,    PROXY    -> false;
+        };
     }
 }
