@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2021, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -21,17 +21,6 @@
  * questions.
  */
 
-/*
- * @test
- * @bug 8216498
- * @summary Tests Exception detail message when too few response bytes are
- *          received before a socket exception or eof.
- * @library /test/lib
- * @build jdk.test.lib.net.SimpleSSLContext
- * @run testng/othervm
- *       -Djdk.httpclient.HttpClient.log=headers,errors,channel
- *       ShortResponseBody
- */
 
 import java.io.IOException;
 import java.io.InputStream;
@@ -44,12 +33,9 @@ import java.net.Socket;
 import java.net.URI;
 import java.net.http.HttpClient;
 import java.net.http.HttpRequest;
-import java.net.http.HttpRequest.BodyPublishers;
 import java.net.http.HttpResponse;
-import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
-import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ThreadFactory;
@@ -57,6 +43,7 @@ import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
+
 import jdk.test.lib.net.SimpleSSLContext;
 import org.testng.ITestContext;
 import org.testng.ITestResult;
@@ -81,7 +68,7 @@ import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.fail;
 
-public class ShortResponseBody {
+public abstract class ShortResponseBody {
 
     Server closeImmediatelyServer;
     Server closeImmediatelyHttpsServer;
@@ -97,11 +84,21 @@ public class ShortResponseBody {
 
     SSLContext sslContext;
     SSLParameters sslParameters;
+    HttpClient client;
+    int numberOfRequests;
+
+    static final int REQUESTS_PER_CLIENT = 10; // create new client every 10 requests
+    static final long PAUSE_FOR_GC = 5; // 5ms to let gc work
+    static final long PAUSE_FOR_PEER = 5; // 5ms to let server react
 
     static final String EXPECTED_RESPONSE_BODY =
             "<html><body><h1>Heading</h1><p>Some Text</p></body></html>";
 
-    final static AtomicLong ids = new AtomicLong();
+    // A request number used to replace %reqnb% in URLs with a unique
+    // number for better log analysis
+    static final AtomicLong reqnb = new AtomicLong();
+
+    static final AtomicLong ids = new AtomicLong();
     final ThreadFactory factory = new ThreadFactory() {
         @Override
         public Thread newThread(Runnable r) {
@@ -125,6 +122,18 @@ public class ShortResponseBody {
 
     @BeforeMethod
     void beforeMethod(ITestContext context) {
+        if (client == null || numberOfRequests == REQUESTS_PER_CLIENT) {
+            numberOfRequests = 0;
+            out.println("--- new client");
+            client = newHttpClient();
+            System.gc();
+            try {
+                Thread.sleep(PAUSE_FOR_GC);
+            } catch (InterruptedException x) {
+
+            }
+        }
+        numberOfRequests++;
         if (context.getFailedTests().size() > 0) {
             if (skiptests.get() == null) {
                 SkipException skip = new SkipException("some tests failed");
@@ -162,10 +171,15 @@ public class ShortResponseBody {
         };
     }
 
+    public static String uniqueURL(String url) {
+        return url.replace("%reqnb%", String.valueOf(reqnb.incrementAndGet()));
+    }
+
     @Test(dataProvider = "sanity")
     void sanity(String url) throws Exception {
-        HttpClient client = newHttpClient();
+        url = uniqueURL(url);
         HttpRequest request = HttpRequest.newBuilder(URI.create(url)).build();
+        out.println("Request: " + request);
         HttpResponse<String> response = client.send(request, ofString());
         String body = response.body();
         assertEquals(body, EXPECTED_RESPONSE_BODY);
@@ -175,6 +189,25 @@ public class ShortResponseBody {
                 .join();
     }
 
+    @DataProvider(name = "sanityBadRequest")
+    public Object[][] sanityBadRequest() {
+        return new Object[][]{
+                { httpURIVarLen  }, // no query string
+                { httpsURIVarLen },
+                { httpURIFixLen  },
+        };
+    }
+
+    @Test(dataProvider = "sanityBadRequest")
+    void sanityBadRequest(String url) throws Exception {
+        url = uniqueURL(url);
+        HttpRequest request = HttpRequest.newBuilder(URI.create(url)).build();
+        out.println("Request: " + request);
+        HttpResponse<String> response = client.send(request, ofString());
+        assertEquals(response.statusCode(), 400);
+        assertEquals(response.body(), "");
+    }
+
     @DataProvider(name = "uris")
     public Object[][] variants(ITestContext context) {
         String[][] cases = new String[][] {
@@ -242,18 +275,9 @@ public class ShortResponseBody {
             return new Object[0][];
         }
 
-        List<Object[]> list = new ArrayList<>();
-        Arrays.asList(cases).stream()
-                .map(e -> new Object[] {e[0], e[1], true})  // reuse client
-                .forEach(list::add);
-        Arrays.asList(cases).stream()
-                .map(e -> new Object[] {e[0], e[1], false}) // do not reuse client
-                .forEach(list::add);
-        return list.stream().toArray(Object[][]::new);
+        return cases;
     }
 
-    static final int ITERATION_COUNT = 3;
-
     HttpClient newHttpClient() {
         return HttpClient.newBuilder()
                 .proxy(NO_PROXY)
@@ -263,61 +287,6 @@ public class ShortResponseBody {
                 .build();
     }
 
-    @Test(dataProvider = "uris")
-    void testSynchronousGET(String url, String expectedMsg, boolean sameClient)
-        throws Exception
-    {
-        checkSkip();
-        out.print("---\n");
-        HttpClient client = null;
-        for (int i=0; i< ITERATION_COUNT; i++) {
-            if (!sameClient || client == null)
-                client = newHttpClient();
-            HttpRequest request = HttpRequest.newBuilder(URI.create(url)).build();
-            try {
-                HttpResponse<String> response = client.send(request, ofString());
-                String body = response.body();
-                out.println(response + ": " + body);
-                fail("UNEXPECTED RESPONSE: " + response);
-            } catch (IOException ioe) {
-                out.println("Caught expected exception:" + ioe);
-                assertExpectedMessage(request, ioe, expectedMsg);
-                // synchronous API must have the send method on the stack
-                assertSendMethodOnStack(ioe);
-                assertNoConnectionExpiredException(ioe);
-            }
-        }
-    }
-
-    @Test(dataProvider = "uris")
-    void testAsynchronousGET(String url, String expectedMsg, boolean sameClient)
-        throws Exception
-    {
-        checkSkip();
-        out.print("---\n");
-        HttpClient client = null;
-        for (int i=0; i< ITERATION_COUNT; i++) {
-            if (!sameClient || client == null)
-                client = newHttpClient();
-            HttpRequest request = HttpRequest.newBuilder(URI.create(url)).build();
-            try {
-                HttpResponse<String> response = client.sendAsync(request, ofString()).get();
-                String body = response.body();
-                out.println(response + ": " + body);
-                fail("UNEXPECTED RESPONSE: " + response);
-            } catch (ExecutionException ee) {
-                if (ee.getCause() instanceof IOException) {
-                    IOException ioe = (IOException) ee.getCause();
-                    out.println("Caught expected exception:" + ioe);
-                    assertExpectedMessage(request, ioe, expectedMsg);
-                    assertNoConnectionExpiredException(ioe);
-                } else {
-                    throw ee;
-                }
-            }
-        }
-    }
-
     // can be used to prolong request body publication
     static final class InfiniteInputStream extends InputStream {
         int count = 0;
@@ -331,6 +300,11 @@ public class ShortResponseBody {
                 k16++;
                 System.out.println("... 16K sent.");
                 count = count % (16 * 1024);
+                try {
+                    Thread.sleep(PAUSE_FOR_PEER);
+                } catch (InterruptedException x) {
+                    // ignore
+                }
             }
             if (k16 > 128) {
                 System.out.println("WARNING: InfiniteInputStream: " +
@@ -353,6 +327,11 @@ public class ShortResponseBody {
                 k16++;
                 System.out.println("... 16K sent.");
                 count = count % (16 * 1024);
+                try {
+                    Thread.sleep(PAUSE_FOR_PEER);
+                } catch (InterruptedException x) {
+                    // ignore
+                }
             }
             if (k16 > 128) {
                 System.out.println("WARNING: InfiniteInputStream: " +
@@ -364,86 +343,6 @@ public class ShortResponseBody {
         }
     }
 
-    // POST tests are racy in what may be received before writing may cause a
-    // broken pipe or reset exception, before all the received data can be read.
-    // Any message up to, and including, the "expected" error message can occur.
-    // Strictly ordered list, in order of possible occurrence.
-    static final List<String> MSGS_ORDER =
-            List.of("no bytes", "status line", "header");
-
-
-    @Test(dataProvider = "uris")
-    void testSynchronousPOST(String url, String expectedMsg, boolean sameClient)
-        throws Exception
-    {
-        checkSkip();
-        out.print("---\n");
-        HttpClient client = null;
-        for (int i=0; i< ITERATION_COUNT; i++) {
-            if (!sameClient || client == null)
-                client = newHttpClient();
-            HttpRequest request = HttpRequest.newBuilder(URI.create(url))
-                    .POST(BodyPublishers.ofInputStream(() -> new InfiniteInputStream()))
-                    .build();
-            try {
-                HttpResponse<String> response = client.send(request, ofString());
-                String body = response.body();
-                out.println(response + ": " + body);
-                fail("UNEXPECTED RESPONSE: " + response);
-            } catch (IOException ioe) {
-                out.println("Caught expected exception:" + ioe);
-
-                List<String> expectedMessages = new ArrayList<>();
-                expectedMessages.add(expectedMsg);
-                MSGS_ORDER.stream().takeWhile(s -> !s.equals(expectedMsg))
-                                   .forEach(expectedMessages::add);
-
-                assertExpectedMessage(request, ioe, expectedMessages);
-                // synchronous API must have the send method on the stack
-                assertSendMethodOnStack(ioe);
-                assertNoConnectionExpiredException(ioe);
-            }
-        }
-    }
-
-    @Test(dataProvider = "uris")
-    void testAsynchronousPOST(String url, String expectedMsg, boolean sameClient)
-        throws Exception
-    {
-        checkSkip();
-        out.print("---\n");
-        HttpClient client = null;
-        for (int i=0; i< ITERATION_COUNT; i++) {
-            if (!sameClient || client == null)
-                client = newHttpClient();
-            HttpRequest request = HttpRequest.newBuilder(URI.create(url))
-                    .POST(BodyPublishers.ofInputStream(() -> new InfiniteInputStream()))
-                    .build();
-            try {
-                HttpResponse<String> response = client.sendAsync(request, ofString()).get();
-                String body = response.body();
-                out.println(response + ": " + body);
-                fail("UNEXPECTED RESPONSE: " + response);
-            } catch (ExecutionException ee) {
-                if (ee.getCause() instanceof IOException) {
-                    IOException ioe = (IOException) ee.getCause();
-                    out.println("Caught expected exception:" + ioe);
-
-                    List<String> expectedMessages = new ArrayList<>();
-                    expectedMessages.add(expectedMsg);
-                    MSGS_ORDER.stream().takeWhile(s -> !s.equals(expectedMsg))
-                            .forEach(expectedMessages::add);
-
-                    assertExpectedMessage(request, ioe, expectedMessages);
-                    assertNoConnectionExpiredException(ioe);
-                } else {
-                    throw ee;
-                }
-            }
-        }
-    }
-
-
     void assertExpectedMessage(HttpRequest request, Throwable t, String expected) {
         if (request.uri().getScheme().equalsIgnoreCase("https")
                 && (t instanceof SSLHandshakeException)) {
@@ -593,6 +492,11 @@ public class ShortResponseBody {
             this.name = name;
         }
 
+        private static final String BAD_REQUEST_RESPONSE =
+                "HTTP/1.1 400 Bad Request\r\n" +
+                        "Content-Length: 0\r\n" +
+                        "Connection: close\r\n\r\n";
+
         abstract String response();
 
         @Override
@@ -609,9 +513,11 @@ public class ShortResponseBody {
 
                     String query = uriPath.getRawQuery();
                     if (query == null) {
-                        out.println("Request headers: [" + headers + "]");
+                        out.println("Unexpected request without query string received. Got headers: [" + headers + "]");
+                        out.println("Replying with 400 Bad Request");
+                        writeResponse(s, BAD_REQUEST_RESPONSE, BAD_REQUEST_RESPONSE.length());
+                        continue;
                     }
-                    assert query != null : "null query for uriPath: " + uriPath;
                     String qv = query.split("=")[1];
                     int len;
                     if (qv.equals("all")) {
@@ -620,13 +526,8 @@ public class ShortResponseBody {
                         len = Integer.parseInt(query.split("=")[1]);
                     }
 
-                    OutputStream os = s.getOutputStream();
                     out.println(name + ": writing " + len  + " bytes");
-                    byte[] responseBytes = response().getBytes(US_ASCII);
-                    for (int i = 0; i< len; i++) {
-                        os.write(responseBytes[i]);
-                        os.flush();
-                    }
+                    writeResponse(s, response(), len);
                 } catch (Throwable e) {
                     if (!closed) {
                         out.println("Unexpected exception in server: " + e);
@@ -637,6 +538,13 @@ public class ShortResponseBody {
             }
         }
 
+        private static void writeResponse(Socket socket, String response, int len) throws IOException {
+            OutputStream os = socket.getOutputStream();
+            byte[] responseBytes = response.getBytes(US_ASCII);
+            os.write(responseBytes, 0, len);
+            os.flush();
+        }
+
         static final byte[] requestEnd = new byte[] { '\r', '\n', '\r', '\n' };
 
         // Read the request method
@@ -758,23 +666,23 @@ public class ShortResponseBody {
 
         closeImmediatelyServer = new PlainCloseImmediatelyServer();
         httpURIClsImed = "http://" + serverAuthority(closeImmediatelyServer)
-                + "/http1/closeImmediately/foo";
+                + "/http1/closeImmediately/req=%reqnb%/foo";
 
         closeImmediatelyHttpsServer = new SSLCloseImmediatelyServer();
         httpsURIClsImed = "https://" + serverAuthority(closeImmediatelyHttpsServer)
-                + "/https1/closeImmediately/foo";
+                + "/https1/closeImmediately/req=%reqnb%/foo";
 
         variableLengthServer = new PlainVariableLengthServer();
         httpURIVarLen = "http://" + serverAuthority(variableLengthServer)
-                + "/http1/variable/bar";
+                + "/http1/variable/req=%reqnb%/bar";
 
         variableLengthHttpsServer = new SSLVariableLengthServer();
         httpsURIVarLen = "https://" + serverAuthority(variableLengthHttpsServer)
-                + "/https1/variable/bar";
+                + "/https1/variable/req=%reqnb%/bar";
 
         fixedLengthServer = new FixedLengthServer();
         httpURIFixLen = "http://" + serverAuthority(fixedLengthServer)
-                + "/http1/fixed/baz";
+                + "/http1/fixed/req=%reqnb%/baz";
     }
 
     @AfterTest
