@@ -26,7 +26,7 @@
  * @bug 4691089 4819436 4942982 5104960 6544471 6627549 7066203 7195759
  *      8039317 8074350 8074351 8145952 8187946 8193552 8202026 8204269
  *      8208746 8209775 8264792 8274658 8283277 8296239 8321480 8334653
- *      8354343 8354344 8356096
+ *      8354343 8354344 8356096 8368308
  * @summary Validate ISO 4217 data for Currency class.
  * @modules java.base/java.util:open
  *          jdk.localedata
@@ -116,7 +116,7 @@ public class ValidateISO4217 {
     private static final Set<String> currenciesNotYetDefined = new HashSet<>();
     // Codes that are obsolete, do not have related country, extra currency
     private static final String otherCodes =
-            "ADP-AFA-ATS-AYM-AZM-BEF-BGL-BOV-BYB-BYR-CHE-CHW-CLF-COU-CUC-CYP-"
+            "ADP-AFA-ATS-AYM-AZM-BEF-BGL-BGN-BOV-BYB-BYR-CHE-CHW-CLF-COU-CUC-CYP-"
                     + "DEM-EEK-ESP-FIM-FRF-GHC-GRD-GWP-HRK-IEP-ITL-LTL-LUF-LVL-MGF-MRO-MTL-MXV-MZM-NLG-"
                     + "PTE-ROL-RUR-SDD-SIT-SLL-SKK-SRG-STD-TMM-TPE-TRL-VEF-UYI-USN-USS-VEB-VED-"
                     + "XAD-XAG-XAU-XBA-XBB-XBC-XBD-XDR-XFO-XFU-XPD-XPT-XSU-XTS-XUA-XXX-"
@@ -131,7 +131,7 @@ public class ValidateISO4217 {
              */
             {"GS", "GBP", "826", "2"},      // South Georgia And The South Sandwich Islands
             /* Not defined in ISO 4217 list, but defined in .properties file. */
-            {"AX", "EUR", "978", "2"},      // \u00c5LAND ISLANDS
+            {"AX", "EUR", "978", "2"},      // ÅLAND ISLANDS
             {"PS", "ILS", "376", "2"},      // Palestinian Territory, Occupied
             /* Not defined in ISO 4217 list, but added in ISO 3166 country code list */
             {"JE", "GBP", "826", "2"},      // Jersey
