@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2022, 2024, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -23,7 +23,7 @@
 
 /*
  * @test
- * @summary Testing Classfile bootstrap methods.
+ * @summary Testing ClassFile bootstrap methods.
  * @run junit BSMTest
  */
 import java.lang.constant.ClassDesc;
@@ -35,14 +35,14 @@ import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.List;
 
-import jdk.internal.classfile.*;
+import java.lang.classfile.*;
 import helpers.ByteArrayClassLoader;
-import jdk.internal.classfile.instruction.ConstantInstruction;
-import jdk.internal.classfile.constantpool.ConstantDynamicEntry;
-import jdk.internal.classfile.constantpool.ConstantPoolBuilder;
-import jdk.internal.classfile.constantpool.LoadableConstantEntry;
-import jdk.internal.classfile.constantpool.MemberRefEntry;
-import jdk.internal.classfile.constantpool.MethodHandleEntry;
+import java.lang.classfile.instruction.ConstantInstruction;
+import java.lang.classfile.constantpool.ConstantDynamicEntry;
+import java.lang.classfile.constantpool.ConstantPoolBuilder;
+import java.lang.classfile.constantpool.LoadableConstantEntry;
+import java.lang.classfile.constantpool.MemberRefEntry;
+import java.lang.classfile.constantpool.MethodHandleEntry;
 import org.junit.jupiter.api.Test;
 
 import static java.lang.constant.ConstantDescs.CD_String;
@@ -58,9 +58,9 @@ public class BSMTest {
 
     @Test
     void testSevenOfThirteenIterator() throws Exception {
-        var cc = Classfile.of();
+        var cc = ClassFile.of();
         ClassModel cm = cc.parse(testClassPath);
-        byte[] newBytes = cc.transform(cm, (cb, ce) -> {
+        byte[] newBytes = cc.transformClass(cm, (cb, ce) -> {
             if (ce instanceof MethodModel mm) {
                 cb.transformMethod(mm, (mb, me) -> {
                     if (me instanceof CodeModel xm) {
@@ -78,7 +78,7 @@ public class BSMTest {
                                     BootstrapMethodEntry bme = cpb.bsmEntry(methodHandleEntry, staticArgs);
                                     ConstantDynamicEntry cde = cpb.constantDynamicEntry(bme, cpb.nameAndTypeEntry("name", CD_String));
 
-                                    codeB.constantInstruction(Opcode.LDC, cde.constantValue());
+                                    codeB.ldc(cde.constantValue());
                                 }
                                 default -> codeB.with(codeE);
                             }
