@@ -23,7 +23,7 @@
 
 /*
  * @test
- * @bug 8335927
+ * @bug 8335927 8345773
  * @summary Testing ClassFile ClassPrinter.
  * @run junit ClassPrinterTest
  */
@@ -35,7 +35,7 @@ import java.util.List;
 import java.util.Optional;
 import java.lang.classfile.*;
 import java.lang.classfile.attribute.*;
-import java.lang.classfile.components.ClassPrinter;
+import jdk.internal.classfile.components.ClassPrinter;
 import java.lang.constant.DirectMethodHandleDesc;
 import java.lang.constant.DynamicCallSiteDesc;
 import java.lang.constant.MethodHandleDesc;
@@ -122,8 +122,7 @@ class ClassPrinterTest {
 
     @Test
     void testPrintYamlTraceAll() throws IOException {
-        var out = new StringBuilder();
-        ClassPrinter.toYaml(getClassModel(), ClassPrinter.Verbosity.TRACE_ALL, out::append);
+        var out = getClassModel().toDebugString();
         assertOut(out,
                 """
                   - class name: Foo
@@ -472,7 +471,7 @@ class ClassPrinterTest {
                     "source file": "Foo.java",
                     "inner classes": [
                         {"inner class": "Phee", "outer class": "Phoo", "inner name": "InnerName", "flags": ["PROTECTED"]},
-                        {"inner class": "Phoo", "outer class": "null", "inner name": "null", "flags": ["PRIVATE"]}],
+                        {"inner class": "Phoo", "outer class": null, "inner name": null, "flags": ["PRIVATE"]}],
                     "enclosing method": {"class": "Phee", "method name": "enclosingMethod", "method type": "(Ljava/util/Collection;)Ljava/lang/Double;"},
                     "signature": "LBoo;LPhee;LPhoo;",
                     "nest host": "Phee",
@@ -725,7 +724,7 @@ class ClassPrinterTest {
                     <source_file>Foo.java</source_file>
                     <inner_classes>
                         <cls><inner_class>Phee</inner_class><outer_class>Phoo</outer_class><inner_name>InnerName</inner_name><flags><flag>PROTECTED</flag></flags></cls>
-                        <cls><inner_class>Phoo</inner_class><outer_class>null</outer_class><inner_name>null</inner_name><flags><flag>PRIVATE</flag></flags></cls></inner_classes>
+                        <cls><inner_class>Phoo</inner_class><outer_class><null/></outer_class><inner_name><null/></inner_name><flags><flag>PRIVATE</flag></flags></cls></inner_classes>
                     <enclosing_method><class>Phee</class><method_name>enclosingMethod</method_name><method_type>(Ljava/util/Collection;)Ljava/lang/Double;</method_type></enclosing_method>
                     <signature>LBoo;LPhee;LPhoo;</signature>
                     <nest_host>Phee</nest_host>
@@ -904,7 +903,7 @@ class ClassPrinterTest {
         assertEquals(node.walk().count(), 42);
     }
 
-    private static void assertOut(StringBuilder out, String expected) {
+    private static void assertOut(CharSequence out, String expected) {
 //        System.out.println("-----------------");
 //        System.out.println(out.toString());
 //        System.out.println("-----------------");
