@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013, 2025, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -38,7 +38,7 @@ import jdk.test.lib.jfr.Events;
 
 /**
  * @test
- * @key jfr
+ * @requires vm.flagless
  * @requires vm.hasJFR
  * @library /test/lib
  * @run main/othervm jdk.jfr.event.runtime.TestExceptionEvents
@@ -102,7 +102,7 @@ public class TestExceptionEvents {
                 RecordedStackTrace rs = e.getStackTrace();
                 RecordedClass rc = e.getValue("thrownClass");
                 List<RecordedFrame> frames = rs.getFrames();
-                RecordedFrame topFrame = frames.get(0);
+                RecordedFrame topFrame = frames.getFirst();
                 System.out.println(rc.getName() + " Top frame: " + topFrame.getMethod().getName());
                 if (!topFrame.getMethod().getName().equals("<init>")) {
                     throw new Exception("Expected name of top frame to be <init>");
